//
//  IOButton.swift
//  InspectionApp
//
//  Created by Devesh Mevada on 3/10/20.
//  Copyright © 2020 Devesh Mevada. All rights reserved.
//

import Foundation
import UIKit

/// UIButton used for dropping shadow to button
extension UIButton {
    
    func basicDropShadow() {
        layer.shadowOpacity = 0.7
        layer.shadowRadius = 5.0
        layer.cornerRadius = 15.0
        layer.shadowOffset = CGSize(width: 0.0, height: 0.0)
        layer.shadowColor = #colorLiteral(red: 0.431372549, green: 0.7058823529, blue: 0.2470588235, alpha: 1)
    }
    
    func centerVertically(padding: CGFloat = 6.0) {
        guard
            let imageViewSize = self.imageView?.frame.size,
            let titleLabelSize = self.titleLabel?.frame.size else {
            return
        }

        self.titleLabel?.numberOfLines = 0
        self.titleLabel?.textAlignment = .center
        let totalHeight = imageViewSize.height + titleLabelSize.height + padding

        self.imageEdgeInsets = UIEdgeInsets(
            top: -(totalHeight - imageViewSize.height),
            left: 0.0,
            bottom: 0.0,
            right: -titleLabelSize.width
        )

        self.titleEdgeInsets = UIEdgeInsets(
            top: 0.0,
            left: -imageViewSize.width,
            bottom: -(totalHeight - titleLabelSize.height),
            right: 0.0
        )

        self.contentEdgeInsets = UIEdgeInsets(
            top: 0.0,
            left: 0.0,
            bottom: titleLabelSize.height,
            right: 0.0
        )
    }

}
