//
//  SignalRService.swift
//  EBV-IoT
//
//  Created by Milan Patel on 15/02/22.
//

import Foundation
import SignalRClient

public protocol SignalRServiceDelegate: AnyObject {
    func passData2(dict:[String: Any])
}

public class SignalRService {
    public var connection: HubConnection
    public weak var delegate: SignalRServiceDelegate?

    public init(url: URL) {
        connection = HubConnectionBuilder(url: url).withLogging(minLogLevel: .error).build()
        connection.on(method: "ReceiveMessage", callback: { (message: Result) in
            do {
                self.handleMessage(message)
            }
            catch {
                print(error)
            }
        })

        connection.start()
    }
    
    private func handleMessage(_ message: Result) {
        // Do something with the message.
      //  print("message: \(message)")
        do {
            if let jsonData = try JSONSerialization.jsonObject(with: message.message.data(using: .utf8)!, options : .allowFragments) as? [String: Any]
            {
                let dic : [String : Any] = jsonData["data"] as! [String : Any]
                delegate?.passData2(dict: dic)
            } else {
                print("bad json")
            }
        } catch let error as NSError {
            print(error)
        }
    }
}

struct Result : Decodable {
    let message : String
}
