//
//  ProgressHud.swift
//  Procell
//
//  Created by Megha Salla on 10/11/21.

import Foundation
import UIKit
import MBProgressHUD

class ProHudManager: NSObject {
    static let shared = ProHudManager()
    private var hud: MBProgressHUD?
    
    public func show(view: UIView? = nil, animated: Bool = true) {
        DispatchQueue.main.async {
            if let window = UIApplication.shared.delegate?.window!, view == nil {
                self.hud = MBProgressHUD.showAdded(to: window, animated: animated)
            } else {
                self.hud = MBProgressHUD.showAdded(to: view!, animated: animated)
            }
            self.hud?.setAppearance()
        }
    }
    
    public func show(view: UIView? = nil, title: String, animated: Bool = true) {
        DispatchQueue.main.async {
            if let window = UIApplication.shared.delegate?.window!, view == nil {
                self.hud = MBProgressHUD.showAdded(to: window, animated: animated)
            } else {
                self.hud = MBProgressHUD.showAdded(to: view!, animated: animated)
            }
            self.hud?.setAppearance()
            self.hud?.label.text = title
        }
    }
    
    public func show(view: UIView? = nil, title: String, message: String, animated: Bool = true) {
        DispatchQueue.main.async {
            if let window = UIApplication.shared.delegate?.window!, view == nil {
                self.hud = MBProgressHUD.showAdded(to: window, animated: animated)
            } else {
                self.hud = MBProgressHUD.showAdded(to: view!, animated: animated)
            }
            self.hud?.setAppearance()
            self.hud?.label.text = title
            self.hud?.detailsLabel.text = message
        }
    }
    
    public func hide(animated: Bool = true) {
        DispatchQueue.main.async {
            self.hud?.hide(animated: animated)
        }
    }
    
    public func setTitle(title: String) {
        hud?.label.text = title
    }
}

// MARK:- MBProgressHUD Default appearance extension
extension MBProgressHUD {
    func setAppearance() {
        self.contentColor = #colorLiteral(red: 0, green: 0.4390631318, blue: 0.7479397655, alpha: 1)
        self.bezelView.style = .solidColor
        self.bezelView.backgroundColor = UIColor.white
        self.backgroundView.style = .solidColor
        self.backgroundView.color = UIColor(red: 0, green: 0, blue: 0, alpha: 0.7)
    }
}

