//
//  AddDeviceVC.swift
//  EBV-IoT
//
//  Created by krutik vora on 04/02/22.
//

import UIKit

class AddDeviceVC: UIViewController,barcodeScanDelegate,UITextFieldDelegate,UIPickerViewDelegate,UIPickerViewDataSource {
    @IBOutlet weak var btnSideMenu: UIButton!
    @IBOutlet weak var viewTop: UIView!
    @IBOutlet weak var txtDeviceIdentifier: UITextField!
    @IBOutlet weak var txtName: UITextField!
    @IBOutlet weak var txtDeviceTemplate: UITextField!

    
    @IBOutlet  var viewPickerBack: UIView!
    @IBOutlet  var viewPicker: UIView!
    @IBOutlet  var pickerview: UIPickerView!
    @IBOutlet  var toolbar: UIToolbar!
    @IBOutlet  var barbuttoncancel: UIBarButtonItem!
    @IBOutlet  var barbuttondone: UIBarButtonItem!
    var selectedRow : Int = 0
    var strDeviceTempID : String = ""
    var arrTemplatelists : [TemplatesListData] = [TemplatesListData]()

    var isFromDeviceList : Bool = false
    override func viewDidLoad() {
        super.viewDidLoad()
        
        let objSideMenuController = self.sideMenuController()?.sideMenu?.menuViewController as! SideMenuViewController
        objSideMenuController.lbl_username.text = String(format: "%@ %@", (UserDefaultManager.shared[.firstname] as? String) ?? "",(UserDefaultManager.shared[.lastname] as? String) ?? "")

        
        let objRightView = UIImageView(frame: CGRect(x: 0, y: 0, width: 30, height: 45))
        objRightView.image = UIImage(named: "down-arrow.png")
        self.txtDeviceTemplate.rightViewMode = .always
        objRightView.backgroundColor = .clear
        objRightView.contentMode = .bottom
        objRightView.tintColor = UIColor(red: 0.0/255.0, green: 57.0/255.0, blue: 139.0/255.0, alpha: 1.0)
        self.txtDeviceTemplate.rightView = objRightView

        
        hideSideMenuView()

        if isFromDeviceList{
            btnSideMenu.setImage(UIImage(named: "back"), for: .normal)
        } else {
            btnSideMenu.setImage(UIImage(named: "menu"), for: .normal)
        }
        viewTop.layer.cornerRadius = 15
        viewTop.layer.maskedCorners = [.layerMinXMaxYCorner, .layerMaxXMaxYCorner]
        viewTop.clipsToBounds = true

        // Do any additional setup after loading the view.
        if Constant.appDelegate.isConnectedToNetwork(){
            print("Internet Connection Available!")
            self.callTemplateLists()
        }else{
            print("Internet Connection not Available!")
            ProHudManager.shared.hide(animated: true)
            AlertConstant.showAlertMessage(self, Message: "No network available. Please check your Wifi or Data connections")
        }
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated) // No need for semicolon
        
        Constant.AppUtility.lockOrientation(.portrait)
    }

    
    func callTemplateLists()
    {
        let dict : [String : Any] = [:]
        DeviceManager.shared.templateListsWebService(paramers: dict ,successBlock: { (response) in
            print(response)
            let decoder = JSONDecoder()
            let templatelists = try? decoder.decode(TemplatesList.self, from: response)
            if templatelists != nil
            {
                self.arrTemplatelists = templatelists?.data ?? []
//                let arrfilter : [TemplatesListData] = self.arrTemplatelists.filter{$0.name == "EBV Mobile"}
//                if arrfilter.count > 0 {
//                    UserDefaultManager.shared[.templateGuid] = ((arrfilter[0].guid)! as String)
//                } else {
//                    UserDefaultManager.shared[.templateGuid] = "1CD121D5-1896-4AC7-B192-A767EDD96BC7"
//                }
            }
        }, failureBlock: { error in
            print("\n\n===========Error===========")
            print("Error Code: \(error._code)")
            print("Error Messsage: \(error.localizedDescription)")
            ProHudManager.shared.hide(animated: true)
        })
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        self.sideMenuController()?.sideMenu?.allowPanGesture = false
        self.sideMenuController()?.sideMenu?.allowRightSwipe = false

        selectedRow  = 0
        strDeviceTempID  = ""
        self.txtDeviceTemplate.text = "Select Device Template"
        self.viewPickerBack.isHidden = true
        self.viewPicker.isHidden = true
    }
    
    @IBAction func barClickCancel(_ sender: AnyObject)    {
        self.viewPicker.isHidden = true
        self.viewPickerBack.isHidden = true
    }

    @IBAction func barClickDone(_ sender: AnyObject)    {
        self.txtDeviceTemplate.text = self.arrTemplatelists[selectedRow].name
        self.strDeviceTempID = self.arrTemplatelists[selectedRow].guid!
        self.viewPickerBack.isHidden = true
        self.viewPicker.isHidden = true
    }

    // MARK: - PickerView Delegate
    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        return 1
    }
    
    func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        if self.arrTemplatelists != nil{
            return self.arrTemplatelists.count
        }
        return 0
    }
    
    func pickerView(_ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
            return self.arrTemplatelists[row].name
    }
    
    func pickerView(_ pickerView: UIPickerView, didSelectRow row: Int, inComponent component: Int)
    {
         selectedRow = row
    }
    
    // MARK: - TextField Delegate
    
    func textFieldShouldBeginEditing(_ textField: UITextField) -> Bool {
         
        if textField == txtDeviceTemplate{
            self.view.endEditing(true)
            if self.viewPicker.isHidden == true {
                self.viewPickerBack.isHidden = false
                self.viewPicker.isHidden = false
                self.pickerview.isHidden = false
                self.pickerview.reloadAllComponents()
                
                //Set scrollview frame
                UIView.animate(withDuration: 0.5,
                               delay: 0.0,
                               options: UIView.AnimationOptions.curveEaseIn,
                               animations: { () -> Void in
                                
                                self.viewPicker.frame = CGRect(origin: CGPoint(x: 0.0, y: (self.view.frame.size.height - self.viewPicker.frame.size.height) ), size: CGSize(width: self.viewPicker.frame.size.width, height:self.viewPicker.frame.size.height))
                                
                               }, completion: { (finished) -> Void in
                                
                               })
            }
            return false
        } else {
            return true
        }
    }


    
    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destination.
        // Pass the selected object to the new view controller.
    }
    */

    //MARK:- Button Action Event
    @IBAction func btnSideMEnuClick(sender : UIButton){
        if isFromDeviceList{
            self.navigationController?.popViewController(animated: true)
        } else {
            toggleSideMenuView()
        }
    }

    @IBAction func btnBarcodeClick(sender : UIButton){
        let objAddDevice = UIStoryboard(name: Constant.StoryboardIdentifier.device, bundle: nil).instantiateViewController(withIdentifier: "BarcodeViewController") as! BarcodeViewController
        objAddDevice.delegate = self
        self.navigationController?.pushViewController(objAddDevice, animated: true)
    }
    
    //MARK: -  barcode scan delegate
    func nextActionForDevice(isScanedFromBarcode: Bool,deviceID: String?)
    {
        if isScanedFromBarcode{
            if self.checkForUrls(text: deviceID ?? "" ).count > 0{
                AlertConstant.displayAlertMessage(viewController: self, strMessage: "Please enter valid Device ID.")
                self.txtDeviceIdentifier.text = deviceID
                return
            }
            txtDeviceIdentifier.text = deviceID
        }
        else{
            self.txtDeviceIdentifier.becomeFirstResponder()
        }
    }
    
    func checkForUrls(text: String) -> [URL] {
        let types: NSTextCheckingResult.CheckingType = .link

        do {
            let detector = try NSDataDetector(types: types.rawValue)

            let matches = detector.matches(in: text, options: .reportCompletion, range: NSMakeRange(0, text.count))
        
            return matches.compactMap({$0.url})
        } catch let error {
            debugPrint(error.localizedDescription)
        }

        return []
    }
    
    func getdeviceGuid() {
        ProHudManager.shared.show(view: self.view, animated: true)
        let param = [Constant.RequestConstant.uniqueId:self.txtDeviceIdentifier.text as Any] as  [String : Any]
        DeviceManager.shared.singledeviceDetailsWebService(paramers: param ,successBlock: { [self] (response) in
            print(response)
            let decoder = JSONDecoder()
            let devicelists = try? decoder.decode(SingleDevice.self, from: response)
            var arrDeviceName : [SingleDeviceInfo] = [SingleDeviceInfo]()
            arrDeviceName += (devicelists?.data) ?? []
            if arrDeviceName.count > 0
            {
                DispatchQueue.main.async {
//                    self.callAddDevice(deviceguid: arrDeviceName[0].guid ?? "")
                    self.getDeviceGrant(deviceguid: arrDeviceName[0].guid ?? "")
                }
            }
            else
            {
                ProHudManager.shared.hide(animated: true)
            }
        }, failureBlock: { error in
            print("\n\n===========Error===========")
            print("Error Code: \(error._code)")
            print("Error Messsage: \(error.localizedDescription)")
            ProHudManager.shared.hide(animated: true)
            if (error as! EBVHttpErrors).errorDescription == "Unknown error" {
                AlertConstant.showAlertMessage(self, Message: "Device not found")
            } else {
                AlertConstant.showAlertMessage(self, Message: (error as! EBVHttpErrors).errorDescription)
            }
        })
    }
    
    func getDeviceGrant(deviceguid : String) {
        let param = [Constant.RequestConstant.uniqueId:deviceguid] as  [String : Any]
        DeviceManager.shared.getdevicegrantDetailsWebService(paramers: param ,successBlock: { [self] (response) in
            print(response)
            let decoder = JSONDecoder()
            let devicelists = try? decoder.decode(DeviceGrant.self, from: response)
            if devicelists != nil
            {
                DispatchQueue.main.async {
                    if devicelists?.data?.count ?? 0 > 0 {
                        let arruser = devicelists?.data?.filter{($0.userGuid == UserDefaultManager.shared[.userGuid] as? String ?? "")}
                        if arruser?.count ?? 0 > 0 {
                            ProHudManager.shared.hide(animated: true)
                            AlertConstant.showAlertMessage(self, Message: "Failed to add device. This device was already registered with you.")
                        } else {
                            ProHudManager.shared.hide(animated: true)
                            AlertConstant.showAlertMessage(self, Message: "Failed to add device. This device was already registered to another user.")
                        }
                    } else {
                        self.postDeviceGrant(deviceguid: deviceguid)
                    }
                }
            } else {
                self.postDeviceGrant(deviceguid: deviceguid)
            }
        }, failureBlock: { error in
            print("\n\n===========Error===========")
            print("Error Code: \(error._code)")
            print("Error Messsage: \(error.localizedDescription)")
            ProHudManager.shared.hide(animated: true)
            if (error as! EBVHttpErrors).errorDescription == "Unknown error" {
                AlertConstant.showAlertMessage(self, Message: "Device delete,acquire and release not allowed")
            } else {
                AlertConstant.showAlertMessage(self, Message: (error as! EBVHttpErrors).errorDescription)
            }
        })
    }
    
    func postDeviceGrant(deviceguid : String) {
        let param = [Constant.RequestConstant.uniqueId:deviceguid] as  [String : Any]
        DeviceManager.shared.postdevicegrantDetailsWebService(paramers: param ,successBlock: { [self] (response) in
            print(response)
            let decoder = JSONDecoder()
            let devicelists = try? decoder.decode(PostDeviceGrant.self, from: response)
            if devicelists != nil
            {
                DispatchQueue.main.async {
                    if devicelists?.data?.count ?? 0 > 0 {
                        if devicelists?.data?[0].success?.count ?? 0 > 0 {
                            self.callAddDevice(deviceguid: deviceguid)
                        } else if devicelists?.data?[0].error?.count ?? 0 > 0 {
                            ProHudManager.shared.hide(animated: true)
                            AlertConstant.showAlertMessage(self, Message: devicelists?.data?[0].error?[0].errorLog ?? "")
                        }
                    }
                }
            } else {
                ProHudManager.shared.hide(animated: true)
            }
        }, failureBlock: { error in
            print("\n\n===========Error===========")
            print("Error Code: \(error._code)")
            print("Error Messsage: \(error.localizedDescription)")
            ProHudManager.shared.hide(animated: true)
            if (error as! EBVHttpErrors).errorDescription == "Unknown error" {
                AlertConstant.showAlertMessage(self, Message: "Device delete,acquire and release not allowed")
            } else {
                AlertConstant.showAlertMessage(self, Message: (error as! EBVHttpErrors).errorDescription)
            }
        })
    }
    
    func callAddDevice(deviceguid : String){
        let param = [Constant.RequestConstant.uniqueId:deviceguid,"entityGuid":UserDefaultManager.shared[.entityGuid] as! String,"Deleted":"0"] as  [String : Any]
        DeviceManager.shared.deviceEntityWebService(paramers: param ,successBlock: { [self] (response) in
            print(response)
            let decoder = JSONDecoder()
            let devicelists = try? decoder.decode(Devices.self, from: response)
            if devicelists == nil
            {
                DispatchQueue.main.async {
                  // Devicename update
                  self.call_Device_Update(deviceguid: deviceguid)
                }
            }
        }, failureBlock: { error in
            print("\n\n===========Error===========")
            print("Error Code: \(error._code)")
            print("Error Messsage: \(error.localizedDescription)")
            ProHudManager.shared.hide(animated: true)
            if (error as! EBVHttpErrors).errorDescription == "Unknown error" {
                AlertConstant.showAlertMessage(self, Message: "Device delete,acquire and release not allowed")
            } else {
                AlertConstant.showAlertMessage(self, Message: (error as! EBVHttpErrors).errorDescription)
            }
        })
    }
    
    func call_Device_Update(deviceguid : String) {
        let param = [Constant.RequestConstant.displayName:self.txtName.text ?? "","entityGuid":UserDefaultManager.shared[.entityGuid] as! String,"deviceguid":deviceguid] as  [String : Any]
        DeviceManager.shared.deviceUpdateWebService(paramers: param ,successBlock: { [self] (response) in
            print(response)
            let decoder = JSONDecoder()
            let devicelists = try? decoder.decode(Devices.self, from: response)
            if devicelists == nil
            {
                DispatchQueue.main.async {
                    ProHudManager.shared.hide(animated: true)
                    let alertController = UIAlertController(title:APP_NAME, message: "Device added successfully", preferredStyle: UIAlertController.Style.alert)
                    alertController.view.tintColor = UIColor.black
                    alertController.addAction(UIAlertAction(title: "OK", style: UIAlertAction.Style.default, handler: { alert in
                        let mainStoryboard: UIStoryboard = UIStoryboard(name: Constant.StoryboardIdentifier.device,bundle: nil)
                        let destViewController = mainStoryboard.instantiateViewController(withIdentifier: "DeviceVC")
                        self.sideMenuController()?.setContentViewController(destViewController)
                    }))
                    self.present(alertController, animated: true, completion: nil)
                }
            }
        }, failureBlock: { error in
            print("\n\n===========Error===========")
            print("Error Code: \(error._code)")
            print("Error Messsage: \(error.localizedDescription)")
            ProHudManager.shared.hide(animated: true)
            if (error as! EBVHttpErrors).errorDescription == "Unknown error" {
                AlertConstant.showAlertMessage(self, Message: "Device delete,acquire and release not allowed")
            } else {
                AlertConstant.showAlertMessage(self, Message: (error as! EBVHttpErrors).errorDescription)
            }
        })
    }

    @IBAction func btnAddDeviceClick(sender : UIButton)
    {
        if self.txtDeviceIdentifier.text?.trimmingCharacters(in: .whitespaces).count == 0 {
            AlertConstant.showAlertMessage(self, Message: MessageConstant.commonMessage.enterIdent)
            self.txtDeviceIdentifier.becomeFirstResponder()
        } else if self.txtName.text?.trimmingCharacters(in: .whitespaces).count == 0 {
            AlertConstant.showAlertMessage(self, Message: MessageConstant.commonMessage.enterdevice)
            self.txtName.becomeFirstResponder()

        }
//        else if self.strDeviceTempID.trimmingCharacters(in: .whitespaces).count == 0 {
//            self.txtName.resignFirstResponder()
//            AlertConstant.showAlertMessage(self, Message: MessageConstant.commonMessage.enterdevicetemplate)
//        }
        else {
            self.txtName.resignFirstResponder()
            self.txtDeviceIdentifier.resignFirstResponder()
            if Constant.appDelegate.isConnectedToNetwork(){
                print("Internet Connection Available!")
                self.getdeviceGuid()
            }else{
                print("Internet Connection not Available!")
                ProHudManager.shared.hide(animated: true)
                AlertConstant.showAlertMessage(self, Message: "No network available. Please check your Wifi or Data connections")
            }
        }
    }

}
