//
//  AddDeviceViewController.swift
//  Procell
//
//  Created by Rumit Khakhi on 28/10/21.
//

import UIKit
import AVFoundation

protocol barcodeScanDelegate: AnyObject {
    // The following command (ie, method) must be obeyed by any
    // underling (ie, delegate) of the older sibling.
    func nextActionForDevice(isScanedFromBarcode: Bool,deviceID: String?)
}

class BarcodeViewController: UIViewController {
    weak var delegate: barcodeScanDelegate?
    @IBOutlet weak var imgvBarcode: UIImageView!
    @IBOutlet weak var viewTop: UIView!
    @IBOutlet weak var scannerView: QRScannerView! {
        didSet {
            scannerView.delegate = self
        }
    }

    
    var qrData: QRData? = nil {
        didSet {
            if qrData != nil {
                if  qrData?.codeString?.count != 0{
                    self.delegate?.nextActionForDevice(isScanedFromBarcode: true, deviceID: qrData?.codeString ?? "")
                    self.navigationController?.popViewController(animated: true)
//                    print("******* QR Code is  ******* : \(String(describing: qrData?.codeString))")
//                    UserDefaults.standard.set(qrData?.codeString ?? "", forKey: "barcode")
                }
            }
        }
    }
    
    @IBAction func btnManualbuttonClicked(){
        self.delegate?.nextActionForDevice(isScanedFromBarcode: false, deviceID: qrData?.codeString ?? "")
        self.navigationController?.popViewController(animated: true)
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        viewTop.layer.cornerRadius = 15
        viewTop.layer.maskedCorners = [.layerMinXMaxYCorner, .layerMaxXMaxYCorner]
        viewTop.clipsToBounds = true

    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)

        if !scannerView.isRunning {
               //      Camera
            AVCaptureDevice.requestAccess(for: AVMediaType.video) { response in
                if response {
                    //access granted
                    DispatchQueue.main.async {
                        self.imgvBarcode.isHidden = true
                    }
                    self.scannerView.startScanning()
                } else {
                    DispatchQueue.main.async {
                        self.imgvBarcode.isHidden = false
                    }
                    DispatchQueue.main.async {
                        AlertConstant.displayAlertMessageWithTwoButtonBlock(viewController: self, strMessage: MessageConstant.AlertMessage.cameraAccess, buttonTitle1: "Cancel", buttonTitle2: "Settings") { (result) in
                            if result.title == "Cancel" {
                                
                            }
                            else if result.title == "Settings"{
                                UIApplication.shared.open(URL(string: UIApplication.openSettingsURLString)!, options: [:], completionHandler: nil)
                            }
                        }
                    }
                }
            }
        }
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        if !scannerView.isRunning {
            scannerView.stopScanning()
        }
    }
    
    
    @objc func backButtonClicked()  {
        print("Back button Click....")
        _ = self.navigationController?.popViewController(animated: true)
    }


    func createScanningIndicator() {
        
        let height: CGFloat = 15
        let opacity: Float = 0.4
        let topColor = UIColor.green.withAlphaComponent(0)
        let bottomColor = UIColor.green

        let layer = CAGradientLayer()
        layer.colors = [topColor.cgColor, bottomColor.cgColor]
        layer.opacity = opacity
        
        let squareWidth = view.frame.width * 0.6
        let xOffset = view.frame.width * 0.2
        let yOffset = view.frame.midY - (squareWidth / 2)
        layer.frame = CGRect(x: xOffset, y: yOffset, width: squareWidth, height: height)
        
        self.scannerView.layer.insertSublayer(layer, at: 0)

        let initialYPosition = layer.position.y
        let finalYPosition = initialYPosition + squareWidth - height
        let duration: CFTimeInterval = 2

        let animation = CABasicAnimation(keyPath: "position.y")
        animation.fromValue = initialYPosition as NSNumber
        animation.toValue = finalYPosition as NSNumber
        animation.duration = duration
        animation.repeatCount = .infinity
        animation.isRemovedOnCompletion = false
        
        layer.add(animation, forKey: nil)
    }
    
    @IBAction func btnSideMEnuClick(sender : UIButton){
            self.navigationController?.popViewController(animated: true)
    }
    
    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destination.
        // Pass the selected object to the new view controller.
    }
    */
 
}
extension BarcodeViewController: QRScannerViewDelegate {
    func qrScanningDidStop() {
       // let buttonTitle = scannerView.isRunning ? "STOP" : "SCAN"
       // scanButton.setTitle(buttonTitle, for: .normal)
    }
    
    func qrScanningDidFail() {
        
        print("Scanning Failed. Please try again")
    }
    
    func qrScanningSucceededWithCode(_ str: String?) {
        self.qrData = QRData(codeString: str)
    }
}



