//
//  PropertiesVC.swift
//  EBV-IoT
//
//  Created by krutik vora on 08/02/22.
//

import UIKit
import SkyFloatingLabelTextField
class CommandVC: UIViewController,UITextFieldDelegate,UIPickerViewDelegate,UIPickerViewDataSource {

    @IBOutlet weak var txtCommand: SkyFloatingLabelTextField!
    @IBOutlet weak var txtCommandValue: SkyFloatingLabelTextField!
    @IBOutlet weak var tblCommand: UITableView!
    var arrCommandType : [CommandTypeDetail]!
    var arrCommandHistory : [CommandHistoryDetail]!

    var strDeviceID : String = ""
    var strDeviceGUID : String = ""
    var strDeviceTemplateGUID : String = ""
    var strDeviceName : String = ""

    @IBOutlet  var viewPickerBack: UIView!
    @IBOutlet  var viewPicker: UIView!
    @IBOutlet  var pickerview: UIPickerView!
    @IBOutlet  var toolbar: UIToolbar!
    @IBOutlet  var barbuttoncancel: UIBarButtonItem!
    @IBOutlet  var barbuttondone: UIBarButtonItem!
    var selectedRow : Int = 0
    var selectedCommandID : String = ""
    var pageNo : Int = 1
    var totalRecords : Int = 0
    var isPaging : Bool = false
    var isLoading : Bool = false
    @IBOutlet weak var tblBottomView: UIView!

    override func viewDidLoad() {
        super.viewDidLoad()
        let objRightView = UIImageView(frame: CGRect(x: 0, y: 0, width: 30, height: 45))
        objRightView.image = UIImage(named: "down-arrow.png")
        self.txtCommand.rightViewMode = .always
        objRightView.backgroundColor = .clear
        objRightView.contentMode = .bottom
        objRightView.tintColor = UIColor(red: 0.0/255.0, green: 57.0/255.0, blue: 139.0/255.0, alpha: 1.0)
        self.txtCommand.rightView = objRightView
        tblCommand.register(UINib(nibName: "CommandCustomCell", bundle: nil), forCellReuseIdentifier: "CommandCustomCell")
        print("CommandVC viewDidLoad")
        tblCommand.tableFooterView = tblBottomView
        tblCommand.tableFooterView?.isHidden = true

        // Do any additional setup after loading the view.
    }
    
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        
        if Constant.appDelegate.isConnectedToNetwork(){
            print("Internet Connection Available!")
            pageNo = 1
            totalRecords = 0
            callCommandTypeWebservice()
            selectedRow  = 0
            selectedCommandID  = ""
            self.txtCommand.text = "Select Command"
            self.txtCommandValue.text = ""

            self.viewPickerBack.isHidden = true
            self.viewPicker.isHidden = true
        }else{
            print("Internet Connection not Available!")
            ProHudManager.shared.hide(animated: true)
            AlertConstant.showAlertMessage(self, Message: "No network available. Please check your Wifi or Data connections")
        }

        print("CommandVC viewWillAppear")
    }

    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated) // No need for semicolon
        
        Constant.AppUtility.lockOrientation(.portrait)
    }

    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        print("CommandVC viewWillDisappear")
    }

    @IBAction func barClickCancel(_ sender: AnyObject)    {
        self.viewPicker.isHidden = true
        self.viewPickerBack.isHidden = true

    }

    @IBAction func barClickDone(_ sender: AnyObject)    {
        
        if self.arrCommandType != nil {
            if self.arrCommandType.count > 0 {
                self.txtCommand.text = self.arrCommandType[selectedRow].name
                self.selectedCommandID = self.arrCommandType[selectedRow].guid!
            }
        }
        
        self.viewPickerBack.isHidden = true
        self.viewPicker.isHidden = true
    }

    
    // MARK: - PickerView Delegate
    func numberOfComponents(in pickerView: UIPickerView) -> Int {
        return 1
    }
    
    func pickerView(_ pickerView: UIPickerView, numberOfRowsInComponent component: Int) -> Int {
        if self.arrCommandType != nil{
            return self.arrCommandType.count
        }
        return 0
    }
    
    func pickerView(_ pickerView: UIPickerView, titleForRow row: Int, forComponent component: Int) -> String? {
            return self.arrCommandType[row].name
    }
    
    func pickerView(_ pickerView: UIPickerView, didSelectRow row: Int, inComponent component: Int)
    {
            selectedRow = row
    }


    // MARK: - DeviceLists Web-Service
    func callCommandTypeWebservice()
    {
        DispatchQueue.main.async { [self] in
            let dict : [String : Any] = [:]
            ProHudManager.shared.show(view: self.view.superview?.superview, animated: true)
            DeviceManager.shared.getCommandTypesWebService(deviceTemplateGUID: self.strDeviceTemplateGUID, paramers: dict ,successBlock: { (response) in
                print(response)
                let decoder = JSONDecoder()
                let commandType = try? decoder.decode(CommandTypeList.self, from: response)
                if commandType != nil{
                    self.arrCommandType = commandType?.data
                    self.callCommandHistoryWebService()
                } else {
                    self.callCommandHistoryWebService()
                }
            }, failureBlock: { error in
                print("\n\n===========Error===========")
                print("Error Code: \(error._code)")
                print("Error Messsage: \(error.localizedDescription)")
                self.callCommandHistoryWebService()
            })
        }
    }

    func callCommandHistoryWebService()
    {
        DispatchQueue.main.async {
            let dict : [String : Any] = [:]
            var dictUrlPerameter : [String : String] = [:]

            dictUrlPerameter["pageNumber"] = String(format: "%d", self.pageNo)
            dictUrlPerameter["pageSize"] =  String(format: "%d",Constant.k_PAGE_SIZE)
            dictUrlPerameter["deviceGuid"] = self.strDeviceGUID
            
            DeviceManager.shared.getCommandHistoryWebService(paramers: dict, urlParamers: dictUrlPerameter ,successBlock: { (response) in
                ProHudManager.shared.hide(animated: true)
                print(response)
                let decoder = JSONDecoder()
                let objCommandHistory = try? decoder.decode(CommandHistory.self, from: response)
                if objCommandHistory != nil{
                    DispatchQueue.main.async {

                        if objCommandHistory!.count! > 0 {
                            self.tblCommand.isHidden = false
                            if  self.isLoading == true {
                                DispatchQueue.main.async {
                                    self.arrCommandHistory += (objCommandHistory?.data) ?? []
                                    self.tblCommand.reloadData()
                                    self.isLoading = false
                                }
                            }
                            else {
                                if  self.arrCommandHistory != nil {
                                    self.arrCommandHistory.removeAll()
                                    
                                }
                                self.arrCommandHistory = (objCommandHistory?.data) ?? []
                                self.isLoading = false
                            }
                        }
                        else
                        {
                            self.tblCommand.isHidden = true
                        }
                        self.totalRecords = objCommandHistory!.count ?? 0
                        self.tblCommand.reloadData()
                        if(self.totalRecords > (self.pageNo*Constant.k_PAGE_SIZE))
                        {
                            self.tblCommand.tableFooterView?.isHidden = false
                            self.tblCommand.tableFooterView = self.tblBottomView
                        } else {
                            self.tblCommand.tableFooterView = nil
                            self.tblCommand.tableFooterView?.isHidden = true


                        }


                    }
                }
            }, failureBlock: { error in
                print("\n\n===========Error===========")
                print("Error Code: \(error._code)")
                print("Error Messsage: \(error.localizedDescription)")
                DispatchQueue.main.async {
                    self.tblCommand.isHidden = true
                }
                ProHudManager.shared.hide(animated: true)
            })
        }
    }
    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        let offsetY = scrollView.contentOffset.y
        let contentHeight = scrollView.contentSize.height

        if (offsetY > contentHeight - scrollView.frame.height) && !isLoading {
            self.loadMoreData()
        }
    }

    func loadMoreData() {
        if !self.isLoading {
            self.isLoading = true
            DispatchQueue.main.async { [self] in
                if(self.totalRecords > (self.pageNo*Constant.k_PAGE_SIZE))
                {
                        self.pageNo += 1
                        self.tblCommand.tableFooterView = self.tblBottomView
                        self.tblCommand.tableFooterView?.isHidden = false
//                        ProHudManager.shared.show(view: self.view, animated: true)
                        self.callCommandHistoryWebService()
                }
                else
                {
                    self.isLoading = false
                    self.tblCommand.tableFooterView = nil
                    self.tblCommand.tableFooterView?.isHidden = true

                }
            }
        }
    }
    

    func callGenerateCommandWebService()
    {
        DispatchQueue.main.async {
            var dict : [String : Any] = [:]
            dict["commandGuid"] = self.selectedCommandID
            dict["parameterValue"] = self.txtCommandValue.text
            ProHudManager.shared.show(view: self.view.superview?.superview, animated: true)
            DeviceManager.shared.generateCommandWebService(strDeviceGUID:self.strDeviceGUID ,paramers: dict ,successBlock: { (response) in
//                ProHudManager.shared.hide(animated: true)
                do {
                    if let json = try JSONSerialization.jsonObject(with: response, options: []) as? [String: Any] {
                        // try to read out a string array
                        //AlertConstant.showAlertMessage(self, Message: json["message"] as! String)
                        self.pageNo = 1
                        self.totalRecords = 0

                        self.callCommandHistoryWebService()

                    }
                } catch let error as NSError {
                    AlertConstant.showAlertMessage(self, Message: error.localizedDescription)
                    print("Failed to load: \(error.localizedDescription)")
                }
            }, failureBlock: { error in
                print("\n\n===========Error===========")
                print("Error Code: \(error._code)")
                print("Error Messsage: \(error.localizedDescription)")
                ProHudManager.shared.hide(animated: true)
                AlertConstant.showAlertMessage(self, Message: (error as! EBVHttpErrors).errorDescription)

            })
        }
    }

    // MARK: - Button Action
    
    @IBAction func btnExecuteCommand(sender : UIButton){
        if selectedCommandID == "" {
            AlertConstant.showAlertMessage(self, Message: "Please select command.")

        } else if self.txtCommandValue.text?.trimmingCharacters(in: .whitespaces).count == 0{
            AlertConstant.showAlertMessage(self, Message: "Please enter parameter value.")
        } else {
            if Constant.appDelegate.isConnectedToNetwork(){
                callGenerateCommandWebService()
            }else{
                print("Internet Connection not Available!")
                ProHudManager.shared.hide(animated: true)
                AlertConstant.showAlertMessage(self, Message: "No network available. Please check your Wifi or Data connections")
            }
        }
    }

    // MARK: - TextField Delegate
    
    func textFieldShouldBeginEditing(_ textField: UITextField) -> Bool {
        if textField == txtCommand{
            if self.arrCommandType.count > 0 {
                if self.viewPicker.isHidden == true{
                    self.viewPickerBack.isHidden = false
                    self.viewPicker.isHidden = false
                    self.pickerview.isHidden = false
                    self.pickerview.reloadAllComponents()
                    
                    //Set scrollview frame
                    UIView.animate(withDuration: 0.5,
                                   delay: 0.0,
                                   options: UIView.AnimationOptions.curveEaseIn,
                                   animations: { () -> Void in
                                    
                                    self.viewPicker.frame = CGRect(origin: CGPoint(x: 0.0, y: (self.view.frame.size.height - self.viewPicker.frame.size.height) ), size: CGSize(width: self.viewPicker.frame.size.width, height:self.viewPicker.frame.size.height))
                                    
                                   }, completion: { (finished) -> Void in
                                    
                                   })
                }
                return false
            }
            return true
        } else {
            return true
        }
    }

    
    func formatDate(strDate:String,currentFormat:String,newDateFormat:String) -> String{
        
        let dtFormat = DateFormatter()
        dtFormat.dateFormat = currentFormat
        let dt = dtFormat.date(from: strDate)
        dtFormat.dateFormat = newDateFormat
        let strDate = dtFormat.string(from: dt!)
        return strDate

    }
    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destination.
        // Pass the selected object to the new view controller.
    }
    */

}
extension CommandVC: UITableViewDelegate {
    
    // didSelectRowAtIndexPath - change background color
//    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
//        tableView.deselectRow(at: indexPath, animated: true)
//    }
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 250
    }
    

}

extension CommandVC: UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        if self.arrCommandHistory == nil {
            return 0
        }
        return self.arrCommandHistory.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "CommandCustomCell", for: indexPath) as! CommandCustomCell
        cell.selectionStyle = .none
        let objCommandHistory = self.arrCommandHistory[indexPath.row]
        cell.lblCommand.text = objCommandHistory.cmdText
        let strExecuteOn = self.formatDate(strDate: objCommandHistory.statusDate!, currentFormat: "YYYY-MM-dd'T'HH:mm:ss", newDateFormat: "MMM d, yyyy HH:mm:ss")
        cell.lblExecutedOn.text = strExecuteOn
        cell.lblExecutedBy.text = objCommandHistory.createdByName
        cell.lblStatus.text = objCommandHistory.status
        let count : Int = objCommandHistory.resendHistory?.execTime?.count ?? 0
        cell.lblCount.text = String(format: "%d", count)

        return cell
    }
    
    
}
