
//
//  DataVC.swift
//  EBV-IoT
//
//  Created by krutik vora on 09/02/22.
//

import UIKit
import Charts
//import AVFAudio

class DataVC: UIViewController,SignalRServiceDelegate {
    
    @IBOutlet var lblData : UILabel!
    @IBOutlet var tblData : UITableView!
    var strDeviceGUID : String = ""
    var strDeviceID : String = ""
    var strDeviceTemplateGUID : String = ""
    var strDeviceName : String = ""
    var timeInterval = [String]()
    let formatter = DateFormatter()
    var indtemp : Int = 0
    
    var livedataSetsList : [LineChartDataSet] = []
    var liveDataSetList : [[String: Any]] = []
    
    var arrSensors : [DataInfo] = [DataInfo]()
    var arr_lines : [DataInfo] = [DataInfo]()
    var arr_gauges : [DataInfo] = [DataInfo]()
    var arr_btsensors : [DataInfo] = [DataInfo]()
    var arr_led : [DataInfo] = [DataInfo]()
    var arr_sliders : [DataInfo] = [DataInfo]()
    var arr_nonconsider : [DataInfo] = [DataInfo]()
    var commandsarr : [Commands] = [Commands]()
    var teledefaultarray : [[String:Any]] = []
    var telelivearray : [[String:Any]] = []
    
    var connect : Bool = false
    var isDeviceConnected:((Bool)->())?
    
    var parentvc : UIViewController?
    var objSignalRService : SignalRService?
    var isSelectedOrientedViewIndex  : Int = -1
    var isDefaultDataReady: Bool  = false
    
    @IBOutlet weak var layLeading: NSLayoutConstraint!
    @IBOutlet weak var layTrailing: NSLayoutConstraint!
    @IBOutlet weak var layTop: NSLayoutConstraint!
    @IBOutlet weak var layBottom: NSLayoutConstraint!

    override func viewDidLoad() {
        super.viewDidLoad()
        print("DataVC viewDidLoad")

        tblData.register(UINib(nibName: "GaugeCell", bundle: nil), forCellReuseIdentifier: "GaugeCell")
        tblData.register(UINib(nibName: "SliderCell", bundle: nil), forCellReuseIdentifier: "SliderCell")
        tblData.register(UINib(nibName: "BTSensorCell", bundle: nil), forCellReuseIdentifier: "BTSensorCell")
        tblData.register(UINib(nibName: "LEDCell", bundle: nil), forCellReuseIdentifier: "LEDCell")
        tblData.register(UINib(nibName: "UnknownCell", bundle: nil), forCellReuseIdentifier: "UnknownCell")

        tblData.separatorStyle = .none
        lblData.isHidden = true
        tblData.isHidden = true
        
        self.resetAllarray()
        
        if Constant.appDelegate.isConnectedToNetwork(){
            print("Internet Connection Available!")
            ProHudManager.shared.show(view: self.parentvc?.view, animated: true)
            self.callTelemetryDefaultWebService(strDeviceGUID: strDeviceGUID)
        }else{
            print("Internet Connection not Available!")
            ProHudManager.shared.hide(animated: true)
            AlertConstant.showAlertMessage(self, Message: "No network available. Please check your Wifi or Data connections")
        }
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated) // No need for semicolon
        Constant.AppUtility.lockOrientation(.portrait)
    }

    
    // MARK: - Button Actions
    @IBAction func btnSwitchClick(sender: UIButton)
    {
        sender.isSelected = !sender.isSelected
        let arrcommandsensors = self.getCommandArray(str_sensor: self.arrSensors[sender.tag].localName ?? "")
        if arrcommandsensors.count > 0 {
            self.callGenerateCommandWebService(str_commandguid: arrcommandsensors[0].guid!, isOn: sender.isSelected, slider: 0,isSlider: false)
        }
    }
    
    @IBAction func btnOrient(sender: UIButton)
    {
        isSelectedOrientedViewIndex = sender.tag
        if UIDevice.current.orientation.isLandscape {
            Constant.AppUtility.lockOrientation(.portrait)
            let value = UIInterfaceOrientation.portrait.rawValue
            UIDevice.current.setValue(value, forKey: "orientation")
        } else {
            Constant.AppUtility.lockOrientation(.landscape)
            let value = UIInterfaceOrientation.landscapeLeft.rawValue
            UIDevice.current.setValue(value, forKey: "orientation")
        }
    }
    
    @objc func onSliderValChanged(slider: UISlider, event: UIEvent) {
     
        let step = (slider.accessibilityLabel)
        if let touchEvent = event.allTouches?.first {
            switch touchEvent.phase {
            case .began:
                let str_sensor : String = self.arrSensors[slider.tag].localName ?? ""
                print(str_sensor)
                // handle drag began
            case .moved:
                // handle drag moved
                let str_sensor : String = self.arrSensors[slider.tag].localName ?? ""
                print(str_sensor)
            case .ended:
                // handle drag ended
                let arrcommandsensors = self.getCommandArray(str_sensor: self.arrSensors[slider.tag].localName ?? "")
                if arrcommandsensors.count > 0 {
                    let indexPath = IndexPath(item: 0, section: 4)
                    let fullyVisibleCell = tblData.cellForRow(at: indexPath) as? SliderCell
                    print("slider values",slider.value)
                    
                    let roundedStepValueCeil = floor(slider.value / Float(step!)!) * Float(step!)!
                    var roundedStepValue = round(slider.value / Float(step!)!) * Float(step!)!
                   
                   
                    if roundedStepValue > slider.maximumValue{
                        roundedStepValue = slider.maximumValue
                        print("RoundedMaxStepValue: \(roundedStepValue)")
                    } else if roundedStepValue < slider.minimumValue{
                        roundedStepValue = slider.minimumValue
                        print("RoundedMinSetpValue: \(roundedStepValue)")
                    } else if roundedStepValueCeil > slider.maximumValue {
                        roundedStepValue = slider.maximumValue
                    }
                    fullyVisibleCell?.objSlider.value = roundedStepValue
                    self.callGenerateCommandWebService(str_commandguid: arrcommandsensors[0].guid!, isOn: false,slider: Int(slider.value),isSlider: true)
                }
            default:
                break
            }
        }
    }
    
    @IBAction func btnLedCommand(sender: UIButton)
    {
        var isled : Bool = false
        let arrcommandsensors = self.getCommandArray(str_sensor: self.arrSensors[sender.tag].localName ?? "")
        if arrcommandsensors.count > 0 {
            let indexPath = IndexPath(item: sender.tag, section: 0)
            let fullyVisibleCell = tblData.cellForRow(at: indexPath) as? LEDCell
            if fullyVisibleCell?.objBtn.accessibilityLabel == "led_on_editable" {
                isled = false
                fullyVisibleCell?.objBtn.accessibilityLabel = "led_off_editable"
                fullyVisibleCell?.objBtn.setImage(UIImage(named: "led_off_editable"), for: .normal)
            } else {
                isled = true
                fullyVisibleCell?.objBtn.accessibilityLabel = "led_on_editable"
                fullyVisibleCell?.objBtn.setImage(UIImage(named: "led_on_editable"), for: .normal)
            }
            self.callGenerateCommandWebService(str_commandguid: arrcommandsensors[0].guid!, isOn: isled,slider: 0,isSlider: false)
        }
    }
    
    //MARK:- Telemetry Default Data Web-Service
    func callTelemetryDefaultWebService(strDeviceGUID : String)
    {
        let param = [Constant.RequestConstant.devicetemplateguid:strDeviceGUID] as [String : Any]
        DeviceManager.shared.templateDefaultWebService(paramers: param ,successBlock: { [self]  (response) in
            print(String(data: response, encoding: .utf8) as Any)
            
            do {
                if let json = try JSONSerialization.jsonObject(with: response, options: []) as? [String: Any] {
                teledefaultarray = json["data"] as! [[String : Any]]
              }
            } catch let error as NSError {
                print("Failed to load: \(error.localizedDescription)")
            }
            self.callSensorCommandListsWebService(strDeviceTemplateGUID: strDeviceTemplateGUID)
        }, failureBlock: { error in
            print("\n\n===========Error===========")
            print("Error Code: \(error._code)")
            print("Error Messsage: \(error.localizedDescription)")
            ProHudManager.shared.hide(animated: true)
        })
    }
    
    //MARK:- Send Command Web-Service
    func callGenerateCommandWebService(str_commandguid : String, isOn : Bool, slider : Int, isSlider : Bool)
    {
        DispatchQueue.main.async {
            var dict : [String : Any] = [:]
            dict["commandGuid"] = str_commandguid
            if isSlider {
                dict["parameterValue"] = slider
            } else {
                if isOn {
                    dict["parameterValue"] = 1
                } else {
                    dict["parameterValue"] = 0
                }
            }
            ProHudManager.shared.show(view: self.view.superview?.superview, animated: true)
            DeviceManager.shared.generateCommandWebService(strDeviceGUID:self.strDeviceGUID ,paramers: dict ,successBlock: { (response) in
                ProHudManager.shared.hide(animated: true)
                do {
                    if let json = try JSONSerialization.jsonObject(with: response, options: []) as? [String: Any] {
                        print(json)
//                        AlertConstant.showAlertMessage(self, Message: json["message"] as! String)
                    }
                } catch let error as NSError {
                    AlertConstant.showAlertMessage(self, Message: error.localizedDescription)
                    print("Failed to load: \(error.localizedDescription)")
                }
            }, failureBlock: { error in
                print("\n\n===========Error===========")
                print("Error Code: \(error._code)")
                print("Error Messsage: \(error.localizedDescription)")
                ProHudManager.shared.hide(animated: true)
                AlertConstant.showAlertMessage(self, Message: (error as! EBVHttpErrors).errorDescription)
            })
        }
    }
    
    //MARK:- SensorList Web-Service
    func callSensorLists(strDeviceTemplateGUID : String)
    {
        let param = [Constant.RequestConstant.pageindex:1,Constant.RequestConstant.pagesize:Constant.k_PAGE_SIZE,Constant.RequestConstant.devicetemplateguid:strDeviceTemplateGUID] as [String : Any]
        DeviceManager.shared.deviceTemplateListsWebService(paramers: param ,successBlock: { [self]  (response) in
            ProHudManager.shared.hide(animated: true)
            print(String(data: response, encoding: .utf8) as Any)
            let decoder = JSONDecoder()
            let devicesensorlists = try? decoder.decode(DeviceTemplate.self, from: response)
            if devicesensorlists != nil
            {
                self.arrSensors = self.setarray2(array_filter : devicesensorlists?.data ?? [])
                
                for j in 0..<self.arrSensors.count {
                    if self.arrSensors[j].attributes?.count ?? 0 > 0 {
                        for l in 0..<self.arrSensors[j].attributes!.count {
                            var dict_color : [String : Any] = [:]
                            dict_color["name"] = String(format: "%@.%@", self.arrSensors[j].localName ?? "",self.arrSensors[j].attributes?[l].localName ?? "")
                            dict_color["colorcode"] = self.arrSensors[j].attributes?[l].attributeColor
                            Constant.appDelegate.arr_Color_Sensor.append(dict_color)
                        }
                    } else {
                        var dict_color : [String : Any] = [:]
                        dict_color["name"] = String(format: "%@", self.arrSensors[j].localName ?? "")
                        dict_color["colorcode"] = self.arrSensors[j].attributeColor
                        Constant.appDelegate.arr_Color_Sensor.append(dict_color)
                    }
                }
                
                
                //For Lines Arrays
                self.arr_lines = self.setarray (array_filter : self.arrSensors, name : Constant.SENSOR_TYPE.GRAPH)
                
                for i in 0..<self.arr_lines.count
                {
                    let cellname : String = String(format: "LineGraphCell-%d", i)

                    DispatchQueue.main.async {
                        self.tblData.register(UINib(nibName: "LineGraphCell", bundle: nil), forCellReuseIdentifier: cellname)
                    }
                    
                    var dataentries : [ChartDataSet] = []
                    if self.arr_lines[i].attributes?.count ?? 0 > 0 {
                        for k in 0..<self.arr_lines[i].attributes!.count
                        {
                            let lineChartDataSet = LineChartDataSet(entries: [], label: "")
                            lineChartDataSet.circleRadius = 4.0
                            lineChartDataSet.circleHoleRadius = 2.0
                            lineChartDataSet.drawCircleHoleEnabled = true
                            lineChartDataSet.mode = .cubicBezier
                            lineChartDataSet.drawValuesEnabled = false
                            lineChartDataSet.lineWidth = 1.5
                            if self.arr_lines[i].attributeColor != "" {
                                lineChartDataSet.circleColors = [NSUIColor(cgColor: hexStringToUIColor(hex: self.arr_lines[i].attributes?[k].attributeColor ?? "").cgColor)]
                                lineChartDataSet.colors = [NSUIColor(cgColor: hexStringToUIColor(hex: self.arr_lines[i].attributes?[k].attributeColor ?? "").cgColor)]
                            } else {
                                lineChartDataSet.circleColors = [NSUIColor(cgColor: hexStringToUIColor(hex: Constant.colorsubArrays[k]).cgColor)]
                                lineChartDataSet.colors = [NSUIColor(cgColor: hexStringToUIColor(hex: Constant.colorsubArrays[k]).cgColor)]
                            }
                            let devicename = self.getlocalname(desc: self.arr_lines[i].attributes?[k].description ?? "",localname: self.arr_lines[i].attributes?[k].localName ?? "")
                            lineChartDataSet.label = devicename
                            dataentries.append(lineChartDataSet)
                        }
                        var dict : [String:Any] = [:]
                        dict["title"] = self.arr_lines[i].localName
                        dict["entries"] = dataentries
                        dict["cellname"] = cellname
                        self.liveDataSetList.append(dict)
                        

                    } else {
                        let lineChartDataSet = LineChartDataSet(entries: [], label: "")
                        lineChartDataSet.circleRadius = 4.0
                        lineChartDataSet.circleHoleRadius = 2.0
                        lineChartDataSet.drawCircleHoleEnabled = true
                        lineChartDataSet.mode = .cubicBezier
                        lineChartDataSet.drawValuesEnabled = false
                        lineChartDataSet.lineWidth = 1.5
                        let devicename = self.getlocalname(desc: self.arr_lines[i].description ?? "",localname: self.arr_lines[i].localName ?? "")
                        lineChartDataSet.label = devicename
                        if self.arr_lines[i].attributeColor != "" {
                            lineChartDataSet.circleColors = [NSUIColor(cgColor: hexStringToUIColor(hex: self.arr_lines[i].attributeColor ?? "").cgColor)]
                            lineChartDataSet.colors = [NSUIColor(cgColor: hexStringToUIColor(hex: self.arr_lines[i].attributeColor ?? "").cgColor)]
                        } else {
                            lineChartDataSet.circleColors = [NSUIColor(cgColor: hexStringToUIColor(hex: Constant.colorArrays[i]).cgColor)]
                            lineChartDataSet.colors = [NSUIColor(cgColor: hexStringToUIColor(hex: Constant.colorArrays[i]).cgColor)]
                        }
                        dataentries.append(lineChartDataSet)
                        
                        var dict : [String:Any] = [:]
                        dict["title"] = self.arr_lines[i].localName
                        dict["entries"] = dataentries
                        dict["cellname"] = cellname
                        self.liveDataSetList.append(dict)
                    }
                }
                DispatchQueue.main.asyncAfter(deadline: .now()) {
                    if self.arrSensors.count > 0 {
                        self.lblData.isHidden = true
                        self.tblData.isHidden = false
                        self.tblData.reloadData()
                        self.isDefaultDataReady = true
                      //  self.startMQTT()
                    } else {
                        self.lblData.isHidden = false
                        self.tblData.isHidden = true
                        self.isDefaultDataReady = true
                      //  self.startMQTT()
                    }
                }
            }
            else
            {
                DispatchQueue.main.async {
                    self.lblData.isHidden = false
                    self.tblData.isHidden = true
                    self.isDefaultDataReady = true
                   // self.startMQTT()
                }
            }
        }, failureBlock: { error in
            print("\n\n===========Error===========")
            print("Error Code: \(error._code)")
            print("Error Messsage: \(error.localizedDescription)")
            ProHudManager.shared.hide(animated: true)
        })
    }
    
    func setData (arraays : [DataInfo], name : String) {
        self.setdata2(arraays: arraays, name: name)
    }
    
    func setdata2 (arraays : [DataInfo], name : String){
        for j in 0..<arraays.count {
            var str_desc : String = arraays[j].description ?? ""
            let devicename = self.getlocalname(desc: arraays[j].description ?? "",localname: arraays[j].localName ?? "")
            str_desc = str_desc.slice(from: "[", to: "]") ?? ""
            if str_desc.uppercased().contains(Constant.SENSOR_TYPE.GAUGE){
                print(Constant.SENSOR_TYPE.GAUGE)
                str_desc = self.removeWhiteSpaceandUpparcase(desc: str_desc, type: Constant.SENSOR_TYPE.GAUGE)
                var data: DataInfo = arraays[j]
                data.type = Constant.SENSOR_TYPE.GAUGE
                let guagearay = str_desc.components(separatedBy: ":")
                if guagearay.count == 2 {
                    if self.removeWhiteSpace(desc: guagearay[0]).isNumeric && self.removeWhiteSpace(desc: guagearay[1]).isNumeric {
                        let min: Double = Double(self.removeWhiteSpace(desc: guagearay[0])) ?? 0
                        let max: Double = Double(self.removeWhiteSpace(desc: guagearay[1])) ?? 0
                        if max > min {
                            data.min = min
                            data.max = max
                            data.devicename = devicename
                            self.arr_gauges.append(data)
                        }
                    }
                }
            }
            else if name == Constant.SENSOR_TYPE.BT {
                print(Constant.SENSOR_TYPE.BT)
                str_desc = self.removeWhiteSpaceandUpparcase(desc: str_desc, type: Constant.SENSOR_TYPE.BT)
                var data: DataInfo = arraays[j]
                data.type = Constant.SENSOR_TYPE.BT
                data.devicename = devicename
                self.arr_btsensors.append(data)
            }
            else if name == Constant.SENSOR_TYPE.LED {
                print(Constant.SENSOR_TYPE.LED)
                str_desc = self.removeWhiteSpaceandUpparcase(desc: str_desc, type: Constant.SENSOR_TYPE.SLIDER)
                var data: DataInfo = arraays[j]
                data.type = Constant.SENSOR_TYPE.LED
                data.devicename = devicename
                self.arr_led.append(data)
            } else if name == Constant.SENSOR_TYPE.SLIDER {
                print(Constant.SENSOR_TYPE.SLIDER)
                str_desc = self.removeWhiteSpaceandUpparcase(desc: str_desc, type: Constant.SENSOR_TYPE.SLIDER)
                var data: DataInfo = arraays[j]
                data.type = Constant.SENSOR_TYPE.SLIDER
                let sliderarray = str_desc.components(separatedBy: ":")
                if sliderarray.count > 1 {
                    if self.removeWhiteSpace(desc: sliderarray[0]).isNumeric && self.removeWhiteSpace(desc: sliderarray[1]).isNumeric {
                        let min: Double = Double(self.removeWhiteSpace(desc: sliderarray[0])) ?? 0
                        let max: Double = Double(self.removeWhiteSpace(desc: sliderarray[1])) ?? 0
                        if max > min {
                            data.min = min
                            data.max = max
                            data.devicename = devicename
                            self.arr_sliders.append(data)
                        }
                    }
                }
            }
        }
    }
    
    func getlocalname(desc: String, localname: String) -> String {
        var locallname : String = ""
        if let strName = (desc.slice(from: "[", to: "]") ?? "").appending("]") as? String, strName.count > 1 {
            locallname = desc.components(separatedBy: strName)[1]
            locallname = locallname.replacingOccurrences(of: "<[^>]+>\n", with: "", options: .regularExpression, range: nil)
            locallname = locallname.trimmingCharacters(in: .whitespaces)
            if locallname == "" {
                locallname = localname
            }
        } else { locallname = localname }
        return locallname
    }
    
    func removeWhiteSpace(desc: String) -> String {
        var value = desc
        value = value.trimmingCharacters(in: .whitespaces)
        return value
    }
    
    func removeWhiteSpaceandUpparcase(desc: String, type: String) -> String{
        var value = desc.uppercased()
        value = value.replacingOccurrences(of: type, with: "") //remove guage string
        value = value.replacingOccurrences(of: "-", with: ":") //remove guage string
        value = value.trimmingCharacters(in: .whitespaces)
        return value
    }
    
    func setarray (array_filter : [DataInfo], name : String) -> [DataInfo] {
        var array : [DataInfo] = [DataInfo]()
        if name == Constant.SENSOR_TYPE.GRAPH {
            array = array_filter.filter{($0.description == "" && $0.dataTypeName == "NUMBER") || ((($0.description?.uppercased().contains("GRAPH"))!) && $0.dataTypeName == "NUMBER") || ($0.description == "" && $0.dataTypeName == "LONG") || ((($0.description?.uppercased().contains("GRAPH"))!) && $0.dataTypeName == "LONG") || ($0.description == "" && $0.dataTypeName == "INTEGER") || ((($0.description?.uppercased().contains("GRAPH"))!) && $0.dataTypeName == "INTEGER") || ($0.description == "" && $0.dataTypeName == "DECIMAL") || ((($0.description?.uppercased().contains("GRAPH"))!) && $0.dataTypeName == "DECIMAL") || ($0.description == "" && $0.dataTypeName == "OBJECT") || ((($0.description?.uppercased().contains("GRAPH"))!) && $0.dataTypeName == "OBJECT")}
        } else if name == Constant.SENSOR_TYPE.GAUGE || name == Constant.SENSOR_TYPE.SLIDER  {
            array = array_filter.filter{($0.dataTypeName == "NUMBER") || ($0.dataTypeName == "LONG") || ($0.dataTypeName == "INTEGER") || ($0.dataTypeName == "DECIMAL")}
        } else if name == Constant.SENSOR_TYPE.BT || name == Constant.SENSOR_TYPE.LED {
            array = array_filter.filter{($0.dataTypeName == "BOOLEAN") || ($0.dataTypeName == "NUMBER")}
        } else {
            array = array_filter.filter{($0.dataTypeName == "STRING")}
        }
        return array
    }
    
    func setarray2 (array_filter : [DataInfo]) -> [DataInfo] {
        var array : [DataInfo] = [DataInfo]()
        for j in 0..<array_filter.count {
            let data: DataInfo = array_filter[j]
            var str_desc : String = array_filter[j].description ?? ""
            if str_desc.contains(Constant.SENSOR_TYPE.UNKNOWN) || array_filter[j].dataTypeName == "STRING" {
                if (self.call_data_set(dict_data: data, name: Constant.SENSOR_TYPE.UNKNOWN)) {
                    array.append(data)
                }
            }
            let devicename = self.getlocalname(desc: array_filter[j].description ?? "",localname: array_filter[j].localName ?? "")
            str_desc = str_desc.slice(from: "[", to: "]") ?? ""
            if str_desc.uppercased().contains(Constant.SENSOR_TYPE.GRAPH) || str_desc == "" {
                if (self.call_data_set(dict_data: data, name: Constant.SENSOR_TYPE.GRAPH)) {
                    array.append(data)
                } else if str_desc == "" {
                    if (self.call_data_set(dict_data: data, name: Constant.SENSOR_TYPE.UNKNOWN)) {
                        array.append(data)
                    }
                }
            } else if str_desc.uppercased().contains(Constant.SENSOR_TYPE.GAUGE){
                if (self.call_data_set(dict_data: data, name: Constant.SENSOR_TYPE.GAUGE)) {
                    array.append(data)
                }
            } else if str_desc.uppercased().contains(Constant.SENSOR_TYPE.SLIDER){
                if (self.call_data_set(dict_data: data, name: Constant.SENSOR_TYPE.SLIDER)) {
                    array.append(data)
                }
            } else if str_desc.uppercased().contains(Constant.SENSOR_TYPE.BT){
                if (self.call_data_set(dict_data: data, name: Constant.SENSOR_TYPE.BT)) {
                    array.append(data)
                }
            } else if str_desc.uppercased().contains(Constant.SENSOR_TYPE.LED){
                if (self.call_data_set(dict_data: data, name: Constant.SENSOR_TYPE.LED)) {
                    array.append(data)
                }
            }
        }
        let arr = Array(NSOrderedSet.init(array: array)) as? [DataInfo] ?? []
        return arr
    }
    
    func call_data_set(dict_data : DataInfo, name : String) -> Bool {
        var array : [DataInfo] = [DataInfo]()
        if name == Constant.SENSOR_TYPE.GRAPH {
            array = [dict_data].filter{($0.description == "" && $0.dataTypeName == "NUMBER") || ((($0.description?.uppercased().contains("GRAPH"))!) && $0.dataTypeName == "NUMBER") || ($0.description == "" && $0.dataTypeName == "LONG") || ((($0.description?.uppercased().contains("GRAPH"))!) && $0.dataTypeName == "LONG") || ($0.description == "" && $0.dataTypeName == "INTEGER") || ((($0.description?.uppercased().contains("GRAPH"))!) && $0.dataTypeName == "INTEGER") || ($0.description == "" && $0.dataTypeName == "DECIMAL") || ((($0.description?.uppercased().contains("GRAPH"))!) && $0.dataTypeName == "DECIMAL") || ($0.description == "" && $0.dataTypeName == "OBJECT") || ((($0.description?.uppercased().contains("GRAPH"))!) && $0.dataTypeName == "OBJECT")}
            if array.count > 0 {
                return true
            }
        }else if name == Constant.SENSOR_TYPE.GAUGE  {
            array = [dict_data].filter{($0.dataTypeName == "NUMBER") || ($0.dataTypeName == "LONG") || ($0.dataTypeName == "INTEGER") || ($0.dataTypeName == "DECIMAL")}
            
            var str_desc = dict_data.description?.slice(from: "[", to: "]") ?? ""
            str_desc = self.removeWhiteSpaceandUpparcase(desc: str_desc, type: Constant.SENSOR_TYPE.GAUGE)
            let guagearay = str_desc.components(separatedBy: ":")
            if guagearay.count == 2 {
                if self.removeWhiteSpace(desc: guagearay[0]).isNumeric && self.removeWhiteSpace(desc: guagearay[1]).isNumeric {
                    let min: Double = Double(self.removeWhiteSpace(desc: guagearay[0])) ?? 0
                    let max: Double = Double(self.removeWhiteSpace(desc: guagearay[1])) ?? 0
                    if max > min {
                        if array.count > 0 {
                            return true
                        }
                    }
                }
            }
            return false
        }
        else if name == Constant.SENSOR_TYPE.SLIDER  {
            array = [dict_data].filter{($0.dataTypeName == "NUMBER") || ($0.dataTypeName == "LONG") || ($0.dataTypeName == "INTEGER") || ($0.dataTypeName == "DECIMAL")}
            if array.count > 0 {
                return true
            }
        } else if name == Constant.SENSOR_TYPE.BT || name == Constant.SENSOR_TYPE.LED {
            array = [dict_data].filter{($0.dataTypeName == "BOOLEAN") || ($0.dataTypeName == "NUMBER")}
            if array.count > 0 {
                return true
            }
        } else {
            array = [dict_data].filter{($0.dataTypeName == "STRING")}
            if array.count > 0 {
                return true
            }
        }
        return false
    }
    
    // MARK: - get signalR Data from mqtt
    func passData2(dict: [String : Any]) {
        
        if !self.isDefaultDataReady{
            return
        }
        
        let finaldict : [String : Any] = dict["data"] as! [String : Any]
        if let status = finaldict["status"] {
            if status as? String == "off" {
                print("**** Status is OFF *******")
                connect = false
                isDeviceConnected?(false)
                self.callTelemetryDefaultWebService(strDeviceGUID: strDeviceGUID)
            }else{
                print("**** Status is ON *******")
                connect = true
                isDeviceConnected?(true)
            }
       }
    
        if finaldict["reporting"] != nil{
        connect = true
        isDeviceConnected?(true)
        let reportdict : [String : Any] = finaldict["reporting"] as! [String : Any]
        if reportdict.count > 0 {
             let myKeys: [String] = reportdict.map{String($0.key) }
             let values = reportdict.values.compactMap { $0 }
             if self.telelivearray.count > 0 {
                self.telelivearray.removeAll()
             }
                for j in 0..<self.arrSensors.count {
                    var str_desc : String = self.arrSensors[j].description ?? ""
                    let arrgauges : [DataInfo] = self.arrSensors.filter{($0.localName == self.arrSensors[j].localName)}
                    let indexPath = IndexPath(item: j, section: 0)

                    if str_desc.contains(Constant.SENSOR_TYPE.UNKNOWN) || self.arrSensors[j].dataTypeName == "STRING"  {
                        if arrgauges.count > 0
                        {
                            self.callNotconsider(indexpath: indexPath, arrnot: arrgauges, myKeys: myKeys, values: values)
                        }
                    }
                    str_desc = str_desc.slice(from: "[", to: "]") ?? ""
                    let strname = self.getlocalname(desc: self.arrSensors[j].description ?? "",localname: self.arrSensors[j].localName ?? "")

                    if str_desc.uppercased().contains(Constant.SENSOR_TYPE.GRAPH) || str_desc == "" {
                        
                        self.callGrpah(indexpath: indexPath, myKeys: myKeys, values: values, j: j, finaldict: finaldict, strname: strname)
                    } else if str_desc.uppercased().contains(Constant.SENSOR_TYPE.GAUGE) {
                        if arrgauges.count > 0
                        {
                            self.callGauge(indexpath: indexPath, arrgauges: arrgauges, myKeys: myKeys, values: values,j: j)
                        }
                    } else if str_desc.uppercased().contains(Constant.SENSOR_TYPE.SLIDER) {
                        if arrgauges.count > 0
                        {
                            self.callSlider(indexpath: indexPath, arrslider: arrgauges, myKeys: myKeys, values: values,j: j)
                        }
                    } else if str_desc.uppercased().contains(Constant.SENSOR_TYPE.BT) {
                        if arrgauges.count > 0
                        {
                            self.callBT(indexpath: indexPath, arrbt: arrgauges, myKeys: myKeys, values: values, j: j)
                        }
                    } else if str_desc.uppercased().contains(Constant.SENSOR_TYPE.LED) {
                        if arrgauges.count > 0
                        {
                            self.callLed(indexpath: indexPath, arrled: arrgauges, myKeys: myKeys, values: values, j: j)
                        }
                    }
                }
                indtemp = indtemp + 1
           }
        }
    }
    
    // MARK: - Set notconsider from mqtt
    func callNotconsider (indexpath : IndexPath, arrnot : [DataInfo], myKeys : [String], values : [Any]) {
        if let index = myKeys.firstIndex(where: {$0 == arrnot[0].localName})
        {
            if ((tblData.indexPathsForVisibleRows?.contains(indexpath))!) {
              // Your code here
                guard let cell : UnknownCell = tblData.cellForRow(at: indexpath) as? UnknownCell else {return}
                cell.objLbl.text = (values[index] as! String)
            }
        }
    }
    
    // MARK: - Set gauge from mqtt
    func callGauge (indexpath : IndexPath, arrgauges : [DataInfo], myKeys : [String], values : [Any], j: Int) {
        if let index = myKeys.firstIndex(where: {$0 == arrgauges[0].localName})
        {
            if ((tblData.indexPathsForVisibleRows?.contains(indexpath))!) {
                guard let cell : GaugeCell = tblData.cellForRow(at: indexpath) as? GaugeCell else {return}
//                let replaced = (values[index] as AnyObject).replacingOccurrences(of: ",", with: "") // "aaaaaaaa"
//                cell.loadGraph(value: self.getval(value: replaced) as? Float ?? 0.0)
                if let stringArray = values[index] as? NSNumber {
                    let replaced = stringArray // "aaaaaaaa"
                    cell.loadGraph(value: self.getval(value: replaced) as? Float ?? 0.0)
                }
                else{
                    let replaced = (values[index] as AnyObject).replacingOccurrences(of: ",", with: "") // "aaaaaaaa"
                    cell.loadGraph(value: self.getval(value: replaced) as? Float ?? 0.0)
                }

            } else {
            }
            var dict_gauge : [String : Any] = [:]
            dict_gauge["templateAttributeGuid"] = self.arrSensors[j].guid
            if let stringArray = values[index] as? NSNumber {
                let replaced = stringArray // "aaaaaaaa"
                dict_gauge["attributeValue"] = String(format: "%g", self.getval(value: replaced) as? Float ?? 0.0)
            }
            else{
                let replaced = (values[index] as AnyObject).replacingOccurrences(of: ",", with: "") // "aaaaaaaa"
                dict_gauge["attributeValue"] = String(format: "%g", self.getval(value: replaced) as? Float ?? 0.0)
            }
            self.telelivearray.append(dict_gauge)
        }
    }
    
    // MARK: - Set slider from mqtt
    func callSlider (indexpath : IndexPath, arrslider : [DataInfo], myKeys : [String], values : [Any], j: Int) {
        if let index = myKeys.firstIndex(where: {$0 == arrslider[0].localName})
        {
            if ((tblData.indexPathsForVisibleRows?.contains(indexpath))!) {
              // Your code here
                guard let cell : SliderCell = tblData.cellForRow(at: indexpath) as? SliderCell else {return}
                var sliderValue: String = values[index] as? String ?? ""
                sliderValue = sliderValue.replacingOccurrences(of: ",", with: "", options: NSString.CompareOptions.literal, range: nil)
//                let sliderValue: String = values[index].replacingOccurrences(of: ",", with: "", options: NSString.CompareOptions.literal, range: nil) ?? ""
//                cell.objSlider.value = self.getval(value: values[index]) as! Float
                cell.objSlider.value = self.getval(value: sliderValue) as! Float
                cell.objlblCurrentValue.text = String(format: "%.f", self.getval(value: sliderValue) as! Float)
            } else {
            }
            
            var sliderValue: String = values[index] as? String ?? ""
            sliderValue = sliderValue.replacingOccurrences(of: ",", with: "", options: NSString.CompareOptions.literal, range: nil)
            var dict_gauge : [String : Any] = [:]
            dict_gauge["templateAttributeGuid"] = self.arrSensors[j].guid
            dict_gauge["attributeValue"] = String(format: "%g", self.getval(value: sliderValue) as? Float ?? 0.0)
            self.telelivearray.append(dict_gauge)
            
            for j in 0..<self.teledefaultarray.count {
                let attributeID: String = dict_gauge["templateAttributeGuid"] as? String ?? ""
                if self.teledefaultarray[j]["templateAttributeGuid"] as! String == attributeID{
                 //   let val = self.teledefaultarray[j]["attributeValue"] ?? ""
                    self.teledefaultarray[j]["attributeValue"] = dict_gauge["attributeValue"]
                }
            }
        }
    }
    
    // MARK: - Set bt from mqtt
    func callBT (indexpath : IndexPath, arrbt : [DataInfo], myKeys : [String], values : [Any], j: Int) {
        if let index = myKeys.firstIndex(where: {$0 == arrbt[0].localName})
        {
            if ((tblData.indexPathsForVisibleRows?.contains(indexpath))!) {
                guard let cell : BTSensorCell = tblData.cellForRow(at: indexpath) as? BTSensorCell else {return}
                let str_sensor : String = self.arrSensors[j].localName ?? ""
                let val = values[index]
                if let num = val as? NSNumber {
                    let onoff : Bool =  num.boolValue
                    cell.objBtnswitch.isSelected = onoff
//                    print("Number")
                } else if let num = val as? String {
                    let onoff : Bool = NSString(string:  num).boolValue
                    cell.objBtnswitch.isSelected = onoff
//                    print("String")
                } else if let num = val as? Int {
                    var onoff : Bool = false
                    if num == 0 {
                        onoff = false
                    } else {
                        onoff = true
                    }
                    cell.objBtnswitch.isSelected = onoff
//                    print("Int")
                } else if let num = val as? Bool {
                    let onoff : Bool = num
                    cell.objBtnswitch.isSelected = onoff
//                    print("Bool")
                }
                
                let arrCommand = self.commandsarr.filter{($0.command?.contains(str_sensor))!}
                if arrCommand.count > 0 {
                    cell.objBtnswitch.isUserInteractionEnabled = true
                    cell.objBtnswitch.alpha = 1.0
                }
                else {
                    cell.objBtnswitch.isUserInteractionEnabled = false
                    cell.objBtnswitch.alpha = 0.5
                }
            } else {
            }
            var dict_gauge : [String : Any] = [:]
            dict_gauge["templateAttributeGuid"] = self.arrSensors[j].guid
            dict_gauge["attributeValue"] = String(format: "%g", self.getvalBTLED(value: values[index]) as? Float ?? 0.0)
            self.telelivearray.append(dict_gauge)
            
            for j in 0..<self.teledefaultarray.count {
                let attributeID: String = dict_gauge["templateAttributeGuid"] as? String ?? ""
                if self.teledefaultarray[j]["templateAttributeGuid"] as! String == attributeID{
                 //   let val = self.teledefaultarray[j]["attributeValue"] ?? ""
                    self.teledefaultarray[j]["attributeValue"] = dict_gauge["attributeValue"]
                }
            }
        }
    }
    
    // MARK: - Set led from mqtt
    func callLed (indexpath : IndexPath, arrled : [DataInfo], myKeys : [String], values : [Any], j: Int) {
        if let index = myKeys.firstIndex(where: {$0 == arrled[0].localName})
        {
            if ((tblData.indexPathsForVisibleRows?.contains(indexpath))!) {
                guard let cell : LEDCell = tblData.cellForRow(at: indexpath) as? LEDCell else {return}
                let str_sensor : String = self.arrSensors[j].localName ?? ""
                let arrCommand = self.commandsarr.filter{($0.command?.contains(str_sensor))!}
                if arrCommand.count > 0 {
                    cell.objBtn.isUserInteractionEnabled = true
                    let val = values[index]
                    if let num = val as? NSNumber {
                        let onoff : Bool =  num.boolValue
                        if (onoff) {
                            cell.objBtn.setImage(UIImage(named: "led_on_editable"), for: .normal)
                        }
                        else {
                            cell.objBtn.setImage(UIImage(named: "led_off_editable"), for: .normal)
                        }
//                        print("Number")
                    } else if let num = val as? String {
                        let onoff : Bool = NSString(string:  num).boolValue
                        if (onoff) {
                            cell.objBtn.setImage(UIImage(named: "led_on_editable"), for: .normal)
                        }
                        else {
                            cell.objBtn.setImage(UIImage(named: "led_off_editable"), for: .normal)
                        }
//                        print("String")
                    } else if let num = val as? Int {
                        var onoff : Bool = false
                        if num == 0 {
                            onoff = false
                        } else {
                            onoff = true
                        }
                        if (onoff) {
                            cell.objBtn.setImage(UIImage(named: "led_on_editable"), for: .normal)
                        }
                        else {
                            cell.objBtn.setImage(UIImage(named: "led_off_editable"), for: .normal)
                        }
//                        print("Int")
                    } else if let num = val as? Bool {
                        let onoff : Bool = num
                        if (onoff) {
                            cell.objBtn.setImage(UIImage(named: "led_on_editable"), for: .normal)
                        }
                        else {
                            cell.objBtn.setImage(UIImage(named: "led_off_editable"), for: .normal)
                        }
//                        print("Bool")
                    }
                }
                else {
                    cell.objBtn.isUserInteractionEnabled = false
                    let val = values[index]
                    if let num = val as? NSNumber {
                        let onoff : Bool =  num.boolValue
                        if (onoff) {
                            cell.objBtn.setImage(UIImage(named: "led_on_noneditable"), for: .normal)
                        }
                        else {
                            cell.objBtn.setImage(UIImage(named: "led_off_noneditable"), for: .normal)
                        }
//                        print("Number")
                    } else if let num = val as? String {
                        let onoff : Bool = NSString(string:  num).boolValue
                        if (onoff) {
                            cell.objBtn.setImage(UIImage(named: "led_on_noneditable"), for: .normal)
                        }
                        else {
                            cell.objBtn.setImage(UIImage(named: "led_off_noneditable"), for: .normal)
                        }
//                        print("String")
                    } else if let num = val as? Int {
                        var onoff : Bool = false
                        if num == 0 {
                            onoff = false
                        } else {
                            onoff = true
                        }
                        if (onoff) {
                            cell.objBtn.setImage(UIImage(named: "led_on_noneditable"), for: .normal)
                        }
                        else {
                            cell.objBtn.setImage(UIImage(named: "led_off_noneditable"), for: .normal)
                        }
//                        print("Int")
                    } else if let num = val as? Bool {
                        let onoff : Bool = num
                        if (onoff) {
                            cell.objBtn.setImage(UIImage(named: "led_on_noneditable"), for: .normal)
                        }
                        else {
                            cell.objBtn.setImage(UIImage(named: "led_off_noneditable"), for: .normal)
                        }
//                        print("Bool")
                    }
                }
            } else {
            }
            var dict_gauge : [String : Any] = [:]
            dict_gauge["templateAttributeGuid"] = self.arrSensors[j].guid
            dict_gauge["attributeValue"] = String(format: "%g", self.getvalBTLED(value: values[index]) as? Float ?? 0.0)
            self.telelivearray.append(dict_gauge)
            
            for j in 0..<self.teledefaultarray.count {
                let attributeID: String = dict_gauge["templateAttributeGuid"] as? String ?? ""
                if self.teledefaultarray[j]["templateAttributeGuid"] as! String == attributeID{
                 //   let val = self.teledefaultarray[j]["attributeValue"] ?? ""
                    self.teledefaultarray[j]["attributeValue"] = dict_gauge["attributeValue"]
                }
            }

        }
    }
    
    // MARK: - Set live grpah from mqtt
    func callGrpah (indexpath : IndexPath, myKeys : [String], values : [Any], j : Int, finaldict : [String : Any], strname : String) {
        if let index = myKeys.firstIndex(where: {$0 == self.arrSensors[j].localName})
        {
            if let vall = values[index] as? NSDictionary {
                let dict : [String:Any] = vall as! [String : Any]
                let myKeys: [String] = dict.map{String($0.key) }
                let values = dict.values.compactMap { $0 }
                let arr_data = liveDataSetList.filter { (obj) -> Bool in
                    return obj["title"] as? String == self.arrSensors[j].localName
                }
                if arr_data.count > 0 {
                    var arrEntries : [ChartDataSet] = []
                    arrEntries = arr_data[0]["entries"] as! [ChartDataSet]
                    
                    for k in 0..<arrEntries.count {
                        if let index = myKeys.firstIndex(where: {$0 == arrEntries[k].label})
                        {
                            let dataEntry = ChartDataEntry(x:Double(indtemp), y:Double(self.getval(value: values[index]) as! Float))
                            arrEntries[k].append(dataEntry)
                        }
                    }
                    self.updateArray(strDate: finaldict["time"] as! String,idx: j, name: strname, isObj : true,DataEntries: arrEntries)
                }
            } else {
                let arr_data = liveDataSetList.filter { (obj) -> Bool in
                    return obj["title"] as? String == self.arrSensors[j].localName
                }
                
                if arr_data.count > 0 {
                    var arrEntries : [ChartDataSet] = []
                    arrEntries = arr_data[0]["entries"] as! [ChartDataSet]
                    for k in 0..<arrEntries.count {
                        if let index = myKeys.firstIndex(where: {$0 == arr_data[0]["title"] as! String})
                        {
                            let dataEntry = ChartDataEntry(x:Double(indtemp), y:Double(self.getval(value: values[index]) as! Float))
                            arrEntries[k].append(dataEntry)
                        }
                    }
                    self.updateArray(strDate: finaldict["time"] as! String,idx: j, name: strname, isObj : true,DataEntries: arrEntries)
                }
            }
        }
    }
    
    func getval(value : Any) -> Any {
        let val = value
        if let num = val as? NSNumber {
            return num.floatValue
        } else if let num = val as? String {
            return num.floatValue
        } else if let num = val as? Int {
            return Float(num)
        }
        return 0.0
    }
    
    func getvalBTLED(value : Any) -> Any {
        let val = value
        if let num = val as? NSNumber {
            return num.floatValue
        } else if let num = val as? String {
            let numberFormatter = NumberFormatter()
            let number = numberFormatter.number(from: num)
            return number?.floatValue as Any
        } else if let num = val as? Int {
            return Float(num)
        } else if let num = val as? Bool {
            return Float(num ? 1 : 0)
        }
        return 0.0
    }
    
    //MARK:- SensorCommandList Web-Service
    func callSensorCommandListsWebService(strDeviceTemplateGUID : String)
    {
        let param = [Constant.RequestConstant.pageindex:1,Constant.RequestConstant.pagesize:Constant.k_PAGE_SIZE,Constant.RequestConstant.devicetemplateguid:strDeviceTemplateGUID] as [String : Any]
        DeviceManager.shared.sensorCommandListsWebService(paramers: param ,successBlock: { [self]  (response) in
            print(String(data: response, encoding: .utf8) as Any)
            let decoder = JSONDecoder()
            let commandlists = try? decoder.decode(CommandList.self, from: response)
            if commandlists != nil
            {
                if (commandlists?.data)!.count > 0  {
                    if( commandlists?.data)![0].commands != nil {
                        self.commandsarr = (commandlists?.data)![0].commands!
                    }
                }
                self.callSensorLists(strDeviceTemplateGUID: strDeviceTemplateGUID)

            }
            else
            {
                self.callSensorLists(strDeviceTemplateGUID: strDeviceTemplateGUID)
            }
        }, failureBlock: { error in
            print("\n\n===========Error===========")
            print("Error Code: \(error._code)")
            print("Error Messsage: \(error.localizedDescription)")
            ProHudManager.shared.hide(animated: true)
        })
    }
    
    
    func hexStringToUIColor (hex:String) -> UIColor {
        var cString:String = hex.trimmingCharacters(in: .whitespacesAndNewlines).uppercased()

        if (cString.hasPrefix("#")) {
            cString.remove(at: cString.startIndex)
        }

        if ((cString.count) != 6) {
            return UIColor.gray
        }

        var rgbValue:UInt64 = 0
        Scanner(string: cString).scanHexInt64(&rgbValue)

        return UIColor(
            red: CGFloat((rgbValue & 0xFF0000) >> 16) / 255.0,
            green: CGFloat((rgbValue & 0x00FF00) >> 8) / 255.0,
            blue: CGFloat(rgbValue & 0x0000FF) / 255.0,
            alpha: CGFloat(1.0)
        )
    }
    
    //MARK:- Graph Integration

    func setDataToSplineGraph(indxx : Int, name: String, isObj : Bool, DataEntries : [ChartDataSet])
    {
        if livedataSetsList.count > 500 {
            livedataSetsList.removeAll()
            self.timeInterval.removeAll()
        }

        let indexPath = IndexPath(item: indxx, section: 0)
        guard (tblData.indexPathsForVisibleRows?.contains(indexPath)) != nil else {
            self.tblData.reloadData()
           return
        }
        
        if ((tblData.indexPathsForVisibleRows?.contains(indexPath))!) {
          // Your code here
            let cell : LineGraphCell = tblData.cellForRow(at: indexPath)! as! LineGraphCell

            cell.chartView.setVisibleXRangeMaximum(4)
            cell.chartView.xAxis.setLabelCount(4, force: false)
            cell.chartView.minOffset = 20.0
            cell.chartView.xAxis.valueFormatter = IndexAxisValueFormatter(values: self.timeInterval)

            let lineChartData = LineChartData(dataSets: DataEntries)
            cell.chartView.data = lineChartData
            let lastXAxisValue: Double = Double(self.timeInterval.count)
            cell.chartView.moveViewToX(lastXAxisValue)
            cell.chartView.notifyDataSetChanged()
        } else {
            print("Not Visible")
            if UIDevice.current.orientation.isPortrait {
                var dict_gauge : [String : Any] = [:]
                dict_gauge["templateAttributeGuid"] = self.arrSensors[indxx].guid
                dict_gauge["attributeValue"] = DataEntries
                self.telelivearray.append(dict_gauge)
            }
        }
    }
    
    private func setup(_ dataSet: LineChartDataSet) {
            dataSet.lineDashLengths = [5, 2.5]
            dataSet.highlightLineDashLengths = [5, 2.5]
            dataSet.setColor(.red)
            dataSet.setCircleColor(.yellow)
            dataSet.gradientPositions = nil
            dataSet.lineWidth = 1
            dataSet.circleRadius = 3
            dataSet.drawCircleHoleEnabled = false
            dataSet.valueFont = .systemFont(ofSize: 9)
            dataSet.formLineDashLengths = [5, 2.5]
            dataSet.formLineWidth = 1
            dataSet.formSize = 15
    }

    func updateArray(strDate: String, idx : Int, name: String, isObj : Bool, DataEntries : [ChartDataSet]) {
        let receivedDate = self.getFormattedDateTime(strDate: strDate)
        self.timeInterval.append(receivedDate)
        setDataToSplineGraph(indxx: idx,name: name, isObj : isObj, DataEntries: DataEntries)
    }

    func getFormattedDateTime(strDate: String) -> String {
        var newDate: String = ""
        let arr = strDate.split(separator: "+")
        if arr.count > 0 {
            newDate = String(arr[0])
        } else {
            newDate = strDate
        }
        
        let dot = "."
        let strHasDot = newDate.localizedStandardContains(dot)
        var finalDate: String = ""
        if strHasDot {
            let arrDate = newDate.components(separatedBy: dot)
            if arrDate.count > 0 {
                finalDate = arrDate[0]
                //finalDate = String(newDate.dropLast(4))
            }
        } else {
            finalDate = newDate
        }

        let dateFormt = DateFormatter()
        dateFormt.dateFormat = "yyyy-MM-dd'T'HH:mm:ss"
        let parsedDate = dateFormt.date(from: String(finalDate))
        formatter.locale = Locale(identifier: "en_US_POSIX")
        formatter.timeStyle = .medium
        formatter.dateStyle = .none
        formatter.string(from: parsedDate!)
        let strFromDate = formatter.string(from: parsedDate!)
        return strFromDate
    }
    
    func resetAllarray () {
        livedataSetsList.removeAll()
        arrSensors.removeAll()
        arr_lines.removeAll()
        arr_gauges.removeAll()
        arr_btsensors.removeAll()
        arr_led.removeAll()
        arr_sliders.removeAll()
        arr_nonconsider.removeAll()
        commandsarr.removeAll()
        teledefaultarray.removeAll()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        self.startMQTT()
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        objSignalRService?.connection.stop()
        print("DataVC viewWillDisappear")
    }
    
    func startMQTT(){
        let str_format = String(format: "https://avnetsignalr.iotconnect.io/notificationhub?groupName=%@-%@", UserDefaultManager.shared[.cpId] as! CVarArg,strDeviceID)
        objSignalRService = SignalRService(url: URL(string: str_format)!)
        objSignalRService?.delegate = self
        print("DataVC viewWillAppear")
    }

    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destination.
        // Pass the selected object to the new view controller.
    }
    */

}

extension DataVC: UITableViewDelegate {
    
    //New
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat
    {
        var str_desc : String = self.arrSensors[indexPath.row].description ?? ""
        if str_desc.contains(Constant.SENSOR_TYPE.UNKNOWN) || self.arrSensors[indexPath.row].dataTypeName == "STRING" {
            return 63
        }
        str_desc = str_desc.slice(from: "[", to: "]") ?? ""
     //   print("Device Name:===> %@",str_desc)
        if str_desc.uppercased().contains(Constant.SENSOR_TYPE.GRAPH) || str_desc == ""{
            if UIDevice.current.orientation.isLandscape {
                 return 400
            } else {
                return 257
            }
        } else if str_desc.uppercased().contains(Constant.SENSOR_TYPE.GAUGE){
            return 220
        } else if str_desc.uppercased().contains(Constant.SENSOR_TYPE.BT) || str_desc.uppercased().contains(Constant.SENSOR_TYPE.LED) {
            return 63
        } else if str_desc.uppercased().contains(Constant.SENSOR_TYPE.SLIDER){
            return 98
        }
        return 250
    }
}

extension DataVC: UITableViewDataSource {
    
    //New
    func numberOfSections(in tableView: UITableView) -> Int {
            // #warning Incomplete implementation, return the number of sections
            return 1
     }
    
    //New
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.arrSensors.count
    }

    func getObject(datainfo: DataInfo)-> String {
        for j in 0..<self.teledefaultarray.count {
            if self.teledefaultarray[j]["templateAttributeGuid"] as! String == datainfo.guid ?? ""{
                let val = self.teledefaultarray[j]["attributeValue"] ?? ""
                return val as! String
            }
        }
        return ""
    }
    
    func getLiveObject(datainfo: DataInfo)-> String {
        for j in 0..<self.telelivearray.count {
            if self.telelivearray[j]["templateAttributeGuid"] as! String == datainfo.guid ?? ""{
                let val = self.telelivearray[j]["attributeValue"] ?? ""
                return val as! String
            }
        }
        for j in 0..<self.teledefaultarray.count {
            if self.teledefaultarray[j]["templateAttributeGuid"] as! String == datainfo.guid ?? ""{
                let val = self.teledefaultarray[j]["attributeValue"] ?? ""
                return val as! String
            }
        }

        return ""
    }
    
    func getLiveGrpahObject(datainfo: DataInfo)-> [ChartDataSet] {
        for j in 0..<self.telelivearray.count {
            if self.telelivearray[j]["templateAttributeGuid"] as! String == datainfo.guid ?? ""{
                let val = self.telelivearray[j]["attributeValue"] ?? ""
                return val as? [ChartDataSet] ?? []
            }
        }
        return []
    }
    
    func notconsiderCell(indexPath: IndexPath,devicename : String) -> UITableViewCell {
        let cell = tblData.dequeueReusableCell(withIdentifier: "UnknownCell", for: indexPath) as! UnknownCell
            cell.lblSensorName.text = "\(devicename)"
            if connect == false{
                cell.objLbl.text = self.getObject(datainfo: self.arrSensors[indexPath.row]) }
            else {
                if self.telelivearray.count > 0 {
                    cell.objLbl.text = self.getLiveObject(datainfo: self.arrSensors[indexPath.row])
                } else {
                    cell.objLbl.text = self.getObject(datainfo: self.arrSensors[indexPath.row])
                }
            }
        cell.selectionStyle = .none
        return cell
    }
    
    //New
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell
    {
        var str_desc : String = self.arrSensors[indexPath.row].description ?? ""
        let devicename = self.getlocalname(desc: self.arrSensors[indexPath.row].description ?? "",localname: self.arrSensors[indexPath.row].localName ?? "")

        if str_desc.contains(Constant.SENSOR_TYPE.UNKNOWN) || self.arrSensors[indexPath.row].dataTypeName == "STRING" {
            str_desc = self.removeWhiteSpaceandUpparcase(desc: str_desc, type: Constant.SENSOR_TYPE.UNKNOWN)
            return notconsiderCell(indexPath: indexPath, devicename: devicename)
        }
        
        str_desc = str_desc.slice(from: "[", to: "]") ?? ""
        
        if str_desc.uppercased().contains(Constant.SENSOR_TYPE.GRAPH) || str_desc == ""{
            str_desc = self.removeWhiteSpaceandUpparcase(desc: str_desc, type: Constant.SENSOR_TYPE.GRAPH)

            let arr_data = liveDataSetList.filter { (obj) -> Bool in
                return obj["title"] as? String == self.arrSensors[indexPath.row].localName
            }
            if arr_data.count > 0 {
                let cell = tableView.dequeueReusableCell(withIdentifier: arr_data[0]["cellname"] as! String, for: indexPath) as! LineGraphCell
                cell.selectionStyle = .none
                cell.lblTitle.text = "\(devicename)"
                cell.btn_orientation.tag = indexPath.row
                cell.btn_orientation.addTarget(self, action: #selector(btnOrient(sender:)), for: .touchUpInside)
                
                if connect == true {
                    if self.telelivearray.count > 0 {
                        let arrdata : [ChartDataSet] = self.getLiveGrpahObject(datainfo: self.arrSensors[indexPath.row])
                        if arrdata.count > 0 {
                            cell.chartView.setVisibleXRangeMaximum(4)
                            cell.chartView.xAxis.setLabelCount(4, force: false)
                            cell.chartView.minOffset = 20.0
                            cell.chartView.xAxis.valueFormatter = IndexAxisValueFormatter(values: self.timeInterval)
                            let lineChartData = LineChartData(dataSets: arrdata)
                            cell.chartView.data = lineChartData
                            let lastXAxisValue: Double = Double(self.timeInterval.count)
                            cell.chartView.moveViewToX(lastXAxisValue)
                            cell.chartView.notifyDataSetChanged()
                        }
                    }
                }
                return cell
            }
        } else if str_desc.uppercased().contains(Constant.SENSOR_TYPE.GAUGE) {
            str_desc = self.removeWhiteSpaceandUpparcase(desc: str_desc, type: Constant.SENSOR_TYPE.GAUGE)

            let cell = tableView.dequeueReusableCell(withIdentifier: "GaugeCell", for: indexPath) as! GaugeCell
            cell.selectionStyle = .none
            let guagearay = str_desc.components(separatedBy: ":")
            if guagearay.count == 2 {
                if self.removeWhiteSpace(desc: guagearay[0]).isNumeric && self.removeWhiteSpace(desc: guagearay[1]).isNumeric {
                    let min: Double = Double(self.removeWhiteSpace(desc: guagearay[0])) ?? 0
                    let max: Double = Double(self.removeWhiteSpace(desc: guagearay[1])) ?? 0
                    if max > min {
                        cell.initGraph(minValue: Float(min), maxValue:Float(max), sensorName: devicename )
                    }
                }
            }
            if connect == false{
                cell.lblCurrentValue.text = self.getObject(datainfo: self.arrSensors[indexPath.row])
                let value: Float = Float(cell.lblCurrentValue.text!.replacingOccurrences(of: ",", with: "")) ?? 0.0
                cell.chartView.value = value

            } else {
                if self.telelivearray.count > 0 {
                    cell.lblCurrentValue.text = self.getLiveObject(datainfo: self.arrSensors[indexPath.row])
                    let value: Float = Float(cell.lblCurrentValue.text!.replacingOccurrences(of: ",", with: "")) ?? 0.0
                    cell.chartView.value = value
                } else {
                    cell.lblCurrentValue.text = self.getObject(datainfo: self.arrSensors[indexPath.row])
                    let value: Float = Float(cell.lblCurrentValue.text!.replacingOccurrences(of: ",", with: "")) ?? 0.0
                    cell.chartView.value = value
                }
            }
            return cell
        } else if str_desc.uppercased().contains(Constant.SENSOR_TYPE.BT) {
            str_desc = self.removeWhiteSpaceandUpparcase(desc: str_desc, type: Constant.SENSOR_TYPE.BT)

            let cell = tableView.dequeueReusableCell(withIdentifier: "BTSensorCell", for: indexPath) as! BTSensorCell
            cell.selectionStyle = .none
            cell.objBtnswitch.tag = indexPath.row
            cell.objBtnswitch.addTarget(self, action: #selector(btnSwitchClick(sender:)), for: .touchUpInside)
            cell.objBtnswitch.isSelected = false
            cell.objLblSensorName.text = "\(devicename)"
            
            if self.getCommandDisable(str_sensor: self.arrSensors[indexPath.row].localName ?? "") {
                cell.objBtnswitch.isUserInteractionEnabled = true
                cell.objBtnswitch.alpha = 1.0
            } else {
                cell.objBtnswitch.isUserInteractionEnabled = false
                cell.objBtnswitch.alpha = 0.5
            }
            
            if connect == false{
                let strval = self.getObject(datainfo: self.arrSensors[indexPath.row])
                cell.objBtnswitch.isSelected = NSString(string: strval).boolValue
            } else {
                if self.telelivearray.count > 0 {
                    let strval = self.getLiveObject(datainfo: self.arrSensors[indexPath.row])
                    cell.objBtnswitch.isSelected = NSString(string: strval).boolValue
                } else {
                    let strval = self.getObject(datainfo: self.arrSensors[indexPath.row])
                    cell.objBtnswitch.isSelected = NSString(string: strval).boolValue
                }
            }
            return cell
        } else if str_desc.uppercased().contains(Constant.SENSOR_TYPE.LED) {
            str_desc = self.removeWhiteSpaceandUpparcase(desc: str_desc, type: Constant.SENSOR_TYPE.LED)

            let cell = tableView.dequeueReusableCell(withIdentifier: "LEDCell", for: indexPath) as! LEDCell
            cell.selectionStyle = .none
            cell.objLblSensorName.text = "\(devicename)"
            
            if self.getCommandDisable(str_sensor: self.arrSensors[indexPath.row].localName ?? "") {
                cell.objBtn.isUserInteractionEnabled = true
            } else {
                cell.objBtn.isUserInteractionEnabled = false
            }
            
            if connect == false{
                let stron = (self.getObject(datainfo: self.arrSensors[indexPath.row]))
                let isonoff : Bool =  stron.boolValue
                if isonoff {
                    cell.objBtn.setImage(UIImage(named: "led_on_editable"), for: .normal)
                    cell.objBtn.accessibilityLabel = "led_on_editable"
                } else {
                    cell.objBtn.setImage(UIImage(named: "led_off_editable"), for: .normal)
                    cell.objBtn.accessibilityLabel = "led_off_editable"
                }
            } else {
                if self.telelivearray.count > 0 {
                    let stron = (self.getLiveObject(datainfo: self.arrSensors[indexPath.row]))
                    let isonoff : Bool =  stron.boolValue
                    if isonoff {
                        cell.objBtn.setImage(UIImage(named: "led_on_editable"), for: .normal)
                        cell.objBtn.accessibilityLabel = "led_on_editable"
                    } else {
                        cell.objBtn.setImage(UIImage(named: "led_off_editable"), for: .normal)
                        cell.objBtn.accessibilityLabel = "led_off_editable"
                    }
                } else {
                    let stron = (self.getObject(datainfo: self.arrSensors[indexPath.row]))
                    let isonoff : Bool =  stron.boolValue
                    if isonoff {
                        cell.objBtn.setImage(UIImage(named: "led_on_editable"), for: .normal)
                        cell.objBtn.accessibilityLabel = "led_on_editable"
                    } else {
                        cell.objBtn.setImage(UIImage(named: "led_off_editable"), for: .normal)
                        cell.objBtn.accessibilityLabel = "led_off_editable"
                    }
                }
            }
            cell.objBtn.addTarget(self, action: #selector(btnLedCommand(sender:)), for: .touchUpInside)
            cell.objBtn.tag = indexPath.row
            return cell
        } else if str_desc.uppercased().contains(Constant.SENSOR_TYPE.SLIDER) {
            str_desc = self.removeWhiteSpaceandUpparcase(desc: str_desc, type: Constant.SENSOR_TYPE.SLIDER)

            let cell = tableView.dequeueReusableCell(withIdentifier: "SliderCell", for: indexPath) as! SliderCell
            let guagearay = str_desc.components(separatedBy: ":")
            if guagearay.count >= 2 {
                if self.removeWhiteSpace(desc: guagearay[0]).isNumeric && self.removeWhiteSpace(desc: guagearay[1]).isNumeric {
                    let min: Double = Double(self.removeWhiteSpace(desc: guagearay[0])) ?? 0
                    let max: Double = Double(self.removeWhiteSpace(desc: guagearay[1])) ?? 0
                    
                    if guagearay.count >= 3{
                        let step: Double = Double(self.removeWhiteSpace(desc: guagearay[2])) ?? 1
                        if ((isInteger(N: min)) &&  (isInteger(N: max)) ) {

                            cell.objSlider.accessibilityLabel = "\(step)"
                        }
                        else{
                            cell.objSlider.accessibilityLabel = "\(1)"
                        }


                    }
                    else {
                        cell.objSlider.accessibilityLabel = "\(1)"
                    }
                    if max > min {
                        if min.truncatingRemainder(dividingBy: 1) == 0{
                            cell.objlblmin.text = String(Int(Double(min)))
                        }else{
                            
                            cell.objlblmin.text =  String(Int(floor(Double(min))))//String(data.min)
                        }
                        if max.truncatingRemainder(dividingBy: 1) == 0 {
                            cell.objlblmax.text = String(Int(Double(max)))
                        }else{
                            cell.objlblmax.text =  String(Int(floor(Double(max))))//String(data.max)
                        }

                        cell.objSlider.minimumValue = Float(min)
                        cell.objSlider.maximumValue = Float(max)
                    }
                }
            }
            cell.objLblSensorName.text = "\(devicename)"
            cell.selectionStyle = .none
            
            if self.getCommandDisable(str_sensor: self.arrSensors[indexPath.row].localName ?? "") {
                cell.objSlider.isUserInteractionEnabled = true
                cell.objSlider.alpha = 1.0
            } else {
                cell.objSlider.isUserInteractionEnabled = false
                cell.objSlider.alpha = 0.5
            }
            cell.objSlider.tag = indexPath.row
            cell.objSlider.addTarget(self, action: #selector(onSliderValChanged(slider:event:)), for: .valueChanged)
    

            if connect == false{
                let currentValue: String = self.getObject(datainfo: self.arrSensors[indexPath.row])
                let value: String = currentValue.replacingOccurrences(of: ",", with: "", options: NSString.CompareOptions.literal, range: nil)
                cell.objlblCurrentValue.text = value
                cell.objSlider.value = value.floatValue
            } else {
                if self.telelivearray.count > 0 {
                    
                    let currentValue: String = self.getLiveObject(datainfo: self.arrSensors[indexPath.row])
                    let value: String = currentValue.replacingOccurrences(of: ",", with: "", options: NSString.CompareOptions.literal, range: nil)
                    cell.objlblCurrentValue.text = value
                    cell.objSlider.value = value.floatValue
                } else {
                    let currentValue: String = self.getObject(datainfo: self.arrSensors[indexPath.row])
                    let value: String = currentValue.replacingOccurrences(of: ",", with: "", options: NSString.CompareOptions.literal, range: nil) 
                    cell.objlblCurrentValue.text = value
                    cell.objSlider.value = value.floatValue
//                    cell.objSlider.value = Float(cell.objlblCurrentValue.text ?? "") ?? 0
                }
            }
            return cell
        }
        return UITableViewCell()
    }
    
    
      func isInteger(N: Double) -> Bool {
         // Convert float value
         // of N to integer
        let X = Int(N)
        let temp2 = N - Double(X)

         // If N is not equivalent
         // to any integer
        
        if (temp2 > 0) {
            return false
        } else {
            return true
     }
      }
    
    func getCommandDisable(str_sensor : String) -> Bool {
        var commandName = String(format: "set %@", self.removeWhiteSpace(desc: str_sensor.lowercased()))
        var arrCommand = self.commandsarr.filter{($0.name?.lowercased() == commandName)}
        if arrCommand.count == 0 {
            commandName = String(format: "set%@", self.removeWhiteSpace(desc: str_sensor.lowercased()))
            arrCommand = self.commandsarr.filter{($0.name?.lowercased() == commandName)}
        }

        if arrCommand.count > 0 {
            return true
        }
        return false
    }
    
    func getCommandArray(str_sensor : String) -> [Commands] {
        var commandName = String(format: "set %@", self.removeWhiteSpace(desc: str_sensor.lowercased()))

        var arrCommand = self.commandsarr.filter{($0.name?.lowercased() == commandName)}
        if arrCommand.count == 0 {
            commandName = String(format: "set%@", self.removeWhiteSpace(desc: str_sensor.lowercased()))
            arrCommand = self.commandsarr.filter{($0.name?.lowercased() == commandName)}
        }
        if arrCommand.count > 0 {
            return arrCommand
        }
        return []
    }
    
    override func viewWillTransition(to size: CGSize, with coordinator: UIViewControllerTransitionCoordinator) {
        super.viewWillTransition(to: size, with: coordinator)
        if self.view.isHidden == true {
            return
        }
        if UIDevice.current.orientation.isLandscape {

            if let parentVC = self.parent {
                if let parentVC = parentVC as? DeviceDetailVC {
                    parentVC.containerlayout.constant = -70
                    parentVC.containenavigationlayout.constant = 0
                    let deviceType = UIDevice.current.name
                    
                    layLeading.constant = 5
                    layTrailing.constant = 5
                    layTop.constant = 15
                    
                    layBottom.constant = 0
                    parentVC.view.layoutIfNeeded()
                    tblData.isScrollEnabled = false
                    parentVC.titleCollection.isHidden = true
                    let indexPath = IndexPath(item: isSelectedOrientedViewIndex, section: 0)
                    tblData.scrollToRow(at: indexPath, at: .top, animated: true)
                    if ((tblData.indexPathsForVisibleRows?.contains(indexPath))!) {
                      // Your code here
                        tblData.reloadRows(at: [indexPath], with: .fade)
                        DispatchQueue.main.async {
                            self.tblData.scrollToRow(at: indexPath, at: .top, animated: true)
                        }
                    }
                }
            }
        } else {
            if let parentVC = self.parent {
                if let parentVC = parentVC as? DeviceDetailVC {
                    parentVC.containerlayout.constant = 25
                    parentVC.containenavigationlayout.constant = 90
                    layTrailing.constant = 25
                    layLeading.constant = 25
                    layBottom.constant = 5
                    layTop.constant = 0
                    parentVC.view.layoutIfNeeded()
                    parentVC.titleCollection.isHidden = false
                    parentVC.titleCollection.reloadData()
                    tblData.isScrollEnabled = true
                    let indexPath = IndexPath(item: 0, section: 0)
                    if ((tblData.indexPathsForVisibleRows?.contains(indexPath))!) {
                      // Your code here
                        tblData.reloadRows(at: [indexPath], with: .fade)
                        DispatchQueue.main.async {
                            self.tblData.scrollToRow(at: indexPath, at: .top, animated: true)
                        }

                    }

                }
            }
        }
    }
}
    
extension String {
    
    func slice(from: String, to: String) -> String? {
        guard let rangeFrom = range(of: from)?.upperBound else { return nil }
        guard let rangeTo = self[rangeFrom...].range(of: to)?.lowerBound else { return nil }
        return String(self[rangeFrom..<rangeTo])
    }
    
    var isNumeric: Bool {
            guard self.count > 0 else { return false }
            let nums: Set<Character> = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "."]
            return Set(self).isSubset(of: nums)
   }
    
   var withoutSpecialCharacters: String {
        return self.components(separatedBy: CharacterSet.punctuationCharacters).joined(separator: "")
   }
    
    var boolValue: Bool {
        return (self as NSString).boolValue
    }
    
    func forTrailingZero(temp: Float) -> String {
        let tempVar = String(format: "%g", temp)
        return tempVar
    }
    
    var floatValue: Float {
        return (self as NSString).floatValue
    }
}

