//
//  HistoryVC.swift
//  EBV-IoT
//
//  Created by krutik vora on 09/02/22.
//

import UIKit
import Charts


class HistoryVC: UIViewController, HistorySelectionCellDelegate {
   
    var strDeviceGUID : String = ""
    var strDeviceID : String = ""
    var strDeviceTemplateGUID : String = ""
    var strDeviceName : String = ""
    var arrSensors : [DataInfo] = [DataInfo]()
    var arr_lines : [DataInfo] = [DataInfo]()
    var livedataSetsList : [LineChartDataSet] = []
    var timeInterval = [String]()
    var DictSensorLists : [String : AnyObject] = [:]
    var duplicatedataSetsList : [LineChartDataSet] = []

    @IBOutlet weak var chartView: LineChartView!
    var indtemp : Int = 0
    let formatter = DateFormatter()
    @IBOutlet weak var legendListTable: UITableView!
    
    @IBOutlet weak var tblhistory: UITableView!
    @IBOutlet weak var vwHistory: UIView!
    @IBOutlet weak var btnOrient: UIButton!

    @IBOutlet weak var layLeading: NSLayoutConstraint!
    @IBOutlet weak var layTrailing: NSLayoutConstraint!
    @IBOutlet weak var layTop: NSLayoutConstraint!
    @IBOutlet weak var layBottom: NSLayoutConstraint!
    @IBOutlet weak var lbl_nodata: UILabel!
    @IBOutlet weak var layTop_orient: NSLayoutConstraint!

    
    var parentvc : UIViewController?
    var onetimeclick : Bool = false
    var goballivedataSetsListss : [LineChartDataSet] = []
    
    override func viewDidLoad() {
        super.viewDidLoad()
            
        tblhistory.isHidden = true
        self.lbl_nodata.isHidden = true
        self.tblhistory.register(UINib(nibName: "HistoryGraphCell", bundle: nil), forCellReuseIdentifier: "HistoryGraphCell")
        self.tblhistory.register(UINib(nibName: "HistorySelectionCell", bundle: nil), forCellReuseIdentifier: "HistorySelectionCell")
        
        self.resetAllarray()
        
        if Constant.appDelegate.isConnectedToNetwork(){
            self.callSensorLists(strDeviceTemplateGUID: strDeviceTemplateGUID)
        }else{
            ProHudManager.shared.hide(animated: true)
            AlertConstant.showAlertMessage(self, Message: "No network available. Please check your Wifi or Data connections")
        }

    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated) // No need for semicolon
        Constant.AppUtility.lockOrientation(.portrait)
        Constant.appDelegate.objhistoryvc = self
    }
    
    //MARK:- SensorList Web-Service
    func callSensorLists(strDeviceTemplateGUID : String)
    {
        ProHudManager.shared.show(view: self.parentvc?.view, animated: true)
        let param = [Constant.RequestConstant.pageindex:1,Constant.RequestConstant.pagesize:Constant.k_PAGE_SIZE,Constant.RequestConstant.devicetemplateguid:strDeviceTemplateGUID] as [String : Any]
        DeviceManager.shared.deviceTemplateListsWebService(paramers: param ,successBlock: { [self]  (response) in
            let decoder = JSONDecoder()
            let devicesensorlists = try? decoder.decode(DeviceTemplate.self, from: response)
            
            if devicesensorlists != nil
            {
                self.arrSensors = (devicesensorlists?.data)!
                
                //For Lines Arrays
                self.arr_lines = self.setarray (array_filter : self.arrSensors, name : Constant.SENSOR_TYPE.GRAPH)
                
                    for i in 0..<self.arr_lines.count
                    {
                        let lineChartDataSet = LineChartDataSet(entries: [], label: "")
                        lineChartDataSet.circleRadius = 4.0
                        lineChartDataSet.circleHoleRadius = 2.0
                        lineChartDataSet.drawCircleHoleEnabled = true
                        lineChartDataSet.mode = .cubicBezier
                        lineChartDataSet.drawValuesEnabled = false
                        lineChartDataSet.lineWidth = 1.5
                        lineChartDataSet.label = self.arr_lines[i].localName
                        switch self.arr_lines[i].localName {
                        case "magnet":
                            let magnetColorString = [NSUIColor(cgColor: hexStringToUIColor(hex: "#82b0db").cgColor)]
                            lineChartDataSet.circleColors = magnetColorString
                            lineChartDataSet.colors = magnetColorString
                        case "co2":
                            let co2ColorString = [NSUIColor(cgColor: hexStringToUIColor(hex: "#FF00FF").cgColor)]
                            lineChartDataSet.circleColors = co2ColorString
                            lineChartDataSet.colors = co2ColorString
                        default:
                            let magnetColorString = [NSUIColor(cgColor: hexStringToUIColor(hex: "#FF00FF").cgColor)]
                            lineChartDataSet.circleColors = magnetColorString
                            lineChartDataSet.colors = magnetColorString
                            break
                        }
                        self.livedataSetsList.append(lineChartDataSet)
                    }
                    DispatchQueue.main.async {
                        self.callHistoricalDataWebService(str_deviceid: self.strDeviceID)
                    }
//                } else {
                    DispatchQueue.main.async {
                        self.tblhistory.isHidden = true
                        self.btnOrient.isHidden = true
                    }
//                }
            }
            else
            {
                DispatchQueue.main.async {
                    ProHudManager.shared.hide(animated: true)
                    self.tblhistory.isHidden = true
                    self.btnOrient.isHidden = true
                    self.lbl_nodata.isHidden = false
                }
            }
        }, failureBlock: { error in
            print("\n\n===========Error===========")
            print("Error Code: \(error._code)")
            print("Error Messsage: \(error.localizedDescription)")
            ProHudManager.shared.hide(animated: true)
        })
    }
    
    func setarray (array_filter : [DataInfo], name : String) -> [DataInfo] {
        var array : [DataInfo] = [DataInfo]()
        if name == Constant.SENSOR_TYPE.GRAPH {
            array = array_filter.filter{($0.description == "" && $0.dataTypeName == "NUMBER") || ((($0.description?.uppercased().contains("GRAPH"))!) && $0.dataTypeName == "NUMBER") || ($0.description == "" && $0.dataTypeName == "LONG") || ((($0.description?.uppercased().contains("GRAPH"))!) && $0.dataTypeName == "LONG") || ($0.description == "" && $0.dataTypeName == "INTEGER") || ((($0.description?.uppercased().contains("GRAPH"))!) && $0.dataTypeName == "INTEGER") || ($0.description == "" && $0.dataTypeName == "DECIMAL") || ((($0.description?.uppercased().contains("GRAPH"))!) && $0.dataTypeName == "DECIMAL")}
        } else if name == Constant.SENSOR_TYPE.GAUGE || name == Constant.SENSOR_TYPE.SLIDER  {
            array = array_filter.filter{($0.dataTypeName == "NUMBER") || ($0.dataTypeName == "LONG") || ($0.dataTypeName == "INTEGER") || ($0.dataTypeName == "DECIMAL")}
        } else if name == Constant.SENSOR_TYPE.BT || name == Constant.SENSOR_TYPE.LED {
            array = array_filter.filter{($0.dataTypeName == "BOOLEAN") || ($0.dataTypeName == "BIT")}
        } else {
            array = array_filter.filter{($0.dataTypeName == "STRING")}
        }
        return array
    }

    
    //MARK:- HistoricalData Web-Service
    func callHistoricalDataWebService(str_deviceid : String)
    {
        DispatchQueue.main.async {
            let today = Date().string(format: "YYYY-MM-dd HH:mm:ss")
            let thirtyDaysBeforeToday = Calendar.current.date(byAdding: .day, value: -1, to: Date())!.string(format: "YYYY-MM-dd HH:mm:ss")
            
//            let fromdate = Date.yesterday.string(format: "YYYY-MM-dd HH:mm:ss")
//            let todate = Date().string(format: "YYYY-MM-dd HH:mm:ss")
            let param = [Constant.RequestConstant.uniqueId:str_deviceid,Constant.RequestConstant.fromdate:thirtyDaysBeforeToday,Constant.RequestConstant.todate:today] as [String : Any]
            DeviceManager.shared.HistroryDataWebService(paramers: param ,successBlock: { [self] (response) in
                do {
                    if let json = try JSONSerialization.jsonObject(with: response, options: []) as? [String: Any] {
                        let data : [String:Any] = json["data"] as! [String : Any]
                        var feedrr : [[String:Any]] = []
                        feedrr = data["feed"] as! [[String : Any]]

                        if feedrr.count > 0 {
                            var arrDates: [String] = []
                            for i in 0..<feedrr.count {
                                var reportdict : [String : Any] = [:]
                                reportdict = feedrr[i]["attr"] as? [String : Any] ?? [:]
                                
                                let myKeys: [String] = reportdict.map{String($0.key) }
                                    for k in 0..<myKeys.count {
                                        let strKey = myKeys[k]
                                        if let vall = reportdict[myKeys[k]] as? NSDictionary {
                                            for j in 0..<vall.count {
                                                var strValue:String = ""
//                                                if let numberValue = vall.allValues[j] as? NSNumber{
//                                                    print(numberValue)
//                                                    strValue = "\(numberValue)"
//                                                }
                                                if let num = vall.allValues[j] as? NSNumber {
                                                    strValue = "\(num)"
                                                } else if let num = vall.allValues[j] as? String {
                                                    strValue = num
                                                } else if let num = vall.allValues[j] as? Int {
                                                    strValue = "\(num)"
                                                }
                                                let str = strValue
                                                let scan = Scanner(string: str)
                                                var val: Int32 = 0
                                                var val1: Float = 0
                                                if ((scan.scanInt32(&val) && scan.isAtEnd) || (scan.scanFloat(&val1) && scan.isAtEnd)) {
                                                    let strsubKey = String(format: "%@.%@", strKey,vall.allKeys[j] as! CVarArg)
                                                    if self.DictSensorLists[strsubKey] == nil {
                                                        let lineChartDataSet = LineChartDataSet(entries: [], label: "")
                                                        lineChartDataSet.circleRadius = 4.0
                                                        lineChartDataSet.circleHoleRadius = 2.0
                                                        lineChartDataSet.drawCircleHoleEnabled = true
                                                        lineChartDataSet.mode = .cubicBezier
                                                        lineChartDataSet.drawValuesEnabled = false
                                                        lineChartDataSet.lineWidth = 1.5
                                                        lineChartDataSet.label = strsubKey
                                                        let arrfilter = Constant.appDelegate.arr_Color_Sensor.filter({$0["name"] as? String == strsubKey})
                                                        var strcolor : String = ""
                                                        if arrfilter.count > 0 {
                                                            if arrfilter[0]["colorcode"] as? String  == "" {
                                                                strcolor = Constant.colorsubArrays[j]
                                                                lineChartDataSet.circleColors = [NSUIColor(cgColor: hexStringToUIColor(hex: Constant.colorsubArrays[j]).cgColor)]
                                                                lineChartDataSet.colors = [NSUIColor(cgColor: hexStringToUIColor(hex: Constant.colorsubArrays[j]).cgColor)]
                                                            } else {
                                                                strcolor = arrfilter[0]["colorcode"] as! String
                                                                lineChartDataSet.circleColors = [NSUIColor(cgColor: hexStringToUIColor(hex: arrfilter[0]["colorcode"] as! String).cgColor)]
                                                                lineChartDataSet.colors = [NSUIColor(cgColor: hexStringToUIColor(hex: arrfilter[0]["colorcode"] as! String).cgColor)]
                                                            }
                                                        } else {
                                                            strcolor = Constant.colorsubArrays[j]
                                                            lineChartDataSet.circleColors = [NSUIColor(cgColor: hexStringToUIColor(hex: Constant.colorsubArrays[j]).cgColor)]
                                                            lineChartDataSet.colors = [NSUIColor(cgColor: hexStringToUIColor(hex: Constant.colorsubArrays[j]).cgColor)]
                                                        }
                                                        self.DictSensorLists[strsubKey] = ["isvisible":true,"dataChart":[lineChartDataSet],"colorcode":strcolor] as AnyObject
                                                    } else {
                                                        var str = Double(0.0)
                                                        if let objVal1 = vall.allValues[j] as? String {
                                                            str = Double(objVal1) ?? 0.0
                                                        } else if let objVal2 = vall.allValues[j] as? Bool {
                                                            str = objVal2 ? 1.0 : 0.0
                                                        } else if let objVal2 = vall.allValues[j] as? NSNumber {
                                                            str = Double(truncating: objVal2)
                                                        }
                                                        let data = self.DictSensorLists[strsubKey]
                                                        var dataChart = data!["dataChart"] as! [LineChartDataSet]
                                                        let dataEntry = ChartDataEntry(x:Double(dataChart[0].entries.count), y:str)
                                                        dataChart[0].append(dataEntry)
                                                        var isVisible = data!["isvisible"] as! Bool
                                                        self.DictSensorLists[strsubKey] = ["isvisible":isVisible,"dataChart":dataChart,"colorcode":data!["colorcode"]] as AnyObject
                                                    }

                                                } else {
                                                    
                                                }
                                            }
                                        } else {
                                            var strValue:String = ""
                                            if let num = reportdict[strKey] as? NSNumber {
                                                strValue = "\(num)"
                                            } else if let num = reportdict[strKey] as? String {
                                                strValue = num
                                            } else if let num = reportdict[strKey] as? Int {
                                                strValue = "\(num)"
                                            }
                                            let str = strValue
                                            let scan = Scanner(string: str)
                                            var val: Int32 = 0
                                            var val1: Float = 0
                                            if ((scan.scanInt32(&val) && scan.isAtEnd) || (scan.scanFloat(&val1) && scan.isAtEnd)) {
                                                if self.DictSensorLists[strKey] == nil {
                                                    let lineChartDataSet = LineChartDataSet(entries: [], label: "")
                                                    lineChartDataSet.circleRadius = 4.0
                                                    lineChartDataSet.circleHoleRadius = 2.0
                                                    lineChartDataSet.drawCircleHoleEnabled = true
                                                    lineChartDataSet.mode = .cubicBezier
                                                    lineChartDataSet.drawValuesEnabled = false
                                                    lineChartDataSet.lineWidth = 1.5
                                                    lineChartDataSet.label = strKey
                                                    let arrfilter = Constant.appDelegate.arr_Color_Sensor.filter({$0["name"] as? String == strKey})
                                                    var strcolor : String = ""
                                                    if arrfilter.count > 0 {
                                                        if arrfilter[0]["colorcode"] as? String  == "" {
                                                            strcolor = Constant.colorArrays[k]
                                                            lineChartDataSet.circleColors = [NSUIColor(cgColor: hexStringToUIColor(hex: Constant.colorArrays[k]).cgColor)]
                                                            lineChartDataSet.colors = [NSUIColor(cgColor: hexStringToUIColor(hex: Constant.colorArrays[k]).cgColor)]
                                                        } else {
                                                            strcolor = arrfilter[0]["colorcode"] as! String
                                                            lineChartDataSet.circleColors = [NSUIColor(cgColor: hexStringToUIColor(hex: arrfilter[0]["colorcode"] as! String).cgColor)]
                                                            lineChartDataSet.colors = [NSUIColor(cgColor: hexStringToUIColor(hex: arrfilter[0]["colorcode"] as! String).cgColor)]
                                                        }
                                                    } else {
                                                        strcolor = Constant.colorArrays[k]
                                                        lineChartDataSet.circleColors = [NSUIColor(cgColor: hexStringToUIColor(hex: Constant.colorArrays[k]).cgColor)]
                                                        lineChartDataSet.colors = [NSUIColor(cgColor: hexStringToUIColor(hex: Constant.colorArrays[k]).cgColor)]
                                                    }
                                                    lineChartDataSet.circleColors = [NSUIColor(cgColor: hexStringToUIColor(hex: Constant.colorArrays[k]).cgColor)]
                                                    lineChartDataSet.colors = [NSUIColor(cgColor: hexStringToUIColor(hex: Constant.colorArrays[k]).cgColor)]
                                                    self.DictSensorLists[strKey] = ["isvisible":true,"dataChart":[lineChartDataSet],"colorcode":strcolor] as AnyObject
                                                } else {
                                                    let vall = reportdict[strKey]
                                                    var str = Double(0.0)
                                                    if let objVal1 = vall as? String {
                                                        str = Double(objVal1) ?? 0.0
                                                    } else if let objVal2 = vall as? Bool {
                                                        str = objVal2 ? 1.0 : 0.0
                                                    } else if let objVal2 = vall as? NSNumber {
                                                        str = Double(truncating: objVal2)
                                                    }
                                                    let data = self.DictSensorLists[strKey]
                                                    var dataChart = data!["dataChart"] as! [LineChartDataSet]
                                                    let dataEntry = ChartDataEntry(x:Double(dataChart[0].entries.count), y:str)
                                                    dataChart[0].append(dataEntry)
                                                    var isVisible = data!["isvisible"] as! Bool
                                                    self.DictSensorLists[strKey] = ["isvisible":isVisible,"dataChart":dataChart,"colorcode":data!["colorcode"]] as AnyObject
                                                }
                                            } else {
                                            }
                                        }
                                    }
//                                }
                                if arrDates.contains(feedrr[i]["dTime"] as? String ?? "") {
                                    
                                } else {
                                    arrDates.append(feedrr[i]["dTime"] as! String)
                                }
                            }
                            var cntMax1: Int = 0
                            (self.DictSensorLists as? NSDictionary)?.allKeys.forEach{ (objKey) in
                                let data = self.DictSensorLists[String(format: "%@", objKey as! CVarArg)]
                                if (data!["dataChart"] as! [LineChartDataSet])[0].entries.count > cntMax1 {
                                    cntMax1 = (data!["dataChart"] as! [LineChartDataSet])[0].entries.count
                                }
                            }

                            var indX1 = 0
                            (self.DictSensorLists as? NSDictionary)?.allKeys.forEach { (objKey) in
                                let data = self.DictSensorLists[String(format: "%@", objKey as! CVarArg)]
                                if cntMax1 != (data!["dataChart"] as! [LineChartDataSet])[0].entries.count {
                                    let cntToAdd = cntMax1 - (data!["dataChart"] as! [LineChartDataSet])[0].entries.count
                                    var idX = (data!["dataChart"] as! [LineChartDataSet])[0].entries.count
                                    for j in 0..<cntToAdd {
                                        let data = self.DictSensorLists[String(format: "%@", objKey as! CVarArg)]
                                        var dataChart = data!["dataChart"] as! [LineChartDataSet]
                                        dataChart[0].append((ChartDataEntry(x:Double(idX), y:0.0)))
                                        let isVisible = data!["isvisible"] as! Bool
                                        self.DictSensorLists[String(format: "%@", objKey as! CVarArg)] = ["isvisible":isVisible,"dataChart":dataChart,"colorcode":data!["colorcode"]] as AnyObject
                                        idX += 1
                                    }
                                    indX1 += 1
                                }
                            }

                            arrDates.forEach { (strDt) in
                                self.updateArray(strDate: strDt)
                            }
                            
                            DispatchQueue.main.async {
                                //self.legendListTable.reloadData()
                                if self.DictSensorLists.count > 0 {
                                    self.duplicatedataSetsList = livedataSetsList
                                    self.tblhistory.isHidden = false
                                    self.btnOrient.isHidden = false
                                    self.lbl_nodata.isHidden = true
                                    self.tblhistory.reloadData()
                                } else {
                                    self.tblhistory.isHidden = true
                                    self.btnOrient.isHidden = true
                                    self.lbl_nodata.isHidden = false
                                }
                                ProHudManager.shared.hide(animated: true)
                            }
                        } else {
                            DispatchQueue.main.async {
                                ProHudManager.shared.hide(animated: true)
                                self.tblhistory.isHidden = true
                                self.btnOrient.isHidden = true
                                self.lbl_nodata.isHidden = false
                            }
                        }
                  }
                } catch let error as NSError {
                    print("Failed to load: \(error.localizedDescription)")
                    ProHudManager.shared.hide(animated: true)
                }
            }, failureBlock: { error in
                print("\n\n===========Error===========")
                print("Error Code: \(error._code)")
                print("Error Messsage: \(error.localizedDescription)")
                ProHudManager.shared.hide(animated: true)
                AlertConstant.showAlertMessage(self, Message: (error as! EBVHttpErrors).errorDescription)
            })
        }
    }
    
    //MARK:- Graph Integration

    func updateArray(strDate: String) {
        let receivedDate = self.getFormattedDateTime(strDate: strDate)
        self.timeInterval.append(receivedDate)
        setDataToSplineGraph()
    }

    func getFormattedDateTime(strDate: String) -> String {
        var newDate: String = ""
        let arr = strDate.split(separator: "+")
        if arr.count > 0 {
            newDate = String(arr[0])
        } else {
            newDate = strDate
        }
        
        let dot = "."
        let strHasDot = newDate.localizedStandardContains(dot)
        var finalDate: String = ""
        if strHasDot {
            let arrDate = newDate.components(separatedBy: dot)
            if arrDate.count > 0 {
                finalDate = arrDate[0]
                //finalDate = String(newDate.dropLast(4))
            }
        } else {
            finalDate = newDate
        }

        let dateFormt = DateFormatter()
        dateFormt.dateFormat = "yyyy-MM-dd'T'HH:mm:ss"
        let parsedDate = dateFormt.date(from: String(finalDate))
        formatter.locale = Locale(identifier: "en_US_POSIX")
        formatter.timeStyle = .medium
        formatter.dateStyle = .none
        formatter.string(from: parsedDate!)
        let strFromDate = formatter.string(from: parsedDate!)
        return strFromDate
    }

    
    func setDataToSplineGraph()
    {
        DispatchQueue.main.async { [self] in
            if self.livedataSetsList.count > 500 {
                self.livedataSetsList.removeAll()
                self.timeInterval.removeAll()
            }
            let indexPath = IndexPath(item: 0, section: 0)
            
            var livedataSetsListss : [LineChartDataSet] = []
            (self.DictSensorLists as? NSDictionary)?.allKeys.forEach { (objKey) in
              let data = self.DictSensorLists[String(format: "%@", objKey as! CVarArg)]
              let dataChart = data!["dataChart"] as! [LineChartDataSet]
              if dataChart.count > 0 {
                  dataChart[0].circleColors = [NSUIColor(cgColor: hexStringToUIColor(hex: data!["colorcode"] as! String).cgColor)]
                  dataChart[0].colors = [NSUIColor(cgColor: hexStringToUIColor(hex: data!["colorcode"] as! String).cgColor)]
                  if let isvisible = data!["isvisible"] as? Bool {
                      if isvisible {
                          livedataSetsListss.append(contentsOf: dataChart)
                      }
                  }
              }
            }
            goballivedataSetsListss = livedataSetsListss
            
            if ((self.tblhistory.indexPathsForVisibleRows?.contains(indexPath))!) {
                
                let cell : HistoryGraphCell = self.tblhistory.cellForRow(at: indexPath)! as! HistoryGraphCell

                if goballivedataSetsListss.count > 0 {
                    // Your code here

                   cell.chartView.setVisibleXRangeMaximum(4)
                  //cell.chartView.setVisibleXRange(minXRange: 10.0, maxXRange: 12.0)
                    
                    cell.chartView.xAxis.setLabelCount(4, force: false)
                    cell.chartView.minOffset = 20.0
                    cell.chartView.xAxis.valueFormatter = IndexAxisValueFormatter(values: self.timeInterval)
                 
                    let lineChartData = LineChartData(dataSets: livedataSetsListss)
                    cell.chartView.data = lineChartData
                    cell.chartView.data?.setDrawValues(true)

                    let lastXAxisValue: Double = Double(self.timeInterval.count)
                    cell.chartView.moveViewToX(lastXAxisValue)
                    cell.chartView.notifyDataSetChanged()
                } else {
                    cell.chartView.clear()
                    cell.chartView.notifyDataSetChanged()
                }
            } else {
                
            }
        }
    }
        
    @IBAction func btnOrientation(_ sender: UIButton) {
        if UIDevice.current.orientation.isLandscape {
            Constant.AppUtility.lockOrientation(.portrait)
            let value = UIInterfaceOrientation.portrait.rawValue
            UIDevice.current.setValue(value, forKey: "orientation")
        } else {
            Constant.AppUtility.lockOrientation(.landscape)
            let value = UIInterfaceOrientation.landscapeLeft.rawValue
            UIDevice.current.setValue(value, forKey: "orientation")
        }
    }

    
    func hexStringToUIColor (hex:String) -> UIColor {
        var cString:String = hex.trimmingCharacters(in: .whitespacesAndNewlines).uppercased()

        if (cString.hasPrefix("#")) {
            cString.remove(at: cString.startIndex)
        }

        if ((cString.count) != 6) {
            return UIColor.gray
        }

        var rgbValue:UInt64 = 0
        Scanner(string: cString).scanHexInt64(&rgbValue)

        return UIColor(
            red: CGFloat((rgbValue & 0xFF0000) >> 16) / 255.0,
            green: CGFloat((rgbValue & 0x00FF00) >> 8) / 255.0,
            blue: CGFloat(rgbValue & 0x0000FF) / 255.0,
            alpha: CGFloat(1.0)
        )
    }
    
    func resetAllarray () {
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
    }
    
    func delegateConnected(sender: UIButton!) {

        sender.isSelected = !sender.isSelected
        
        let strkey = Constant.appDelegate.arr_History_Sensor[sender.tag]["localName"]
        if self.DictSensorLists[strkey as! String] != nil {
            let data = self.DictSensorLists[strkey as! String]
            let dataChart = data!["dataChart"] as! [LineChartDataSet]
            var isVisiblee : Bool = false
            if let isVisible = data!["isvisible"] as? Bool {
                if isVisible {
                    isVisiblee = false
                } else {
                    isVisiblee = true
                }
            }
            var dict : [String:Any] = [:]
            dict["localName"] = strkey
            dict["isvisible"] = isVisiblee
            dict["dataChart"] = dataChart
            dict["colorcode"] = data!["colorcode"]
            Constant.appDelegate.arr_History_Sensor[sender.tag] = dict
            self.DictSensorLists[strkey as! String] = ["isvisible":isVisiblee,"dataChart":dataChart,"colorcode":data!["colorcode"]] as AnyObject
            self.setDataToSplineGraph()
            
            let indexPath = IndexPath(item: 1, section: 0)
            if ((self.tblhistory.indexPathsForVisibleRows?.contains(indexPath))!) {                // Your code here
                  let cell : HistorySelectionCell = self.tblhistory.cellForRow(at: indexPath)! as! HistorySelectionCell
                cell.tblSelection.reloadData()
            }
        }
    }
    
    
    override func viewWillTransition(to size: CGSize, with coordinator: UIViewControllerTransitionCoordinator) {
        super.viewWillTransition(to: size, with: coordinator)
        if self.view.isHidden == true {
            return
        }
        if UIDevice.current.orientation.isLandscape {
            if let parentVC = self.parent {
                if let parentVC = parentVC as? DeviceDetailVC {
                    parentVC.containerlayout.constant = -70
                    parentVC.containenavigationlayout.constant = 0
                    let deviceType = UIDevice.current.name
                    
                    layLeading.constant = 5
                    layTrailing.constant = 5
                    layBottom.constant = 0
                    layTop_orient.constant = 30
                    parentVC.titleCollection.isHidden = true
                }
            }
        } else {
            if let parentVC = self.parent {
                if let parentVC = parentVC as? DeviceDetailVC {
                    parentVC.containerlayout.constant = 25
                    parentVC.containenavigationlayout.constant = 90
                    layTrailing.constant = 25
                    layLeading.constant = 25
                    layBottom.constant = 25
                    layTop_orient.constant = 13
                    parentVC.titleCollection.isHidden = false
                    parentVC.titleCollection.reloadData()

                }
            }
        }
    }
}



extension HistorySelectionCell: UITableViewDelegate {
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
      
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 44
    }
    
    func tableView(_ tableView: UITableView, viewForHeaderInSection section: Int) -> UIView? {
        
            let headerView = UIView.init(frame: CGRect.init(x: 0, y: 0, width: tableView.frame.width, height: 44))
            headerView.backgroundColor = .white
            let label = UILabel()
            label.frame = CGRect.init(x: 15, y: 5, width: headerView.frame.width-10, height: headerView.frame.height-10)
            label.text = "Input Data Selections"
            label.font = UIFont.boldSystemFont(ofSize: 16.0)
        
            let labelline = UILabel()
            labelline.frame = CGRect.init(x: 0, y: label.frame.size.height + label.frame.origin.y + 5, width: headerView.frame.width, height: 1)
            labelline.text = ""
            labelline.backgroundColor = UIColor.lightGray.withAlphaComponent(0.4)

            headerView.addSubview(label)
            headerView.addSubview(labelline)

            return headerView
   }
    
   func tableView(_ tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        return 44
   }
}

extension HistorySelectionCell: UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return Constant.appDelegate.arr_History_Sensor.count
        
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "LegendCell", for: indexPath) as! LegendCell
        cell.lblname.text = Constant.appDelegate.arr_History_Sensor[indexPath.row]["localName"] as? String
        cell.btnright.tag = indexPath.row
        cell.btnright.isSelected = Constant.appDelegate.arr_History_Sensor[indexPath.row]["isvisible"] as? Bool ?? true
        cell.vwbox.backgroundColor = self.hexStringToUIColor(hex: Constant.appDelegate.arr_History_Sensor[indexPath.row]["colorcode"] as? String ?? Constant.colorArrays[indexPath.row])
        cell.btnright.addTarget(self, action: #selector(self.connnected(sender:)), for: .touchUpInside)
        cell.selectionStyle = .none
        return cell
    }
        
    func hexStringToUIColor (hex:String) -> UIColor {
        var cString:String = hex.trimmingCharacters(in: .whitespacesAndNewlines).uppercased()

        if (cString.hasPrefix("#")) {
            cString.remove(at: cString.startIndex)
        }

        if ((cString.count) != 6) {
            return UIColor.gray
        }

        var rgbValue:UInt64 = 0
        Scanner(string: cString).scanHexInt64(&rgbValue)

        return UIColor(
            red: CGFloat((rgbValue & 0xFF0000) >> 16) / 255.0,
            green: CGFloat((rgbValue & 0x00FF00) >> 8) / 255.0,
            blue: CGFloat(rgbValue & 0x0000FF) / 255.0,
            alpha: CGFloat(1.0)
        )
    }
    
    @objc func connnected (sender : UIButton) {
        print("connnected")
        
        delegate?.delegateConnected(sender: sender)
    }
}

extension HistoryVC: UITableViewDelegate {
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        if indexPath.row == 0 {
            return 367
        } else if indexPath.row == 1 {
            return CGFloat(Constant.appDelegate.arr_History_Sensor.count * 44) + 88
        }
        return 44
    }
}

extension HistoryVC: UITableViewDataSource {
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return 2
    }
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        if tableView == tblhistory {
            if indexPath.row == 0 {
                let cell = tableView.dequeueReusableCell(withIdentifier: "HistoryGraphCell", for: indexPath) as! HistoryGraphCell
                cell.selectionStyle = .none
                if goballivedataSetsListss.count > 0 {
                    cell.chartView.setVisibleXRangeMaximum(4)
                    cell.chartView.xAxis.setLabelCount(4, force: false)
                    cell.chartView.minOffset = 20.0
                    cell.chartView.xAxis.valueFormatter = IndexAxisValueFormatter(values: self.timeInterval)
                    let lineChartData = LineChartData(dataSets: goballivedataSetsListss)
                    cell.chartView.data = lineChartData
                    cell.chartView.data?.setDrawValues(true)
                    let lastXAxisValue: Double = Double(self.timeInterval.count)
                    cell.chartView.moveViewToX(lastXAxisValue)
                    cell.chartView.notifyDataSetChanged()
                } else {
                    cell.chartView.clear()
                    cell.chartView.notifyDataSetChanged()
                }
                return cell
            } else if indexPath.row == 1 {
                let cell = tableView.dequeueReusableCell(withIdentifier: "HistorySelectionCell", for: indexPath) as! HistorySelectionCell
                cell.selectionStyle = .none
                cell.arr_selections = self.arr_lines
                if (self.DictSensorLists as? NSDictionary)?.allKeys.count ?? 0 > 0 {
                    Constant.appDelegate.arr_History_Sensor.removeAll()
                    for (key, value) in self.DictSensorLists {
//                        print("\(key) -> \(value)")
                        var dict : [String:Any] = [:]
                        dict["localName"] = key
                        dict["dataChart"] = value["dataChart"]
                        dict["isvisible"] = value["isvisible"]
                        dict["colorcode"] = value["colorcode"]
                        Constant.appDelegate.arr_History_Sensor.append(dict)
                    }
                }
                cell.delegate = self
                cell.tblSelection.reloadData()
                return cell
            }
        }
        return UITableViewCell()
    }
}


extension Date {
    static var yesterday: Date { return Date().dayBefore }
    static var tomorrow:  Date { return Date().dayAfter }
    var dayBefore: Date {
        return Calendar.current.date(byAdding: .day, value: -1, to: noon)!
    }
    var dayAfter: Date {
        return Calendar.current.date(byAdding: .day, value: 1, to: noon)!
    }
    var noon: Date {
        return Calendar.current.date(bySettingHour: 12, minute: 0, second: 0, of: self)!
    }
    var month: Int {
        return Calendar.current.component(.month,  from: self)
    }
    var isLastDayOfMonth: Bool {
        return dayAfter.month != month
    }
}

extension Date {
    func string(format: String) -> String {
        let formatter = DateFormatter()
        formatter.dateFormat = format
        return formatter.string(from: self)
    }
}
