//
//  DeviceVC.swift
//  EBV-IoT
//
//  Created by krutik vora on 02/02/22.
//

import UIKit

class DeviceVC: UIViewController {
    @IBOutlet weak var deviceListTable: UITableView!
    @IBOutlet weak var viewTop: UIView!

    var arrDeviceName : [DeviceInfo] = [DeviceInfo]()
    private let refreshControl = UIRefreshControl()
    var pageNo : Int = 1
    var totalRecords : Int = 0
    var isPaging : Bool = false
    var isLoading : Bool = false
    var fetchingMore = false
    @IBOutlet weak var tblBottomView: UIView!
    @IBOutlet weak var lblNoData: UILabel!
    let refreshControlTop = UIRefreshControl()
    
    override func viewDidLoad()
    {
        super.viewDidLoad()
        
        let objSideMenuController = self.sideMenuController()?.sideMenu?.menuViewController as! SideMenuViewController
        objSideMenuController.lbl_username.text = String(format: "%@ %@", (UserDefaultManager.shared[.firstname] as? String) ?? "",(UserDefaultManager.shared[.lastname] as? String) ?? "")

        hideSideMenuView()
        deviceListTable.tableFooterView = tblBottomView
        deviceListTable.tableFooterView?.isHidden = true
        self.lblNoData.isHidden = true

        deviceListTable.register(UINib(nibName: "DeviceListCell", bundle: nil), forCellReuseIdentifier: "DeviceListCell")
        let loadingNib = UINib(nibName: "LoadingCell", bundle: nil)
        deviceListTable.register(loadingNib, forCellReuseIdentifier: "loadingCell")

        viewTop.layer.cornerRadius = 15
        viewTop.layer.maskedCorners = [.layerMinXMaxYCorner, .layerMaxXMaxYCorner]
        viewTop.clipsToBounds = true
        NotificationCenter.default.addObserver(self, selector: #selector(callDeviceListWS), name: Notification.Name(rawValue: "callDeviceListWS"), object: nil)
        refreshControlTop.attributedTitle = NSAttributedString(string: "Refreshing list")
        refreshControlTop.addTarget(self, action: #selector(self.refreshTop(_:)), for: .valueChanged)
        deviceListTable.insertSubview(refreshControlTop, at: 0)
        
//        refreshControl.attributedTitle = NSAttributedString(string: "Pull to refresh")
//        refreshControl.addTarget(self, action: #selector(self.refresh(_:)), for: .valueChanged)
//        tblBottomView.addSubview(refreshControl) // not required when using UITableViewController
        
        self.callDeviceListWS()
   }
    
    @objc func refreshTop(_ sender: AnyObject)
    {
        if self.arrDeviceName.count > 0 {
            print("**** Top Refresh control call ****")
            self.callDeviceListWS()
        } else {
            self.refreshControlTop.endRefreshing()
        }
    }
    
//    func passData(dict: [String : Any])
//    {
//        if dict["uniqueId"] != nil {
//            let arronoff : [DeviceInfo] = self.arrDeviceName.filter{($0.uniqueId == (dict["uniqueId"] as! String))}
//            if arronoff.count > 0 {
//                let finaldict : [String : Any] = dict["data"] as! [String : Any]
//                if let status = finaldict["status"] {
//                    if status as? String == "off" {
//                        print("**** Status is OFF *******")
//                    }else{
//                        print("**** Status is ON *******")
//                    }
//               }
//            }
//        }
//    }

    @objc func refresh(_ sender: AnyObject) {
       // Code to refresh table view
        self.callDeviceListWS()
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated) // No need for semicolon
        
        Constant.AppUtility.lockOrientation(.portrait)
    }
    
    override var supportedInterfaceOrientations: UIInterfaceOrientationMask {
        return .portrait
    }

    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated) // No need for semicolon
        self.sideMenuController()?.sideMenu?.allowPanGesture = true
        self.sideMenuController()?.sideMenu?.allowRightSwipe = true
    }
    
    @objc func callDeviceListWS(){
        if Constant.appDelegate.isConnectedToNetwork(){
            print("Internet Connection Available!")
            pageNo = 1
            totalRecords = 0
            ProHudManager.shared.show(view: self.view, animated: true)
            self.callDeviceLists()
        }else{
            print("Internet Connection not Available!")
            ProHudManager.shared.hide(animated: true)
            AlertConstant.showAlertMessage(self, Message: "No network available. Please check your Wifi or Data connections")
        }
    }
    
    @objc private func refreshDeviceLists(_ sender: Any)
    {
        self.resetPagingvariable()
        if(totalRecords > (pageNo*Constant.k_PAGE_SIZE))
        {
            pageNo += 1
            self.isPaging = true
            //Call webservice to get Inquiry List
            self.callDeviceLists()
        }
        else
        {
            self.refreshControl.endRefreshing()
        }
    }

    func resetPagingvariable()
    {
//        pageNo = 1
//        totalRecords = 0
//        isPaging = false
        self.deviceListTable.setContentOffset(CGPoint(x:0,y:0), animated: false)
    }

    // MARK: - TemplateLists Web-Service
    func callTemplateLists()
    {
        let dict : [String : Any] = [:]
        DeviceManager.shared.templateListsWebService(paramers: dict ,successBlock: { (response) in
            print(response)
            let decoder = JSONDecoder()
            let templatelists = try? decoder.decode(TemplatesList.self, from: response)
            if templatelists != nil
            {
            }
        }, failureBlock: { error in
            print("\n\n===========Error===========")
            print("Error Code: \(error._code)")
            print("Error Messsage: \(error.localizedDescription)")
            ProHudManager.shared.hide(animated: true)
        })
    }

    
    // MARK: - DeviceLists Web-Service
    func callDeviceLists() {
            DispatchQueue.main.async { [self] in
                let param = [Constant.RequestConstant.pageindex:pageNo,Constant.RequestConstant.pagesize:Constant.k_PAGE_SIZE] as           [String : Any]
                DeviceManager.shared.deviceListsWebService(paramers: param ,successBlock: { [self] (response) in
                    ProHudManager.shared.hide(animated: true)
                    print(response)
                    let decoder = JSONDecoder()
                    let devicelists = try? decoder.decode(Devices.self, from: response)
                    if devicelists != nil
                    {
                        DispatchQueue.main.async {
//                            self.refreshControl.endRefreshing()
                            self.refreshControlTop.endRefreshing()
                            if devicelists!.count > 0 {
                                if  self.isLoading == true {
                                    DispatchQueue.main.async {
                                        self.arrDeviceName += devicelists?.data?.filter{($0.deviceStatus == "Acquired")} ?? []
//                                        self.arrDeviceName += (devicelists?.data) ?? []
                                        self.deviceListTable.reloadData()
                                        self.isLoading = false
                                    }
                                }
                                else {
                                    self.arrDeviceName.removeAll()
//                                    self.arrDeviceName = (devicelists?.data) ?? []
                                    self.arrDeviceName = devicelists?.data?.filter{($0.deviceStatus == "Acquired")} ?? []
                                    self.isLoading = false
                                }
                                self.lblNoData.isHidden = true
                            }
                            else
                            {
                                print("\n\n===========Add Device===========")
                                
                                self.lblNoData.isHidden = false
                            }
                            self.totalRecords = devicelists!.count
                            if(self.totalRecords > (self.pageNo*Constant.k_PAGE_SIZE))
                            {
                                deviceListTable.tableFooterView?.isHidden = false
                                self.deviceListTable.tableFooterView = self.tblBottomView
                            } else {
                                self.deviceListTable.tableFooterView = nil
                                deviceListTable.tableFooterView?.isHidden = true
                            }
                            self.deviceListTable.reloadData()
                        }
                    }
                    else
                    {
                        DispatchQueue.main.async {
                            self.lblNoData.isHidden = false }
                    }
                }, failureBlock: { error in
                    print("\n\n===========Error===========")
                    print("Error Code: \(error._code)")
                    print("Error Messsage: \(error.localizedDescription)")
                    ProHudManager.shared.hide(animated: true)
                })
            }
    }
    
    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        let offsetY = scrollView.contentOffset.y
        let contentHeight = scrollView.contentSize.height

        if (offsetY > contentHeight - scrollView.frame.height) && !isLoading {
            self.loadMoreData()
        }
    }

    func loadMoreData() {
        if !self.isLoading {
            DispatchQueue.main.async {
                self.isLoading = false
                if(self.totalRecords > (self.pageNo*Constant.k_PAGE_SIZE))
                {
                    self.deviceListTable.tableFooterView = self.tblBottomView
                    self.deviceListTable.tableFooterView?.isHidden = false
                    self.isLoading = true
                    self.pageNo += 1
//                        ProHudManager.shared.show(view: self.view, animated: true)
                        self.callDeviceLists()
                }
                else
                {
                    self.isLoading = false
                    self.deviceListTable.tableFooterView = nil
                    self.deviceListTable.tableFooterView?.isHidden = true
//                    self.callDeviceLists()
                }
            }
        }
    }
    
    /*func callDeviceLists()
    {
        DispatchQueue.main.async { [self] in
            let param = [Constant.RequestConstant.pageindex:pageNo,            Constant.RequestConstant.pagesize:Constant.k_PAGE_SIZE] as           [String : Any]
            DeviceManager.shared.deviceListsWebService(paramers: param ,successBlock: { [self] (response) in
                ProHudManager.shared.hide(animated: true)
                print(response)
                let decoder = JSONDecoder()
                let devicelists = try? decoder.decode(Devices.self, from: response)
                if devicelists != nil
                {
                    DispatchQueue.main.async {
                        self.refreshControl.endRefreshing()
                        if devicelists!.count > 0 {
                            if  self.isPaging == true {
                                self.arrDeviceName += (devicelists?.data) ?? [] }
                            else {
                                self.arrDeviceName.removeAll()
                                self.arrDeviceName = (devicelists?.data) ?? []}
                        }
                        else
                        {
                            print("\n\n===========Add Device===========")
                        }
                        self.totalRecords = devicelists!.count
                        self.deviceListTable.reloadData()
                    }
                }
            }, failureBlock: { error in
                print("\n\n===========Error===========")
                print("Error Code: \(error._code)")
                print("Error Messsage: \(error.localizedDescription)")
                ProHudManager.shared.hide(animated: true)
            })
        }
    }*/
    
    //MARK:- Button Action Event
    @IBAction func btnSideMEnuClick(sender : UIButton){
        toggleSideMenuView()
    }

    @IBAction func btnAddDeviceClick(sender : UIButton){
        
        let objAddDevice = UIStoryboard(name: Constant.StoryboardIdentifier.device, bundle: nil).instantiateViewController(withIdentifier: "AddDeviceVC") as! AddDeviceVC
        objAddDevice.isFromDeviceList = true
        self.navigationController?.pushViewController(objAddDevice, animated: true)
    }
    
    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destination.
        // Pass the selected object to the new view controller.
    }
    */
}

extension DeviceVC: UITableViewDelegate {
    
    // didSelectRowAtIndexPath - change background color
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if Constant.appDelegate.isConnectedToNetwork(){
            let objDeviceDetail = UIStoryboard(name: Constant.StoryboardIdentifier.device, bundle: nil).instantiateViewController(withIdentifier: "DeviceDetailVC") as! DeviceDetailVC
            objDeviceDetail.strNavigationTitle = arrDeviceName[indexPath.row].displayName
            objDeviceDetail.strDeviceID = arrDeviceName[indexPath.row].uniqueId!
            objDeviceDetail.strDeviceGUID = arrDeviceName[indexPath.row].guid!
            objDeviceDetail.strDeviceTemplateGUID = arrDeviceName[indexPath.row].deviceTemplateGuid!
            objDeviceDetail.strDeviceName = arrDeviceName[indexPath.row].displayName!
            objDeviceDetail.connected = self.arrDeviceName[indexPath.row].isConnected!
            self.navigationController?.pushViewController(objDeviceDetail, animated: true)
        }else{
            print("Internet Connection not Available!")
            ProHudManager.shared.hide(animated: true)
            AlertConstant.showAlertMessage(self, Message: "No network available. Please check your Wifi or Data connections")
        }
    }
    
    func tableView(_ tableView: UITableView, didUnhighlightRowAt indexPath: IndexPath) {
        if let cell = tableView.cellForRow(at: indexPath) as? DeviceListCell {
            cell.bgView.backgroundColor = UIColor.white
            cell.imgArrow.tintColor = UIColor.black
            cell.imgDevice.tintColor = UIColor.black
            cell.lblDeviceName.textColor = UIColor.black
        }
    }

    func tableView(_ tableView: UITableView, didHighlightRowAt indexPath: IndexPath) {
        if let cell = tableView.cellForRow(at: indexPath) as? DeviceListCell {
            cell.bgView.backgroundColor = UIColor(red: 0.0/255.0, green: 57.0/255.0, blue: 139.0/255.0, alpha: 1.0)
            cell.imgArrow.tintColor = UIColor.white
            cell.imgDevice.tintColor = UIColor.white
            cell.lblDeviceName.textColor = UIColor.white

        }

    }

    // didDeselectRowAtIndexPath - change background color

//    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
//        tableView.deselectRow(at: indexPath, animated: true)
//    }
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 58
    }
    
    func tableView(tableView: UITableView, canEditRowAtIndexPath indexPath: NSIndexPath) -> Bool {
        return true
    }

}

extension DeviceVC: UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        
            return arrDeviceName.count

    }
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "DeviceListCell", for: indexPath) as! DeviceListCell
        cell.lblDeviceName.text = self.arrDeviceName[indexPath.row].displayName
        if  self.arrDeviceName[indexPath.row].isConnected == true {
            cell.lblStatus.backgroundColor = UIColor(red: 0.0/255.0, green: 206.0/255.0, blue: 39.0/255.0, alpha: 1.0)
        } else {
            cell.lblStatus.backgroundColor = UIColor(red: 251.0/255.0, green: 84.0/255.0, blue: 84.0/255.0, alpha: 1.0)
        }
        cell.selectionStyle = .none
        return cell
    }
    
    func tableView(_ tableView: UITableView, trailingSwipeActionsConfigurationForRowAt indexPath: IndexPath) -> UISwipeActionsConfiguration? {
//        if UserDefaultManager.shared[.parentEntityGuid] != nil {
            var actions = [UIContextualAction]()

            let delete = UIContextualAction(style: .normal, title: nil) { (contextualAction, view, completion) in
                let alertController = UIAlertController(title:APP_NAME, message: String(format: "Do you want to release device %@?", self.arrDeviceName[indexPath.row].displayName!), preferredStyle: UIAlertController.Style.alert)
                alertController.view.tintColor = UIColor.black
                alertController.addAction(UIAlertAction(title: "No", style: UIAlertAction.Style.default, handler: { alert in
                }))
                alertController.addAction(UIAlertAction(title: "Yes", style: UIAlertAction.Style.default, handler: {  alert in
                    ProHudManager.shared.show(view: self.view, animated: true)
                    self.getDeviceGrant(deviceguid: self.arrDeviceName[indexPath.row].guid ?? "",roww: indexPath.row, indexpath: indexPath)
                }))
                self.present(alertController, animated: true, completion: nil)
                // Delete something
                completion(true)
            }

            let largeConfig = UIImage.SymbolConfiguration(pointSize: 17.0, weight: .bold, scale: .large)
            delete.image = UIImage(systemName: "trash", withConfiguration: largeConfig)?.withTintColor(.white, renderingMode: .alwaysTemplate).addBackgroundCircle(.systemRed)
            delete.title = "Delete"
            delete.backgroundColor = UIColor.init(red: 0/255.0, green: 0/255.0, blue: 0/255.0, alpha: 0.0)
            actions.append(delete)

            let config = UISwipeActionsConfiguration(actions: actions)
            config.performsFirstActionWithFullSwipe = false

            return config
//        } else {
//
//        }
//        return .none
    }
    
    func getDeviceGrant(deviceguid : String,roww : Int, indexpath : IndexPath) {
        let param = [Constant.RequestConstant.uniqueId:deviceguid] as  [String : Any]
        DeviceManager.shared.getdevicegrantDetailsWebService(paramers: param ,successBlock: { [self] (response) in
            print(response)
            let decoder = JSONDecoder()
            let devicelists = try? decoder.decode(DeviceGrant.self, from: response)
            if devicelists != nil
            {
                DispatchQueue.main.async {
                    if devicelists?.data?.count ?? 0 > 0 {
                        let arruser = devicelists?.data?.filter{($0.userGuid == UserDefaultManager.shared[.userGuid] as? String ?? "")}
                        if arruser?.count ?? 0 > 0 {
                            self.invokedevicefromuser(userDevicePermissionGuid : devicelists?.data?[0].userDevicePermissionGuid ?? "",roww: roww,indexpath: indexpath)
                        } else {
                            ProHudManager.shared.hide(animated: true)
                            AlertConstant.showAlertMessage(self, Message: "Failed to delete the device from user. This device was already registered to another user.")
                        }
                    } else {
                        self.deletedevicefromroot(userDevicePermissionGuid: "", deviceguid: deviceguid, roww: roww, indexpath: indexpath)
                    }
                }
            } else {
                ProHudManager.shared.hide(animated: true)
            }
        }, failureBlock: { error in
            print("\n\n===========Error===========")
            print("Error Code: \(error._code)")
            print("Error Messsage: \(error.localizedDescription)")
            ProHudManager.shared.hide(animated: true)
            if (error as! EBVHttpErrors).errorDescription == "Unknown error" {
                AlertConstant.showAlertMessage(self, Message: "Device delete,acquire and release not allowed")
            } else {
                AlertConstant.showAlertMessage(self, Message: (error as! EBVHttpErrors).errorDescription)
            }
        })
    }
    
    func invokedevicefromuser(userDevicePermissionGuid : String,roww : Int, indexpath : IndexPath) {
        let param = ["entityGuid":userDevicePermissionGuid] as  [String : Any]
        DeviceManager.shared.deviceInvokeWebService(paramers: param ,successBlock: { [self] (response) in
            print(response)
            self.deletedevicefromuser(roww: roww, indexpath: indexpath)
        }, failureBlock: { error in
            print("\n\n===========Error===========")
            print("Error Code: \(error._code)")
            print("Error Messsage: \(error.localizedDescription)")
            ProHudManager.shared.hide(animated: true)
            if (error as! EBVHttpErrors).errorDescription == "Unknown error" {
                AlertConstant.showAlertMessage(self, Message: "Device delete,acquire and release not allowed")
            } else {
                AlertConstant.showAlertMessage(self, Message: (error as! EBVHttpErrors).errorDescription)
            }
        })
    }
    
    func deletedevicefromroot(userDevicePermissionGuid : String,deviceguid : String,roww : Int, indexpath : IndexPath) {
        let param = [Constant.RequestConstant.uniqueId:deviceguid ,"entityGuid":UserDefaultManager.shared[.rootentityGuid] as! String,"Deleted":"1"] as  [String : Any]
        DeviceManager.shared.deviceEntityWebService(paramers: param ,successBlock: { [self] (response) in
            print(response)
            let decoder = JSONDecoder()
            let devicelists = try? decoder.decode(Devices.self, from: response)
            if devicelists == nil
            {
                DispatchQueue.main.async {
                    ProHudManager.shared.hide(animated: true)
                    deviceListTable.beginUpdates()
                    arrDeviceName.remove(at: roww)
                    deviceListTable.deleteRows(at: [indexpath], with: .none)
                    deviceListTable.endUpdates()
                }
            }
        }, failureBlock: { error in
            print("\n\n===========Error===========")
            print("Error Code: \(error._code)")
            print("Error Messsage: \(error.localizedDescription)")
            ProHudManager.shared.hide(animated: true)
            if (error as! EBVHttpErrors).errorDescription == "Unknown error" {
                AlertConstant.showAlertMessage(self, Message: "Device delete,acquire and release not allowed")
            } else {
                AlertConstant.showAlertMessage(self, Message: (error as! EBVHttpErrors).errorDescription)
            }
        })
    }
    
    func deletedevicefromuser(roww : Int, indexpath : IndexPath)
    {
        let param = [Constant.RequestConstant.uniqueId:self.arrDeviceName[roww].guid ?? "","entityGuid":UserDefaultManager.shared[.rootentityGuid] as! String,"Deleted":"1"] as  [String : Any]
        DeviceManager.shared.deviceEntityWebService(paramers: param ,successBlock: { [self] (response) in
            print(response)
            let decoder = JSONDecoder()
            let devicelists = try? decoder.decode(Devices.self, from: response)
            if devicelists == nil
            {
                DispatchQueue.main.async {
                    ProHudManager.shared.hide(animated: true)
                    deviceListTable.beginUpdates()
                    arrDeviceName.remove(at: roww)
                    deviceListTable.deleteRows(at: [indexpath], with: .none)
                    deviceListTable.endUpdates()
                }
            }
        }, failureBlock: { error in
            print("\n\n===========Error===========")
            print("Error Code: \(error._code)")
            print("Error Messsage: \(error.localizedDescription)")
            ProHudManager.shared.hide(animated: true)
            if (error as! EBVHttpErrors).errorDescription == "Unknown error" {
                AlertConstant.showAlertMessage(self, Message: "Device delete,acquire and release not allowed")
            } else {
                AlertConstant.showAlertMessage(self, Message: (error as! EBVHttpErrors).errorDescription)
            }
        })
    }


//    func tableView(_ tableView: UITableView, commit editingStyle: UITableViewCell.EditingStyle, forRowAt indexPath: IndexPath) {
//        if (editingStyle == .delete) {
//            let alertController = UIAlertController(title:APP_NAME, message: String(format: "Are you sure you want to delete the %@?", self.arrDeviceName[indexPath.row].displayName!), preferredStyle: UIAlertController.Style.alert)
//            alertController.view.tintColor = UIColor.black
//            alertController.addAction(UIAlertAction(title: "Cancel", style: UIAlertAction.Style.cancel, handler: { alert in
//            }))
//            alertController.addAction(UIAlertAction(title: "Delete", style: UIAlertAction.Style.destructive, handler: {  alert in
//                ProHudManager.shared.show(view: self.view, animated: true)
//                let param = [Constant.RequestConstant.uniqueId:self.arrDeviceName[indexPath.row].guid ?? "","entityGuid":UserDefaultManager.shared[.rootentityGuid] as! String,"Deleted":"1"] as  [String : Any]
//                DeviceManager.shared.deviceEntityWebService(paramers: param ,successBlock: { [self] (response) in
//                    print(response)
//                    let decoder = JSONDecoder()
//                    let devicelists = try? decoder.decode(Devices.self, from: response)
//                    if devicelists == nil
//                    {
//                        DispatchQueue.main.async {
//                            ProHudManager.shared.hide(animated: true)
//                            deviceListTable.beginUpdates()
//                            arrDeviceName.remove(at: indexPath.row)
//                            deviceListTable.deleteRows(at: [indexPath], with: .none)
//                            deviceListTable.endUpdates()
//                        }
//                    }
//                }, failureBlock: { error in
//                    print("\n\n===========Error===========")
//                    print("Error Code: \(error._code)")
//                    print("Error Messsage: \(error.localizedDescription)")
//                    ProHudManager.shared.hide(animated: true)
//                    if (error as! EBVHttpErrors).errorDescription == "Unknown error" {
//                        AlertConstant.showAlertMessage(self, Message: "Device delete,acquire and release not allowed")
//                    } else {
//                        AlertConstant.showAlertMessage(self, Message: (error as! EBVHttpErrors).errorDescription)
//                    }
//                })
//            }))
//            self.present(alertController, animated: true, completion: nil)
//        }
//    }
}

extension UIImage {

    func addBackgroundCircle(_ color: UIColor?) -> UIImage? {

        let circleDiameter = max(size.width * 1.5, size.height * 1.5)
        let circleRadius = circleDiameter * 0.5
        let circleSize = CGSize(width: circleDiameter, height: circleDiameter)
        let circleFrame = CGRect(x: 0, y: 0, width: circleSize.width, height: circleSize.height)
        let imageFrame = CGRect(x: circleRadius - (size.width * 0.5), y: circleRadius - (size.height * 0.5), width: size.width, height: size.height)

        let view = UIView(frame: circleFrame)
        view.backgroundColor = color ?? .systemRed
        view.layer.cornerRadius = circleDiameter * 0.5

        UIGraphicsBeginImageContextWithOptions(circleSize, false, UIScreen.main.scale)

        let renderer = UIGraphicsImageRenderer(size: circleSize)
        let circleImage = renderer.image { ctx in
            view.drawHierarchy(in: circleFrame, afterScreenUpdates: true)
        }

        circleImage.draw(in: circleFrame, blendMode: .normal, alpha: 1.0)
        draw(in: imageFrame, blendMode: .normal, alpha: 1.0)

        let image: UIImage? = UIGraphicsGetImageFromCurrentImageContext()

        UIGraphicsEndImageContext()

        return image
    }
}
