//
//  SideMenuViewController.swift

import UIKit

enum MenuItem: Int
{
    case Devices = 0
    case Statistics = 1
    case UserSettings = 2
    case SwitchUser = 3
    case Logout = 4
    
    func description() -> String
    {
        switch self
        {
        case .Devices:
            return "Devices"
            
        case .Statistics:
            return "Statistics"
            
        case .UserSettings:
            return "User Settings"
            
        case .SwitchUser:
            return "Switch User"

        case .Logout:
            return "Logout"
        }
    }
}

class SideMenuViewController: UIViewController,UITableViewDataSource,UITableViewDelegate
{
    @IBOutlet weak var vw_Box: UIView!
    @IBOutlet weak var lbl_username: UILabel!

    @IBOutlet weak var tblSideMenu: UITableView!
    @IBOutlet weak var constraintViewTrailing : NSLayoutConstraint!
    @IBOutlet weak var btnUserName: UIButton!
    var arrMenuItems = [["Name": MenuItem.Devices.description()],
                        ["Name": MenuItem.Statistics.description()],
                        ["Name": MenuItem.UserSettings.description()],
                        ["Name": MenuItem.SwitchUser.description()],
                        ["Name": MenuItem.Logout.description()]]
    
    var selectedMenuIndex : Int = 0
    
    override func viewDidLoad()
    {
        super.viewDidLoad()
        
        self.vw_Box.layer.cornerRadius = 5.0
        self.vw_Box.layer.masksToBounds = true
       
        //Register TableView Cell
        let nib = UINib(nibName:"SideMenuCell", bundle: nil)
        tblSideMenu.register(nib, forCellReuseIdentifier:"SideMenuCell")
    }
    
    // handle notification
    @objc func setMenu(_ notification: NSNotification)
    {
        view.endEditing(true)

        NotificationCenter.default.removeObserver(self)
        self.tblSideMenu.reloadData()
    }
        
    //MARK : Tableview Delegate Methods
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return self.arrMenuItems.count
    }
    
    func numberOfSections(in tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat
    {
        return 60.0
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        
        let cell = tableView.dequeueReusableCell(withIdentifier: "SideMenuCell") as! SideMenuCell
        cell.selectionStyle = .none
        
        switch (indexPath.row)
        {
            case 0:
                if selectedMenuIndex == indexPath.row {
                    cell.imgIcon.image = UIImage(named: "device_side_hover")}
                else{
                    cell.imgIcon.image = UIImage(named: "device_side_normal")}
            case 1:
                if selectedMenuIndex == indexPath.row {
                    cell.imgIcon.image = UIImage(named: "statistics_hover")}
                else{
                    cell.imgIcon.image = UIImage(named: "statistics_normal")}
            case 2:
                if selectedMenuIndex == indexPath.row {
                    cell.imgIcon.image = UIImage(named: "user_settings_hover")}
                else{
                    cell.imgIcon.image = UIImage(named: "user_settings_normal")}
            case 3:
                if selectedMenuIndex == indexPath.row {
                    cell.imgIcon.image = UIImage(named: "change_user_hover")}
                else{
                    cell.imgIcon.image = UIImage(named: "change_user_normal")}
            case 4:
                if selectedMenuIndex == indexPath.row {
                    cell.imgIcon.image = UIImage(named: "logout_hover")}
                else{
                    cell.imgIcon.image = UIImage(named: "logout_normal")}
            case 5:
                if selectedMenuIndex == indexPath.row {
                    cell.imgIcon.image = UIImage(named: "logout_hover")}
                else{
                    cell.imgIcon.image = UIImage(named: "logout_normal")}
            default:
                break
        }
       
        //Set corner radius to Button
         cell.btnMenu.layer.cornerRadius =  cell.btnMenu.frame.height / 2.0
         cell.btnMenu.clipsToBounds = true
        
        let dictFromArray = self.arrMenuItems[indexPath.row]
        cell.lblTitle.text = dictFromArray["Name"]
        
        let totalrows = tableView.numberOfRows(inSection: indexPath.section)
        if indexPath.row == totalrows - 1
        {
            cell.lblLine.isHidden = true
        }
        else
        {
            cell.lblLine.isHidden = false
        }
        
        //Click on Button Menu
        cell.btnMenu.tag = indexPath.row
        cell.btnMenu.addTarget(self, action: #selector(SideMenuViewController.buttonClickedMenu(_:)), for: UIControl.Event.touchUpInside)
        
        
        if selectedMenuIndex == indexPath.row
        {
            cell.lblTitle.textColor = hexStringToUIColor(hex: "#236eda")
        }
        else
        {
            cell.lblTitle.textColor = .black
        }
        return cell
    }
    
    func tableView(_ tableView: UITableView, willDisplay cell: UITableViewCell, forRowAt indexPath: IndexPath) {
        cell.separatorInset = UIEdgeInsets.zero
        cell.layoutMargins = UIEdgeInsets.zero
    }
    
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
    }
    
    //********************************
    //MARK:- Button Click on Close Menu
    //********************************
    
    @IBAction func buttonClickCloseMenu(_ sender : UIButton){
        hideSideMenuView()
    }
    
    func hexStringToUIColor (hex:String) -> UIColor {
        var cString:String = hex.trimmingCharacters(in: .whitespacesAndNewlines).uppercased()

        if (cString.hasPrefix("#")) {
            cString.remove(at: cString.startIndex)
        }

        if ((cString.count) != 6) {
            return UIColor.gray
        }

        var rgbValue:UInt64 = 0
        Scanner(string: cString).scanHexInt64(&rgbValue)

        return UIColor(
            red: CGFloat((rgbValue & 0xFF0000) >> 16) / 255.0,
            green: CGFloat((rgbValue & 0x00FF00) >> 8) / 255.0,
            blue: CGFloat(rgbValue & 0x0000FF) / 255.0,
            alpha: CGFloat(1.0)
        )
    }
    
    //********************************
    //MARK:- Button Click on Stock Info
    //********************************
    
    @objc func buttonClickedMenu(_ sender : UIButton){
        
        if (sender.tag == selectedMenuIndex && sender.tag != 3 && sender.tag != 4) {
            return
        }

        let dictFromArray = self.arrMenuItems[sender.tag]
        selectedMenuIndex =  sender.tag
        tblSideMenu.beginUpdates()
        tblSideMenu.reloadRows(at: tblSideMenu.indexPathsForVisibleRows!, with: .none)
        tblSideMenu.endUpdates()
        
        if (dictFromArray["Name"]) == MenuItem.Devices.description(){
            let mainStoryboard: UIStoryboard = UIStoryboard(name: Constant.StoryboardIdentifier.device,bundle: nil)
            let destViewController = mainStoryboard.instantiateViewController(withIdentifier: "DeviceVC")
            sideMenuController()?.setContentViewController(destViewController)
        }
        else if (dictFromArray["Name"]) == MenuItem.Statistics.description(){
            let mainStoryboard: UIStoryboard = UIStoryboard(name: Constant.StoryboardIdentifier.sidemenu,bundle: nil)
            let destViewController = mainStoryboard.instantiateViewController(withIdentifier: "StatisticsVC")
            sideMenuController()?.setContentViewController(destViewController)

        }
        else if (dictFromArray["Name"]) == MenuItem.UserSettings.description(){
            let mainStoryboard: UIStoryboard = UIStoryboard(name: Constant.StoryboardIdentifier.sidemenu,bundle: nil)
            let destViewController = mainStoryboard.instantiateViewController(withIdentifier: "UserSettingsVC")
            sideMenuController()?.setContentViewController(destViewController)

        }
        else if (dictFromArray["Name"]) == MenuItem.SwitchUser.description(){
            let alertController = UIAlertController(title:APP_NAME, message: MessageConstant.validationMessage.changeuser, preferredStyle: UIAlertController.Style.alert)
            alertController.view.tintColor = UIColor.black
            alertController.addAction(UIAlertAction(title: "Cancel", style: UIAlertAction.Style.cancel, handler: { alert in

            }))
            alertController.addAction(UIAlertAction(title: "Yes", style: UIAlertAction.Style.default, handler: { [self] alert in
                
                self.selectedMenuIndex = 0
                tblSideMenu.beginUpdates()
                tblSideMenu.reloadRows(at: tblSideMenu.indexPathsForVisibleRows!, with: .none)
                tblSideMenu.endUpdates()
                UserDefaultManager.shared[.accesstoken] =  nil
                UserDefaultManager.shared[.refreshtoken] =  nil
                UserDefaultManager.shared[.email] =  nil
                UserDefaultManager.shared[.password] =  nil
                UserDefaultManager.shared[.entityGuid] =  nil
                UserDefaultManager.shared[.templateGuid] =  nil
                UserDefaultManager.shared[.firstname]  = nil
                UserDefaultManager.shared[.lastname]  = nil
                UserDefaultManager.shared[.cpId]  = nil
                UserDefaultManager.shared[.isLogin] = nil
                UserDefaultManager.shared[.deviceAdded] = nil
                UserDefaultManager.shared[.accesstokenSubscription] =  nil
                UserDefaultManager.shared[.userGuid] = nil
                UserDefaultManager.shared[.parentEntityGuid] = nil
                
                let mainStoryboard: UIStoryboard = UIStoryboard(name: Constant.StoryboardIdentifier.login,bundle: nil)
                let destViewController = mainStoryboard.instantiateViewController(withIdentifier: "LoginVC")
                self.sideMenuController()?.setContentViewController(destViewController)
            }))
            self.present(alertController, animated: true, completion: nil)

        }
        else if (dictFromArray["Name"]) == MenuItem.Logout.description(){
        
            let alertController = UIAlertController(title:APP_NAME, message: MessageConstant.validationMessage.logout, preferredStyle: UIAlertController.Style.alert)
            alertController.view.tintColor = UIColor.black
            alertController.addAction(UIAlertAction(title: "Cancel", style: UIAlertAction.Style.cancel, handler: { alert in


            }))
            alertController.addAction(UIAlertAction(title: "Logout", style: UIAlertAction.Style.default, handler: { [self] alert in
                
                self.selectedMenuIndex = 0
                tblSideMenu.beginUpdates()
                tblSideMenu.reloadRows(at: tblSideMenu.indexPathsForVisibleRows!, with: .none)
                tblSideMenu.endUpdates()

                UserDefaultManager.shared[.email] =  nil
                UserDefaultManager.shared[.password] =  nil
                UserDefaultManager.shared[.basictoken] =  nil
                UserDefaultManager.shared[.accesstoken] =  nil
                UserDefaultManager.shared[.refreshtoken] =  nil
                UserDefaultManager.shared[.rootaccesstoken] =  nil
                UserDefaultManager.shared[.rootrefreshtoken] =  nil
                UserDefaultManager.shared[.accesstokenSubscription] =  nil
                UserDefaultManager.shared[.entityGuid] =  nil
                UserDefaultManager.shared[.templateGuid] =  nil
                UserDefaultManager.shared[.firstname]  = nil
                UserDefaultManager.shared[.lastname]  = nil
                UserDefaultManager.shared[.cpId]  = nil
                UserDefaultManager.shared[.isLogin] = nil
                UserDefaultManager.shared[.deviceAdded] = nil
                UserDefaultManager.shared[.rootemail] = nil
                UserDefaultManager.shared[.isRootLogin] = nil
                UserDefaultManager.shared[.rootemail] = nil
                UserDefaultManager.shared[.rootpass] = nil
                UserDefaultManager.shared[.userGuid] = nil
                UserDefaultManager.shared[.parentEntityGuid] = nil
                UserDefaultManager.shared[.rootentityGuid] = nil
                
                let mainStoryboard: UIStoryboard = UIStoryboard(name: Constant.StoryboardIdentifier.login,bundle: nil)
                let destViewController = mainStoryboard.instantiateViewController(withIdentifier: "LoginVC")
                self.sideMenuController()?.setContentViewController(destViewController)
            }))
            self.present(alertController, animated: true, completion: nil)
        }

        /*if #available(iOS 11.0, *)
        {
            if Constant.appDelegate.myNavigation?.viewControllers.last is CustomerListVC
            {
                return
            }
            else
            {
                //Navigate to ItemInquiry View Controller
                Constant.appDelegate.myNavigation?.pushViewController(self.storyboard!.instantiateViewController(withIdentifier: "CustomerListVC"), animated: false)
            }
        }
        else
        {
            // Fallback on earlier versions
        }
        else if (dictFromArray["Name"] as! String) == "Logout"
        {
                //Logout
            SharedAlert.alert.displayAlertMessageWithTwoButtonBlock(viewController: self, strMessage: MessageConstant.validationMessage.logout, buttonTitle1: MessageConstant.commonButtonTitle.yes, buttonTitle2: MessageConstant.commonButtonTitle.no, handler: { (result) in
                
                if result.title == MessageConstant.commonButtonTitle.yes {
                    
               SBPreferenceManager.shared[.userName] =  nil
                SBPreferenceManager.shared[.password] =  nil
                    SBPreferenceManager.shared[.isRemember]  = false
                   //Set default selected index
                    
                    self.selectedMenuIndex = 0
                    self.tblSideMenu.reloadData()
                    
                    self.setEmptyBackButton(self.storyboard!.instantiateViewController(withIdentifier: "LoginViewController"))
                Constant.appDelegate.myNavigation?.pushViewController(self.storyboard!.instantiateViewController(withIdentifier: "LoginViewController"), animated: true)
                }
                else
                {
                    print("No")
                }
            })
        }*/
    }
    /*
     // MARK: - Navigation
     
     // In a storyboard-based application, you will often want to do a little preparation before navigation
     override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
     // Get the new view controller using segue.destination.
     // Pass the selected object to the new view controller.
     }
     */
    
    func setEmptyBackButton(_ viewController: UIViewController) {
        viewController.navigationItem.backBarButtonItem =
            UIBarButtonItem(title: "", style: .plain, target: nil, action: nil)
    }
}
