//
//  StatisticsVC.swift
//  EBV-IoT
//
//  Created by krutik vora on 08/02/22.
//

import UIKit

class StatisticsVC: UIViewController {
    @IBOutlet weak var viewTop: UIView!
    
    @IBOutlet weak var lblUsers: UILabel!
    @IBOutlet weak var lblDevices: UILabel!
    @IBOutlet weak var lblMessages: UILabel!
    @IBOutlet weak var lblComp: UILabel!
    @IBOutlet weak var lblDate: UILabel!
    var dataConsumption : [String:Any] = [:]
    
    override func viewDidLoad() {
        super.viewDidLoad()
        

        DispatchQueue.main.async {
            self.viewTop.layer.cornerRadius = 15
            self.viewTop.layer.maskedCorners = [.layerMinXMaxYCorner, .layerMaxXMaxYCorner]
            self.viewTop.clipsToBounds = true
            
            self.lblComp.text = ""
            self.lblDate.text = ""

            if Constant.appDelegate.isConnectedToNetwork(){
                print("Internet Connection Available!")
                ProHudManager.shared.show(view: self.view, animated: true)
                self.call_subscription()
            }else{
                print("Internet Connection not Available!")
                ProHudManager.shared.hide(animated: true)
                AlertConstant.showAlertMessage(self, Message: "No network available. Please check your Wifi or Data connections")
            }
        }
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated) // No need for semicolon
        
        Constant.AppUtility.lockOrientation(.portrait)
        
    }

    
    func call_subscription() {
        let strurl = String(format: "%@%@", WebConstant.BASE_URL_SUBSCRIPTION,WebConstant.SUBSCRIPTION_TOKEN_PATH)
        var request = URLRequest(url: URL(string: strurl)!,timeoutInterval: Double.infinity)
        let param = [Constant.RequestConstant.clientid:Constant.Client_ID,Constant.RequestConstant.clientsecret:Constant.Client_Secret,Constant.RequestConstant.username:UserDefaultManager.shared[.rootemail] as Any] as [String : Any]
        if let data = try? JSONSerialization.data(withJSONObject: param, options: .prettyPrinted) {
            request.httpBody = data
        }
        request.addValue("application/json", forHTTPHeaderField: "Content-Type")
        request.httpMethod = "POST"
        let task = URLSession.shared.dataTask(with: request) {data, response, error in
          guard let data = data else {
            print(String(describing: error))
            ProHudManager.shared.hide(animated: true)
            return
          }
          do {
                let json = try JSONSerialization.jsonObject(with: data, options: .mutableContainers) as? [String:AnyObject]
                let data : [String:Any] = json!
                self.call_EnvironmentId(str_accesstoken: data["accessToken"] as! String)
          } catch {
                print("Something went wrong")
                ProHudManager.shared.hide(animated: true)
                AlertConstant.showAlertMessage(self, Message: (error as! EBVHttpErrors).errorDescription)
          }
        }
        task.resume()
    }
    
    func call_EnvironmentId(str_accesstoken : String) {
//        let strApp = String(format: "%@%@/%@", WebConstant.BASE_URL_SUBSCRIPTION,WebConstant.SUBSCRIPTION_ENVIORNMENT_PATH,UserDefaultManager.shared[.companyGuid] as! CVarArg)
        
        let companyGuid = UserDefaultManager.shared[.companyGuid] as? String ?? ""
        
        let strApp = String(format: "%@%@/%@", WebConstant.BASE_URL_SUBSCRIPTION,WebConstant.SUBSCRIPTION_ENVIORNMENT_PATH,companyGuid)
        var request = URLRequest(url: URL(string: strApp)!,timeoutInterval: Double.infinity)
        request.addValue("text/plain", forHTTPHeaderField: "Accept")
        let str_bearer = String(format: "Bearer %@", str_accesstoken)
        request.addValue(str_bearer, forHTTPHeaderField: "Authorization")
        request.httpMethod = "GET"

        let task = URLSession.shared.dataTask(with: request) { data, response, error in
          guard let data = data else {
            print(String(describing: error))
            ProHudManager.shared.hide(animated: true)
            return
          }
            if (response as? HTTPURLResponse)?.statusCode == 200 {
                self.call_Consumption_Data(str_accesstoken: str_accesstoken, str_enironment: String(data: data, encoding: .utf8)!)
            } else {
                ProHudManager.shared.hide(animated: true)
            }
        }
        task.resume()
    }
    
    func call_Consumption_Data(str_accesstoken : String,str_enironment : String) {
        let strApp = String(format: "%@%@/%@/%@/consumption/active", WebConstant.BASE_URL_SUBSCRIPTION,WebConstant.SUBSCRIPTION_CONSUMPTION_PATH,Constant.Solution_Env_ID,str_enironment)
        var request = URLRequest(url: URL(string: strApp)!,timeoutInterval: Double.infinity)
        request.addValue("text/plain", forHTTPHeaderField: "Accept")
        let str_bearer = String(format: "Bearer %@", str_accesstoken)
        request.addValue(str_bearer, forHTTPHeaderField: "Authorization")
        request.httpMethod = "GET"

        let task = URLSession.shared.dataTask(with: request) { [self] data, response, error in
          guard let data = data else {
            print(String(describing: error))
            ProHudManager.shared.hide(animated: true)
            return
          }
          do {
              let json = try JSONSerialization.jsonObject(with: data, options: .mutableContainers) as? [String:AnyObject]
              self.dataConsumption = json!
              print("Data of Consumption %@",data)
              self.call_Plan_Data(str_accesstoken: str_accesstoken, str_enironment: str_enironment)
          } catch {
              print("Something went wrong")
              ProHudManager.shared.hide(animated: true)
          }
        }
        task.resume()
    }
    
    func call_Plan_Data(str_accesstoken : String,str_enironment : String) {
        let strApp = String(format: "%@%@/%@/plans?pageNo=1&pageSize=10&consumerId=%@", WebConstant.BASE_URL_SUBSCRIPTION,WebConstant.SUBSCRIPTION_PLAN_PATH,Constant.Solution_Env_ID,str_enironment)
        var request = URLRequest(url: URL(string: strApp)!,timeoutInterval: Double.infinity)
        request.addValue("text/plain", forHTTPHeaderField: "Accept")
        let str_bearer = String(format: "Bearer %@", str_accesstoken)
        request.addValue(str_bearer, forHTTPHeaderField: "Authorization")
        request.httpMethod = "GET"

        let task = URLSession.shared.dataTask(with: request) { data, response, error in
          guard let data = data else {
            print(String(describing: error))
            ProHudManager.shared.hide(animated: true)
            return
          }
          do {
                let json = try JSONSerialization.jsonObject(with: data, options: .mutableContainers) as? [String:AnyObject]
                let data : [String:Any] = json!
                let dict_data : [[String:Any]] = data["data"] as! [[String : Any]]
              
//              if let theJSONData = try? JSONSerialization.data(
//                  withJSONObject: dictionary,
//                  options: []) {
//                  let theJSONText = String(data: theJSONData,
//                                             encoding: .ascii)
//                  print("JSON string = \(theJSONText!)")
//              }

              
                if dict_data.count > 0 {
                    DispatchQueue.main.async {
                        var totaldevices : Double = 0.0
                        var totalusers : Double = 0.0
                        var totalAvailableMessages : Double = 0.0

                        for i in 0..<dict_data.count {
                            let arr_features = dict_data[i]["features"] as! [[String : Any]]
                            let arr_devices = arr_features.filter{($0["displayName"] as! String) == "Devices"}
                            let arr_users = arr_features.filter{($0["displayName"] as! String) == "Users"}

                            if arr_devices.count > 0 {
                                if totaldevices <= (arr_devices[0]["featureValue"] as! NSString).doubleValue {
                                    totaldevices = (arr_devices[0]["featureValue"] as! NSString).doubleValue
                                }
                            }
                            if arr_users.count > 0 {
                                if totalusers <= (arr_users[0]["featureValue"] as! NSString).doubleValue {
                                    totalusers = (arr_users[0]["featureValue"] as! NSString).doubleValue
                                }
                            }
                            
                            if totalAvailableMessages <= dict_data[i]["totalAvailableMessages"] as! Double {
                                totalAvailableMessages = dict_data[i]["totalAvailableMessages"] as! Double
                            }
                        }
                      
                        self.lblUsers.text = String(format:"%d/%@", self.dataConsumption["noOfUsers"] as! Int , String(format: "%g", totalusers))
                        self.lblDevices.text = String(format:"%d/%@", self.dataConsumption["noOfDevices"] as! Int , String(format: "%g", totaldevices))
                        
                        self.lblComp.text = self.dataConsumption["planName"] as? String

                        if (self.dataConsumption["renewalDate"] as? String) != nil{
                            let renewalDate: String = self.dataConsumption["renewalDate"] as! String
                            let strdesiredUpdatedDate = self.formatDate(strDate: (renewalDate) , currentFormat: "YYYY-MM-dd'T'HH:mm:ssZ", newDateFormat: "MM/dd/yyyy")
                               self.lblDate.text = strdesiredUpdatedDate
                        }
                        
                        let perCent = 100 * (self.dataConsumption["totalMessage"] as! Double) / totalAvailableMessages
                            
                        self.lblMessages.text = String(format:"%@/%@ (%@%@)", self.formatPoints(num: Double(self.dataConsumption["totalMessage"] as! Int)), self.formatPoints(num: totalAvailableMessages), String(format: "%.1g", floor(perCent)),"%")
                      }
                }
                ProHudManager.shared.hide(animated: true)
          } catch {
                print("Something went wrong")
                ProHudManager.shared.hide(animated: true)
          }
        }
        task.resume()
    }
    
    func formatPoints(num: Double) ->String{
        var thousandNum = num/1000
        var millionNum = num/1000000
        if num >= 1000 && num < 1000000{
            if(floor(thousandNum) == thousandNum){
                return("\(Int(thousandNum))k")
            }
            return("\(thousandNum.roundToPlaces(places: 0))k")
        }
        if num > 1000000{
            if(floor(millionNum) == millionNum){
                return("\(Int(thousandNum))k")
            }
            return ("\(millionNum.roundToPlaces(places: 1))m")
        }
        else{
            if(floor(num) == num){
                return ("\(Int(num))")
            }
            return ("\(num)")
        }

    }
    
    func formatDate(strDate:String,currentFormat:String,newDateFormat:String) -> String{
        
        let dtFormat = DateFormatter()
        dtFormat.dateFormat = currentFormat
        let dt = dtFormat.date(from: strDate)
        dtFormat.dateFormat = newDateFormat
        let strDate = dtFormat.string(from: dt!)
        return strDate
    }
    
    func convertStringToDictionary(text: String) -> [String:AnyObject]? {
        if let data = text.data(using: .utf8) {
            do {
                let json = try JSONSerialization.jsonObject(with: data, options: .mutableContainers) as? [String:AnyObject]
                return json
            } catch {
                print("Something went wrong")
            }
        }
        return nil
    }
    
    //MARK:- Button Action Event
    @IBAction func btnSideMEnuClick(sender : UIButton){
        toggleSideMenuView()
    }


    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destination.
        // Pass the selected object to the new view controller.
    }
    */
}


extension Double {
    /// Rounds the double to decimal places value
    mutating func roundToPlaces(places:Int) -> Double {
        let divisor = pow(10.0, Double(places))
        return Darwin.round(self * divisor) / divisor
    }
}


extension Float {
    var clean: String {
       return self.truncatingRemainder(dividingBy: 1) == 0 ? String(format: "%.0f", self) : String(self)
    }
}

//extension Reachability {
//
//    class func isConnectedToNetwork() -> Bool {
//
//        var zeroAddress = sockaddr_in(sin_len: 0, sin_family: 0, sin_port: 0, sin_addr: in_addr(s_addr: 0), sin_zero: (0, 0, 0, 0, 0, 0, 0, 0))
//        zeroAddress.sin_len = UInt8(MemoryLayout.size(ofValue: zeroAddress))
//        zeroAddress.sin_family = sa_family_t(AF_INET)
//
//        let defaultRouteReachability = withUnsafePointer(to: &zeroAddress) {
//            $0.withMemoryRebound(to: sockaddr.self, capacity: 1) {zeroSockAddress in
//                SCNetworkReachabilityCreateWithAddress(nil, zeroSockAddress)
//            }
//        }
//
//        var flags: SCNetworkReachabilityFlags = SCNetworkReachabilityFlags(rawValue: 0)
//        if SCNetworkReachabilityGetFlags(defaultRouteReachability!, &flags) == false {
//            return false
//        }
//
//        /* Only Working for WIFI
//        let isReachable = flags == .reachable
//        let needsConnection = flags == .connectionRequired
//
//        return isReachable && !needsConnection
//        */
//
//        // Working for Cellular and WIFI
//        let isReachable = (flags.rawValue & UInt32(kSCNetworkFlagsReachable)) != 0
//        let needsConnection = (flags.rawValue & UInt32(kSCNetworkFlagsConnectionRequired)) != 0
//        let ret = (isReachable && !needsConnection)
//
//        return ret
//    }
//}
