//
//  AuthenticationManager.swift
//  IoTConnect
//
//  Created by Devesh Mevada on 8/20/21.
//

import Foundation

struct AuthenticationManager {
    typealias CompletionHandler = (_ success:Bool) -> Void
    
    static let shared = AuthenticationManager()
    
    // MARK: - Basic Token

    func basicToken(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let config = getRequestConfigbasicToken(param: paramers,
                                           endPointURL: WebConstant.BASIC_TOKEN_PATH,
                                           method: .get)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }
    
    fileprivate func getRequestConfigbasicToken(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfig {
        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_AUTHENTICATION,
                                 path:WebConstant.BASIC_TOKEN_PATH,
                                 body:param,
                                 urlParams:nil,
                                 method: .get)
    }
    
    // MARK: - Refresh Token
    func getRefreshToken(paramers:[String:Any], isUser:Bool,  successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()){
        let param = [Constant.RequestConstant.refreshtoken:UserDefaultManager.shared[.refreshtoken] ?? ""] as [String : Any]
        self.refreshToken(paramers: param ,successBlock: { (response) in
            do {
                if let json = try JSONSerialization.jsonObject(with: response, options: []) as? [String: Any] {
                    if(isUser){
                        UserDefaultManager.shared[.accesstoken] = json["access_token"]
                        UserDefaultManager.shared[.refreshtoken] = json["refresh_token"]
                    }
                    else{
                        UserDefaultManager.shared[.rootaccesstoken] = json["access_token"]
                        UserDefaultManager.shared[.rootrefreshtoken] = json["refresh_token"]
                    }
                    successBlock(response)
                }
            } catch let error as NSError {
                print("Failed to load: \(error.localizedDescription)")
            }
        }, failureBlock: { error in
            print("\n\n===========Error===========")
            print("Error Code: \(error._code)")
            print("Error Messsage: \(error.localizedDescription)")
            failureBlock(error)
        })
    }
    
    
    func refreshToken(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let config = getRequestConfigrefreshToken(param: paramers,
                                           endPointURL: WebConstant.REFRESH_TOKEN_PATH,
                                           method: .post)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }
    
    // MARK: - Create Role List
    func createRole(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        
        let config = getRequestConfigCreateRole(param: paramers,
                                                endPointURL: WebConstant.CREATE_ROLE_PATH,
                                                method: .post)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
            
        })
    }
    
    fileprivate func getRequestConfigCreateRole(param:[String:Any],
                                                
                                                endPointURL:String,
                                                
                                                method:HTTPMethod) -> HTTPRequestConfig {
        
        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_USER,
                                 path:WebConstant.CREATE_ROLE_PATH,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)],
                                 body:param,
                                 urlParams:nil,
                                 method: .post)
    }
    
    // MARK: - Create Role Permission
    func createRolePermission(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        
        let config = getRequestConfigCreateRolePermission(param: paramers,
                                                endPointURL: WebConstant.CREATE_ROLE_PERMISSION,
                                                method: .post)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
            
        })
    }
    
    fileprivate func getRequestConfigCreateRolePermission(param:[String:Any],
                                                
                                                endPointURL:String,
                                                
                                                method:HTTPMethod) -> HTTPRequestConfig {
        
        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_AUTHORIZE,
                                 path:WebConstant.CREATE_ROLE_PERMISSION,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)],
                                 body:param,
                                 urlParams:nil,
                                 method: .post)
    }


    
    fileprivate func getRequestConfigrefreshToken(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfig {
        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_AUTHENTICATION,
                                 path:WebConstant.REFRESH_TOKEN_PATH,
                                 body:param,
                                 urlParams:nil,
                                 method: .post)
    }
    
    func rootrefreshToken(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let config = getRequestConfigrootrefreshToken(param: paramers,
                                           endPointURL: WebConstant.REFRESH_TOKEN_PATH,
                                           method: .post)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }
    
    fileprivate func getRequestConfigrootrefreshToken(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfig {
        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_AUTHENTICATION,
                                 path:WebConstant.REFRESH_TOKEN_PATH,
                                 body:param,
                                 urlParams:nil,
                                 method: .post)
    }

    
    // MARK: - Login
    func loginWebService(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let config = getRequestConfigLogin(param: paramers,
                                           endPointURL: WebConstant.LOGIN_PATH,
                                           method: .post)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }
    
    fileprivate func getRequestConfigLogin(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfig {
        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_AUTHENTICATION,
                                 path:WebConstant.LOGIN_PATH,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Basic " + (UserDefaultManager.shared[.basictoken] as! String),WebConstant.requestConstants.kSolutionkey: WebConstant.requestValueConstants.ksolution_key],
                                 body:param,
                                 urlParams:nil,
                                 method: .post)
    }
    
    // MARK: - User Detail
    func getUserDetail(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let config = getRequestConfigUserInfo(param: paramers,
                                           endPointURL: WebConstant.USER_DETAIL_PATH,
                                           method: .get)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestConfigUserInfo(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfig {
        if UserDefaultManager.shared[.isRootLogin] != nil {
            if (UserDefaultManager.shared[.isRootLogin] as! Bool) {
                if UserDefaultManager.shared[.accesstoken] == nil {
                    return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_USER,
                                             path:WebConstant.USER_DETAIL_PATH,
                                             headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)],
                                             body:param,
                                             urlParams:nil,
                                             method: .get)
                } else {
                    return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_USER,
                                             path:WebConstant.USER_DETAIL_PATH,
                                             headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)],
                                             body:param,
                                             urlParams:nil,
                                             method: .get)
                }
            } else {
                return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_USER,
                                         path:WebConstant.USER_DETAIL_PATH,
                                         headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)],
                                         body:param,
                                         urlParams:nil,
                                         method: .get)
            }
        } else {
            return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_USER,
                                     path:WebConstant.USER_DETAIL_PATH,
                                     headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)],
                                     body:param,
                                     urlParams:nil,
                                     method: .get)
        }
    }
    
    // MARK: - Company Subscription Detail
    func getCompanySubscriptionDetail(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let strApp = String(format: "%@%@/subscription", WebConstant.COMPANY_SUBSCRIPTION_DETAIL_PATH,paramers["companyGuid"] as! String)
        let config = getRequestConfigCompanySubscriptionDetail(param: paramers,
                                                 endPointURL: strApp,
                                           method: .get)
        HTTPService.shared.makeRequestwithoutBody(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestConfigCompanySubscriptionDetail(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfigwithoutBody {
        let strApp = String(format: "%@%@/subscription", WebConstant.COMPANY_SUBSCRIPTION_DETAIL_PATH,param["companyGuid"] as! String)
        return HTTPRequestConfigwithoutBody(baseUrl: WebConstant.BASE_URL_COMPANY,
                                            path:strApp,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)], method: .get)
    }
    
    // MARK: - Authorize Module
    func getAuthorizeModuleDetail(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let strApp = String(format: "%@", WebConstant.AUTHORIZE_MODULE_DETAIL_PATH)
        let config = getRequestConfigAuthorizeModuleDetail(param: paramers,
                                                 endPointURL: strApp,
                                           method: .get)
        HTTPService.shared.makeRequestwithoutBody(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestConfigAuthorizeModuleDetail(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfigwithoutBody {
        let strApp = String(format: "%@", WebConstant.AUTHORIZE_MODULE_DETAIL_PATH)
        return HTTPRequestConfigwithoutBody(baseUrl: WebConstant.BASE_URL_AUTHORIZE,
                                            path:strApp,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)], method: .get)
    }

    
    // MARK: - Country List
    func getCountryList(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let config = getRequestConfigCountryList(param: paramers,
                                           endPointURL: WebConstant.COUNTRY_LIST_PATH,
                                           method: .get)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestConfigCountryList(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfig {
        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_MASTER,
                                 path:WebConstant.COUNTRY_LIST_PATH,
                                 body:param,
                                 urlParams:nil,
                                 method: .get)
    }
    
    // MARK: - State List
    func getStateList(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let config = getRequestConfigStateList(param: paramers,
                                           endPointURL: String(format: "%@/%@/state", WebConstant.COUNTRY_LIST_PATH,paramers["countryGuid"] as! String),
                                           method: .get)
        HTTPService.shared.makeRequestwithoutBody(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestConfigStateList(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfigwithoutBody {
        return HTTPRequestConfigwithoutBody(baseUrl: WebConstant.BASE_URL_MASTER,
                                 path:String(format: "%@/%@/state", WebConstant.COUNTRY_LIST_PATH,param["countryGuid"] as! String),
                                 method: .get)
    }
        
    // MARK: - Entity List
    func getEntityList(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let config = getRequestConfigEntityList(param: paramers,
                                           endPointURL: WebConstant.ENTITY_LOOKUP_PATH,
                                           method: .get)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestConfigEntityList(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfig {
        if UserDefaultManager.shared[.isRootLogin] != nil {
            if (UserDefaultManager.shared[.isRootLogin] as! Bool) {
                if UserDefaultManager.shared[.accesstoken] == nil {
                    return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_USER,
                                             path:WebConstant.ENTITY_LOOKUP_PATH,
                                             headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)],
                                             body:param,
                                             urlParams:nil,
                                             method: .get)
                } else {
                    return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_USER,
                                             path:WebConstant.ENTITY_LOOKUP_PATH,
                                             headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)],
                                             body:param,
                                             urlParams:nil,
                                             method: .get)
                }
            } else {
                return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_USER,
                                         path:WebConstant.ENTITY_LOOKUP_PATH,
                                         headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)],
                                         body:param,
                                         urlParams:nil,
                                         method: .get)
            }
        } else {
            return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_USER,
                                     path:WebConstant.ENTITY_LOOKUP_PATH,
                                     headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)],
                                     body:param,
                                     urlParams:nil,
                                     method: .get)
        }
    }

    // MARK: - Role List
    func getRoleList(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let config = getRequestConfigRoleList(param: paramers,
                                           endPointURL: WebConstant.ROLE_LOOKUP_PATH,
                                           method: .get)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestConfigRoleList(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfig {
        if UserDefaultManager.shared[.isRootLogin] != nil {
            if (UserDefaultManager.shared[.isRootLogin] as! Bool) {
                if UserDefaultManager.shared[.accesstoken] == nil {
                    return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_USER,
                                             path:WebConstant.ROLE_LOOKUP_PATH,
                                             headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)],
                                             body:param,
                                             urlParams:nil,
                                             method: .get)
                } else {
                    return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_USER,
                                             path:WebConstant.ROLE_LOOKUP_PATH,
                                             headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)],
                                             body:param,
                                             urlParams:nil,
                                             method: .get)
                }
            } else {
                return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_USER,
                                         path:WebConstant.ROLE_LOOKUP_PATH,
                                         headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)],
                                         body:param,
                                         urlParams:nil,
                                         method: .get)
            }
        } else {
            return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_USER,
                                     path:WebConstant.ROLE_LOOKUP_PATH,
                                     headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)],
                                     body:param,
                                     urlParams:nil,
                                     method: .get)
        }
    }

    // MARK: - Signup
    func SignupWebService(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let config = getRequestConfigSignup(param: paramers,
                                           endPointURL: WebConstant.ENTITY_ADD_PATH,
                                           method: .post)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }
    
    fileprivate func getRequestConfigSignup(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfig {
        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_USER,
                                 path:WebConstant.ENTITY_ADD_PATH,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)],
                                 body:param,
                                 urlParams:nil,
                                 method: .post)
    }
    
    // MARK: - addUser
//    func AddUSerWebService(entityID:String,paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
//        let config = getRequestConfigAddUser(param: paramers,
//                                           endPointURL: WebConstant.USER_ADD_PATH,
//                                           method: .post)
//        HTTPService.shared.makeRequest(config: config, success: { (response) in
//            print("---- response ----  \(response)")
//            successBlock(response)
//        },failure: { error in
//            print("\n\n---- Error ----")
//            failureBlock(error)
//        })
//    }
//
//    fileprivate func getRequestConfigAddUser(param:[String:Any],
//                                           endPointURL:String,
//                                           method:HTTPMethod) -> HTTPRequestConfig {
//        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_USER,
//                                 path:WebConstant.USER_ADD_PATH,
//                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)],
//                                 body:param,
//                                 urlParams:nil,
//                                 method: .post)
//    }

}




