//
//  DeviceManager.swift
//  Procell
//
//  Created by Megha Salla on 26/11/21.

import Foundation

struct DeviceManager {
    
    typealias CompletionHandler = (_ success:Bool) -> Void
    
    static let shared = DeviceManager()
    
    // MARK: - Device Lists
    func deviceListsWebService(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let strApp = String(format: "%@?pageNumber=%d&pageSize=%d", WebConstant.DEVICE_LIST_PATH,paramers["pageNumber"] as! Int,paramers["pageSize"] as! Int)
        let config = getRequestConfigDeviceLists(param: paramers,
                                                 endPointURL: strApp,
                                           method: .get)
        HTTPService.shared.makeRequestwithoutBody(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestConfigDeviceLists(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfigwithoutBody {
        let strApp = String(format: "%@?pageNumber=%d&pageSize=%d", WebConstant.DEVICE_LIST_PATH,param["pageNumber"] as! Int,param["pageSize"] as! Int)
        return HTTPRequestConfigwithoutBody(baseUrl: WebConstant.BASE_URL_DEVICE,
                                            path:strApp,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)], method: .get)
    }
    
    
    // MARK: - Single Device Detail
    func singledeviceDetailsWebService(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let strApp = String(format: "%@/%@", WebConstant.SINGLE_DEVICE_LIST_PATH,paramers["uniqueId"] as? String ?? "")
        let config = getRequestConfigsingledeviceDetail(param: paramers,
                                                 endPointURL: strApp,
                                           method: .get)
        HTTPService.shared.makeRequestwithoutBody(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestConfigsingledeviceDetail(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfigwithoutBody {
        let strApp = String(format: "%@/%@", WebConstant.SINGLE_DEVICE_LIST_PATH,param["uniqueId"] as? String ?? "")
        return HTTPRequestConfigwithoutBody(baseUrl: WebConstant.BASE_URL_DEVICE,
                                            path:strApp,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)], method: .get)
    }
    
    // MARK: - Get Device Grant Detail
    func getdevicegrantDetailsWebService(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let strApp = String(format: "%@/%@/grant", WebConstant.DEVICE_LIST_PATH,paramers["uniqueId"] as? String ?? "")
        let config = getRequestConfiggetdevicegrantDetail(param: paramers,
                                                 endPointURL: strApp,
                                           method: .get)
        HTTPService.shared.makeRequestwithoutBody(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestConfiggetdevicegrantDetail(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfigwithoutBody {
        let strApp = String(format: "%@/%@/grant", WebConstant.DEVICE_LIST_PATH,param["uniqueId"] as? String ?? "")
        return HTTPRequestConfigwithoutBody(baseUrl: WebConstant.BASE_URL_DEVICE,
                                            path:strApp,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)], method: .get)
    }

    // MARK: - Post Device Grant Detail
    func postdevicegrantDetailsWebService(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let strApp = String(format: "%@/%@/grant", WebConstant.DEVICE_LIST_PATH,paramers["uniqueId"] as? String ?? "")
        let config = postRequestConfiggetdevicegrantDetail(param: paramers, endPointURL: strApp, method: .post)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func postRequestConfiggetdevicegrantDetail(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfig {
        let strApp = String(format: "%@/%@/grant", WebConstant.DEVICE_LIST_PATH,param["uniqueId"] as? String ?? "")
        var dict : [String : Any] = [:]
        dict["emails"] = [UserDefaultManager.shared[.email]]
        dict["permission"] = ["view":true,"operation":true]
        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_DEVICE, path: strApp, headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)], body: dict, urlParams: nil, method: .post)
    }

    // MARK: - Device Invoke Service
    func deviceInvokeWebService(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let strApp = String(format: "%@/user-device-permission/%@/revoke", WebConstant.DEVICE_LIST_PATH,paramers["entityGuid"] as? String ?? "")
        let config = deleteRequestConfigDeviceRevoke(param: paramers, endPointURL: strApp, method: .delete)
        HTTPService.shared.makeRequestwithoutBody(config: config, success: { (response) in
              print("---- response ----  \(response)")
              successBlock(response)
          },failure: { error in
              print("\n\n---- Error ----")
              failureBlock(error)
          })
    }

    fileprivate func deleteRequestConfigDeviceRevoke(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfigwithoutBody {
        let strApp = String(format: "%@/user-device-permission/%@/revoke", WebConstant.DEVICE_LIST_PATH,param["entityGuid"] as? String ?? "")
        return HTTPRequestConfigwithoutBody(baseUrl: WebConstant.BASE_URL_DEVICE, path: strApp, headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)], method: method)
    }
    
    // MARK: - Device Entity Change
    func deviceEntityWebService(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        var arrsdevice : [[String:Any]] = []
        var dict : [String : Any] = [:]
        dict["deviceguid"] = paramers["uniqueId"]
        arrsdevice.append(dict)
        let strApp = String(format: "%@/%@/device", WebConstant.DEVICE_ENTITY_PATH,paramers["entityGuid"] as? String ?? "")
        let config = getRequestConfigDeviceEntity(param: paramers, endPointURL: strApp, method: .put)
        HTTPService.shared.makeRequestarray(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestConfigDeviceEntity(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfigArray {
        var arrsdevice : [[String:Any]] = []
        var dict : [String : Any] = [:]
        dict["deviceguid"] = param["uniqueId"]
        arrsdevice.append(dict)
        let strApp = String(format: "%@/%@/device", WebConstant.DEVICE_ENTITY_PATH,param["entityGuid"] as? String ?? "")
        if param["Deleted"] as? String ?? "" == "1" {
            return HTTPRequestConfigArray(baseUrl: WebConstant.BASE_URL_DEVICE,
                                     path:strApp,
                                     headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)],
                                     body:arrsdevice,
                                     urlParams:nil,
                                     method: method)
        } else {
            return HTTPRequestConfigArray(baseUrl: WebConstant.BASE_URL_DEVICE,
                                     path:strApp,
                                     headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)],
                                     body:arrsdevice,
                                     urlParams:nil,
                                     method: method)
        }
    }
    
    // MARK: - Device Update
    func deviceUpdateWebService(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let strApp = String(format: "%@/%@", WebConstant.DEVICE_UPDATE,paramers["deviceguid"] as? String ?? "")
        let config = getRequestConfigDeviceUpdate(param: paramers, endPointURL: strApp, method: .put)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestConfigDeviceUpdate(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfig {
        var dict : [String : Any] = [:]
        dict["entityGuid"] = param["uniqueId"]
        dict["displayName"] = param["displayName"]
        let strApp = String(format: "%@/%@", WebConstant.DEVICE_UPDATE,param["deviceguid"] as? String ?? "")
//        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_DEVICE,
//                                 path:strApp,
//                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.rootaccesstoken] as! String)],
//                                 body:dict,
//                                 urlParams:nil,
//                                 method: method)
        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_DEVICE,
                                 path:strApp,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)],
                                 body:dict,
                                 urlParams:nil,
                                 method: method)
    }

    
    // MARK: - Delete Device
    func DeletedeviceWebService(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let strApp = String(format: "%@/%@", WebConstant.DEVICE_LIST_PATH,paramers["uniqueId"] as! String as CVarArg)
        let config = getRequestDeleteDevice(param: paramers,
                                                 endPointURL: strApp,
                                           method: .delete)
        HTTPService.shared.makeRequestwithoutBody(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestDeleteDevice(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfigwithoutBody {
        let strApp = String(format: "%@/%@", WebConstant.DEVICE_LIST_PATH,param["uniqueId"] as! String as CVarArg)
        return HTTPRequestConfigwithoutBody(baseUrl: WebConstant.BASE_URL_DEVICE,
                                            path:strApp,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)], method: .delete)
    }

    
    // MARK: - Sensor Lists
    func deviceTemplateListsWebService(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let strApp = String(format: "%@/%@?pageSize=%d&pageNumber=%d", WebConstant.TEMPLATE_ATTRIBUTEL_LIST_PATH,paramers["devicetemplateguid"] as! CVarArg,paramers["pageSize"] as! Int,paramers["pageNumber"] as! Int)
        let config = getRequestConfigDeviceTemplateLists(param: paramers,
                                                 endPointURL: strApp,
                                           method: .get)
        HTTPService.shared.makeRequestwithoutBody(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestConfigDeviceTemplateLists(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfigwithoutBody {
        return HTTPRequestConfigwithoutBody(baseUrl: WebConstant.BASE_URL_DEVICE,
                                            path:endPointURL,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)], method: .get)
    }
    
    // MARK: - Template Default Service
    func templateDefaultWebService(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        
        let strApp = String(format: "%@/%@", WebConstant.TELEMETRY_DEFAULT_DATA_PATH,paramers["devicetemplateguid"] as! CVarArg)
        let config = getRequestConfigTemplateDefault(param: paramers,
                                                 endPointURL: strApp,
                                           method: .get)
        HTTPService.shared.makeRequestwithoutBody(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestConfigTemplateDefault(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfigwithoutBody {
        return HTTPRequestConfigwithoutBody(baseUrl: WebConstant.BASE_URL_TELEMETRY,
                                            path:endPointURL,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)], method: .get)
    }

    // MARK: - Template Lists
    func templateListsWebService(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        
        let strApp = String(format: "%@", WebConstant.TEMPLATE_LOOKUP_PATH)
        let config = getRequestConfigTemplateLists(param: paramers,
                                                 endPointURL: strApp,
                                           method: .get)
        HTTPService.shared.makeRequestwithoutBody(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestConfigTemplateLists(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfigwithoutBody {
        return HTTPRequestConfigwithoutBody(baseUrl: WebConstant.BASE_URL_DEVICE,
                                            path:endPointURL,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)], method: .get)
    }

    
    // MARK: - Sensor Command Lists
    func sensorCommandListsWebService(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        
        let strApp = String(format: "%@/%@", WebConstant.TEMPLATE_LOOKUP_PATH,paramers["devicetemplateguid"] as! CVarArg)
        let config = getRequestConfigSensorcommandLists(param: paramers,
                                                 endPointURL: strApp,
                                           method: .get)
        HTTPService.shared.makeRequestwithoutBody(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestConfigSensorcommandLists(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfigwithoutBody {
        return HTTPRequestConfigwithoutBody(baseUrl: WebConstant.BASE_URL_DEVICE,
                                            path:endPointURL,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)], method: .get)
    }
    
    func HistroryDataWebService(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        
        var strApp = String(format: "%@/%@/from/%@/to/%@", WebConstant.HISTORICAL_DATA_PATH,paramers["uniqueId"] as! CVarArg,paramers["fromdate"] as! CVarArg,paramers["todate"] as! CVarArg)
//        strApp = strApp.replacingOccurrences(of: " ", with: "%20")
        strApp = strApp.addingPercentEncoding(withAllowedCharacters:.urlQueryAllowed) ?? ""
        let config = getRequestConfighistroryData(param: paramers,
                                                 endPointURL: strApp,
                                           method: .get)
        HTTPService.shared.makeRequestwithoutBody(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }

    fileprivate func getRequestConfighistroryData(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfigwithoutBody {
        return HTTPRequestConfigwithoutBody(baseUrl: WebConstant.BASE_URL_TELEMETRY,
                                            path:endPointURL,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)], method: .get)
    }


    
    // MARK: - Add Device Web-Service
    func addDeviceWebService(paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let config = getRequestConfigAddDevice(param: paramers,
                                           endPointURL: WebConstant.DEVICE_LIST_PATH,
                                           method: .post)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }
    
    fileprivate func getRequestConfigAddDevice(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfig {
        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_DEVICE,
                                 path:WebConstant.DEVICE_LIST_PATH,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)],
                                 body:param,
                                 urlParams:nil,
                                 method: .post)
    }
    
    
    func twinPropertiesListWebService(deviceTemplateGUID:String,paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let endPoint = String(format: "%@/%@/lookup", WebConstant.TWIN_PROPERTY_LOOKUP_PATH,deviceTemplateGUID)

        let config = getRequestConfigtwinPropertiesList(param: paramers,
                                           endPointURL: endPoint,
                                           method: .get)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }
    
    fileprivate func getRequestConfigtwinPropertiesList(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfig {
        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_DEVICE,
                                 path:endPointURL,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)],
                                 body:param,
                                 urlParams:nil,
                                 method: .get)
    }

    
    func propertiesHistoryWebService(strDeviceID:String,paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        
        let strEndPoint = String(format: "%@/%@/twin", WebConstant.TWIN_PROPERTY_HISTORY_PATH,strDeviceID)

        let config = getRequestConfigPropertyHistory(param: paramers,
                                           endPointURL: strEndPoint,
                                           method: .get)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }
    
    fileprivate func getRequestConfigPropertyHistory(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfig {
        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_DEVICE,
                                 path:endPointURL,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)],
                                 body:param,
                                 urlParams:nil,
                                 method: .get)
    }

    func executeTwinPropertyWebService(deviceName:String,paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let endPoint = String(format: "%@/%@/twin", WebConstant.EXECUTE_TWIN_PROPERTY_PATH,deviceName)

        let config = getRequestConfigExecuteTwinProperty(param: paramers,
                                           endPointURL: endPoint,
                                           method: .put)

        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }
    
    fileprivate func getRequestConfigExecuteTwinProperty(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfig {
        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_DEVICE,
                                 path:endPointURL,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)],
                                 body:param,
                                 urlParams:nil,
                                 method: method)
    }
    
    func getCommandTypesWebService(deviceTemplateGUID:String, paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        
        let endPoint = String(format: "%@/%@/lookup", WebConstant.COMMAND_LOOUP_PATH,deviceTemplateGUID)

        let config = getRequestConfigCommandTypes(param: paramers,
                                           endPointURL: endPoint,
                                           method: .get)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }
    
    fileprivate func getRequestConfigCommandTypes(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfig {
        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_DEVICE,
                                 path:endPointURL,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)],
                                 body:param,
                                 urlParams:nil,
                                 method: .get)
    }


    func getCommandHistoryWebService(paramers:[String:Any],urlParamers:[String:String], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        
        let strEndPoint = String(format: "%@", WebConstant.COMMAND_HISTORY_PATH)

        let config = getRequestConfigCommandHistory(param: paramers,
                                                    endPointURL: strEndPoint, urlParam: urlParamers,
                                           method: .get)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }
    
    fileprivate func getRequestConfigCommandHistory(param:[String:Any],
                                           endPointURL:String,urlParam:[String:String],
                                           method:HTTPMethod) -> HTTPRequestConfig {
        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_DEVICE,
                                 path:endPointURL,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)],
                                 body:param,
                                 urlParams:urlParam,
                                 method: .get)
    }
    
    func generateCommandWebService(strDeviceGUID:String,paramers:[String:Any], successBlock:@escaping (Data) -> (), failureBlock: @escaping (Error) -> ()) {
        let endPoint = String(format: "%@/%@/send", WebConstant.EXECUTE_COMMAND_PATH,strDeviceGUID)

        let config = getRequestConfigGenerateCommand(param: paramers,
                                           endPointURL: endPoint,
                                           method: .post)
        HTTPService.shared.makeRequest(config: config, success: { (response) in
            print("---- response ----  \(response)")
            successBlock(response)
        },failure: { error in
            print("\n\n---- Error ----")
            failureBlock(error)
        })
    }
    
    fileprivate func getRequestConfigGenerateCommand(param:[String:Any],
                                           endPointURL:String,
                                           method:HTTPMethod) -> HTTPRequestConfig {
        return HTTPRequestConfig(baseUrl: WebConstant.BASE_URL_DEVICE,
                                 path:endPointURL,
                                 headers: [WebConstant.requestConstants.kAuthorization: "Bearer " + (UserDefaultManager.shared[.accesstoken] as! String)],
                                 body:param,
                                 urlParams:nil,
                                 method: method)
    }



}
