/*
 * ebv_logs.h
 *
 *  Created on: 1. jun. 2023
 *      Author: MaliU
 */

#ifndef EBV_LOGS_H_
#define EBV_LOGS_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

typedef enum
{
	TOOL_ENVIRONMENT_INFINEON_MTB = 0,
	TOOL_ENVIRONMENT_INFINEON_MTB_FREERTOS,
	TOOL_ENVIRONMENT_SMT32
} tool_environment_t;

#define	TOOL_ENVIRONMENT_INFINEON_MTB  0
#define	TOOL_ENVIRONMENT_INFINEON_MTB_FREERTOS 1
#define	TOOL_ENVIRONMENT_SMT32 2


#define TOOL_ENVIRONMENT TOOL_ENVIRONMENT_INFINEON_MTB_FREERTOS

#if (TOOL_ENVIRONMENT == TOOL_ENVIRONMENT_INFINEON_MTB) | (TOOL_ENVIRONMENT == TOOL_ENVIRONMENT_INFINEON_MTB_FREERTOS)
	#include "cy_retarget_io.h"
#endif

#if TOOL_ENVIRONMENT == TOOL_ENVIRONMENT_STM32

#endif

/******************************************************
 *                   Enumerations
 ******************************************************/

/** Logging levels. NOTE: Default value for all modules is passed during init call */
typedef enum
{
    EBV_LOGS_OFF = 0,     /**< No logs output */
    EBV_LOGS_MENU,        /**< User interface messages */
    EBV_LOGS_ERROR,       /**< Critical error, caller function crashes the application. Prints log message if run-time level is <= EBV_LOGS_ERROR       */
    EBV_LOGS_WARNING,     /**< Error recovered. Print log message if run-time level is <= EBV_LOGS_WARNING   */
    EBV_LOGS_NOTICE,      /**< Notice on possible error. Print log message if run-time level is <= EBV_LOGS_NOTICE    */
    EBV_LOGS_INFO,        /**< Information describing expected outcome. Print log message if run-time level is <= EBV_LOGS_INFO      */
    EBV_LOGS_DEBUG,       /**< Additional information useful for debugging. Print log message if run-time level is <= EBV_LOGS_DEBUG     */
    EBV_LOGS_DEBUG1,      /**< One more level for more debug messages. Print log message if run-time level is <= EBV_LOGS_DEBUG1    */

    EBV_LOGS_ALL		  /**< Max level, bigger chunks of debug information, or similar. May flood the output. Have to be last, used for checking*/
} ebv_logs_level_t;

/** Log Module type
 *  Log module allow for separate modules or subsystems to have different run-time log levels for output.
 *  This allows for targeted subsystem debugging without turning other modules levels
 *  - focusing on wanted output during debugging / testing.
 */
typedef enum
{
    EBV_LOGS_MODULE_DEF = 0,                   /**< General log message not associated with any specific module */
	EBV_LOGS_MODULE_TEST,                      /**< Test module */
	EBV_LOGS_MODULE_DRIVER,                    /**< Driver module */
	EBV_LOGS_MODULE_LOW_POWER,                 /**< Low Power module */
	EBV_LOGS_MODULE_MIDDLEWARE,                /**< Middleware module */
	EBV_LOGS_MODULE_AUDIO,                     /**< Audio module */
	EBV_LOGS_MODULE_APP,					   /**< Application module */

	EBV_LOGS_MODULE_ALL                        /**< Have to be last, used for checking */
} ebv_logs_module_t;

/******************************************************
 *                 Type Definitions
 ******************************************************/

typedef uint32_t ebv_logs_rslt_t;

extern ebv_logs_level_t ebv_logs_level;


/******************************************************
 *                      Macros
 ******************************************************/

/******************************************************
 *                    Constants
 ******************************************************/
#define EBV_LOGS_COLOR 	1 //0/1 - disable/enable colors
#define EBV_LOGS_IS_TIME_FORMATED 	1 //0/1 - disable/enable time formating

#if EBV_LOGS_COLOR
#define LOG_COLOR_BLACK   "30"
#define LOG_COLOR_RED     "31"
#define LOG_COLOR_GREEN   "32"
#define LOG_COLOR_BROWN   "33"
#define LOG_COLOR_BLUE    "34"
#define LOG_COLOR_PURPLE  "35"
#define LOG_COLOR_CYAN    "36"
#define LOG_COLOR(COLOR)  "\033[0;" COLOR "m"
#define LOG_BOLD(COLOR)   "\033[1;" COLOR "m"
#define LOG_RESET_COLOR   "\033[0m"
#define LOG_COLOR_M
#define LOG_COLOR_E       LOG_COLOR(LOG_COLOR_RED)
#define LOG_COLOR_W       LOG_COLOR(LOG_COLOR_BROWN)
#define LOG_COLOR_N       LOG_COLOR(LOG_COLOR_CYAN)
#define LOG_COLOR_I       LOG_COLOR(LOG_COLOR_GREEN)
#define LOG_COLOR_D		  LOG_COLOR(LOG_COLOR_BLUE)
#define LOG_COLOR_A
#else //EBV_LOGS_COLOR
#define LOG_COLOR_M
#define LOG_COLOR_E
#define LOG_COLOR_W
#define LOG_COLOR_N
#define LOG_COLOR_I
#define LOG_COLOR_D
#define LOG_COLOR_A
#define LOG_RESET_COLOR
#endif //EBV_LOGS_COLOR

//#define LOG_FORMAT(letter, format)  		LOG_COLOR_##letter #letter "(%u) [%s]: " format LOG_RESET_COLOR "\r\n"	//includes letter
//#define LOG_FORMAT_CONT(letter, format)  	LOG_COLOR_##letter #letter "(%u) [%s]: " format //continuous write with the same format //includes letter
#define LOG_FORMAT(letter, format)  		LOG_COLOR_##letter "(%s) [%s]: " format LOG_RESET_COLOR "\r\n"
//#define LOG_FORMAT_MENU(letter, format)  	LOG_COLOR_##letter "[%s]: " format LOG_RESET_COLOR "\r\n" //includes tag print
#define LOG_FORMAT_MENU(letter, format)  	LOG_COLOR_##letter format LOG_RESET_COLOR "\r\n" //includes tag print
#define LOG_FORMAT_CONT(letter, format)  	LOG_COLOR_##letter "(%s) [%s]: " format //continuous write for lines to be continued
#define LOG_FORMAT_NEXT(format)  			format //next item in the continuous line, no format change
#define LOG_FORMAT_RESET(format)  			format LOG_RESET_COLOR "\r\n" //close format

/* \x1b[2J\x1b[;H - ANSI ESC sequence to clear screen. */
#define EBV_LOGS_CLEAR_SCREEEN()			ebv_logs_printf("\x1b[2J\x1b[;H");

#if EBV_LOGS_COLOR

//#define EBV_LOGS_M(tag, module, format, ...) 	ebv_logs_write(EBV_LOGS_MENU, module, LOG_FORMAT_MENU(M, format), tag, ##__VA_ARGS__); //includes tag print
#define EBV_LOGS_M(tag, module, format, ...) 	ebv_logs_write(EBV_LOGS_MENU, module, LOG_FORMAT_MENU(M, format), ##__VA_ARGS__);
#define EBV_LOGS_E(tag, module, format, ...) 	ebv_logs_write(EBV_LOGS_ERROR, module, LOG_FORMAT(E, format), ebv_logs_timestamp(), tag, ##__VA_ARGS__);
#define EBV_LOGS_W(tag, module, format, ...) 	ebv_logs_write(EBV_LOGS_WARNING, module, LOG_FORMAT(W, format), ebv_logs_timestamp(), tag, ##__VA_ARGS__);
#define EBV_LOGS_N(tag, module, format, ...) 	ebv_logs_write(EBV_LOGS_NOTICE, module, LOG_FORMAT(N, format), ebv_logs_timestamp(), tag, ##__VA_ARGS__);
#define EBV_LOGS_I(tag, module, format, ...) 	ebv_logs_write(EBV_LOGS_INFO, module, LOG_FORMAT(I, format), ebv_logs_timestamp(), tag, ##__VA_ARGS__);
#define EBV_LOGS_D(tag, module, format, ...) 	ebv_logs_write(EBV_LOGS_DEBUG, module, LOG_FORMAT(D, format), ebv_logs_timestamp(), tag, ##__VA_ARGS__);

// log with no LF and CR
#define EBV_LOGS_M_CONT(tag, module, format, ...) 	ebv_logs_write(EBV_LOGS_MENU, module, LOG_FORMAT_CONT(M, format), tag, ##__VA_ARGS__); //includes tag print
#define EBV_LOGS_N_CONT(tag, module, format, ...) 	ebv_logs_write(EBV_LOGS_NOTICE, module, LOG_FORMAT_CONT(N, format), ebv_logs_timestamp(), tag, ##__VA_ARGS__);
#define EBV_LOGS_I_CONT(tag, module, format, ...) 	ebv_logs_write(EBV_LOGS_INFO, module, LOG_FORMAT_CONT(I, format), ebv_logs_timestamp(), tag, ##__VA_ARGS__);
#define EBV_LOGS_D_CONT(tag, module, format, ...) 	ebv_logs_write(EBV_LOGS_DEBUG, module, LOG_FORMAT_CONT(D, format), ebv_logs_timestamp(), tag, ##__VA_ARGS__);

// log next on continuous line, no LF and CR
#define EBV_LOGS_I_NEXT(tag, module, format, ...) 	ebv_logs_write(EBV_LOGS_INFO, module, LOG_FORMAT_NEXT(format), ##__VA_ARGS__);
#define EBV_LOGS_D_NEXT(tag, module, format, ...) 	ebv_logs_write(EBV_LOGS_DEBUG, module, LOG_FORMAT_NEXT(format), ##__VA_ARGS__);

// close formatting and add LF and CR
#define EBV_LOGS_I_RESET(tag, module, format) 		ebv_logs_write(EBV_LOGS_INFO, module, LOG_FORMAT_RESET(format));
#define EBV_LOGS_D_RESET(tag, module, format) 		ebv_logs_write(EBV_LOGS_DEBUG, module, LOG_FORMAT_RESET(format));

#define EBV_LOGS_PRINTF(format, ...) 				ebv_logs_printf(format, ##__VA_ARGS__);

#else //EBV_LOGS_COLOR


#endif //EBV_LOGS_COLOR


/**
 * @brief Write message into the log
 *
 * This function is not intended to be used directly. Use one of the follwoing instead:
 * EBV_LOGE, EBV_LOGW, EBV_LOGI, EBV_LOGD, EBV_LOGV macros.
 *
 * This function or these macros should not be used from an interrupt.
 */
void ebv_logs_write(ebv_logs_level_t level, ebv_logs_module_t module, const char* format, ...); // __attribute__ ((format (printf, 3, 4)));

/**
 * @brief Function which returns timestamp to be used in log outputs
 *
 * This function is used in expansion of EBV_LOGS_x macros.
 *
 * @return timestamp, in milliseconds
 */
char* ebv_logs_timestamp(void);

/**
 * @brief Function which returns system timestamp to be used in log output
 *
 * This function is used in expansion of EBV_LOGS_x macros to print
 * the time as "HH:MM:SS.sss". The time is initialized to
 * 0 on startup.
 *
  * @return timestamp, in "HH:MM:SS.sss"
 */
char* ebv_logs_timestamp_formated(void);


/**
 * @brief Write message into the log, va_list variant
 * @see ebv_log_write()
 *
 * This function is provided to ease integration toward other logging framework,
 * so that ebv_log can be used as a log sink.
 */
void ebv_log_writev(ebv_logs_level_t level, ebv_logs_module_t module, const char* tag, const char* format, va_list args);


//void ebv_logs_printf(const char *format, va_list args);
void ebv_logs_printf(const char* format, ...);


#endif /* EBV_LOGS_H_ */
