/*
 * drum_fonts.h
 *
 *  Created on: 15. feb. 2022
 *      Author: MaliU
 */

#ifndef DRUM_FONTS_H_
#define DRUM_FONTS_H_

#include "string.h"
#include "stdint.h"


//H font - pixels described horizontally
//V font - pixels described vertically
typedef enum e_drum_font_type
{
    FONT_SIMPLE_H           = 1,  ///< original font
    FONT_MIKROE_GLDC_1V_VAR = 2,  ///< original GLDC font from MikroElektronika, vertical, 1 byte, variable width
    FONT_MIKROE_GLDC_2V_VAR = 3,  ///< original GLDC font from MikroElektronika, vertical, 2 bytes, variable width
    FONT_MIKROE_GLDC_4V_VAR = 4,  ///< original GLDC font from MikroElektronika, vertical, 4 bytes, variable width
} drum_font_type_t;

/**
 * @brief  Font structure used on DrUM libraries
 */
typedef struct {
    uint8_t FontType;           /*!< Font type */
    uint8_t FontWidth;          /*!< Font width in pixels */
    uint8_t FontHeight;         /*!< Font height in pixels */
    uint8_t FontFirstChar;      /*!< Font first char */
    uint8_t FontLastChar;       /*!< Font last char */
    uint8_t FontSpaceH;         /*!< Font horizontal space used with fully compressed fonts */
    uint8_t FontSpaceV;         /*!< Font vertical space used with fully compressed fonts */
    const uint16_t *widths;     /*!< Pointer to font widths data array */
    const uint16_t *pointers;   /*!< Pointer to font data pointers data array */
    const uint8_t *data;        /*!< Pointer to data font data array */
} DrUM_FontDef_t;


extern DrUM_FontDef_t MS_Gothic6x12;
extern DrUM_FontDef_t IsidoraSansMedium16x16;
extern DrUM_FontDef_t Small5x8;
extern DrUM_FontDef_t Small8x10;
extern DrUM_FontDef_t SmallBold9x10;
extern DrUM_FontDef_t ArialRoundedMTBold28x31;
extern DrUM_FontDef_t SmallSym10x10;

#endif /* DRUM_FONTS_H_ */
