/*
 * drum_oled.h
 *
 *  Created on: 15. feb. 2022
 *      Author: MaliU
 */

#ifndef DRUM_OLED_H_
#define DRUM_OLED_H_

#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include "drum_fonts.h"

/* Private SSD1306 structure */
typedef struct {
    uint16_t CurrentX;
    uint16_t CurrentY;
    uint8_t Inverted;
    uint8_t Initialized;
    uint16_t MinUpdatedX;
    uint16_t MaxUpdatedX;
    uint16_t MinUpdatedY;
    uint16_t MaxUpdatedY;
    uint8_t *Buffer;
} OLED_t;

//extern OLED_t OLED;

/**
 * @brief  OLED types
 */
#define OLED_TYPE_SSD1306           0x01       /*!< basic black & white, 1 bit per pixel */
#define OLED_TYPE_SSD1327           0x02       /*!< 4 levels, 2 bits per pixel */

#define OLED_TYPE           OLED_TYPE_SSD1327

#define OLED_WIDTH      128
#define OLED_HEIGHT     128

#if OLED_TYPE == OLED_TYPE_SSD1327
//#define OLED_GRAYSCALE_LEVELS   OLED_GRAYSCALE_LEVELS_16
#define OLED_GRAYSCALE_LEVELS   OLED_GRAYSCALE_LEVELS_4
#endif

#if OLED_TYPE == OLED_TYPE_SSD1306
#define OLED_GRAYSCALE_LEVELS   OLED_GRAYSCALE_LEVELS_2
#endif
/**
 * @brief  OLED gray scale levels
 */

#define OLED_GRAYSCALE_LEVELS_2     0x01        /*!< basic black & white, 1 bit per pixel */
#define OLED_GRAYSCALE_LEVELS_4     0x02        /*!< 4 levels, 2 bits per pixel */
//#define OLED_GRAYSCALE_LEVELS_8  0x03        /*!< 8 levels, 3 bits per pixel */ // does not make much sense since it is not aligned
#define OLED_GRAYSCALE_LEVELS_16    0x04        /*!< 16 levels, 4 bits per pixel */

/**
 * @brief  OLED color enumeration
 */
#if (OLED_GRAYSCALE_LEVELS == OLED_GRAYSCALE_LEVELS_16)
#define OLED_COLOR_BLACK        0x00 /*!< Black color, no pixel */
#define OLED_COLOR_DARK_GRAY    0x04
#define OLED_COLOR_GRAY         0x08
#define OLED_COLOR_LIGHT_GRAY   0x0C
#define OLED_COLOR_WHITE        0x0F  /*!< Pixel is set. Color depends on LCD */
#endif

#if (OLED_GRAYSCALE_LEVELS == OLED_GRAYSCALE_LEVELS_2)
#define OLED_COLOR_BLACK        0x00 /*!< Black color, no pixel */
#define OLED_COLOR_DARK_GRAY    0x00
#define OLED_COLOR_GRAY         0x01
#define OLED_COLOR_LIGHT_GRAY   0x01
#define OLED_COLOR_WHITE        0x01  /*!< Pixel is set. Color depends on LCD */
#endif

#if (OLED_GRAYSCALE_LEVELS == OLED_GRAYSCALE_LEVELS_4)
#define OLED_COLOR_BLACK        0x00 /*!< Black color, no pixel */
#define OLED_COLOR_DARK_GRAY    0x00
#define OLED_COLOR_GRAY         0x01
#define OLED_COLOR_LIGHT_GRAY   0x02
#define OLED_COLOR_WHITE        0x03  /*!< Pixel is set. Color depends on LCD */
#endif

typedef uint8_t OLED_COLOR_t;

#define OLED_BUFFERSIZE (OLED_WIDTH * OLED_HEIGHT) / 8 * OLED_GRAYSCALE_LEVELS

#define OLED_ROTATE_0       0x01
#define OLED_ROTATE_90      0x02
#define OLED_ROTATE_180     0x03
#define OLED_ROTATE_270     0x04
#define OLED_ROTATE         OLED_ROTATE_90 //for no square displays only 0 and 180 rotations possible, yet

void OLED_Init();
void OLED_SetContrast(uint8_t contrast);
void OLED_UpdateScreen();
void OLED_Fill(OLED_COLOR_t color);
void OLED_DrawPixel(uint16_t x, uint16_t y, OLED_COLOR_t color);
unsigned char OLED_PrintXY(uint16_t x, uint16_t y, DrUM_FontDef_t* Font, OLED_COLOR_t text_color, OLED_COLOR_t back_color, uint8_t enable_background, const char* format, ...);
#endif /* DRUM_OLED_H_ */
