/* USER CODE BEGIN */
/**
* original author:  Tilen Majerle<tilen@majerle.eu>
* modification for SSD1327: Uros Mali, 2022

----------------------------------------------------------------------
Copyright (C) Alexander Lutsai, 2016
Copyright (C) Tilen Majerle, 2015

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
----------------------------------------------------------------------
*/
#include "ssd1327.h"
//#include "i2c_hal.h"
#include "cyhal_i2c.h"
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include "drum_oled.h"
#include "app_task.h"
#include "FreeRTOS.h"
#include "task.h"

extern cyhal_i2c_t i2c_master_obj;

#define I2C_RESULT_OK		0
#define I2C_RESULT_ERROR	1

/* Write command */
#define SSD1327_WRITECOMMAND(command)      SSD1327_I2C_Write(SSD1327_I2C_ADDR, 0x00, (command))
/* Write data */
#define SSD1327_WRITEDATA(data)            SSD1327_I2C_Write(SSD1327_I2C_ADDR, 0x40, (data))
/* Absolute value */
#define ABS(x)   ((x) > 0 ? (x) : -(x))


typedef uint32_t status_t;


//void HAL_Delay(int delay){
//    tx_thread_sleep(delay / 10);
//}

uint8_t SSD1327_Init(void) {
  /* Init I2C */

  /* A little delay for the SSD1327 initialization */
  //cyhal_system_delay_ms(50);
  vTaskDelay(pdMS_TO_TICKS(50));
  
  /* Init LCD */
  SSD1327_WRITECOMMAND(SSD1327_DISPLAY_OFF); //display off

  SSD1327_WRITECOMMAND(SSD1327_SET_MULTIPLEX); //--set multiplex ratio(1 to 64)
  SSD1327_WRITECOMMAND(SSD1327_HEIGHT - 1); //
  //SSD1327_WRITECOMMAND(0x5F);

  SSD1327_WRITECOMMAND(SSD1327_SET_DISPLAY_START_LINE); //--set start line address
  SSD1327_WRITECOMMAND(0x00); //

  SSD1327_WRITECOMMAND(SSD1327_SET_DISPLAY_OFFSET); //-set display offset
  SSD1327_WRITECOMMAND(0x00);
  
  SSD1327_WRITECOMMAND(SSD1327_SET_REMAP);
  SSD1327_WRITECOMMAND(0x51);

  SSD1327_SetContrast(0x7F);

  SSD1327_WRITECOMMAND(SSD1327_SET_PHASE_LENGTH);
  SSD1327_WRITECOMMAND(0x22);

  SSD1327_WRITECOMMAND(SSD1327_SET_FRONT_CLOCK_DIVIDER_OSCILLATOR_FREQUENCY);
  SSD1327_WRITECOMMAND(0x50);

  SSD1327_WRITECOMMAND(SSD1327_SELECT_DEFAULT_LINEAR_GRAYSCALE_TABLE);

  SSD1327_WRITECOMMAND(SSD1327_SET_PRECHARGE_VOLTAGE);
  SSD1327_WRITECOMMAND(0x10);

  SSD1327_WRITECOMMAND(SSD1327_SET_SET_VCOM_VOLTAGE);
  SSD1327_WRITECOMMAND(0x05);

  SSD1327_WRITECOMMAND(SSD1327_SET_SECOND_PRECHARGE_PERIOD);
  SSD1327_WRITECOMMAND(0x0A);

  SSD1327_WRITECOMMAND(SSD1327_FUNCTION_SELECTION_B);
  SSD1327_WRITECOMMAND(0x62);

  SSD1327_WRITECOMMAND(SSD1327_SET_COLUMN_ADDRESS);
  SSD1327_WRITECOMMAND(0x00);
  SSD1327_WRITECOMMAND(0x3F);

  SSD1327_WRITECOMMAND(SSD1327_SET_ROW_ADDRESS);
  SSD1327_WRITECOMMAND(0x00);
  SSD1327_WRITECOMMAND(0x7F);

  SSD1327_WRITECOMMAND(SSD1327_NORMAL_DISPLAY);  // Set Normal Display

  SSD1327_WRITECOMMAND(SSD1327_DISPLAY_ALL_ON_RESUME);  // Entire Display ON

  SSD1327_WRITECOMMAND(SSD1327_DEACTIVATE_SCROLL);

  SSD1327_WRITECOMMAND(SSD1327_DISPLAY_ON);
  
  /* Return OK */
  return 0;
}

#define I2C_MAX_BUFFER_LENGTH   32 //WIDTH bust be multiple of the buffer length
//void SSD1327_UpdateScreenFull(void) {
//	uint8_t m;
//	uint8_t n = 0;
//
//	// Writing one row at once
//	for (m = 0; m < (SSD1327_HEIGHT); m++) {
////        n = 0;
////
////	    while(n < SSD1327_WIDTH){
////	        SSD1327_WRITECOMMAND(SSD1327_SET_ROW_ADDRESS);
////	        SSD1327_WRITECOMMAND(m);
////	        SSD1327_WRITECOMMAND(m+1);
////
////	        SSD1327_WRITECOMMAND(SSD1327_SET_COLUMN_ADDRESS);
////            SSD1327_WRITECOMMAND(n);
////            SSD1327_WRITECOMMAND(n + I2C_MAX_BUFFER_LENGTH - 1);
////
////            SSD1327_I2C_WriteMulti(SSD1327_I2C_ADDR, 0x40, &SSD1327_buffer[SSD1327_WIDTH / 2 * m + n], I2C_MAX_BUFFER_LENGTH);
////            n += I2C_MAX_BUFFER_LENGTH;
////
////	    }
//        SSD1327_WRITECOMMAND(SSD1327_SET_COLUMN_ADDRESS);
//        SSD1327_WRITECOMMAND(0x00);
//        SSD1327_WRITECOMMAND(0x3F);
//
//        SSD1327_WRITECOMMAND(SSD1327_SET_ROW_ADDRESS);
//        SSD1327_WRITECOMMAND(m);
//        SSD1327_WRITECOMMAND(m+1);
//
//        /* Write multi data */
//        SSD1327_I2C_WriteMulti(SSD1327_I2C_ADDR, 0x40, &SSD1327_buffer[SSD1327_WIDTH / 2 * m], SSD1327_WIDTH / 2);
//
//
////        SSD1327_WRITECOMMAND(SSD1327_SET_ROW_ADDRESS);
////        SSD1327_WRITECOMMAND(m);
////        SSD1327_WRITECOMMAND(m+1);
////
////        SSD1327_WRITECOMMAND(SSD1327_SET_COLUMN_ADDRESS);
////        SSD1327_WRITECOMMAND(0x00);
////        SSD1327_WRITECOMMAND(0x1F);
////
////
////        /* Write multi data */
////        SSD1327_I2C_WriteMulti(SSD1327_I2C_ADDR, 0x40, &SSD1327_buffer[SSD1327_WIDTH / 2 * m], SSD1327_WIDTH / 4);
////        SSD1327_WRITECOMMAND(SSD1327_SET_ROW_ADDRESS);
////        SSD1327_WRITECOMMAND(m);
////        SSD1327_WRITECOMMAND(m+1);
////
////        SSD1327_WRITECOMMAND(SSD1327_SET_COLUMN_ADDRESS);
////        SSD1327_WRITECOMMAND(0x20);
////        SSD1327_WRITECOMMAND(0x3F);
////
////        /* Write multi data */
////        SSD1327_I2C_WriteMulti(SSD1327_I2C_ADDR, 0x40, &SSD1327_buffer[SSD1327_WIDTH / 2 * m + 0x20], SSD1327_WIDTH / 4);
////
//
//	}
//}

void SSD1327_UpdateScreen(OLED_t UpdateOLED) {
    uint8_t m;
    #if (OLED_GRAYSCALE_LEVELS == OLED_GRAYSCALE_LEVELS_4)
    uint8_t exp_buffer[64];
    uint8_t n;
    uint8_t cur_set;
    #endif



//	// Writing one row at once
//  for (m = 0; m < (SSD1327_HEIGHT); m++) {
//	  SSD1327_WRITECOMMAND(SSD1327_SET_COLUMN_ADDRESS);
//	  SSD1327_WRITECOMMAND(0x00);
//	  SSD1327_WRITECOMMAND(0x3F);
//
//	  SSD1327_WRITECOMMAND(SSD1327_SET_ROW_ADDRESS);
//	  SSD1327_WRITECOMMAND(m);
//	  SSD1327_WRITECOMMAND(m+1);
//
//    /* Write multi data */
//    SSD1327_I2C_WriteMulti(SSD1327_I2C_ADDR, 0x40, &SSD1327_buffer[SSD1327_WIDTH / 2 * m], SSD1327_WIDTH / 2);
//  }


    // update prev and next pixel
    //if(SSD1327.MinUpdatedX > 0) SSD1327.MinUpdatedX--;
    if(UpdateOLED.MaxUpdatedX < (SSD1327_WIDTH - 2)) UpdateOLED.MaxUpdatedX+=2;
    // have to take care of align bytes. Each byte represents 2 pixels
    //if(UpdateOLED.MaxUpdatedY < (SSD1327_HEIGHT - 1)) UpdateOLED.MaxUpdatedY++;
    if(UpdateOLED.MaxUpdatedY < (SSD1327_HEIGHT)) UpdateOLED.MaxUpdatedY++;

    UpdateOLED.MinUpdatedX = UpdateOLED.MinUpdatedX & 0xFFFE;
    UpdateOLED.MaxUpdatedX = UpdateOLED.MaxUpdatedX & 0xFFFE;
	if(UpdateOLED.MinUpdatedY == UpdateOLED.MaxUpdatedY){
		if(UpdateOLED.MinUpdatedY > 0){
		    UpdateOLED.MinUpdatedY--;
		} else {
		    UpdateOLED.MaxUpdatedY++;
		}
	}

	// Writing one row at once
	for (m = UpdateOLED.MinUpdatedY; m < (UpdateOLED.MaxUpdatedY); m++) {
//		SSD1327_WRITECOMMAND(SSD1327_SET_COLUMN_ADDRESS);
//		SSD1327_WRITECOMMAND(SSD1327.MinUpdatedX / 2);
//		//SSD1327_WRITECOMMAND(0x3F);
//		SSD1327_WRITECOMMAND(SSD1327.MaxUpdatedX / 2 + 1);

		SSD1327_WRITECOMMAND(SSD1327_SET_ROW_ADDRESS);
		SSD1327_WRITECOMMAND(m);
		SSD1327_WRITECOMMAND(m+1);


		if((UpdateOLED.MaxUpdatedX == UpdateOLED.MinUpdatedX)){
			//update at least one (two) pixels)
			SSD1327_WRITECOMMAND(SSD1327_SET_COLUMN_ADDRESS);
			SSD1327_WRITECOMMAND(UpdateOLED.MinUpdatedX / 2);
			//SSD1327_WRITECOMMAND(0x3F);
			SSD1327_WRITECOMMAND(UpdateOLED.MaxUpdatedX / 2 + 1);

            #if (OLED_GRAYSCALE_LEVELS == OLED_GRAYSCALE_LEVELS_4)
			//we only need to compose one byte
            cur_set = UpdateOLED.Buffer[SSD1327_WIDTH / 4 * m + UpdateOLED.MinUpdatedX / 4];
            exp_buffer[0] = (cur_set & 0xC0) | ((cur_set & 0x30) >> 4);
			SSD1327_I2C_WriteMulti(SSD1327_I2C_ADDR, 0x40, exp_buffer, 1);
            #endif

            #if (OLED_GRAYSCALE_LEVELS == OLED_GRAYSCALE_LEVELS_16)
            SSD1327_I2C_WriteMulti(SSD1327_I2C_ADDR, 0x40, &UpdateOLED.Buffer[SSD1327_WIDTH / 2 * m + UpdateOLED.MinUpdatedX / 2 ], 1);
            //SSD1327_I2C_WriteMulti(SSD1327_I2C_ADDR, 0x40, UpdateOLED.Buffer[SSD1327_WIDTH / 2 * m + UpdateOLED.MinUpdatedX / 2], 1);
            #endif


		} else{
			/* Write multi data */
			SSD1327_WRITECOMMAND(SSD1327_SET_COLUMN_ADDRESS);
			SSD1327_WRITECOMMAND(UpdateOLED.MinUpdatedX / 2);
			//SSD1327_WRITECOMMAND(0x3F);
			SSD1327_WRITECOMMAND(UpdateOLED.MaxUpdatedX / 2);

			#if (OLED_GRAYSCALE_LEVELS == OLED_GRAYSCALE_LEVELS_4)
			for(n = 0; n < (UpdateOLED.MaxUpdatedX - UpdateOLED.MinUpdatedX) / 4 + 1; n++){
                cur_set = UpdateOLED.Buffer[SSD1327_WIDTH / 4 * m + UpdateOLED.MinUpdatedX / 4 + n ];
                exp_buffer[2*n] = (cur_set & 0xC0) | ((cur_set & 0x30) >> 2);
                exp_buffer[2*n + 1] = ((cur_set & 0x0C) << 4) | ((cur_set & 0x03) << 2);
            }
            SSD1327_I2C_WriteMulti(SSD1327_I2C_ADDR, 0x40, exp_buffer, (UpdateOLED.MaxUpdatedX - UpdateOLED.MinUpdatedX) / 2 + 1);
            #endif

            #if (OLED_GRAYSCALE_LEVELS == OLED_GRAYSCALE_LEVELS_16)
			SSD1327_I2C_WriteMulti(SSD1327_I2C_ADDR, 0x40, &UpdateOLED.Buffer[SSD1327_WIDTH / 2 * m + UpdateOLED.MinUpdatedX / 2 ], (UpdateOLED.MaxUpdatedX - UpdateOLED.MinUpdatedX) / 2 + 1);
			//SSD1327_I2C_WriteMulti(SSD1327_I2C_ADDR, 0x40, UpdateOLED.Buffer + (SSD1327_WIDTH / 2 * m + UpdateOLED.MinUpdatedX / 2 ), (UpdateOLED.MaxUpdatedX - UpdateOLED.MinUpdatedX) / 2);
            #endif
		}

}

  //partial update reset range
	UpdateOLED.MinUpdatedX = SSD1327_WIDTH - 1;
	UpdateOLED.MaxUpdatedX = 0;
	UpdateOLED.MinUpdatedY = SSD1327_HEIGHT - 1;
	UpdateOLED.MaxUpdatedY = 0;
}

void SSD1327_ToggleInvert(OLED_t UpdateOLED) {
//  uint16_t i;
  
  /* Toggle invert */
    UpdateOLED.Inverted = !UpdateOLED.Inverted;
  
  /* Do memory toggle */
//  for (i = 0; i < sizeof(SSD1327_Buffer); i++) {
//    SSD1327_Buffer[i] = ~SSD1327_Buffer[i];
//  }
  SSD1327_WRITECOMMAND(UpdateOLED.Inverted ? SSD1327_INVERT_DISPLAY : SSD1327_NORMAL_DISPLAY);
}

void SSD1327_RotateDisplay(uint8_t rotate)
{
	if(rotate > 1) rotate = 1;

	SSD1327_WRITECOMMAND(0xA0 | (0x01 & rotate));  // Set Segment Re-Map Default
							// 0xA0 (0x00) => column Address 0 mapped to 127
                			// 0xA1 (0x01) => Column Address 127 mapped to 0

	SSD1327_WRITECOMMAND(0xC0 | (0x08 & (rotate<<3)));  // Set COM Output Scan Direction
							// 0xC0	(0x00) => normal mode (RESET) Scan from COM0 to COM[N-1];Where N is the Multiplex ratio.
							// 0xC8	(0xC8) => remapped mode. Scan from COM[N-1] to COM0;;Where N is the Multiplex ratio.
}


void SSD1327_ON(void) {
//  SSD1327_WRITECOMMAND(0x8D);
//  SSD1327_WRITECOMMAND(0x14);
  SSD1327_WRITECOMMAND(SSD1327_DISPLAY_ON);
}
void SSD1327_OFF(void) {
//  SSD1327_WRITECOMMAND(0x8D);
//  SSD1327_WRITECOMMAND(0x10);
  SSD1327_WRITECOMMAND(SSD1327_DISPLAY_OFF);
}

void SSD1327_SetContrast(uint8_t contrast)
{
	SSD1327_WRITECOMMAND(SSD1327_SET_CONTRAST_CURRENT);	// Set Contrast Control
	SSD1327_WRITECOMMAND(contrast);
}

uint8_t SSD1327_I2C_WriteMulti(uint8_t address, uint8_t reg, uint8_t* data, uint16_t count) {
	uint8_t buff[128];

	buff[0] = reg;
	memcpy(&buff[1], data, count);

	//tx_mutex_get(&mutex_i2c,TX_NO_WAIT);

	/* Try to transmit via I2C */

	cy_rslt_t result = cyhal_i2c_master_write( &i2c_master_obj, address, buff, count + 1, 1000, true);
    if (result != CY_RSLT_SUCCESS)
    {
        return I2C_RESULT_ERROR;
    }
	return I2C_RESULT_OK;
}

uint8_t SSD1327_I2C_Write(uint8_t address, uint8_t reg, uint8_t data) {
	uint8_t buff[2];

	/* Format array to send */
	buff[0] = reg;
	buff[1] = data;

	//tx_mutex_get(&mutex_i2c,TX_NO_WAIT);

	/* Try to transmit via I2C */
	cy_rslt_t result = cyhal_i2c_master_write( &i2c_master_obj, address, (uint8_t *)buff, 2 , 1000, true);
    if (result != CY_RSLT_SUCCESS)
    {
        return I2C_RESULT_ERROR;
    }
	return I2C_RESULT_OK;
}
/* USER CODE END */
