/* USER CODE BEGIN */
/**
 * original author:  Tilen Majerle<tilen@majerle.eu>
 * modification for SSD1327: Uros Mali, 2022

   ----------------------------------------------------------------------
   	Copyright (C) Alexander Lutsai, 2016
    Copyright (C) Tilen Majerle, 2015

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
   ----------------------------------------------------------------------
 */
#ifndef SSD1327_H
#define SSD1327_H 100

/* C++ detection */
#ifdef __cplusplus
extern C {
#endif

/**
 * This SSD1327 LCD uses I2C for communication
 *
 * Library features functions for drawing lines, rectangles and circles.
 *
 * It also allows you to draw texts and characters using appropriate functions provided in library.

 */
#include "stdint.h"
#include "fonts.h"

#include "stdlib.h"
#include "string.h"

#include "drum_oled.h"

/* I2C address */
#ifndef SSD1327_I2C_ADDR
#define SSD1327_I2C_ADDR         0x3C//0x78
//#define SSD1327_I2C_ADDR       0x7A
#endif

/* SSD1327 settings */
/* SSD1327 width in pixels */
#ifndef SSD1327_WIDTH
#define SSD1327_WIDTH            OLED_WIDTH
#endif
/* SSD1327 LCD height in pixels */
#ifndef SSD1327_HEIGHT
#define SSD1327_HEIGHT           OLED_HEIGHT
#endif

// Commands
#define SSD1327_SET_COLUMN_ADDRESS			0x15
#define SSD1327_SET_ROW_ADDRESS				0x75
#define SSD1327_SET_CONTRAST_CURRENT 		0x81
#define SSD1327_NOP							0x84
#define SSD1327_SET_REMAP					0xA0
#define SSD1327_SET_DISPLAY_START_LINE		0xA1
#define SSD1327_SET_DISPLAY_OFFSET			0xA2
#define SSD1327_DISPLAY_ALL_ON_RESUME 		0xA4
#define SSD1327_DISPLAY_ALL_ON 				0xA5
#define SSD1327_NORMAL_DISPLAY 				0xA6
#define SSD1327_INVERT_DISPLAY 				0xA7
#define SSD1327_SET_MULTIPLEX				0xA8
#define SSD1327_FUNCTION_SELECTION_A		0xAB
#define SSD1327_DISPLAY_OFF					0xAE
#define SSD1327_DISPLAY_ON					0xAF
#define SSD1327_SET_PHASE_LENGTH			0xB1
#define SSD1327_SET_FRONT_CLOCK_DIVIDER_OSCILLATOR_FREQUENCY 		0xB3
#define SSD1327_SET_GPIO					0xB5
#define SSD1327_SET_SECOND_PRECHARGE_PERIOD	0xB6
#define SSD1327_SET_GRAYSCALE_TABLE			0xB8
#define SSD1327_SELECT_DEFAULT_LINEAR_GRAYSCALE_TABLE	0xB9
#define SSD1327_SET_PRECHARGE_VOLTAGE		0xBC
#define SSD1327_SET_SET_VCOM_VOLTAGE		0xBE
#define SSD1327_FUNCTION_SELECTION_B		0xD5
#define SSD1327_SET_COMMAND_LOCK			0xFD

// Scrolling
#define SSD1327_ACTIVATE_SCROLL			0x2F
#define SSD1327_DEACTIVATE_SCROLL		0x2E
#define SSD1327_RIGHT_HORIZONTAL_SCROLL	0x26
#define SSD1327_LEFT_HORIZONTAL_SCROLL	0x27


/**
 * @brief  Initializes SSD1327 LCD
 * @param  None
 * @retval Initialization status:
 *           - 0: LCD was not detected on I2C port
 *           - > 0: LCD initialized OK and ready to use
 */
uint8_t SSD1327_Init(void);

/** 
 * @brief  Updates buffer from internal RAM to LCD
 * @note   This function must be called each time you do some changes to LCD, to update buffer from RAM to LCD
 * @param  None
 * @retval None
 */
void SSD1327_UpdateScreen(OLED_t UpdateOLED);

void SSD1327_UpdateScreenFull(void);

/**
 * @brief  Toggles pixels invertion inside internal RAM
 * @note   @ref SSD1327_UpdateScreen() must be called after that in order to see updated LCD screen
 * @param  None
 * @retval None
 */
void SSD1327_ToggleInvert(OLED_t UpdateOLED);

/**
 * @brief  Set contract
 * @note
 * @param  contrast: contrast value
 * @retval None
 */
void SSD1327_SetContrast(uint8_t contrast);

#ifndef SSD1327_I2C_TIMEOUT
#define SSD1327_I2C_TIMEOUT					20000
#endif

/**
 * @brief  Initializes SSD1327 LCD
 * @param  None
 * @retval Initialization status:
 *           - 0: LCD was not detected on I2C port
 *           - > 0: LCD initialized OK and ready to use
 */
//void SSD1327_I2C_Init(void);

/**
 * @brief  Writes single byte to slave
 * @param  *I2Cx: I2C used
 * @param  address: 7 bit slave address, left aligned, bits 7:1 are used, LSB bit is not used
 * @param  reg: register to write to
 * @param  data: data to be written
 * @retval None
 */
uint8_t SSD1327_I2C_Write(uint8_t address, uint8_t reg, uint8_t data);

/**
 * @brief  Writes multi bytes to slave
 * @param  *I2Cx: I2C used
 * @param  address: 7 bit slave address, left aligned, bits 7:1 are used, LSB bit is not used
 * @param  reg: register to write to
 * @param  *data: pointer to data array to write it to slave
 * @param  count: how many bytes will be written
 * @retval None
 */
uint8_t SSD1327_I2C_WriteMulti(uint8_t address, uint8_t reg, uint8_t *data, uint16_t count);

void SSD1327_ON(void);
void SSD1327_OFF(void);

uint8_t SSD1327_Init(void);
void SSD1327_SetContrast(uint8_t contrast);
void SSD1327_UpdateScreen(OLED_t UpdateOLED);


/* C++ detection */
#ifdef __cplusplus
}
#endif

#endif
/* USER CODE END */
