/*
 * input_task.c
 *
 *  Created on: 08.04.2023.
 *      Author: maliu
 */

/* FreeRTOS header files */
#include "FreeRTOS.h"
#include "task.h"

#include "cyhal.h"
#include "cybsp.h"

#include "input_task.h"
#include "cy_retarget_io.h"

#include "wifi_config.h"
#include "app_config.h"

#include "iotc_eeprom.h"

#include "cy_rgb_led.h"
#include "ebv_logs.h"

uint8_t input_data;

#define MAX_INPUT_BUFFER 	128
uint8_t input_bufer[MAX_INPUT_BUFFER + 1];
uint16_t numOfBytesRead, current_byte = 0;

extern char IOTCONNECT_DUID[31];


bool g_event_btn1_occured = false, g_event_btn2_occured = false;
extern uint8_t uLED_RGB_Brightness, uLED_RGB_Brightness_Last;
extern uint32_t uLED_RGB_Color;

extern bool bManualACK;

static const char *TAG = "input_task";

static void MyGPIO_ISR(void)
{
	if ( Cy_GPIO_GetInterruptStatus(CYBSP_USER_BTN1_PORT, CYBSP_USER_BTN1_NUM) ){

		Cy_GPIO_ClearInterrupt(CYBSP_USER_BTN1_PORT, CYBSP_USER_BTN1_NUM);

		g_event_btn1_occured = true;
	}
	if ( Cy_GPIO_GetInterruptStatus(CYBSP_USER_BTN2_PORT, CYBSP_USER_BTN2_NUM) ){

		Cy_GPIO_ClearInterrupt(CYBSP_USER_BTN2_PORT, CYBSP_USER_BTN2_NUM);

		g_event_btn2_occured = true;
	}
}


void input_task(void *pvParameters) {
	//temporary arrays for WiFi credentials
	char sSSID_TMP[WIFI_SSID_MAX_LENGTH];
	char sPWD_TMP[WIFI_PWD_MAX_LENGTH];
	char sLAT_TMP[LAT_LON_MAX_LENGTH];
	char sLON_TMP[LAT_LON_MAX_LENGTH];
	int nTimeoutTick = 0;

	//set tmp credentials to align with current ones
	memcpy(sSSID_TMP, wifi_credentials.sSSID, strlen(wifi_credentials.sSSID)+1);
	memcpy(sPWD_TMP, wifi_credentials.sPWD, strlen(wifi_credentials.sPWD)+1);
	memcpy(sLAT_TMP, lat_lon_data.sLatitude, strlen(lat_lon_data.sLatitude)+1);
	memcpy(sLON_TMP, lat_lon_data.sLongitude, strlen(lat_lon_data.sLongitude)+1);

	//configure buttons and IRQs
    cy_stc_sysint_t irqCfg = {CYBSP_USER_BTN1_IRQ, 7};
    Cy_SysInt_Init(&irqCfg, MyGPIO_ISR);
    NVIC_EnableIRQ(CYBSP_USER_BTN1_IRQ);

    cy_stc_sysint_t irqCfg2 = {CYBSP_USER_BTN2_IRQ, 7};
	Cy_SysInt_Init(&irqCfg2, MyGPIO_ISR);
	NVIC_EnableIRQ(CYBSP_USER_BTN2_IRQ);

	while(1)
	{
		cyhal_uart_getc(&cy_retarget_io_uart_obj, &input_data, 1);
		if((input_data == 'X') | (input_data == 'x')){
			//vTaskSuspendAll();
			//vTaskResume(inputTaskHandle);
			//vTaskSuspend(sensorTaskHandle);
			//vTaskSuspend(wifiConnTaskHandle);

			ebv_logs_level_t ebv_logs_level_tmp;
			ebv_logs_level_tmp = ebv_logs_level; //Store current log level
			ebv_logs_level = EBV_LOGS_ERROR;
			nTimeoutTick = 0;
//			printf("\x1b[2J\x1b[;H");
//			printf("===============================================================\n");
//			printf("Enter credential data using following input\n");
//			printf("---------------------------------------------------------------\n");
//			printf("H followed by 'ENTER' to show this menu\n");
//			printf("S{WiFi SSID} followed by 'ENTER' - SET WiFi SSID\n");
//			printf("P{WiFi Password} followed by 'ENTER' - SET WiFi Password\n");
//			printf("U followed by 'ENTER' to display UID\n");
//			printf("Q followed by 'ENTER' to display QR code web link\n");
//			printf("D followed by 'ENTER' to use default values\n");
//			printf("R followed by 'ENTER' to review credentials\n");
//			printf("W followed by 'ENTER' to write credentials to EEPROM and EXIT\n");
//			printf("E followed by 'ENTER' to EXIT!\n");
//			printf("===============================================================\n\n");

			EBV_LOGS_CLEAR_SCREEEN();
			EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "===============================================================");
			EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "Enter credential data using following input");
			EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "---------------------------------------------------------------");
			EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "H followed by 'ENTER' to show this menu");
			EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "S{WiFi SSID} followed by 'ENTER' - SET WiFi SSID");
			EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "P{WiFi Password} followed by 'ENTER' - SET WiFi Password");
			EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "U followed by 'ENTER' to display UID");
			EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "Q followed by 'ENTER' to display QR code web link");
			EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "D followed by 'ENTER' to use default values");
			EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "R followed by 'ENTER' to review credentials");
			EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "W followed by 'ENTER' to write credentials to EEPROM and EXIT");
			EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "L{x} followed by 'ENTER' to set printf debug level {x} 2-8");
			EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "M{lat,lon} followed by 'ENTER' to set latitude/longitude");
			EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "E followed by 'ENTER' to EXIT!");
			EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "===============================================================\n");

			current_byte = 0;
			while(1u){ //do until break
				input_data = 0;
				cyhal_uart_getc(&cy_retarget_io_uart_obj, &input_data, 1);
				if(input_data > 0){
					nTimeoutTick = 0;
					input_bufer[current_byte] = input_data;
					if((input_data == '\r') | (input_data == '\n')){ //we have enter
						input_bufer[current_byte] = 0;

						if(input_bufer[0]=='H'){
//							printf("===============================================================\n");
//							printf("Enter credential data using following input\n");
//							printf("---------------------------------------------------------------\n");
//							printf("H followed by 'ENTER' to show this menu\n");
//							printf("S{WiFi SSID} followed by 'ENTER' - SET WiFi SSID\n");
//							printf("P{WiFi Password} followed by 'ENTER' - SET WiFi Password\n");
//							printf("U followed by 'ENTER' to display UID\n");
//							printf("Q followed by 'ENTER' to display QR code web link\n");
//							printf("D followed by 'ENTER' to use default values\n");
//							printf("R followed by 'ENTER' to review credentials\n");
//							printf("W followed by 'ENTER' to write credentials to EEPROM and EXIT\n");
//							printf("E followed by 'ENTER' to EXIT!\n");
//							printf("===============================================================\n\n");

							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "===============================================================");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "Enter credential data using following input");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "---------------------------------------------------------------");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "H followed by 'ENTER' to show this menu");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "S{WiFi SSID} followed by 'ENTER' - SET WiFi SSID");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "P{WiFi Password} followed by 'ENTER' - SET WiFi Password");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "U followed by 'ENTER' to display UID");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "Q followed by 'ENTER' to display QR code web link");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "D followed by 'ENTER' to use default values");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "R followed by 'ENTER' to review credentials");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "W followed by 'ENTER' to write credentials to EEPROM and EXIT");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "L{x} followed by 'ENTER' to set printf debug level {x} 2-8");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "M{lat,lon} followed by 'ENTER' to set latitude/longitude");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "E followed by 'ENTER' to EXIT!");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "===============================================================\n");
						}

						if(input_bufer[0]=='S'){
							memcpy(sSSID_TMP, input_bufer+1, current_byte);
							//printf("\nOK.\n");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "\nOK");
						}
						if(input_bufer[0]=='P'){
							memcpy(sPWD_TMP, input_bufer+1, current_byte);
							//printf("\nOK.\n");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "\nOK");
						}
						if(input_bufer[0]=='U'){
							//printf(IOTCONNECT_DUID);
							//printf("\nOK.\n");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "\n%s", IOTCONNECT_DUID);
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "OK");
						}
						if(input_bufer[0]=='Q'){
							//printf("https://chart.googleapis.com/chart?chs=200x200&cht=qr&chl=%.31s&choe=UTF-8%%22\n\n", IOTCONNECT_DUID);
							//printf("\nOK.\n");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "\nhttps://chart.googleapis.com/chart?chs=200x200&cht=qr&chl=%.31s&choe=UTF-8%%22\n", IOTCONNECT_DUID);
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "OK");
						}
						if(input_bufer[0]=='D'){
							memcpy(sSSID_TMP, WIFI_SSID, strlen(WIFI_SSID)+1);
							memcpy(sPWD_TMP, WIFI_PASSWORD, strlen(WIFI_PASSWORD)+1);
							//memcpy(wifi_credentials.sIOTCONNECT_DUID, IOTCONNECT_DUID, strlen(IOTCONNECT_DUID));
							memcpy(sLAT_TMP, LATITUDE, strlen(LATITUDE)+1);
							memcpy(sLON_TMP, LONGITUDE, strlen(LONGITUDE)+1);

							//printf("\nOK.\n");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "\nUsing default values... OK");
						}

						if(input_bufer[0]=='R'){
							//printf("\nWiFi SSID:                        %s\n", sSSID_TMP);
							//printf("WiFi Password:                    %s\n", sPWD_TMP);
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "\nWiFi SSID:                        %s", sSSID_TMP);
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "WiFi Password:                    %s", sPWD_TMP);
							//printf("WiFi IoTConnect Unique Device ID: %s\n", wifi_credentials.sIOTCONNECT_DUID);
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "EBV LOGs level:                   %d", ebv_logs_level_tmp);
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "Your position:");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "https://www.google.com/maps/search/?api=1&query=%s,%s", sLAT_TMP, sLON_TMP);
						}
						if(input_bufer[0]=='W'){

							//printf("\nSaving data...");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "\nSaving data...");
							eeprom_data.ucEEPROM_DataValid = EEPROM_DATA_VALID_ID;
							memcpy(wifi_credentials.sSSID, sSSID_TMP, strlen(sSSID_TMP)+1);
							memcpy(wifi_credentials.sPWD, sPWD_TMP, strlen(sPWD_TMP)+1);
							memcpy(lat_lon_data.sLatitude, sLAT_TMP, strlen(sLAT_TMP)+1);
							memcpy(lat_lon_data.sLongitude, sLON_TMP, strlen(sLON_TMP)+1);

							memcpy(eeprom_data.wifi_credentials_stored.sSSID, sSSID_TMP, strlen(sSSID_TMP)+1);
							memcpy(eeprom_data.wifi_credentials_stored.sPWD, sPWD_TMP, strlen(sPWD_TMP)+1);
							memcpy(eeprom_data.lat_lon_data_stored.sLatitude, sLAT_TMP, strlen(sLAT_TMP)+1);
							memcpy(eeprom_data.lat_lon_data_stored.sLongitude, sLON_TMP, strlen(sLON_TMP)+1);

							vTaskSuspendAll();
							eeprom_result = Cy_Em_EEPROM_Write(0u, &eeprom_data, sizeof(eeprom_data_t), &eepromContext);
							xTaskResumeAll();
							if (CY_EM_EEPROM_SUCCESS != eeprom_result)
							{
								//printf("FAILED!!!\n");//Error check action
								EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "FAILED!!!");
							}
							//printf("OK.\n");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "OK. Exiting menu...");
							break;
						}
						if(input_bufer[0]=='L'){
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "\n");
							switch (input_bufer[1]){

							case '2':
								ebv_logs_level_tmp = EBV_LOGS_ERROR;
								EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "Setting print log message level for Critical error, caller function crashes the application");
								break;
							case '3':
								ebv_logs_level_tmp = EBV_LOGS_WARNING;
								EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "Setting print log message level for Error recovered");
								break;
							case '4':
								ebv_logs_level_tmp = EBV_LOGS_NOTICE;
								EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "Setting print log message level for Notice on possible error");
								break;
							case '5':
								ebv_logs_level_tmp = EBV_LOGS_INFO;
								EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "Setting print log message level for Information describing expected outcome");
								break;
							case '6':
								ebv_logs_level_tmp = EBV_LOGS_DEBUG;
								EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "Setting print log message level for Additional information useful for debugging");
								break;
							case '7':
								ebv_logs_level_tmp = EBV_LOGS_DEBUG1;
								EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "Setting print log message level for Less important information useful for debugging");
								break;
							case '8':
								ebv_logs_level_tmp = EBV_LOGS_ALL;
								EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "Setting print log message level for all levels.");
								break;
							default:
								break;
							}

							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "OK");
						}
						if(input_bufer[0]=='M'){
							unsigned char i, j;
							i = 1;
							j = 0;
							while((input_bufer[i]!=',') & (j < LAT_LON_MAX_LENGTH-1)){
								if(input_bufer[i]!=' '){
									sLAT_TMP[j] = input_bufer[i];
									j++;
								}
								i++;
							}
							sLAT_TMP[j] = 0; //string termination
							j=0;
							i++;
							while((i < current_byte) & (j < LAT_LON_MAX_LENGTH-1)){
								if(input_bufer[i]!=' '){
									sLON_TMP[j] = input_bufer[i];
									j++;
								}
								i++;
							}
							sLON_TMP[j] = 0; //string termination
							//printf("\nOK.\n");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "\nOK");
						}

						if(input_bufer[0]=='E'){
							//printf("\nEXIT no saving OK.\n");
							EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "\nEXIT & no saving... OK.");
							break;
						}
						current_byte = 0;
					} else {
						if(current_byte < MAX_INPUT_BUFFER) current_byte++;
					}
				}

				vTaskDelay(pdMS_TO_TICKS(INPUT_TASK_INTERVAL_MENU_MS));
				nTimeoutTick++;
				if(nTimeoutTick > INPUT_TASK_TIMEOUT_TICKS){
					EBV_LOGS_M(TAG, EBV_LOGS_MODULE_APP, "\nEnter credential timeout. Exiting without saving.");
					break;
				}
			}

			//xTaskResumeAll();
			//vTaskResume(sensorTaskHandle);
			//vTaskResume(wifiConnTaskHandle);

			ebv_logs_level = ebv_logs_level_tmp; //Restore log level
		}

		if (g_event_btn1_occured ){
			cyhal_gpio_toggle(CYBSP_USER_LED);
			g_event_btn1_occured = false;
			EBV_LOGS_I(TAG, EBV_LOGS_MODULE_APP, "Toggle LED1");
			bManualACK = true;
		}

		if ( g_event_btn2_occured ){
			if ( (uLED_RGB_Brightness == 0) & (uLED_RGB_Brightness_Last == 0)){ //turn on max brightness
				uLED_RGB_Brightness = CY_RGB_LED_MAX_BRIGHTNESS;
				EBV_LOGS_I(TAG, EBV_LOGS_MODULE_APP, "Turn ON RGB LED - max brightness");
			} else if (uLED_RGB_Brightness == 0) { //turn to previous brightness
				uLED_RGB_Brightness = uLED_RGB_Brightness_Last;
				EBV_LOGS_I(TAG, EBV_LOGS_MODULE_APP, "Turn ON RGB LED");
			} else {
				uLED_RGB_Brightness_Last = uLED_RGB_Brightness;
				uLED_RGB_Brightness = 0;
				EBV_LOGS_I(TAG, EBV_LOGS_MODULE_APP, "Turn OFF RGB LED");
			}

			if (uLED_RGB_Brightness == 0){
				cy_rgb_led_off();
			} else {
				cy_rgb_led_on(uLED_RGB_Color, uLED_RGB_Brightness);
			}

			g_event_btn2_occured = false;
			bManualACK = true;
		}

		vTaskDelay(pdMS_TO_TICKS(INPUT_TASK_INTERVAL_MS));
	}
}
