/*
 * optiga_trust_task.c
 *
 */

#include "cybsp.h"
#include "cyhal.h"

#include "optiga/pal/pal_os_event.h"
#include "optiga/pal/pal_i2c.h"
//#include "optiga_trust.h"
#include "optiga_trust_helpers.h"

#include "FreeRTOS.h"
#include "task.h"

#include "app_task.h"
#include "optiga_trust_task.h"

#include "drum_oled.h"
#include "drum_fonts.h"

/* Configuration file for Wi-Fi and MQTT client */
#include "wifi_config.h"
#include "app_config.h"

#include "cy_retarget_io.h"

#include "iotc_eeprom.h"

#include "wifi_connection_task.h"
#include "sensor_task.h"
#include "input_task.h"

#include "mbedtls/base64.h"

#include "ebv_logs.h"

TaskHandle_t inputTaskHandle;
TaskHandle_t sensorTaskHandle;
TaskHandle_t wifiConnTaskHandle;

/* I2C bus frequency */
#define I2C_MASTER_FREQUENCY (400000U)

/* We don't use CLIENT_CERTIFICATE memory but instead allocate a buffer and
 * populate it with teh certificate form the Secure Element */
char certificate[CERT_BUF_SIZE];

char IOTCONNECT_DUID[31];
char IOTCONNECT_CPID [120];//32
char IOTCONNECT_ENV  [120];//8

bool optigaON = true;

uint8_t read_data; /* Variable to store the received character
                    * through terminal */




extern bool use_optiga_certificate(void);

static const char *TAG = "optiga_trust_task";

/* This is a place from which we can poll the status of operation */
void vApplicationTickHook( void );


void vApplicationTickHook( void )
{
    pal_os_event_trigger_registered_callback();
}

bool use_optiga_certificate(void)
{
	uint16_t certificate_size = 0;
    /* This is the place where the certificate is initialized. This is a function
     * which will allow to read it out and populate internal mbedtls settings wit it*/
    read_certificate_from_optiga(0xe0e0, certificate, &certificate_size);

    if (certificate_size && (certificate_size < CERT_BUF_SIZE)) {
    	//printf("Your certificate is:\n%s\n", certificate);
    	EBV_LOGS_I(TAG, EBV_LOGS_MODULE_APP, "Your certificate is:");
    	printf("%s\n", certificate);
    	//EBV_LOGS_PRINTF("Your certificate is:\n%s\n", certificate);
    	return true;
    } else if (certificate_size) {
    	EBV_LOGS_E(TAG, EBV_LOGS_MODULE_APP, "Error: Certificate buffer overflow!");
    	return false;
    } else {
    	EBV_LOGS_E(TAG, EBV_LOGS_MODULE_APP, "Error: Optiga certificate read failed!");
    	return false;
    }
}

bool use_optiga_UID(void)
{
	uint16_t ID_size = 0;
    char CID[CID_BUF_SIZE];
    /* This is the place where the certificate is initialized. This is a function
     * which will allow to read it out and populate internal mbedtls settings wit it*/
    read_certificate_from_optiga(0xe0c2, CID, &ID_size);

    if (ID_size && (ID_size < CID_BUF_SIZE)) {
        printf("Your UID is: %.31s\n\n", CID+28);
        memcpy(IOTCONNECT_DUID, CID+28, 31);
        printf("Your web QR code is:\n");
        printf("https://chart.googleapis.com/chart?chs=200x200&cht=qr&chl=%.31s&choe=UTF-8%%22\n\n", IOTCONNECT_DUID);
    	return true;
    } else if (ID_size) {
    	EBV_LOGS_E(TAG, EBV_LOGS_MODULE_APP, "Error: UID buffer overflow!\n");
    	return false;
    } else {
    	EBV_LOGS_E(TAG, EBV_LOGS_MODULE_APP, "Error: Optiga UID read failed!\n");
    	return false;
    }
}

bool use_optiga_PID(void)
{
	uint16_t ID_size = 0;
	size_t  pid_b64_len = 0;
    uint8_t pid_b64_temp[120];
    char CPID[CID_BUF_SIZE];
    memset(pid_b64_temp, 0, sizeof(pid_b64_temp));
    /* This is the place where the certificate is initialized. This is a function
     * which will allow to read it out and populate internal mbedtls settings wit it*/
    read_certificate_from_optiga(0xf1d3, CPID, &ID_size);
    mbedtls_base64_decode((unsigned char *)pid_b64_temp, sizeof(pid_b64_temp), &pid_b64_len, (unsigned char *)CPID+28, ID_size-56);

    if (ID_size && (ID_size < CID_BUF_SIZE)) {
        //printf("Your CPID is:\n%.32s\n\n", CPID+28);
        printf("Your CPID is: %s\n\n", pid_b64_temp);
        memcpy(IOTCONNECT_CPID, pid_b64_temp, sizeof(pid_b64_temp));
    	return true;
    } else if (ID_size) {
    	EBV_LOGS_E(TAG, EBV_LOGS_MODULE_APP, "Error: CPID buffer overflow!\n");
    	return false;
    } else {
    	EBV_LOGS_E(TAG, EBV_LOGS_MODULE_APP, "Error: Optiga CPID read failed!\n");
    	return false;
    }
}

bool use_optiga_ENV(void)
{
	uint16_t ID_size = 0;
	size_t  env_b64_len = 0;
    uint8_t env_b64_temp[120];
    char ENV[CID_BUF_SIZE];
    memset(env_b64_temp, 0, sizeof(env_b64_temp));
    /* This is the place where the certificate is initialized. This is a function
     * which will allow to read it out and populate internal mbedtls settings wit it*/
    read_certificate_from_optiga(0xf1d2, ENV, &ID_size);
    mbedtls_base64_decode((unsigned char *)env_b64_temp, sizeof(env_b64_temp), &env_b64_len, (unsigned char *)ENV+28, ID_size-56);

    if (ID_size && (ID_size < CID_BUF_SIZE)) {
        //printf("Your ENV is:\n%.8s\n\n", ENV+28);
        printf("Your ENV is: %s\n\n", env_b64_temp);
        memcpy(IOTCONNECT_ENV, env_b64_temp, sizeof(env_b64_temp));
    	return true;
    } else if (ID_size) {
        printf("Error: ENV buffer overflow!\n");
    	return false;
    } else {
        printf("Error: Optiga ENV read failed!\n");
    	return false;
    }
}

eeprom_data_t eeprom_data;
wifi_credentials_t wifi_credentials;
lat_lon_data_t lat_lon_data;



void optiga_client_task(void *pvParameters) {

    /* Optiga initialize and read the certificate */
    optiga_trust_init();
    /* Based on the certificate read from the chip, Optiga uses a hook(optiga-trust-m\release-v3.1.4\examples\mbedtls_port\)
     * to mbedtls to generate signature and so on for TLS handshake */
    bool result = use_optiga_certificate();
    if (!result) {
    	//the called function will print the ERROR.
    	//return;
    	//printf("Error: Optiga use certificate\n");
    	EBV_LOGS_E(TAG, EBV_LOGS_MODULE_APP, "use_optiga_certificate: failed");
    	optigaON = false;
    }
    result = use_optiga_UID();
    if (!result) {
    	//the called function will print the ERROR.
    	//return;
    	//printf("Error: Optiga ID\n");
    	EBV_LOGS_E(TAG, EBV_LOGS_MODULE_APP, "use_optiga_UID: failed");
    }
    result = use_optiga_PID();
    if (!result) {
    	//the called function will print the ERROR.
    	//return;
    	//printf("Error: Optiga PID\n");
    	EBV_LOGS_E(TAG, EBV_LOGS_MODULE_APP, "use_optiga_PID: failed");
    }
    result = use_optiga_ENV();
    if (!result) {
    	//the called function will print the ERROR.
    	//return;
    	//printf("Error: Optiga ENV\n");
    	EBV_LOGS_E(TAG, EBV_LOGS_MODULE_APP, "use_optiga_ENV: failed");
    }

	OLED_Init();
//	OLED_SetContrast(15);

	OLED_PrintXY(10, 0, &IsidoraSansMedium16x16, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "%cEBV-IoT%c ", 127, 127);
	OLED_PrintXY(2, 40, &SmallBold9x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "Infineon PAS CO2");

	OLED_UpdateScreen();

    //***************************************
	//read stored values
    //***************************************
    vTaskSuspendAll();
    eeprom_result = Cy_Em_EEPROM_Init(&eepromConfig, &eepromContext);
    xTaskResumeAll();
	if (CY_EM_EEPROM_SUCCESS != eeprom_result)
	{
		//Error check action
		//printf("EEPROM Init FAILED!!!\n");
		EBV_LOGS_E(TAG, EBV_LOGS_MODULE_APP, "EEPROM Init FAILED!!!");
	}

	//printf("Reading EEPROM data...");
	EBV_LOGS_I_CONT(TAG, EBV_LOGS_MODULE_APP, "Reading EEPROM data...");
	vTaskSuspendAll();
	eeprom_result = Cy_Em_EEPROM_Read(0u, &eeprom_data, sizeof(eeprom_data_t), &eepromContext);
	xTaskResumeAll();
	if (CY_EM_EEPROM_SUCCESS != eeprom_result)
	{
		//reading EEPROM failed - copy defaults
		memcpy(wifi_credentials.sSSID, WIFI_SSID, strlen(WIFI_SSID));
		memcpy(wifi_credentials.sPWD, WIFI_PASSWORD, strlen(WIFI_PASSWORD));
		//memcpy(wifi_credentials.sIOTCONNECT_DUID, IOTCONNECT_DUID, strlen(IOTCONNECT_DUID));

		memcpy(lat_lon_data.sLatitude, LATITUDE, strlen(LATITUDE));
		memcpy(lat_lon_data.sLongitude, LONGITUDE, strlen(LONGITUDE));

		OLED_PrintXY(2, 60, &SmallBold9x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "Using DEFAULTs  ");

		//printf("FAILED!!! Using DEFAULT credentials\n");
		EBV_LOGS_E(TAG, EBV_LOGS_MODULE_APP, "FAILED!!! Using DEFAULT credentials");

	} else {
		//printf("OK.\n");
		EBV_LOGS_I_RESET(TAG, EBV_LOGS_MODULE_APP, "OK.");

		//printf("Checking valid data...");
		EBV_LOGS_I_CONT(TAG, EBV_LOGS_MODULE_APP, "Checking valid data...");
		if(eeprom_data.ucEEPROM_DataValid == EEPROM_DATA_VALID_ID){
			OLED_PrintXY(2, 60, &SmallBold9x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "Using EEPROM vals");

			memcpy(wifi_credentials.sSSID, eeprom_data.wifi_credentials_stored.sSSID, strlen(eeprom_data.wifi_credentials_stored.sSSID));
			memcpy(wifi_credentials.sPWD, eeprom_data.wifi_credentials_stored.sPWD, strlen(eeprom_data.wifi_credentials_stored.sPWD));

			memcpy(lat_lon_data.sLatitude, eeprom_data.lat_lon_data_stored.sLatitude, strlen(eeprom_data.lat_lon_data_stored.sLatitude));
			memcpy(lat_lon_data.sLongitude, eeprom_data.lat_lon_data_stored.sLongitude, strlen(eeprom_data.lat_lon_data_stored.sLongitude));

			//printf("OK. Using EEPROM values!\n");
			EBV_LOGS_I_RESET(TAG, EBV_LOGS_MODULE_APP, "OK. Using EEPROM values!");

		} else {
			memcpy(wifi_credentials.sSSID, WIFI_SSID, strlen(WIFI_SSID));
			memcpy(wifi_credentials.sPWD, WIFI_PASSWORD, strlen(WIFI_PASSWORD));
			//memcpy(wifi_credentials.sIOTCONNECT_DUID, IOTCONNECT_DUID, strlen(IOTCONNECT_DUID));

			memcpy(lat_lon_data.sLatitude, LATITUDE, strlen(LATITUDE));
			memcpy(lat_lon_data.sLongitude, LONGITUDE, strlen(LONGITUDE));

			OLED_PrintXY(2, 60, &SmallBold9x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "Using DEFAULTs");

			//printf("not available! Using DEFAULT values\n");
			EBV_LOGS_I_RESET(TAG, EBV_LOGS_MODULE_APP, "not available! Using DEFAULT values");
		}
	}


//	printf("Your credentials:\n");
//	printf("WiFi SSID:                        %s\n", wifi_credentials.sSSID);
//	printf("WiFi Password:                    %s\n", wifi_credentials.sPWD);
//	printf("WiFi IoTConnect Unique Device ID: %s\n", IOTCONNECT_DUID);

	EBV_LOGS_I(TAG, EBV_LOGS_MODULE_APP, "Your credentials:");
	EBV_LOGS_I(TAG, EBV_LOGS_MODULE_APP, "WiFi SSID:                        %s", wifi_credentials.sSSID);
	EBV_LOGS_I(TAG, EBV_LOGS_MODULE_APP, "WiFi Password:                    %s", wifi_credentials.sPWD);
	EBV_LOGS_I(TAG, EBV_LOGS_MODULE_APP, "Your position:");
	EBV_LOGS_I(TAG, EBV_LOGS_MODULE_APP, "https://www.google.com/maps/search/?api=1&query=%s,%s", lat_lon_data.sLatitude, lat_lon_data.sLongitude);

	OLED_PrintXY(2, 70, &Small8x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "SSID: %s", wifi_credentials.sSSID);
	OLED_PrintXY(2, 80, &Small8x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "PSD: %s", wifi_credentials.sPWD);
	OLED_PrintXY(2, 90, &Small8x10, OLED_COLOR_WHITE, OLED_COLOR_BLACK, true, "DUID: %s", IOTCONNECT_DUID);

	OLED_UpdateScreen();
	vTaskDelay(pdMS_TO_TICKS(2000));



	EBV_LOGS_I(TAG, EBV_LOGS_MODULE_APP, "======================================================================");
	EBV_LOGS_I(TAG, EBV_LOGS_MODULE_APP, "Starting The App (Input & Sensor Tasks)");
	EBV_LOGS_I(TAG, EBV_LOGS_MODULE_APP, "======================================================================");


    xTaskCreate(input_task, "input Task",INPUT_TASK_STACK_SIZE, NULL, INPUT_TASK_PRIORITY, &inputTaskHandle);
    xTaskCreate(sensor_task, "sensor Task",SENSOR_TASK_STACK_SIZE, NULL, SENSOR_TASK_PRIORITY, &sensorTaskHandle);

    // if optiga present we start connectivity task
    if(optigaON == true)
    {
    	EBV_LOGS_I(TAG, EBV_LOGS_MODULE_APP, "======================================================================");
    	EBV_LOGS_I(TAG, EBV_LOGS_MODULE_APP, "Starting The WiFi Connection");
    	EBV_LOGS_I(TAG, EBV_LOGS_MODULE_APP, "======================================================================");

    	xTaskCreate(wifi_conn_task, "wifi connection Task", WIFI_CONNECTION_TASK_STACK_SIZE, NULL, WIFI_CONNECTION_TASK_PRIORITY, &wifiConnTaskHandle);
    }

    while(1);
}




