package com.avnet.ebv.base


import android.app.Dialog
import android.os.Bundle
import android.view.Window
import androidx.annotation.IdRes
import androidx.annotation.NonNull
import androidx.appcompat.app.AppCompatActivity
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentManager
import com.avnet.ebv.R
import dagger.android.AndroidInjection
import dagger.android.support.DaggerAppCompatActivity
import java.text.SimpleDateFormat
import java.util.*


abstract class BaseActivity : DaggerAppCompatActivity() {

    protected abstract fun initViewBinding()
    private lateinit var mDialog: Dialog
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        AndroidInjection.inject(this)
        initViewBinding()
        initiateProgressDialog()
    }

    private fun initiateProgressDialog() {

        mDialog = Dialog(this, R.style.NewDialog)
        mDialog.requestWindowFeature(Window.FEATURE_NO_TITLE)
        mDialog.setContentView(R.layout.progress_loader)
        mDialog.setCancelable(false)
        mDialog.setCanceledOnTouchOutside(false)
    }

    fun showProgressDialog() {

        if (!mDialog.isShowing) {
            mDialog.show()
        }
    }

    fun dismissProgressDialog() {
        if (mDialog.isShowing) {
            mDialog.dismiss()
        }
    }

    public fun getDate(date: String): String {
        try {
            if (date != null) {
                val fmt = SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss", Locale.US)
                val outputFormat = SimpleDateFormat("hh:mm:ss a", Locale.US)
//            val outputFormat = SimpleDateFormat("yyyy-MM-dd", Locale.US)
                val date = fmt.parse(date)
                val formattedDate = outputFormat.format(date!!)

                return formattedDate.toString()
            } else {
                return ""
            }
        } catch (e: Exception) {
            return ""
        }

    }

    public fun getFullDate(date: String): String {
        if (date != null) {
            val fmt = SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss", Locale.US)
            val outputFormat = SimpleDateFormat("MMMM dd, yyyy HH:mm:ss", Locale.US)
//            val outputFormat = SimpleDateFormat("yyyy-MM-dd", Locale.US)
            val date = fmt.parse(date)
            val formattedDate = outputFormat.format(date!!)

            return formattedDate.toString()
        } else {
            return ""
        }
    }

    fun getError() {

    }

    fun replaceFragment(
        fragment: Fragment,
        backStackName: Boolean = false,
        popStack: Boolean = false,
        aTAG: String = "",
        @IdRes containerViewId: Int = R.id.main_content
    ) {


        var transition = supportFragmentManager.beginTransaction()
        transition.setCustomAnimations(
            R.anim.slide_in_from_right,
            R.anim.slide_out_from_left,
            R.anim.slide_in_from_left,
            R.anim.slide_out_from_right
        )

        if (popStack) supportFragmentManager.popBackStack(
            null, FragmentManager.POP_BACK_STACK_INCLUSIVE
        )
        if (backStackName) transition.addToBackStack(aTAG)

        transition.replace(containerViewId, fragment).commit()

    }

    //
    fun addFragment(
        @NonNull fragment: Fragment,
        backStackName: Boolean = false,
        aTAG: String = "",
        @IdRes containerViewId: Int = R.id.nav_host_fragment
    ) {

        val transition = supportFragmentManager.beginTransaction()
        if (backStackName) transition.addToBackStack(aTAG)
        transition.setCustomAnimations(
            R.anim.slide_in_from_right,
            R.anim.slide_out_from_left,
            R.anim.slide_in_from_left,
            R.anim.slide_out_from_right
        )

        transition.add(containerViewId, fragment).commit()
    }

    public fun isFragmentAlreadyInStack(tag1: String): Boolean {
        if (supportFragmentManager.backStackEntryCount > 0) {
            val backEntry =
                supportFragmentManager.getBackStackEntryAt(supportFragmentManager.backStackEntryCount - 1)
            val tag = backEntry.name

            if ((tag.equals(tag1))) {
                return true
            } else {
                return false
            }
        } else {
            return false
        }
    }

}