package com.avnet.ebv.base


import android.content.Context
import android.os.Bundle
import android.text.*
import android.text.method.LinkMovementMethod
import android.text.style.ClickableSpan
import android.view.View
import android.widget.TextView
import androidx.annotation.IdRes
import androidx.annotation.NonNull
import androidx.fragment.app.Fragment
import com.avnet.ebv.R
import com.avnet.ebv.utils.KeyboardUtils
import dagger.android.support.AndroidSupportInjection
import java.text.SimpleDateFormat
import java.util.*

abstract class BaseFragment : Fragment() {


    protected abstract fun initViewModel()
    protected abstract fun observeViewModel()

    var baseActivity: BaseActivity? = null

    override fun onAttach(context: Context) {
        super.onAttach(context)
        AndroidSupportInjection.inject(this)
        baseActivity = activity as BaseActivity
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

    }

    fun showProgressDialog() {
        baseActivity?.showProgressDialog();
    }

    fun dismissProgressDialog() {
        baseActivity?.dismissProgressDialog();
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        initViewModel()
        observeViewModel()
        KeyboardUtils.forceCloseKeyboard(view)
    }

//    fun replaceFragment(
//        @NonNull fragment: Fragment,
//        backStackName: Boolean = false,
//        popStack: Boolean = false,
//        aTAG: String = "",
//        @IdRes containerViewId: Int = R.id.main_content
//    ) {
//        baseActivity!!.replaceFragment(fragment, backStackName, popStack, aTAG, containerViewId)
//    }
//
//    fun addFragment(
//        @NonNull fragment: Fragment,
//        backStackName: Boolean = false,
//        aTAG: String = "",
//        @IdRes containerViewId: Int = R.id.main_content
//
//    ) {
//        baseActivity!!.addFragment(fragment, backStackName, aTAG, containerViewId)
//    }

    public fun isFragmentAlreadyInStack(tag1: String): Boolean {
        if (baseActivity!!.supportFragmentManager.backStackEntryCount > 0) {
            val backEntry =
                baseActivity!!.supportFragmentManager.getBackStackEntryAt(baseActivity!!.supportFragmentManager.backStackEntryCount - 1)
            val tag = backEntry.name

            if ((tag.equals(tag1))) {
                return true
            } else {
                return false
            }
        } else {
            return false
        }
    }

    fun makeLinks(textView: TextView, vararg links: Pair<String, View.OnClickListener>) {
        var textView: TextView = textView
        if (!textView.text.isEmpty()) {

            val spannableString = SpannableString(textView.text)
            for (link in links) {
                val clickableSpan = object : ClickableSpan() {
                    override fun onClick(view: View) {
                        Selection.setSelection((view as TextView).text as Spannable, 0)
                        view.invalidate()
                        link.second.onClick(view)
                    }

                    override fun updateDrawState(ds: TextPaint) {
                        ds.isUnderlineText = true
                    }
                }
                val startIndexOfLink = textView.text.toString().indexOf(link.first)
                spannableString.setSpan(
                    clickableSpan,
                    startIndexOfLink,
                    startIndexOfLink + link.first.length,
                    Spanned.SPAN_EXCLUSIVE_EXCLUSIVE
                )

            }
            textView.movementMethod =
                LinkMovementMethod.getInstance() // without LinkMovementMethod, link can not click
            textView.setText(spannableString, TextView.BufferType.SPANNABLE)
        }
    }

    public fun getDate(date: String): String {
        try {
            if (date != null) {
                val fmt = SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss", Locale.US)
                val outputFormat = SimpleDateFormat("hh:mm:ss a", Locale.US)
//            val outputFormat = SimpleDateFormat("yyyy-MM-dd", Locale.US)
                val date = fmt.parse(date)
                val formattedDate = outputFormat.format(date!!)

                return formattedDate.toString()
            } else {
                return ""
            }
        } catch (e: Exception) {
            return ""
        }
    }

    public fun getFullDate(date: String): String {
        if (date != null) {
            val fmt = SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss", Locale.US)
            val outputFormat = SimpleDateFormat("MMMM d hh:mm:ss a", Locale.US)
//            val outputFormat = SimpleDateFormat("yyyy-MM-dd", Locale.US)
            val date = fmt.parse(date)
            val formattedDate = outputFormat.format(date!!)

            return formattedDate.toString()
        } else {
            return ""
        }
    }

    public fun getFullDateWithYear(date: String): String {
        try {
            if (date != null) {
                val fmt = SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss", Locale.US)
                val outputFormat = SimpleDateFormat("MM/dd/yyyy", Locale.US)
//            val outputFormat = SimpleDateFormat("yyyy-MM-dd", Locale.US)
                val date = fmt.parse(date)
                val formattedDate = outputFormat.format(date!!)

                return formattedDate.toString()
            } else {
                return ""
            }
        } catch (e: Exception) {
            return ""
        }
    }

    fun getValueFromBundle(bundle: Bundle?, keyName: String): String {
        if (bundle != null) {
            if (bundle.containsKey(keyName)) {
                return bundle.getString(keyName).toString()
            } else {
                return ""
            }
        } else {
            return ""
        }
    }

    fun getBooleanValueFromBundle(bundle: Bundle?, keyName: String): Boolean {
        if (bundle != null) {
            if (bundle.containsKey(keyName)) {
                return bundle.getBoolean(keyName)
            } else {
                return false
            }
        } else {
            return false
        }
    }

}
