package com.avnet.ebv.data

import com.avnet.ebv.data.local.LocalData
import com.avnet.ebv.data.remote.RemoteData
import com.avnet.ebv.model.request.CreateRoleRequest
import com.avnet.ebv.model.request.LoginRequest
import com.avnet.ebv.model.request.NewEntityRequest
import com.avnet.ebv.model.response.*
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.flow.flowOn
import javax.inject.Inject


class DataRepository @Inject
constructor(private val remoteRepository: RemoteData, private val localRepository: LocalData) :
    DataRepositorySource {

    override suspend fun requestGetToken(): Flow<Resource<TokenResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.requestGetToken())
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestRefereshToken(): Flow<Resource<RefreshTokenResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.requestRefreshToken())
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestLogin(request: LoginRequest): Flow<Resource<LoginResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.reguestGetLogin(request))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestCountryList(): Flow<Resource<CountryListResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.requestGetCountryList())
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestStateList(countryGuid: String): Flow<Resource<StateResponseList>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.requestGetStateList(countryGuid))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestEntityLookUp(): Flow<Resource<EntityLookUpResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.requestGetEntityLookUp())
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestRoleLookUp(): Flow<Resource<RoleLookUpResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.requestGetRoleLookUp())
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestAddNewEntity(request: NewEntityRequest): Flow<Resource<NewEntityResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.requestAddNewEntity(request))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestDeviceList(params: Map<String, String?>): Flow<Resource<DeviceListResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.requestDeviceList(params))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestCreateRole(request: CreateRoleRequest): Flow<Resource<CreateRoleResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.createRoleApi(request))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestGetRolePermissions(roleGuid: String): Flow<Resource<GetRolePermissionsResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.getRolePermissionsApi(roleGuid))
        }.flowOn(Dispatchers.IO)
    }


}
