package com.avnet.ebv.data

import com.avnet.ebv.data.local.LocalData
import com.avnet.ebv.data.remote.RemoteData
import com.avnet.ebv.model.request.*
import com.avnet.ebv.model.response.AddDeviceResponse
import com.avnet.ebv.model.response.DeviceListResponse
import com.avnet.ebv.model.response.UserIdentityResponse
import com.avnet.ebv.model.response.*
import com.avnet.ebv.model.response.subscribe.GetSubscribtionToken
import com.avnet.ebv.model.response.subscribe.PlanResponse
import com.avnet.ebv.model.response.subscribe.SubscriberResponse
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.flow.Flow
import kotlinx.coroutines.flow.flow
import kotlinx.coroutines.flow.flowOn
import okhttp3.ResponseBody
import javax.inject.Inject


class DeviceRepository @Inject
constructor(private val remoteRepository: RemoteData, private val localRepository: LocalData) :
    DeviceRepositorySource {

    override suspend fun requestDeviceList(params: Map<String, String?>): Flow<Resource<DeviceListResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.requestDeviceList(params))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun addDevice(request: AddDeviceRequest): Flow<Resource<AddDeviceResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.addDeviceList(request))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun propertyLookupList(templateGuid: String): Flow<Resource<PropertyLookupResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.propertyLookupList(templateGuid))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun propertyHistoryList(
        deviceName: String,
    ): Flow<Resource<PropertyHistoryResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.propertyHistoryList(deviceName))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun commandLookupList(templateGuid: String): Flow<Resource<CommandLookupResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.commandLookupList(templateGuid))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun commandHistoryList(params: Map<String, String?>): Flow<Resource<CommandHistoryResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.commandHistoryList(params))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun executeCommand(
        commandGuId: String,
        request: ExecuteCommandRequest
    ): Flow<Resource<ExecuteCommandResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.requestExecuteCommand(commandGuId, request))
        }.flowOn(Dispatchers.IO)
    }


    override suspend fun requestGetUserIdentity(): Flow<Resource<UserIdentityResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.requestGetUserIdentity())
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestGetUserIdentityRoot(): Flow<Resource<UserIdentityResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.requestGetUserIdentityRoot())
        }.flowOn(Dispatchers.IO)
    }


    override suspend fun requestTemplateAttributeList(
        params: Map<String, String?>,
        deviceTemplateGuid: String
    ): Flow<Resource<AttributeTemplateResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.getTemplateAttributeResponse(params, deviceTemplateGuid))
        }.flowOn(Dispatchers.IO)
    }


    override suspend fun executeProperty(
        uniqueId: String,
        request: ExecutePropertyRequest
    ): Flow<Resource<ExecuteCommandResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.requestExecuteProperty(uniqueId, request))

        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestDeviceTemplateList(
        params: Map<String, String?>,
        deviceTemplateGuid: String
    ): Flow<Resource<DeviceTemplateResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.getDeviceTemplateResponse(params, deviceTemplateGuid))

        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestHistoryDataList(
        uniqueId: String, fromDate: String, toDate: String
    ): Flow<Resource<HistoryDataResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.getHistoryDataResponse(uniqueId, fromDate, toDate))

        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestTemplateDevice(deviceTemplateGuid: String): Flow<Resource<TemplateResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.getTelemetryDeviceResponse(deviceTemplateGuid))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestDeviceTemplateList1(): Flow<Resource<MainDeviceTemplateResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.getDeviceTemplateResponse1())
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestSubscribeToken(params: Map<String, String?>): Flow<Resource<GetSubscribtionToken>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.getSubscribeTokenResponse(params))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestConsumerId(iotConnectCompanyId: String): Flow<Resource<ResponseBody>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.getConsumerId(iotConnectCompanyId))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestSubscriber(
        solutionid: String,
        consumerid: String
    ): Flow<Resource<SubscriberResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.getSubscriber(solutionid, consumerid))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun requestPlans(
        solutionid: String,
        params: Map<String, String?>
    ): Flow<Resource<PlanResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.getPlan(solutionid, params))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun deleteDevice(
        deviceGuid: String
    ): Flow<Resource<DeviceDeleteResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.deleteDevice(deviceGuid))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun getSingleDeviceDetail(
        uniqueId: String
    ): Flow<Resource<SingleDeviceDetailResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.getSingleDeviceDetails(uniqueId))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun addDeviceEntity(
        entityGuid: String,
        request: AddDeleteDeviceEntityRequest
    ): Flow<Resource<DeviceUpdateResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.addDeviceEntity(entityGuid, request))
        }.flowOn(Dispatchers.IO)
    }


    override suspend fun deleteDeviceEntity(
        entityGuid: String,
        request: AddDeleteDeviceEntityRequest
    ): Flow<Resource<DeviceUpdateResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.deleteDeviceEntity(entityGuid, request))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun getDeviceGrantDetailsApi(deviceGuid: String): Flow<Resource<GetDeviceAllocatedDetailsResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.getDeviceGrantDetailsApi(deviceGuid))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun addPermissionsDeviceDetails(
        deviceGuid: String,
        request: GrantDevicePermissionRequest
    ): Flow<Resource<DeviceGrantedPermissionsResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.addPermissionsDeviceDetails(deviceGuid, request))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun revokeAlocatedPermissionsApi(userDevicePermissionGuid: String): Flow<Resource<RevokeDevicePermissionsResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.revokeAlocatedPermissionsApi(userDevicePermissionGuid))
        }.flowOn(Dispatchers.IO)
    }

    override suspend fun updateDeviceEntity(
        deviceGuid: String,
        request: UpdateDeviceEntityRequest
    ): Flow<Resource<DeviceUpdateResponse>> {
        return flow {
//                emit(Resource.Loading())
            emit(remoteRepository.updateDeviceEntity(deviceGuid,request))
        }.flowOn(Dispatchers.IO)
    }


}


