package com.avnet.ebv.data

import com.avnet.ebv.model.request.*
import com.avnet.ebv.model.response.AddDeviceResponse
import com.avnet.ebv.model.response.DeviceListResponse
import com.avnet.ebv.model.response.UserIdentityResponse
import com.avnet.ebv.model.response.*
import com.avnet.ebv.model.response.subscribe.GetSubscribtionToken
import com.avnet.ebv.model.response.subscribe.PlanResponse
import com.avnet.ebv.model.response.subscribe.SubscriberResponse
import kotlinx.coroutines.flow.Flow
import okhttp3.ResponseBody


interface DeviceRepositorySource {

    suspend fun requestDeviceList(params: Map<String, String?>): Flow<Resource<DeviceListResponse>>

    suspend fun addDevice(request: AddDeviceRequest): Flow<Resource<AddDeviceResponse>>

    suspend fun requestGetUserIdentity(): Flow<Resource<UserIdentityResponse>>

    suspend fun requestGetUserIdentityRoot(): Flow<Resource<UserIdentityResponse>>

    suspend fun propertyLookupList(templateGuid : String): Flow<Resource<PropertyLookupResponse>>

    suspend fun propertyHistoryList(deviceName: String): Flow<Resource<PropertyHistoryResponse>>

    suspend fun commandLookupList(templateGuid : String): Flow<Resource<CommandLookupResponse>>

    suspend fun commandHistoryList(params: Map<String, String?>): Flow<Resource<CommandHistoryResponse>>

    suspend fun executeCommand(commandGuId: String, request: ExecuteCommandRequest): Flow<Resource<ExecuteCommandResponse>>

    suspend fun requestTemplateAttributeList(params: Map<String, String?>,deviceTemplateGuid: String): Flow<Resource<AttributeTemplateResponse>>

    suspend fun requestDeviceTemplateList(params: Map<String, String?>,deviceTemplateGuid: String): Flow<Resource<DeviceTemplateResponse>>

    suspend fun executeProperty(uniqueId: String, request: ExecutePropertyRequest): Flow<Resource<ExecuteCommandResponse>>

    suspend fun requestHistoryDataList(uniqueId:String,fromDate:String,toDate:String): Flow<Resource<HistoryDataResponse>>

    suspend fun requestTemplateDevice(deviceTemplateGuid:String): Flow<Resource<TemplateResponse>>

    suspend fun requestDeviceTemplateList1(): Flow<Resource<MainDeviceTemplateResponse>>

    suspend fun requestSubscribeToken(params: Map<String, String?>): Flow<Resource<GetSubscribtionToken>>

    suspend fun requestConsumerId(iotConnectCompanyId:String): Flow<Resource<ResponseBody>>

    suspend fun requestSubscriber(solutionid: String,consumerid:String): Flow<Resource<SubscriberResponse>>

    suspend fun requestPlans(solutionid: String,params: Map<String, String?>): Flow<Resource<PlanResponse>>

    suspend fun deleteDevice(deviceGuid: String): Flow<Resource<DeviceDeleteResponse>>

    suspend fun getSingleDeviceDetail(uniqueId:String): Flow<Resource<SingleDeviceDetailResponse>>

    suspend fun addDeviceEntity(entityGuid:String,request: AddDeleteDeviceEntityRequest): Flow<Resource<DeviceUpdateResponse>>

    suspend fun deleteDeviceEntity(entityGuid:String, request: AddDeleteDeviceEntityRequest): Flow<Resource<DeviceUpdateResponse>>

    suspend fun getDeviceGrantDetailsApi(deviceGuid:String): Flow<Resource<GetDeviceAllocatedDetailsResponse>>

    suspend fun addPermissionsDeviceDetails(deviceGuid:String, request: GrantDevicePermissionRequest): Flow<Resource<DeviceGrantedPermissionsResponse>>

    suspend fun revokeAlocatedPermissionsApi(userDevicePermissionGuid:String): Flow<Resource<RevokeDevicePermissionsResponse>>

    suspend fun updateDeviceEntity(deviceGuid:String,request: UpdateDeviceEntityRequest): Flow<Resource<DeviceUpdateResponse>>

}
