package com.avnet.ebv.data


import com.avnet.ebv.BuildConfig
import com.avnet.ebv.data.interceptor.HeaderInterceptor
import com.avnet.ebv.data.interceptor.HeaderInterceptorRoot
import com.avnet.ebv.data.interceptor.TokenAuthenticator
import com.avnet.ebv.data.interceptor.TokenAuthenticatorRoot
import com.avnet.ebv.data.remote.ApiConstant
import okhttp3.OkHttpClient
import okhttp3.logging.HttpLoggingInterceptor
import retrofit2.Retrofit
import retrofit2.converter.gson.GsonConverterFactory
import java.util.concurrent.TimeUnit
import javax.inject.Inject
import javax.inject.Singleton


@Singleton
class ServiceGenerator @Inject
constructor() {

    val retrofitAuth: Retrofit = Retrofit.Builder()
        .baseUrl(ApiConstant.BASE_URL_AUTH)
        .client(getOkHttpClient())
        .addConverterFactory(GsonConverterFactory.create())
        .build()


    private val retrofitDevice: Retrofit = Retrofit.Builder()
        .baseUrl(ApiConstant.BASE_URL_DEVICE)
        .client(getOkHttpClient())
        .addConverterFactory(GsonConverterFactory.create())
        .build()

    private val retrofitDeviceRoot: Retrofit = Retrofit.Builder()
        .baseUrl(ApiConstant.BASE_URL_DEVICE)
        .client(getOkHttpClientRoot())
        .addConverterFactory(GsonConverterFactory.create())
        .build()

    private val retrofitMaster: Retrofit = Retrofit.Builder()
        .baseUrl(ApiConstant.BASE_URL_MASTER)
        .client(getOkHttpClient())
        .addConverterFactory(GsonConverterFactory.create())
        .build()

    private val retrofitMasterRoot: Retrofit = Retrofit.Builder()
        .baseUrl(ApiConstant.BASE_URL_MASTER)
        .client(getOkHttpClientRoot())
        .addConverterFactory(GsonConverterFactory.create())
        .build()

    private val retrofitUser: Retrofit = Retrofit.Builder()
        .baseUrl(ApiConstant.BASE_URL_USER)
        .client(getOkHttpClient())
        .addConverterFactory(GsonConverterFactory.create())
        .build()

    private val retrofitUserRoot: Retrofit = Retrofit.Builder()
        .baseUrl(ApiConstant.BASE_URL_USER)
        .client(getOkHttpClientRoot())
        .addConverterFactory(GsonConverterFactory.create())
        .build()

    private val retrofitTelemetry: Retrofit = Retrofit.Builder()
        .baseUrl(ApiConstant.BASE_URL_TELEMETRY)
        .client(getOkHttpClient())
        .addConverterFactory(GsonConverterFactory.create())
        .build()

    private val retrofitSubscribtion: Retrofit = Retrofit.Builder()
        .baseUrl(ApiConstant.BASE_URL_SUBSCRIBE)
        .client(getOkHttpClient2())
        .addConverterFactory(GsonConverterFactory.create())
        .build()


    private fun getOkHttpClient(): OkHttpClient
    {
        val httpLogger = HttpLoggingInterceptor(HttpLoggingInterceptor.Logger.DEFAULT)
        if (BuildConfig.DEBUG)
        {
            httpLogger.level = HttpLoggingInterceptor.Level.HEADERS
            httpLogger.level = HttpLoggingInterceptor.Level.BODY
        }
        else
        {
            httpLogger.level = HttpLoggingInterceptor.Level.HEADERS
            httpLogger.level = HttpLoggingInterceptor.Level.BODY
        }
        val okHttpBuilder = OkHttpClient.Builder()
        okHttpBuilder.readTimeout(5, TimeUnit.MINUTES)
            .connectTimeout(5, TimeUnit.MINUTES)
            .addInterceptor(httpLogger)
            .authenticator(TokenAuthenticator())
            .addInterceptor(HeaderInterceptor())


        return okHttpBuilder.build()
    }

    private fun getOkHttpClient2(): OkHttpClient
    {
        val httpLogger = HttpLoggingInterceptor(HttpLoggingInterceptor.Logger.DEFAULT)
        if (BuildConfig.DEBUG)
        {
            httpLogger.level = HttpLoggingInterceptor.Level.BODY
        }
        else
        {
            httpLogger.level = HttpLoggingInterceptor.Level.NONE
        }
        val okHttpBuilder = OkHttpClient.Builder()
        okHttpBuilder.readTimeout(5, TimeUnit.MINUTES)
            .connectTimeout(5, TimeUnit.MINUTES)
            .addInterceptor(httpLogger)
            .addInterceptor(HeaderInterceptor())// To set header in the all API's


        return okHttpBuilder.build()
    }

    private fun getOkHttpClientRoot(): OkHttpClient
    {
        val httpLogger = HttpLoggingInterceptor(HttpLoggingInterceptor.Logger.DEFAULT)
        if (BuildConfig.DEBUG)
        {
            httpLogger.level = HttpLoggingInterceptor.Level.BODY
        }
        else
        {
            httpLogger.level = HttpLoggingInterceptor.Level.NONE
        }
        val okHttpBuilder = OkHttpClient.Builder()
        okHttpBuilder.readTimeout(5, TimeUnit.MINUTES)
            .connectTimeout(5, TimeUnit.MINUTES)
            .addInterceptor(httpLogger)
            .authenticator(TokenAuthenticatorRoot())
            .addInterceptor(HeaderInterceptorRoot())


        return okHttpBuilder.build()
    }

    fun <S> createServiceAuth(serviceClass: Class<S>): S {
        return retrofitAuth.create(serviceClass)
    }
    fun <S> createServiceDevice(serviceClass: Class<S>): S {
        return retrofitDevice.create(serviceClass)
    }

    fun <S> createServiceDeviceRoot(serviceClass: Class<S>): S {
        return retrofitDeviceRoot.create(serviceClass)
    }

    fun <S> createServiceMaster(serviceClass: Class<S>): S {
        return retrofitMaster.create(serviceClass)
    }

    fun <S> createServiceMasterRoot(serviceClass: Class<S>): S {
        return retrofitMasterRoot.create(serviceClass)
    }

    fun <S> createServiceUser(serviceClass: Class<S>): S {
        return retrofitUser.create(serviceClass)
    }
    fun <S> createServiceUserRoot(serviceClass: Class<S>): S {
        return retrofitUserRoot.create(serviceClass)
    }

    fun <S> createServiceTelemetry(serviceClass: Class<S>): S {
        return retrofitTelemetry.create(serviceClass)
    }

    fun <S> createServiceSubscribe(serviceClass: Class<S>): S {
        return retrofitSubscribtion.create(serviceClass)
    }

}