package com.avnet.ebv.data.remote

import com.avnet.ebv.model.request.*
import com.avnet.ebv.model.response.*
import com.avnet.ebv.model.response.subscribe.GetSubscribtionToken
import com.avnet.ebv.model.response.subscribe.PlanResponse
import com.avnet.ebv.model.response.subscribe.SubscriberResponse
import okhttp3.ResponseBody
import retrofit2.Response
import retrofit2.http.*

interface DeviceApiService {

    @GET(ApiConstant.getDevice)
    suspend fun getDeviceList(@QueryMap params: Map<String, String?>): Response<DeviceListResponse>

    @POST(ApiConstant.addDevice)
    suspend fun addDeviceApi(@Body request: AddDeviceRequest): Response<AddDeviceResponse>

    @GET(ApiConstant.getUserIdentity)
    suspend fun getUserIdentity(): Response<UserIdentityResponse>

    @GET(ApiConstant.getTemplateAttribute)
    suspend fun getTemplateAttributeResponse(
        @Path(ApiConstant.deviceTemplateGuid1) deviceTemplateGuid: String,
        @QueryMap params: Map<String, String?>
    ): Response<AttributeTemplateResponse>

    @GET(ApiConstant.getDeviceTemplate)
    suspend fun getDeviceTemplateResponse(
        @Path(ApiConstant.deviceTemplateGuid1) deviceTemplateGuid: String,
        @QueryMap params: Map<String, String?>
    ): Response<DeviceTemplateResponse>

    @GET(ApiConstant.getDeviceTemplate1)
    suspend fun getDeviceTemplateResponse1(): Response<MainDeviceTemplateResponse>

    @GET(ApiConstant.twinPropertyLookup)
    suspend fun twinPropertyLookupApi(@Path(ApiConstant.deviceGuid) deviceTemplateGuid: String): Response<PropertyLookupResponse>

    @GET(ApiConstant.twinPropertyHistory)
    suspend fun twinPropertyHistoryApi(@Path(ApiConstant.deviceName) deviceName: String): Response<PropertyHistoryResponse>

    @GET(ApiConstant.CommandLookup)
    suspend fun commandLookupApi(@Path(ApiConstant.deviceName) deviceName: String): Response<CommandLookupResponse>

    @GET(ApiConstant.CommandHistory)
    suspend fun commandHistoryApi(@QueryMap params: Map<String, String?>): Response<CommandHistoryResponse>

    @POST(ApiConstant.CommandExecute)
    suspend fun commandExecuteApi(
        @Path(ApiConstant.deviceName) deviceName: String,
        @Body request: ExecuteCommandRequest
    ): Response<ExecuteCommandResponse>

    @PUT(ApiConstant.PropertyExecute)
    suspend fun propertyExecuteApi(
        @Path(ApiConstant.uniqueId) uniqueId: String,
        @Body request: ExecutePropertyRequest
    ): Response<ExecuteCommandResponse>

    @GET(ApiConstant.getHistoryData)
    suspend fun getHistoryData(
        @Path(ApiConstant.uniqueId) uniqueId: String,
        @Path(ApiConstant.fromDate) fromDate: String,
        @Path(ApiConstant.toDate) toDate: String
    ): Response<HistoryDataResponse>

    @GET(ApiConstant.getTelemetryDevice)
    suspend fun getTelemetryDevice(@Path(ApiConstant.deviceGuid) deviceGuid: String): Response<TemplateResponse>

    @POST(ApiConstant.getSubscribeToken)
    suspend fun getSubscribeToken(@Body params: Map<String, String?>): Response<GetSubscribtionToken>

    @GET(ApiConstant.getConsumerId)
    suspend fun getConsumerId(@Path(ApiConstant.iotConnectCompanyId ) iotConnectCompanyId  : String): Response<ResponseBody>


    @GET(ApiConstant.getSubscriber)
    suspend fun getSubscriber(
        @Path(ApiConstant.solutionid) solutionid: String,
        @Path(ApiConstant.consumerid) consumerid: String
    ): Response<SubscriberResponse>

    @GET(ApiConstant.getPlan)
    suspend fun getPlan(@Path(ApiConstant.solutionid) solutionid: String,@QueryMap params: Map<String, String?>): Response<PlanResponse>


    @DELETE(ApiConstant.deleteDevice)
    suspend fun deleteDevice(@Path(ApiConstant.deviceGuid) deviceGuid: String): Response<DeviceDeleteResponse>

    @GET(ApiConstant.getSingleDeviceDetails)
    suspend fun getSingleDeviceDetails(@Path(ApiConstant.uniqueId) uniqueId:String): Response<SingleDeviceDetailResponse>

    @PUT(ApiConstant.addDeleteDeviceEntity)
    suspend fun addDeleteDeviceEntity(@Path(ApiConstant.entityGuid) entityGuid:String,@Body request: AddDeleteDeviceEntityRequest): Response<DeviceUpdateResponse>

    @GET(ApiConstant.getDeviceGrantDetails)
    suspend fun getDeviceGrantDetailsApi(
        @Path(ApiConstant.deviceGuid) deviceGuid: String
    ): Response<GetDeviceAllocatedDetailsResponse>

    @POST(ApiConstant.addPermissionsDeviceDetails)
    suspend fun addPermissionsDeviceDetails(@Path(ApiConstant.deviceGuid) deviceGuid: String,@Body request: GrantDevicePermissionRequest): Response<DeviceGrantedPermissionsResponse>

    @DELETE(ApiConstant.revokeAlocatedPermissions)
    suspend fun revokeAlocatedPermissions(@Path(ApiConstant.userDevicePermissionGuid) userDevicePermissionGuid: String): Response<RevokeDevicePermissionsResponse>


    @PUT(ApiConstant.updateDeviceEntity)
    suspend fun updateDeviceEntity(@Path(ApiConstant.deviceGuid ) deviceGuid :String,@Body request: UpdateDeviceEntityRequest): Response<DeviceUpdateResponse>


}