package com.avnet.ebv.data.remote

import android.util.Log
import com.avnet.ebv.EbvApplication
import com.avnet.ebv.data.Resource
import com.avnet.ebv.data.ServiceGenerator
import com.avnet.ebv.log.AndroidLog
import com.avnet.ebv.model.request.*
import com.avnet.ebv.model.response.*
import com.avnet.ebv.model.response.subscribe.GetSubscribtionToken
import com.avnet.ebv.model.response.subscribe.PlanResponse
import com.avnet.ebv.model.response.subscribe.SubscriberResponse
import com.avnet.ebv.utils.Network.Utils.isConnected
import okhttp3.ResponseBody
import retrofit2.Response
import java.io.IOException
import javax.inject.Inject


class RemoteData @Inject
constructor(private val serviceGenerator: ServiceGenerator) : RemoteDataSource {

    private suspend fun processCall(responseCall: suspend () -> Response<*>): Any? {
        if (!isConnected(EbvApplication.context)) {
            Log.d("Inside not connected","Network errorr")
//            return NO_INTERNET_CONNECTION
            var errorResponse: ErrorRespose? =
                ErrorRespose(null, "NETWORK ERROR",0)
            return errorResponse
        }
        return try {
            val response = responseCall.invoke()
            AndroidLog.e("TAG", "response.isSuccessful-> ${response.body().toString()}")
            AndroidLog.e("TAG", "response.not success-> ${response.message()}")
            AndroidLog.e("TAG", "response.code-> ${response.code()}")


            val responseCode = response.code()
            val responseErrorBody = response.errorBody()



            var errorResponse: ErrorRespose? =
                ErrorRespose(null, response.message(), response.code())
            var converter = ServiceGenerator().retrofitAuth.responseBodyConverter<ErrorRespose>(
                ErrorRespose::class.java,
                arrayOfNulls<Annotation>(0)
            )
            try {
                if (response != null && response.errorBody() != null) {
                    errorResponse = converter.convert(response.errorBody())!!
                    AndroidLog.d("error", errorResponse.toString());
                }

            } catch (e: Exception) {
                AndroidLog.d("error1", e.toString());
            }
            if (response.isSuccessful) {
                if(response.body()==null){
                    errorResponse
                }else{
                    response.body()
                }

            } else if (!response.isSuccessful) {
                errorResponse
            } else {
                errorResponse
            }
        } catch (e: IOException) {
            Log.d("Inside catch","Network errorr")
            var errorResponse: ErrorRespose? =
                ErrorRespose(null, "NETWORK ERROR",0)
            return errorResponse

        }
    }

    override suspend fun requestGetToken(): Resource<TokenResponse> {
        val authApiService = serviceGenerator.createServiceAuth(AuthApiService::class.java)
        return when (val response = processCall(authApiService::getTokenResponse)) {
            is TokenResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun requestRefreshToken(): Resource<RefreshTokenResponse> {
        val authApiService = serviceGenerator.createServiceAuth(AuthApiService::class.java)
        return when (val response = processCall(authApiService::getRefereshToken)) {
            is RefreshTokenResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }


    override suspend fun reguestGetLogin(request: LoginRequest): Resource<LoginResponse> {
        val authApiService = serviceGenerator.createServiceAuth(AuthApiService::class.java)
        return when (val response = processCall {
            authApiService.getLoginApi(request)
        }) {
            is LoginResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun requestDeviceList(params: Map<String, String?>): Resource<DeviceListResponse> {
        val authApiService = serviceGenerator.createServiceDevice(DeviceApiService::class.java)
        return when (val response = processCall {
            authApiService.getDeviceList(params)
        }) {
            is DeviceListResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun addDeviceList(request: AddDeviceRequest): Resource<AddDeviceResponse> {
        val authApiService = serviceGenerator.createServiceDevice(DeviceApiService::class.java)
        return when (val response = processCall {
            authApiService.addDeviceApi(request)
        }) {
            is AddDeviceResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun requestGetCountryList(): Resource<CountryListResponse> {
        val authApiService = serviceGenerator.createServiceMasterRoot(AuthApiService::class.java)
        return when (val response = processCall(authApiService::getCountryListResponse)) {
            is CountryListResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun requestGetStateList(countryGuid: String): Resource<StateResponseList> {
        val authApiService = serviceGenerator.createServiceMasterRoot(AuthApiService::class.java)
        return when (val response = processCall {
            authApiService.getStateListResponse(countryGuid)
        }) {
            is StateResponseList -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun requestGetUserIdentity(): Resource<UserIdentityResponse> {
        val deviceApiService = serviceGenerator.createServiceUser(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.getUserIdentity()
        }) {
            is UserIdentityResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun requestGetUserIdentityRoot(): Resource<UserIdentityResponse> {
        val deviceApiService = serviceGenerator.createServiceUserRoot(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.getUserIdentity()
        }) {
            is UserIdentityResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun requestGetEntityLookUp(): Resource<EntityLookUpResponse> {
        val deviceApiService = serviceGenerator.createServiceUserRoot(AuthApiService::class.java)
        return when (val response = processCall {
            deviceApiService.getEntityLookUp()
        }) {
            is EntityLookUpResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun requestGetRoleLookUp(): Resource<RoleLookUpResponse> {
        val deviceApiService = serviceGenerator.createServiceUserRoot(AuthApiService::class.java)
        return when (val response = processCall {
            deviceApiService.getRoleLookUp()
        }) {
            is RoleLookUpResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun propertyLookupList(templateGuid :String): Resource<PropertyLookupResponse> {
        val authApiService = serviceGenerator.createServiceDevice(DeviceApiService::class.java)
        return when (val response = processCall {
            authApiService.twinPropertyLookupApi(templateGuid)
        }) {
            is PropertyLookupResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun propertyHistoryList(deviceName: String): Resource<PropertyHistoryResponse> {
        val authApiService = serviceGenerator.createServiceDevice(DeviceApiService::class.java)
        return when (val response = processCall {
            authApiService.twinPropertyHistoryApi(deviceName)
        }) {
            is PropertyHistoryResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun commandLookupList(templateGuid :String): Resource<CommandLookupResponse> {
        val authApiService = serviceGenerator.createServiceDevice(DeviceApiService::class.java)
        return when (val response = processCall {
            authApiService.commandLookupApi(templateGuid)
        }) {
            is CommandLookupResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun commandHistoryList(params: Map<String, String?>): Resource<CommandHistoryResponse> {
        val authApiService = serviceGenerator.createServiceDevice(DeviceApiService::class.java)
        return when (val response = processCall {
            authApiService.commandHistoryApi(params)
        }) {
            is CommandHistoryResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun requestAddNewEntity(request: NewEntityRequest): Resource<NewEntityResponse> {
        val authApiService = serviceGenerator.createServiceUserRoot(AuthApiService::class.java)
        return when (val response = processCall {
            authApiService.getAddNewEntity(request)
        }) {
            is NewEntityResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun requestExecuteCommand(commandGuId: String,request: ExecuteCommandRequest): Resource<ExecuteCommandResponse> {
        val authApiService = serviceGenerator.createServiceDevice(DeviceApiService::class.java)
        return when (val response = processCall {
            authApiService.commandExecuteApi(commandGuId,request)
        }) {
            is ExecuteCommandResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun getTemplateAttributeResponse(
        params: Map<String, String?>,
        deviceTemplateGuid: String
    ): Resource<AttributeTemplateResponse> {
        val deviceApiService = serviceGenerator.createServiceDevice(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.getTemplateAttributeResponse(deviceTemplateGuid,params)
        }) {
            is AttributeTemplateResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun getTelemetryDeviceResponse(
        deviceTemplateGuid: String
    ): Resource<TemplateResponse> {
        val deviceApiService = serviceGenerator.createServiceTelemetry(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.getTelemetryDevice(deviceTemplateGuid)
        }) {
            is TemplateResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun requestExecuteProperty(commandGuId: String,request: ExecutePropertyRequest): Resource<ExecuteCommandResponse> {
        val authApiService = serviceGenerator.createServiceDevice(DeviceApiService::class.java)
        return when (val response = processCall {
            authApiService.propertyExecuteApi(commandGuId,request)
        }) {
            is ExecuteCommandResponse -> {
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun getDeviceTemplateResponse(
        params: Map<String, String?>,
        deviceTemplateGuid: String
    ): Resource<DeviceTemplateResponse> {
        val deviceApiService = serviceGenerator.createServiceDevice(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.getDeviceTemplateResponse(deviceTemplateGuid,params)
        }) {
            is DeviceTemplateResponse -> {

                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun getDeviceTemplateResponse1(
    ): Resource<MainDeviceTemplateResponse> {
        val deviceApiService = serviceGenerator.createServiceDevice(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.getDeviceTemplateResponse1()
        }) {
            is MainDeviceTemplateResponse -> {

                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun getHistoryDataResponse(
        uniqueId:String,fromDate:String,toDate:String
    ): Resource<HistoryDataResponse> {
        val deviceApiService = serviceGenerator.createServiceTelemetry(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.getHistoryData(uniqueId,fromDate,toDate)
        }) {
            is HistoryDataResponse -> {

                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun getSubscribeTokenResponse(
        params: Map<String, String?>
    ): Resource<GetSubscribtionToken> {
        val deviceApiService = serviceGenerator.createServiceSubscribe(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.getSubscribeToken(params)
        }) {
            is GetSubscribtionToken -> {
                response.status=200
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun getConsumerId(iotConnectCompanyId: String): Resource<ResponseBody> {
        val deviceApiService = serviceGenerator.createServiceSubscribe(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.getConsumerId(iotConnectCompanyId)
        }) {
            is ResponseBody -> {

//                Log.d("customerId", response.string())
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun getSubscriber(
        solutionid: String,
        consumerid: String
    ): Resource<SubscriberResponse> {
        val deviceApiService = serviceGenerator.createServiceSubscribe(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.getSubscriber(solutionid,consumerid)
        }) {
            is SubscriberResponse -> {
                response.status=200
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun getPlan(
        solutionid: String,
        params: Map<String, String?>
    ): Resource<PlanResponse> {
        val deviceApiService = serviceGenerator.createServiceSubscribe(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.getPlan(solutionid,params)
        }) {
            is PlanResponse -> {
                response.status=200
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun deleteDevice(
        deviceGuid: String,
    ): Resource<DeviceDeleteResponse> {
        val deviceApiService = serviceGenerator.createServiceDeviceRoot(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.deleteDevice(deviceGuid)
        }) {
            is DeviceDeleteResponse -> {
                response.status=200
                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun getSingleDeviceDetails(
        uniqueId:String
    ): Resource<SingleDeviceDetailResponse> {
        val deviceApiService = serviceGenerator.createServiceDeviceRoot(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.getSingleDeviceDetails(uniqueId)
        }) {
            is SingleDeviceDetailResponse -> {

                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun addDeviceEntity(entityGuid: String,request: AddDeleteDeviceEntityRequest): Resource<DeviceUpdateResponse> {
        val deviceApiService = serviceGenerator.createServiceDevice(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.addDeleteDeviceEntity(entityGuid,request)
        }) {
            is DeviceUpdateResponse -> {

                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun deleteDeviceEntity(entityGuid: String,request: AddDeleteDeviceEntityRequest): Resource<DeviceUpdateResponse> {
        val deviceApiService = serviceGenerator.createServiceDeviceRoot(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.addDeleteDeviceEntity(entityGuid,request)
        }) {
            is DeviceUpdateResponse -> {

                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun createRoleApi(request: CreateRoleRequest): Resource<CreateRoleResponse> {
        val authApiService = serviceGenerator.createServiceUserRoot(AuthApiService::class.java)
        return when (val response = processCall {
            authApiService.createRoleApi(request)
        }) {
            is CreateRoleResponse -> {

                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun getRolePermissionsApi(roleGuid: String): Resource<GetRolePermissionsResponse> {
        val authApiService = serviceGenerator.createServiceUserRoot(AuthApiService::class.java)
        return when (val response = processCall {
            authApiService.getRolePermissionsApi(roleGuid)
        }) {
            is GetRolePermissionsResponse -> {

                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun getDeviceGrantDetailsApi(deviceGuid: String): Resource<GetDeviceAllocatedDetailsResponse> {
        val deviceApiService = serviceGenerator.createServiceDevice(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.getDeviceGrantDetailsApi(deviceGuid)
        }) {
            is GetDeviceAllocatedDetailsResponse -> {

                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun addPermissionsDeviceDetails(
        deviceGuid: String,
        request: GrantDevicePermissionRequest
    ): Resource<DeviceGrantedPermissionsResponse> {
        val deviceApiService = serviceGenerator.createServiceDevice(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.addPermissionsDeviceDetails(deviceGuid,request)
        }) {
            is DeviceGrantedPermissionsResponse -> {

                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun revokeAlocatedPermissionsApi(userDevicePermissionGuid: String): Resource<RevokeDevicePermissionsResponse> {
        val deviceApiService = serviceGenerator.createServiceDevice(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.revokeAlocatedPermissions(userDevicePermissionGuid)
        }) {
            is RevokeDevicePermissionsResponse -> {

                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }

    override suspend fun updateDeviceEntity(
        deviceGuid: String,
        request: UpdateDeviceEntityRequest
    ): Resource<DeviceUpdateResponse> {
        val deviceApiService = serviceGenerator.createServiceDevice(DeviceApiService::class.java)
        return when (val response = processCall {
            deviceApiService.updateDeviceEntity(deviceGuid,request)
        }) {
            is DeviceUpdateResponse -> {

                Resource.Success(data = response)
            }
            else -> {
                Resource.DataErrorBody(errorBody = response as ErrorRespose)
            }
        }
    }


}