/*
 * Copyright (C) 2018 The Android Open Source Project
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.avnet.ebv.di

import androidx.lifecycle.ViewModel
import androidx.lifecycle.ViewModelProvider
import com.avnet.ebv.ViewModelFactory
import com.avnet.ebv.viewmodel.AddDeviceViewModel
import com.avnet.ebv.viewmodel.DeviceListViewModel
import com.avnet.ebv.viewmodel.LoginViewModel
import com.avnet.ebv.viewmodel.SignUpViewModel
import com.avnet.ebv.viewmodel.*
import dagger.Binds
import dagger.Module
import dagger.multibindings.IntoMap

/**
 * Since dagger
 * support multibinding you are free to bind as may ViewModels as you want.
 */
@Suppress("unused")
@Module
abstract class ViewModelModule {
    @Binds
    internal abstract fun bindViewModelFactory(viewModelFactory: ViewModelFactory): ViewModelProvider.Factory

    @Binds
    @IntoMap
    @ViewModelKey(LoginViewModel::class)
    abstract fun bindLoginViewModel(viewModel: LoginViewModel): ViewModel


    @Binds
    @IntoMap
    @ViewModelKey(LoginRootUserViewModel::class)
    abstract fun bindLoginRootUserViewModel(viewModel: LoginRootUserViewModel): ViewModel


    @Binds
    @IntoMap
    @ViewModelKey(DeviceListViewModel::class)
    abstract fun bindDeviceListViewModel(viewModel: DeviceListViewModel): ViewModel


    @Binds
    @IntoMap
    @ViewModelKey(AddDeviceViewModel::class)
    abstract fun bindAddDeviceViewModel(viewModel: AddDeviceViewModel): ViewModel

    @Binds
    @IntoMap
    @ViewModelKey(SignUpViewModel::class)
    abstract fun bindSignUpViewModel(viewModel: SignUpViewModel): ViewModel


    @Binds
    @IntoMap
    @ViewModelKey(UserSettingViewModel::class)
    abstract fun bindUserSettingViewModel(viewModel: UserSettingViewModel): ViewModel


    @Binds
    @IntoMap
    @ViewModelKey(StatisticViewModel::class)
    abstract fun bindUserStatisticViewModel(viewModel: StatisticViewModel): ViewModel

    @Binds
    @IntoMap
    @ViewModelKey(DataViewModel::class)
    abstract fun bindDataViewModel(viewModel: DataViewModel): ViewModel

    @Binds
    @IntoMap
    @ViewModelKey(HistoryDataViewModel::class)
    abstract fun bindHistoryDataViewModel(viewModel: HistoryDataViewModel): ViewModel

    @Binds
    @IntoMap
    @ViewModelKey(PropertiesViewModel::class)
    abstract fun bindPropertiesViewModel(viewModel: PropertiesViewModel): ViewModel

    @Binds
    @IntoMap
    @ViewModelKey(CommandViewModel::class)
    abstract fun bindCommandViewModel(viewModel: CommandViewModel): ViewModel

    @Binds
    @IntoMap
    @ViewModelKey(MainViewModel::class)
    abstract fun bindMainViewModel(viewModel: MainViewModel): ViewModel


}
