package com.avnet.ebv.utils
import android.app.Activity
import android.app.Dialog
import android.content.Context
import android.content.DialogInterface
import android.os.Build
import android.util.Log
import android.view.View
import android.view.WindowManager
import android.view.animation.Animation
import android.view.animation.RotateAnimation
import android.widget.ImageView
import androidx.appcompat.app.AlertDialog
import com.avnet.ebv.R
import com.avnet.ebv.log.AndroidLog


object DialogUtils {

    private val TAG = DialogUtils::class.java.name

    private var progressDialog: Dialog? = null

    private var alertDialog: Dialog? = null
    /*
     * showLoadingDialog
     */
    fun showLoadingDialog(mContext: Activity) {

        try {
            progressDialog = Dialog(mContext, R.style.styleLoadingDialog)
            progressDialog!!.setCancelable(false)

            val mProgressView = View.inflate(mContext, R.layout.layout_loading_dialog, null)
            progressDialog!!.setContentView(mProgressView)

            var imgLoading = mProgressView.findViewById<ImageView>(R.id.imgloading)

            val rotate = RotateAnimation(
                0f, 360f,
                Animation.RELATIVE_TO_SELF, 0.5f,
                Animation.RELATIVE_TO_SELF, 0.5f
            )
            rotate.duration = 900
            rotate.repeatCount = Animation.INFINITE
            imgLoading.startAnimation(rotate)

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                progressDialog!!.window!!.addFlags(WindowManager.LayoutParams.FLAG_DRAWS_SYSTEM_BAR_BACKGROUNDS)
                progressDialog!!.window!!.addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS)
            } else {
                progressDialog!!.window!!.addFlags(WindowManager.LayoutParams.FLAG_TRANSLUCENT_STATUS)
            }

            // Set properties
            mProgressView.fitsSystemWindows = false

            if (progressDialog != null && progressDialog!!.isShowing) {
                progressDialog!!.dismiss()
            }

            if (progressDialog != null) {
                progressDialog!!.show()
            }
        }catch (e:Exception) {
            e.printStackTrace()
        }
    }
    /**
     * dismissDialog
     */
    fun dismissDialog() {
        try {
            if (progressDialog != null && progressDialog!!.isShowing) {
                progressDialog!!.dismiss()
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    fun dialogOK(title: String, message: String, activity: Activity) {
        try {
            val builder = AlertDialog.Builder(activity,R.style.styleAlertDialog)
            builder.setTitle(title)
            builder.setMessage(message)
            builder.setPositiveButton(android.R.string.ok) { dialog, which -> dialog.dismiss() }
            builder.show()

        } catch (e: Exception) {
            Log.e(TAG, e.toString())
        }
    }


    /**
     * Display message according to your parameters
     *
     * @param mContext Context
     * @param title Title of alert dialog
     * @param message Message of alert dialog
     * @param positiveTitle Positive button text to display. It should not null or blank
     * @param negativeTitle Negative button text to display. It should not null or blank
     * @param positiveClick Positive button click listener
     * @param negativeClick Negative button click listener
     */
    fun showAlertDialog(mContext: Context?, title:String, message: String?,positiveTitle: String?, negativeTitle: String?, positiveClick: DialogInterface.OnClickListener?, negativeClick: DialogInterface.OnClickListener?)
    {
        try
        {
            val builder = AlertDialog.Builder(mContext!!)
            builder.setCancelable(false)
            builder.setTitle(title)
            builder.setMessage(message)

            if (positiveClick != null)
            {
                builder.setPositiveButton(positiveTitle, positiveClick)
            }
            if (negativeClick != null)
            {
                builder.setNegativeButton(negativeTitle, negativeClick)
            }

            if (alertDialog != null && alertDialog!!.isShowing) {
                alertDialog!!.dismiss()
                alertDialog = null
            }
            alertDialog = builder.create()
            (alertDialog as AlertDialog).show()
        }
        catch (e: java.lang.Exception)
        {
            AndroidLog.e(TAG, "Display Alert Dialog Error" + e.message)
        }
    }

    /**
     * dismissAlert
     */
    fun dismissAlert() {
        try {
            if (alertDialog != null && alertDialog!!.isShowing) {
                alertDialog!!.dismiss()
            }
        } catch (e: Exception) {
            e.printStackTrace()
        }
    }
}