package com.avnet.ebv.view.activity

import android.content.Context
import android.content.Intent
import android.content.pm.ActivityInfo
import android.content.res.Configuration
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import android.util.Log
import android.view.Display
import android.view.MotionEvent
import android.view.View
import android.view.WindowManager
import androidx.core.content.ContextCompat
import androidx.viewpager.widget.ViewPager
import com.akexorcist.screenorientationhelper.ScreenOrientationHelper
import com.avnet.ebv.R
import com.avnet.ebv.base.BaseActivity
import com.avnet.ebv.databinding.ActivityDeviceDetailBinding
import com.avnet.ebv.model.response.DeviceListResponse
import com.avnet.ebv.utils.AppConstants
import com.avnet.ebv.utils.NetworkUtils
import com.avnet.ebv.view.adater.ViewPagerAdapter
import com.avnet.ebv.view.fragment.*
import com.google.android.material.tabs.TabLayout
import com.google.android.material.tabs.TabLayout.OnTabSelectedListener
import kotlinx.android.synthetic.main.layout_toolbar.view.*


class DeviceDetailActivity : BaseActivity() {

    private val TAG: String = "DeviceDetailActivity"

    lateinit var binding: ActivityDeviceDetailBinding
    var deviceInfo: DeviceListResponse.Data? = null


    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        Log.d(TAG, "onCreate")

        getIntentData()
        clicks()
        setTabView()
    }

    fun changeOrientation() {

        if (binding.tabTablayout.selectedTabPosition == 0 || binding.tabTablayout.selectedTabPosition == 1) {
            val display: Display =
                (getSystemService(Context.WINDOW_SERVICE) as WindowManager?)!!.getDefaultDisplay()
            val orientation: Int = display.orientation
            when (orientation) {
                0 -> {
                    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE)

                }
                1 -> {
                    setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT)
                }
            }
        }

    }

    fun isDeviceOnlineOffline(isConnect: Boolean) {
        binding.toolbar.ivDeviceConnect.visibility = View.VISIBLE
        if (isConnect) {

            binding.toolbar.ivDeviceConnect.setImageResource(0)
//            binding.toolbar.ivDeviceConnect.setImageDrawable(null)
            binding.toolbar.ivDeviceConnect.setImageDrawable(ColorDrawable(ContextCompat.getColor(this, R.color.green)))

        } else {
            binding.toolbar.ivDeviceConnect.setImageResource(0)
//            binding.toolbar.ivDeviceConnect.setImageDrawable(null)
            binding.toolbar.ivDeviceConnect.setImageDrawable(ColorDrawable(ContextCompat.getColor(this, R.color.red)))

        }
    }


    override fun onStart() {
        super.onStart()
        Log.d(TAG, "onStart")
    }

    override fun onResume() {
        super.onResume()
        Log.d(TAG, "onStart")
    }

    override fun onPause() {
        super.onPause()
        Log.d(TAG, "onStart")
    }

    override fun onStop() {
        super.onStop()
        Log.d(TAG, "onStart")
    }

    override fun onDestroy() {
        super.onDestroy()
    }

    override fun initViewBinding() {
        binding = ActivityDeviceDetailBinding.inflate(layoutInflater)
        setContentView(binding.root)
    }

    private fun setTabView() {

        val bundle = Bundle()
        bundle.putSerializable("Data", deviceInfo)

        val adapter = ViewPagerAdapter(supportFragmentManager)
        adapter.addFragment(
            Data2Fragment(
                deviceInfo?.uniqueId.toString(),
                deviceInfo?.deviceTemplateGuid.toString(),
                deviceInfo?.guid.toString(),
                deviceInfo?.isConnected!!, true
            ), "Data"
        )
        adapter.addFragment(
            HistoryFragment(
                deviceInfo?.uniqueId.toString(),
                deviceInfo?.deviceTemplateGuid.toString()
            ), "History"
        )
        adapter.addFragment(PropertiesFragment(bundle), "Properties")
        adapter.addFragment(CommandFragment(bundle), "Command")

        binding.viewPager.adapter = adapter
        binding.viewPager.offscreenPageLimit = 4
        binding.tabTablayout.setupWithViewPager(binding.viewPager)

        binding.viewPager.setOnPageChangeListener(object :
            ViewPager.OnPageChangeListener {
            override fun onPageScrolled(
                position: Int,
                positionOffset: Float,
                positionOffsetPixels: Int
            ) {


            }

            override fun onPageSelected(position: Int) {
                if (position == 0 || position == 1) {
                    requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_PORTRAIT

                } else {
                    requestedOrientation = ActivityInfo.SCREEN_ORIENTATION_PORTRAIT
                }

                if (position == 2) {
                    val intent = Intent(AppConstants.INTENT_REFRESH_PROPERTIES)
                    sendBroadcast(intent)
                } else if (position == 3) {
                    val intent = Intent(AppConstants.INTENT_REFRESH_COMMAND)
                    sendBroadcast(intent)
                } else if (position == 1) {
                    val intent = Intent(AppConstants.INTENT_REFRESH_HISTORY)
                    sendBroadcast(intent)
                }
            }

            override fun onPageScrollStateChanged(state: Int) {

            }
        })
    }

    private fun clicks() {
        binding.toolbar.ivMenu.setOnClickListener { onBackPressed() }

        binding.tabTablayout.setOnTouchListener(object : View.OnTouchListener {
            override fun onTouch(p0: View?, p1: MotionEvent?): Boolean {
                if (NetworkUtils.isNetworkAvailable(this@DeviceDetailActivity)) {
                    return true
                } else {
                    return false
                }
            }

        })
    }

    private fun getIntentData() {
        if (intent != null) {
            deviceInfo = intent.extras?.get("Data") as DeviceListResponse.Data
            binding.toolbar.tvTitle.text = deviceInfo!!.displayName
        }
        binding.toolbar.ivMenu.setImageResource(R.drawable.ic_back)
        binding.toolbar.ivAdd.visibility = View.INVISIBLE

    }


    override fun onConfigurationChanged(newConfig: Configuration) {
        super.onConfigurationChanged(newConfig)
        try {
            val currentOrientation = resources.configuration.orientation
            if (currentOrientation == Configuration.ORIENTATION_LANDSCAPE) {
                Log.v("TAG", "Landscape !!!")
                binding.toolbar.toolbarMain.visibility = View.GONE
                binding.tabTablayout.visibility = View.GONE

            } else {
                Log.v("TAG", "Portrait !!!")
                binding.toolbar.toolbarMain.visibility = View.VISIBLE
                binding.tabTablayout.visibility = View.VISIBLE
            }
        } catch (e: Exception) {

        }

    }

}