package com.avnet.ebv.view.activity

import android.annotation.SuppressLint
import android.content.*
import android.graphics.Typeface
import android.os.Bundle
import android.util.Log
import android.view.Gravity
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import androidx.appcompat.app.AlertDialog
import androidx.core.content.res.ResourcesCompat
import androidx.drawerlayout.widget.DrawerLayout
import androidx.fragment.app.FragmentManager
import androidx.fragment.app.FragmentTransaction
import androidx.lifecycle.Observer
import androidx.navigation.NavController
import androidx.navigation.NavGraph
import androidx.navigation.findNavController
import androidx.navigation.ui.AppBarConfiguration
import androidx.navigation.ui.navigateUp
import com.avnet.ebv.R
import com.avnet.ebv.ViewModelFactory
import com.avnet.ebv.base.BaseActivity
import com.avnet.ebv.common.SharedPref
import com.avnet.ebv.databinding.ActivityMainBinding
import com.avnet.ebv.model.response.UserIdentityResponse
import com.avnet.ebv.utils.AppConstants
import com.avnet.ebv.view.fragment.AddDeviceFragment
import com.avnet.ebv.viewmodel.MainViewModel
import com.google.gson.GsonBuilder
import kotlinx.android.synthetic.main.nav_header_main.view.*
import javax.inject.Inject


class MainActivity() : BaseActivity() {

    private val TAG: String = "MainActivity"

    @Inject
    lateinit var mainViewModel: MainViewModel

    @Inject
    lateinit var viewModelFactory: ViewModelFactory

    private lateinit var appBarConfiguration: AppBarConfiguration
    lateinit var navController: NavController
    lateinit var binding: ActivityMainBinding

    var btnAddDevice: ImageView? = null
    var btnMenu: ImageView? = null
    var tvTitle: TextView? = null
    var drawerLayout: DrawerLayout? = null
    lateinit var userDetails: UserIdentityResponse
    var fragmentHost: Int = 0
    lateinit var graph: NavGraph


    override fun initViewBinding() {

    }

    private val mMessageReceiver: BroadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent) {

            userDetails = GsonBuilder().create().fromJson(
                SharedPref.instance!!.getValueFromPreferences(
                    getString(R.string.pref_user_details)
                ),
                UserIdentityResponse::class.java
            )
            binding.drawerLayout.txt_user_name.setText(userDetails.data?.firstName + " " + userDetails.data?.lastName)
        }
    }

    fun switchToSecondFragment() {
        val manager: FragmentManager = supportFragmentManager
        val fragment = AddDeviceFragment()
        val transaction: FragmentTransaction = manager.beginTransaction()
        transaction.replace(R.id.nav_host_fragment, fragment).addToBackStack(null)
        transaction.commit()
    }

    @SuppressLint("WrongConstant")
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)

        Log.d(TAG, "onCreate")

        binding = ActivityMainBinding.inflate(layoutInflater)
        setContentView(binding.root)


        fragmentHost = R.id.nav_host_fragment
        navController = findNavController(R.id.nav_host_fragment)
        val inflater = navController.navInflater
        graph = inflater.inflate(R.navigation.navigation)

        if (intent.getIntExtra(AppConstants.startDest, 0).equals(0)) {
            graph.setStartDestination(R.id.nav_add_device)
        } else {
            graph.setStartDestination(R.id.nav_home)
        }
//
//
        navController.setGraph(graph, intent.extras)


//        initViewModel()
//        observeViewModel()





        menuItemClicks()
        inits()
        setToolbarTitle("Devices")


        binding.drawerLayout.setDrawerLockMode(DrawerLayout.LOCK_MODE_UNLOCKED)

        binding.appBarMain.viewToolbar.ivMenu.setOnClickListener {
            binding.drawerLayout.openDrawer(Gravity.START)
        }

    }

    override fun onStart() {
        super.onStart()
        Log.d(TAG, "onStart")
    }

    private fun initViewModel() {
        mainViewModel = viewModelFactory.create(MainViewModel::class.java)
        binding.lifecycleOwner = this
        mainViewModel.initializeWithContext(this)
        binding.mainViewModel = mainViewModel

        mainViewModel.getDeviceList()
//        if(SharedPref.instance?.getValueFromPreferences(resources.getString(R.string.pref_is_main_first_time))
//                .equals("0")){
//
//            SharedPref.instance!!.saveStringSession(
//                getString(R.string.pref_is_main_first_time),
//                "1"
//            )
//        }

    }

    private fun observeViewModel() {
        mainViewModel.startDest.observe(this, Observer {
            if(it==0){
                graph.setStartDestination(R.id.nav_add_device)
            }else{
                graph.setStartDestination(R.id.nav_home)
            }
            navController.setGraph(graph, intent.extras)
        })
    }

    private fun inits() {
        btnAddDevice = findViewById<ImageView>(R.id.ivAdd)
        btnMenu = findViewById<ImageView>(R.id.ivMenu)
        tvTitle = findViewById<TextView>(R.id.tvTitle)
        drawerLayout = binding.drawerLayout

        setSelectedMenuItem(1)

    }

    private fun setToolbarTitle(title: String) {
        binding.appBarMain.viewToolbar.tvTitle.text = title
    }


    private fun menuItemClicks() {
        binding.navMainView.ivClose.setOnClickListener {
            closeNavigationDrawer()
        }
        binding.navMainView.viewDevices.setOnClickListener {

            navController.navigateUp()
            if (navController.graph.startDestinationId == R.id.nav_home) // Id as per set up on nav_graph.xml file
            {
                setSelectedMenuItem(1)
            } else {
                val inflater = navController.navInflater
                var graph = inflater.inflate(R.navigation.navigation)
                navController.setGraph(
                    graph,
                    intent.extras
                )
                closeNavigationDrawer()
            }

        }
        binding.navMainView.viewStatistics.setOnClickListener {
            setSelectedMenuItem(2)

            if (navController.graph.startDestinationId == R.id.nav_home) // Id as per set up on nav_graph.xml file
            {
                navController.navigateUp()
                navController.navigate(R.id.action_nav_home_to_nav_statistics)
            } else {
                navController.navigateUp()
                navController.navigate(R.id.action_nav_add_device_to_nav_statistics)
            }


        }
        binding.navMainView.viewUserSetting.setOnClickListener {

            setSelectedMenuItem(3)
            if (navController.graph.startDestinationId == R.id.nav_home) // Id as per set up on nav_graph.xml file
            {
                navController.navigateUp()
                navController.navigate(R.id.action_nav_home_to_nav_user_setting)
            } else {
                navController.navigateUp()
                navController.navigate(R.id.action_nav_add_device_to_nav_user_setting)
            }


        }
        binding.navMainView.viewLogout.setOnClickListener {
//            setSelectedMenuItem(1)
            closeNavigationDrawer()
            showConfimationDialog()

        }
        binding.navMainView.viewUserChange.setOnClickListener {
//            setSelectedMenuItem(1)
            closeNavigationDrawer()
            showChangeUserConfimationDialog()

        }
    }

    fun setSelectedMenuItem(pos: Int) {
        binding.navMainView.ivDevices.setImageResource(R.drawable.ic_devices_normal)
        binding.navMainView.txtDevices.setTextColor(resources.getColor(R.color.black))
        binding.navMainView.ivStatistics.setImageResource(R.drawable.ic_statistics_normal)
        binding.navMainView.txtStatistics.setTextColor(resources.getColor(R.color.black))
        binding.navMainView.ivUserSetting.setImageResource(R.drawable.ic_user_settings_normal)
        binding.navMainView.txtUserSetting.setTextColor(resources.getColor(R.color.black))
        binding.navMainView.ivUserChange.setImageResource(R.drawable.ic_change_user_normal)
        binding.navMainView.txtUserChange.setTextColor(resources.getColor(R.color.black))
        binding.navMainView.ivLogout.setImageResource(R.drawable.ic_logout_normal)
        binding.navMainView.txtLogout.setTextColor(resources.getColor(R.color.black))

        if (pos == 1) {
            binding.navMainView.ivDevices.setImageResource(R.drawable.ic_devices_hover)
            binding.navMainView.txtDevices.setTextColor(resources.getColor(R.color.colorPrimary))
        } else if (pos == 2) {
            binding.navMainView.ivStatistics.setImageResource(R.drawable.ic_statistics_hover)
            binding.navMainView.txtStatistics.setTextColor(resources.getColor(R.color.colorPrimary))
        } else if (pos == 3) {
            binding.navMainView.ivUserSetting.setImageResource(R.drawable.ic_user_settings_hover)
            binding.navMainView.txtUserSetting.setTextColor(resources.getColor(R.color.colorPrimary))
        } else if (pos == 4) {
            binding.navMainView.ivUserChange.setImageResource(R.drawable.ic_change_user_hover)
            binding.navMainView.txtUserChange.setTextColor(resources.getColor(R.color.colorPrimary))
        } else if (pos == 5) {
            binding.navMainView.ivLogout.setImageResource(R.drawable.ic_logout_hover)
            binding.navMainView.txtLogout.setTextColor(resources.getColor(R.color.colorPrimary))
        }
        closeNavigationDrawer()

    }

    private fun showConfimationDialog() {

        val builder: AlertDialog.Builder = AlertDialog.Builder(this)
        builder.setMessage(getString(R.string.are_you_sure_you_want_to_logout))
        builder.setCancelable(true)

        builder.setPositiveButton("Logout") { dialogInterface, _ ->
            val intent = Intent(this, AuthenticationActivity()::class.java)
            startActivity(intent)
            finish()
            SharedPref.instance!!.deleteAllPreferences()
            dialogInterface.dismiss()
        }
        //performing cancel action
        builder.setNegativeButton("Cancel") { dialogInterface, _ ->
            dialogInterface.dismiss()
        }

        val alertDialog: AlertDialog = builder.create()
        // set title for alert dialog box
        alertDialog.setTitle(R.string.app_name)
        alertDialog.show()

        val textView = alertDialog.window?.findViewById(android.R.id.message) as TextView

        val button1: Button = alertDialog.window!!.findViewById(android.R.id.button1) as Button
        val button2: Button = alertDialog.window!!.findViewById(android.R.id.button2) as Button


        val typeface: Typeface? = ResourcesCompat.getFont(this, R.font.arial)
        textView.typeface = typeface
//        button1.typeface = typeface
//        button2.typeface = typeface
    }

    private fun showChangeUserConfimationDialog() {
        val builder: AlertDialog.Builder = AlertDialog.Builder(this)
        builder.setMessage(getString(R.string.are_you_sure_you_want_to_change_user))
        builder.setCancelable(true)

        builder.setPositiveButton("Switch User") { dialogInterface, _ ->
            SharedPref.instance!!.saveStringSession(
                getString(R.string.pref_is_user_logged_in),
                ""
            )
//            SharedPref.instance!!.saveStringSession(
//                getString(R.string.pref_bearer_token),
//                SharedPref.instance!!.getValueFromPreferences(EbvApplication.context.getString(R.string.pref_bearer_token_root))
//            )
//            SharedPref.instance!!.saveStringSession(
//                getString(R.string.pref_refresh_token),
//                SharedPref.instance!!.getValueFromPreferences(EbvApplication.context.getString(R.string.pref_refresh_token_root))
//            )

            val intent = Intent(this, AuthenticationActivity()::class.java)
            startActivity(intent)
            finish()
            dialogInterface.dismiss()
        }
        //performing cancel action
        builder.setNegativeButton("Cancel") { dialogInterface, _ ->
            dialogInterface.dismiss()
        }

        val alertDialog: AlertDialog = builder.create()
        // set title for alert dialog box
        alertDialog.setTitle(R.string.app_name)
        alertDialog.show()

        val textView = alertDialog.window?.findViewById(android.R.id.message) as TextView

        val button1: Button = alertDialog.window!!.findViewById(android.R.id.button1) as Button
        val button2: Button = alertDialog.window!!.findViewById(android.R.id.button2) as Button


        val typeface: Typeface? = ResourcesCompat.getFont(this, R.font.arial)
        textView.typeface = typeface
//        button1.typeface = typeface
//        button2.typeface = typeface
    }


    @SuppressLint("WrongConstant")
    private fun closeNavigationDrawer() {
        if (binding.drawerLayout.isOpen)
            binding.drawerLayout.closeDrawer(Gravity.START)
    }


    override fun onSupportNavigateUp(): Boolean {
        val navController = findNavController(R.id.nav_host_fragment)
        return navController.navigateUp(appBarConfiguration) || super.onSupportNavigateUp()
    }

    //on resume
    override fun onResume() {
        super.onResume()
        Log.d(TAG, "onResume")
        registerReceiver(
            mMessageReceiver,
            IntentFilter(AppConstants.INTENT_USER_DETAILS)
        )
    }

    override fun onPause() {
        super.onPause()
        Log.d(TAG, "onPause")
        unregisterReceiver(mMessageReceiver);
    }

    companion object {
        fun newIntent(context: Context): Intent {
            return Intent(context, MainActivity::class.java)
        }
    }

    override fun onRestart() {
        super.onRestart()
        Log.d(TAG, "onRestart")
    }

    override fun onDestroy() {
        super.onDestroy()
        Log.d(TAG, "onDestroy")
        Log.d("isDestroyed","isDestroyed")
    }

}