package com.avnet.ebv.view.adater

import android.content.Context
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.TextView
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.avnet.ebv.R
import com.avnet.ebv.base.BaseActivity
import com.avnet.ebv.base.BaseFragment
import com.avnet.ebv.databinding.RowTwinPropertyBinding
import com.avnet.ebv.databinding.ViewNoDataFoundBinding
import com.avnet.ebv.databinding.ViewProgressLoadingBinding
import com.avnet.ebv.enums.AdapterViewType
import com.avnet.ebv.model.response.PropertyHistoryDataItem


class PropertyListAdapter(
    var deviceList: ArrayList<PropertyHistoryDataItem?>,
    recyclerView: RecyclerView,
    var totalCount: Int,
    var context: Context
) :
    RecyclerView.Adapter<RecyclerView.ViewHolder>() {
    private var onViewClickListener: OnViewClickListener? = null


    private var totalItemCount = 0
    private var lastVisibleItem = 0
    private var visibleThreshold = 5
    private var isLoading = false
    private var layoutManager: LinearLayoutManager? = null

    private var onLoadMoreListener: OnLoadMoreInterface? = null

    init {
        recyclerView.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)
                layoutManager = recyclerView.layoutManager as LinearLayoutManager

                totalItemCount = layoutManager!!.itemCount
                lastVisibleItem = layoutManager!!.findLastVisibleItemPosition()
                if (!isLoading && lastVisibleItem > -1 && totalItemCount < totalCount
                    && totalItemCount <= (lastVisibleItem + visibleThreshold)
                ) {
                    if (onLoadMoreListener != null) {
                        if (deviceList.size > 0) {
                            isLoading = true
                            onLoadMoreListener!!.onLoadMore()
                        }
                    }
                }
            }
        })
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder =
        when (viewType) {
            AdapterViewType.FILL_VIEW.type -> {
                val view =
                    LayoutInflater.from(parent.context)
                        .inflate(R.layout.row_twin_property, parent, false)

                val binding = RowTwinPropertyBinding.bind(view)
                AlertViewHolder(binding)
            }

            AdapterViewType.PROGRESS.type -> {
                val view = LayoutInflater.from(parent.context)
                    .inflate(R.layout.view_progress_loading, parent, false)
                val binding = ViewProgressLoadingBinding.bind(view)
                ProgressViewHolder(binding)
            }
            else -> {

                val view = LayoutInflater.from(parent.context)
                    .inflate(R.layout.view_no_data_found, parent, false)
                val binding = ViewNoDataFoundBinding.bind(view)
                EmptyViewHolder(binding)

            }
        }

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        if (holder is AlertViewHolder) {
            holder.itemView.tag = position
            holder.txtName.text = deviceList[position]!!.name
            if(deviceList[position]!!.desiredValue!=null){
                holder.txtDesiredValue.text = deviceList[position]!!.desiredValue.toString()
            }else{
                holder.txtDesiredValue.text = "NA"
            }

            if(deviceList[position]!!.desiredUpdatedDate!=null && (!deviceList[position]!!.desiredUpdatedDate?.isEmpty()!!) ){
                val currentString =
                    deviceList[position]!!.desiredUpdatedDate
                val separated = currentString?.split(".")?.toTypedArray()
                var label = (context as BaseActivity).getFullDate(separated?.get(0).toString())

                holder.txtDesiredUpdateDate.text = label
            }else{
                holder.txtDesiredUpdateDate.text = "NA"
            }



            if(deviceList[position]!!.reportedValue!=null){
                holder.txtReportedValue.text = deviceList[position]!!.reportedValue.toString()
            }else{
                holder.txtReportedValue.text = "NA"
            }


            if(deviceList[position]!!.reportedUpdatedDate!=null && (!deviceList[position]!!.reportedUpdatedDate?.isEmpty()!!) ){
                val currentString1 =
                    deviceList[position]!!.reportedUpdatedDate
                val separated1 = currentString1?.split(".")?.toTypedArray()
                var label1 = (context as BaseActivity).getFullDate(separated1?.get(0).toString())

                holder.txtReportedUpdatedDate.text = label1
            }else{
                holder.txtReportedUpdatedDate.text = "NA"
            }



        } else if (holder is EmptyViewHolder) {
            holder.tvNoDataFound.text =
                holder.itemView.context.getString(R.string.no_data_found)
        }
    }

    override fun getItemViewType(position: Int): Int {
        return if (deviceList.size == 0 && !isLoading)
            AdapterViewType.EMPTY_VIEW.type
        else if (deviceList.size > 0 && isLoading && deviceList[position] == null) {
            AdapterViewType.PROGRESS.type
        } else if (deviceList.size == 0 && isLoading) {
            AdapterViewType.PROGRESS.type
        } else
            AdapterViewType.FILL_VIEW.type
    }

    override fun getItemCount(): Int {
        return if (deviceList.size == 0) 1 else deviceList.size
    }

    fun setLoaded(isLoading: Boolean) {
        this.isLoading = isLoading
    }

    fun isLoading(): Boolean {
        return this.isLoading
    }


    fun setOnLoadMoreListener(onLoadMoreInterface: OnLoadMoreInterface) {
        this.onLoadMoreListener = onLoadMoreInterface
    }

    interface OnLoadMoreInterface {
        fun onLoadMore()
    }

    fun setOnViewClickListener(onViewClickListener: OnViewClickListener) {
        this.onViewClickListener = onViewClickListener
    }

    private class AlertViewHolder(view: RowTwinPropertyBinding) :
        RecyclerView.ViewHolder(view.root) {
        val txtName = view.txtName
        val txtDesiredValue = view.txtDesiredValue
        val txtDesiredUpdateDate = view.txtDesiredUpdateDate
        val txtReportedValue = view.txtReportedValue
        val txtReportedUpdatedDate = view.txtReportedUpdatedDate


    }

    private class ProgressViewHolder(view: ViewProgressLoadingBinding) :
        RecyclerView.ViewHolder(view.root)

    private class EmptyViewHolder(view: ViewNoDataFoundBinding) :
        RecyclerView.ViewHolder(view.root) {
        val tvNoDataFound: TextView = view.tvNoDataFound
    }


    interface OnViewClickListener {
        fun onItemClick(position: Int)
    }
}