package com.avnet.ebv.view.fragment

import android.os.Bundle
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.Spinner
import androidx.drawerlayout.widget.DrawerLayout
import androidx.lifecycle.Observer
import com.avnet.ebv.R
import com.avnet.ebv.ViewModelFactory
import com.avnet.ebv.base.BaseFragment
import com.avnet.ebv.databinding.FragmentAddDeviceBinding
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.log.AndroidLog
import com.avnet.ebv.model.response.MainDeviceTemplateResponse
import com.avnet.ebv.utils.KeyboardUtils
import com.avnet.ebv.utils.Network
import com.avnet.ebv.view.activity.MainActivity
import com.avnet.ebv.viewmodel.AddDeviceViewModel
import com.google.zxing.client.android.Intents
import com.journeyapps.barcodescanner.ScanContract
import com.journeyapps.barcodescanner.ScanIntentResult
import com.journeyapps.barcodescanner.ScanOptions
import javax.inject.Inject


class AddDeviceFragment : BaseFragment(), View.OnClickListener {
    private lateinit var mBinding: FragmentAddDeviceBinding

    @Inject
    lateinit var addDeviceViewModel: AddDeviceViewModel

    @Inject
    lateinit var viewModelFactory: ViewModelFactory

    lateinit var templateAdapter: ArrayAdapter<String>

    private var templateList: ArrayList<String> = ArrayList()
    private var templateListMain: ArrayList<MainDeviceTemplateResponse.Data> = ArrayList()

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        mBinding = FragmentAddDeviceBinding.inflate(layoutInflater)
        return mBinding.root
    }

    override fun initViewModel() {

        addDeviceViewModel = viewModelFactory.create(AddDeviceViewModel::class.java)
        mBinding.setLifecycleOwner(this)
        mBinding.addDeviceViewModel = addDeviceViewModel
        addDeviceViewModel.initializeWithContext(requireContext())

    }

    override fun observeViewModel() {
        addDeviceViewModel._ProgreeBarState.observe(this, Observer {
            when (it) {
                ProgressState.HIDE -> dismissProgressDialog()
                ProgressState.SHOW -> showProgressDialog()
            }
        })

//        addDeviceViewModel._templateResponse.observe(this, Observer {
//            if(it.data?.size!!>0){
//                for(i in 0 until it.data!!.size){
//                    if(it.data!!.get(i)?.name.equals(ApiConstant.EBV_MOBILE)){
//                        var guid:String=it.data?.get(i)?.guid.toString()
//                        if(!guid.isEmpty()){
//                            SharedPref.instance!!.saveStringSession(
//                                getString(R.string.deviceTemplateGuid),
//                                guid)
//                            Log.d("MainGuid", SharedPref.instance?.getValueFromPreferences(getString(R.string.deviceTemplateGuid).toString())!!)
//
//                        }else{
//                            SharedPref.instance!!.saveStringSession(
//                                getString(R.string.deviceTemplateGuid),
//                                ApiConstant.deviceTemplateGuid)
//                            Log.d("MainGuid1", SharedPref.instance?.getValueFromPreferences(getString(R.string.deviceTemplateGuid).toString())!!)
//
//                        }
//
//                    }
//                }
//            }
//        })

        addDeviceViewModel._templateResponse.observe(this, Observer {
            if (it.data?.size!! > 0) {

                templateListMain = ArrayList()
                templateListMain.addAll(it.data as ArrayList<MainDeviceTemplateResponse.Data>)
                for (i in 0 until it.data!!.size) {
                    templateList.add(it.data!!.get(i)?.name.toString())
                }
                templateAdapter.notifyDataSetChanged()
            }
        })
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        addDeviceViewModel.getTemplate()
        addDeviceViewModel.getUserIdentity()
        clicks()
        setSpinnerTemplate(mBinding.spSelectTemplate)

    }

    override fun onResume() {
        (activity as MainActivity).tvTitle?.text = getString(R.string.add_device)
        (activity as MainActivity).btnAddDevice?.visibility = View.VISIBLE
        (activity as MainActivity).btnAddDevice?.setImageDrawable(resources.getDrawable(R.drawable.ic_qr_code))
        if ((context as MainActivity).navController.graph.startDestinationId == R.id.nav_home) // Id as per set up on nav_graph.xml file
        {
            (activity as MainActivity).btnMenu?.setImageDrawable(resources.getDrawable(R.drawable.ic_back))
            (activity as MainActivity).drawerLayout?.setDrawerLockMode(DrawerLayout.LOCK_MODE_LOCKED_CLOSED)

        } else {
            (activity as MainActivity).btnMenu?.setImageDrawable(resources.getDrawable(R.drawable.ic_menu))
            (activity as MainActivity).drawerLayout?.setDrawerLockMode(DrawerLayout.LOCK_MODE_UNLOCKED)
        }
        (activity as MainActivity).btnMenu?.setOnClickListener {

            if ((context as MainActivity).navController.graph.startDestinationId == R.id.nav_home) {
                (context as MainActivity).navController.navigateUp()
            } else {
                (activity as MainActivity).drawerLayout?.openDrawer(Gravity.LEFT)

            }

        }


        val scanBarcodeId = BarcodeViewFragment.scanBarcodeId

        if (scanBarcodeId != null && !scanBarcodeId.isEmpty()) {
            if (scanBarcodeId.equals("-1")) {

                AndroidLog.e("scanBarcodeId", "" + scanBarcodeId)
                mBinding.edtDeviceIdentifier.requestFocus()
                Network.showKeyboard(requireContext())
                BarcodeViewFragment.scanBarcodeId = ""

            } else {
                AndroidLog.e("scanBarcodeId", "" + scanBarcodeId)
                mBinding.edtDeviceIdentifier.setText("" + scanBarcodeId)

            }
        }

        mBinding.edtDeviceIdentifier.requestFocus()

        super.onResume()
    }

    private fun clicks() {
        mBinding.btnAddDevice.setOnClickListener(this)
        (activity as MainActivity).btnAddDevice?.setOnClickListener {
            (activity as MainActivity).navController.navigate(R.id.action_nav_add_device_to_BarcodeViewFragment)
        }
    }

    private fun setSpinnerTemplate(s: Spinner) {

        templateList= ArrayList()
        templateList.add("Select Template")
        templateAdapter =
            ArrayAdapter<String>(
                requireContext(),
                R.layout.spinner_item_select,
                templateList
            )
        templateAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        s.adapter = templateAdapter

        s.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onItemSelected(
                parentView: AdapterView<*>?,
                selectedItemView: View?,
                position: Int,
                id: Long
            ) {
                if(position!=0){
                    addDeviceViewModel.deviceTemplateGuid.value =templateListMain.get(position-1).guid
                }

            }

            override fun onNothingSelected(parentView: AdapterView<*>?) {
            }
        }
    }

    private val barcodeLauncher =
        registerForActivityResult(ScanContract()) { result: ScanIntentResult ->
            if (result.contents == null) {
                val originalIntent = result.originalIntent
                if (originalIntent == null) {
                    addDeviceViewModel.showMsg("Cancelled")
                } else if (originalIntent.hasExtra(Intents.Scan.MISSING_CAMERA_PERMISSION)) {
                    addDeviceViewModel.showMsg("Cancelled due to missing camera permission")
                }
            } else {

                addDeviceViewModel.deviceId.value = result.contents
            }
        }

    fun ScanCode() {
        val options = ScanOptions()
        options.setCameraId(0)
        options.setBeepEnabled(true)
        barcodeLauncher.launch(options)
    }

    override fun onClick(v: View?) {

        when (v?.id) {

            R.id.btnAddDevice -> {
                KeyboardUtils.forceCloseKeyboard(mBinding.root)
//                val request = AddDeviceRequest(
//                    addDeviceViewModel.deviceId.value,
//                    addDeviceViewModel.deviceTemplateGuid.value,
//                    addDeviceViewModel.deviceName.value,
//                    addDeviceViewModel.entityGuid.value
//                )
                if (addDeviceViewModel.checkIsValid()) {
                    addDeviceViewModel.getSingleDeviceDetial(addDeviceViewModel.deviceId.value.toString())
                }

            }
        }
    }
}