package com.avnet.ebv.view.fragment

import android.Manifest
import android.content.pm.PackageManager
import android.util.Log
import android.view.View
import androidx.core.content.ContextCompat
import androidx.drawerlayout.widget.DrawerLayout
import androidx.navigation.fragment.findNavController
import com.avnet.ebv.R
import com.avnet.ebv.databinding.FragmentBarcodeViewBinding
import com.avnet.ebv.view.activity.MainActivity

import com.google.zxing.BarcodeFormat
import com.google.zxing.ResultPoint
import com.journeyapps.barcodescanner.BarcodeCallback
import com.journeyapps.barcodescanner.BarcodeResult
import com.journeyapps.barcodescanner.DefaultDecoderFactory


class BarcodeViewFragment : BaseFragment(), View.OnClickListener {

    private var REQUEST_CAMERA = 1
    private lateinit var mBinding: FragmentBarcodeViewBinding

    override fun contentView(): Int = R.layout.fragment_barcode_view

    companion object {
        var scanBarcodeId = ""

    }

    override fun onResume() {

        (activity as MainActivity).tvTitle?.text = getString(R.string.add_device)
        (activity as MainActivity).btnAddDevice?.visibility = View.GONE
        (activity as MainActivity).btnMenu?.setImageDrawable(resources.getDrawable(R.drawable.ic_back))
        (activity as MainActivity).btnMenu?.setOnClickListener {
            (activity as MainActivity).navController.navigateUp()
        }
        (activity as MainActivity).drawerLayout?.setDrawerLockMode(DrawerLayout.LOCK_MODE_LOCKED_CLOSED)

        super.onResume()
    }

    override fun initViews() {

        mBinding = getBindingLayout() as FragmentBarcodeViewBinding
        mBinding.lifecycleOwner = this

        val formats: Collection<BarcodeFormat> = listOf(BarcodeFormat.CODE_39) // Set barcode type

        scanBarcodeId = ""

        mBinding.barcodeView.barcodeView.decoderFactory = DefaultDecoderFactory(formats)
        mBinding.barcodeView.initializeFromIntent(requireActivity().intent)
        mBinding.barcodeView.decodeContinuous(callback)

        mBinding.button.setOnClickListener(this)

        val cameraPermission: String = Manifest.permission.CAMERA
        val permissionCheck = ContextCompat.checkSelfPermission(requireActivity(), cameraPermission)
        if (permissionCheck == PackageManager.PERMISSION_GRANTED) {
            mBinding.barcodeView.resume()
        } else {
            requestPermissions(arrayOf(cameraPermission), REQUEST_CAMERA)
        }

    }

    override fun setTitle(): String {
        return ""
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<String>,
        grantResults: IntArray
    ) {
        if (requestCode == REQUEST_CAMERA) {
            for (i in permissions.indices) {
                if (permissions[i] == Manifest.permission.CAMERA && grantResults[i] == PackageManager.PERMISSION_GRANTED) {
                    mBinding.barcodeView.resume()
                }
            }
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        mBinding.barcodeView.pause()
    }

    private val callback: BarcodeCallback = object : BarcodeCallback {
        override fun barcodeResult(result: BarcodeResult) {
            Log.e("callback", result.text) // QR/Barcode result
            scanBarcodeId = result.text
            if (isAdded) {
                findNavController().navigateUp()
                //requireFragmentManager().popBackStack()
                //requireActivity().onBackPressed()
            }
        }

        override fun possibleResultPoints(resultPoints: List<ResultPoint>) {}
    }


    override fun loadData() {
    }

    override fun onClick(v: View?) {

        when (v?.id) {

            R.id.button -> {
                scanBarcodeId = "-1"
                findNavController().navigateUp()
//                requireFragmentManager().popBackStack()
                //requireActivity().onBackPressed()
            }
        }
    }

}