package com.avnet.ebv.view.fragment

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.pm.ActivityInfo
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.Spinner
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import com.avnet.ebv.R
import com.avnet.ebv.ViewModelFactory
import com.avnet.ebv.base.BaseFragment
import com.avnet.ebv.databinding.FragmentCommandBinding
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.model.request.ExecuteCommandRequest
import com.avnet.ebv.model.response.CommandHistoryDataItem
import com.avnet.ebv.model.response.CommandLookupDataItem
import com.avnet.ebv.model.response.DeviceListResponse
import com.avnet.ebv.utils.AppConstants
import com.avnet.ebv.utils.EndlessScrollRecyclerViewListener
import com.avnet.ebv.utils.hide
import com.avnet.ebv.utils.show
import com.avnet.ebv.view.activity.DeviceDetailActivity
import com.avnet.ebv.view.adater.CommandAdapter
import com.avnet.ebv.viewmodel.CommandViewModel
import javax.inject.Inject


class CommandFragment() : BaseFragment(), CommandAdapter.OnViewClickListener, View.OnClickListener {

    private val TAG: String = "CommandFragment"


    var bundle = Bundle()

    constructor(bundle: Bundle) : this() {
        this.bundle = bundle
    }

    private lateinit var mBinding: FragmentCommandBinding
    var deviceInfo: DeviceListResponse.Data? = null

    @Inject
    lateinit var commandViewModel: CommandViewModel

    @Inject
    lateinit var viewModelFactory: ViewModelFactory
    private lateinit var deviceListAdapter: CommandAdapter
    private var commandNameLookUpList: ArrayList<String> = ArrayList()
    private var commandLookupList: ArrayList<CommandLookupDataItem> = ArrayList()


    companion object {
    }

    private val mMessageReceiver: BroadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent) {
            commandViewModel._pageNumber.value = 1
            commandViewModel.callPagination = false
            commandViewModel.isApiCalled = false
            commandViewModel.propertyHistoryList.clear()
            deviceListAdapter.notifyDataSetChanged()
            commandViewModel.getCommandLookList(deviceInfo?.deviceTemplateGuid.toString())
            commandViewModel.getCommandHistoryList(deviceInfo?.guid.toString())
            setDeviceListAdapter()
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        mBinding = FragmentCommandBinding.inflate(layoutInflater)

        deviceInfo = bundle.get("Data") as DeviceListResponse.Data
        return mBinding.root
    }

    override fun initViewModel() {
        commandViewModel = viewModelFactory.create(CommandViewModel::class.java)
        mBinding.lifecycleOwner = this
        commandViewModel.initializeWithContext(requireContext())
        mBinding.commadData = commandViewModel
        commandViewModel.deviceInfo.value = deviceInfo

        mBinding.btnTwinCommand.setOnClickListener(this)
    }

    private fun setPagination() {
        mBinding.rvCommand.addOnScrollListener(object :
            EndlessScrollRecyclerViewListener() {
            override fun onLoadMore() {
                if (commandViewModel.callPagination) {
                    if (!commandViewModel.isApiCalled) {
                        commandViewModel.isApiCalled = true
                        commandViewModel.getCommandHistoryList(deviceInfo?.guid.toString())
                    }
                } else {
                }

            }
        })
    }

    private fun setDeviceListAdapter() {
        mBinding.rvCommand.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
        deviceListAdapter = CommandAdapter(commandViewModel.propertyHistoryList)
        deviceListAdapter.setOnViewClickListener(this)
        mBinding.rvCommand.adapter = deviceListAdapter



    }

    override fun observeViewModel() {

        commandViewModel._ProgreeBarState1.observe(this, Observer {
            when (it) {
                ProgressState.HIDE -> {
                    if ((commandViewModel.callPagination)) {

                        mBinding.pbProgress.hide()
                    } else {
                        dismissProgressDialog()
                    }
                }
                ProgressState.SHOW -> {
                    if (commandViewModel.callPagination) {

                        mBinding.pbProgress.show()
                    } else {
                        showProgressDialog()

                    }

                }
            }
        })


        commandViewModel._totalDeviceCount.observe(this, Observer {
            if (it == 0) {
                mBinding.noDataFound.show()
                mBinding.rvCommand.hide()

            } else {
                mBinding.rvCommand.show()
                mBinding.noDataFound.hide()
            }
        })

        commandViewModel._ProgreeBarState.observe(this, Observer {
            when (it) {
                ProgressState.HIDE -> dismissProgressDialog()
                ProgressState.SHOW -> showProgressDialog()
            }
        })

        commandViewModel.commandLookup.observe(this, Observer {

            commandNameLookUpList.clear()
            commandLookupList.clear()

            for (i in it.data?.data!!) {

                commandLookupList.add(i!!)
                commandNameLookUpList.add(i!!.name.toString())
            }
            setSpinnerCommand(mBinding.spSelectCommand)
        })

        commandViewModel.commandHistory.observe(this, Observer {

            if (it.data?.data != null) {
                commandViewModel.propertyHistoryList.addAll(it.data.data)
                deviceListAdapter.notifyDataSetChanged()
            }
        })


//        commandViewModel.refreshData.observe(this, Observer {
//
//            if (it.data?.data != null) {
//                propertyHistoryList.clear()
//                propertyHistoryList.addAll(it.data.data)
//                deviceListAdapter.notifyDataSetChanged()
//            }
//        })
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        clicks()
        commandViewModel.getCommandLookList(deviceInfo?.deviceTemplateGuid.toString())
        commandViewModel.getCommandHistoryList(deviceInfo?.guid.toString())
        setDeviceListAdapter()
        setPagination()

    }

    private fun clicks() {
        mBinding.btnTwinCommand.setOnClickListener(this)
    }

    //on resume
    override fun onResume() {
        super.onResume()
        Log.d(TAG, "onResume")
        requireActivity().registerReceiver(
            mMessageReceiver,
            IntentFilter(AppConstants.INTENT_REFRESH_COMMAND)
        )
    }

    override fun onPause() {
        super.onPause()
        Log.d(TAG, "onPause")
        requireActivity().unregisterReceiver(mMessageReceiver);
    }



    override fun onClick(v: View?) {

        when (v?.id) {

            R.id.btn_twin_command -> {
                if (commandLookupList.size>0 && commandViewModel.checkIsValid()) {

                    commandViewModel._pageNumber.value = 1
                    commandViewModel.callPagination = false
                    commandViewModel.isApiCalled = false
                    commandViewModel.propertyHistoryList.clear()
                    deviceListAdapter.notifyDataSetChanged()

                    val request = ExecuteCommandRequest(

                        commandLookupList[mBinding.spSelectCommand.selectedItemPosition].guid,
                        commandViewModel.desiredValue.value.toString()
                    )
                    commandViewModel.executeCommandApiCalL(deviceInfo?.guid.toString(), request)
                }
            }
        }
    }

    private fun setSpinnerCommand(s: Spinner) {

        val dataAdapter: ArrayAdapter<String> =
            ArrayAdapter<String>(
                requireContext(),
                R.layout.spinner_item_select,
                commandNameLookUpList
            )
        dataAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        s.adapter = dataAdapter

        s.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onItemSelected(
                parentView: AdapterView<*>?,
                selectedItemView: View?,
                position: Int,
                id: Long
            ) {


            }

            override fun onNothingSelected(parentView: AdapterView<*>?) {
            }
        }
    }



    override fun onItemClick(position: Int) {

    }

}
