package com.avnet.ebv.view.fragment

import android.content.Context
import android.content.res.Configuration
import android.graphics.Color
import android.graphics.Rect
import android.os.Bundle
import android.os.Handler
import android.util.DisplayMetrics
import android.util.Log
import android.view.*
import android.widget.LinearLayout
import android.widget.SeekBar
import androidx.core.content.ContextCompat
import androidx.core.text.isDigitsOnly
import androidx.core.widget.NestedScrollView
import androidx.lifecycle.Observer
import com.avnet.ebv.R
import com.avnet.ebv.ViewModelFactory
import com.avnet.ebv.base.BaseFragment
import com.avnet.ebv.common.SharedPref
import com.avnet.ebv.databinding.*
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.log.AndroidLog
import com.avnet.ebv.model.request.ExecuteCommandRequest
import com.avnet.ebv.model.response.LiveSensorDataResponse
import com.avnet.ebv.model.response.UserIdentityResponse
import com.avnet.ebv.utils.AppConstants
import com.avnet.ebv.view.activity.DeviceDetailActivity
import com.avnet.ebv.viewmodel.DataViewModel
import com.github.anastr.speedviewlib.components.Section
import com.github.mikephil.charting.charts.LineChart
import com.github.mikephil.charting.components.Legend
import com.github.mikephil.charting.components.XAxis
import com.github.mikephil.charting.data.Entry
import com.github.mikephil.charting.data.LineData
import com.github.mikephil.charting.data.LineDataSet
import com.github.mikephil.charting.formatter.IndexAxisValueFormatter
import com.github.mikephil.charting.interfaces.datasets.ILineDataSet
import com.google.gson.GsonBuilder
import com.microsoft.signalr.HubConnection
import com.microsoft.signalr.HubConnectionBuilder
import com.warkiz.widget.IndicatorSeekBar
import com.warkiz.widget.OnSeekChangeListener
import com.warkiz.widget.SeekParams
import kotlinx.android.synthetic.main.layout_line_chart.view.*
import org.apache.commons.lang3.StringUtils
import org.json.JSONObject
import java.math.RoundingMode
import java.text.DecimalFormat
import java.util.*
import javax.inject.Inject


class Data2Fragment(
    var uniqueId: String,
    var deviceGuidId: String,
    var guid1: String,
    var isConnect: Boolean,
    var isFromDevice: Boolean
) : BaseFragment() {

    private val TAG: String = "Data2Fragment"

    private lateinit var mBinding: FragmentData2Binding

    @Inject
    lateinit var viewModelFactory: ViewModelFactory

    @Inject
    lateinit var dataViewModel: DataViewModel

//    public lateinit var lineChart: LineChart

    var signalR: SignalRClient? = null

    lateinit var userDetails: UserIdentityResponse
    public lateinit var chartDataResponse: LiveSensorDataResponse

    val xAxisLabel: ArrayList<String> = ArrayList()
    var livedataSetsList: ArrayList<LineDataSet> = ArrayList()


    var guageList: ArrayList<String> = ArrayList()
    var sliderSensorList: ArrayList<String> = ArrayList()
    var btsensorList: ArrayList<String> = ArrayList()
    var ledSensorList: ArrayList<String> = ArrayList()
    var notConsiderSensorList: ArrayList<String> = ArrayList()

    var viewAddedList: ArrayList<View> = ArrayList()
    var bindingAddedList: ArrayList<Any> = ArrayList()

    var multipleLiveDataSets: HashMap<String, ArrayList<LineDataSet>> = HashMap()

    lateinit var llMain: LinearLayout


    lateinit var deviceDetailActivity: DeviceDetailActivity
    var landscapeViewTag: String = ""
    var scrollValue: Float = 0f


    override fun onAttach(context: Context) {
        super.onAttach(context)
        Log.d(TAG, "onAttach")
        deviceDetailActivity = (baseActivity as DeviceDetailActivity)
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?, savedInstanceState: Bundle?
    ): View {
        Log.d(TAG, "onCreateView")
        mBinding = FragmentData2Binding.inflate(layoutInflater)
        return mBinding.root
    }

    override fun initViewModel() {
        dataViewModel = viewModelFactory.create(DataViewModel::class.java)
        mBinding.setLifecycleOwner(this)
        mBinding.dataViewModel = dataViewModel
        dataViewModel.initializeWithContext(context!!)
        dataViewModel.deviceGuidId = deviceGuidId
        dataViewModel.guid1 = guid1
        dataViewModel.isConnect = isConnect
        dataViewModel.getTemplateAttributeList(deviceGuidId)
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        Log.d(TAG, "onViewCreated")

        deviceDetailActivity.isDeviceOnlineOffline(isConnect)

        llMain = mBinding.llMain

        if (SharedPref.instance!!.getValueFromPreferences(getString(R.string.pref_user_details)) != null) {
            userDetails = GsonBuilder().create().fromJson(
                SharedPref.instance!!.getValueFromPreferences(
                    getString(R.string.pref_user_details)
                ),
                UserIdentityResponse::class.java
            )
        }

//            lineChart = mBinding.root.findViewById(R.id.chartBar)
//            initGraph(lineChart)

        AndroidLog.d(
            "SignalR",
            "https://avnetsignalr.iotconnect.io/notificationhub?groupName=" + userDetails.data?.cpId + "-" + uniqueId
        )


        mBinding.nsvMain.setOnScrollChangeListener(NestedScrollView.OnScrollChangeListener { v, scrollX, scrollY, oldScrollX, oldScrollY ->
            if (scrollY > oldScrollY) {
                Log.i("TAG", "Scroll DOWN")
            }
            if (scrollY < oldScrollY) {
                Log.i("TAG", "Scroll UP")
            }
            if (scrollY == 0) {
                Log.i("TAG", "TOP SCROLL")
            }
            if (scrollY == v.measuredHeight - v.getChildAt(0).measuredHeight) {
                Log.i("TAG", "BOTTOM SCROLL")
            }

//                var handler: Handler = Handler(Looper.getMainLooper())
//                var myRunnable: Runnable = object : Runnable {
//                    override fun run() {
//
////                        dataViewModel.getTemplate(dataViewModel.guid1)
//                    }
//                }
            if (!dataViewModel.isConnect) {
                if (dataViewModel.isValueChanged) {


                    if ((dataViewModel.view != null) && (!view.isFullyVisible(
                            mBinding.nsvMain,
                            dataViewModel.view!!
                        ))
                    ) {
                        if (!view.isFullyVisible(mBinding.nsvMain, dataViewModel.view!!)) {
                            Log.i("TAG", "not visible")
                        }
                        dataViewModel._templateResponse.value =
                            dataViewModel._templateResponse.value
                        dataViewModel.isValueChanged = false
                        dataViewModel.view = null
                    };

                }

            }


        })


    }

    fun View.isFullyVisible(scrollView: NestedScrollView, view: View): Boolean {
//        val scrollBounds = Rect()
//        scrollView.getDrawingRect(scrollBounds)
//        val top = y
//        val bottom = top + height
//        return scrollBounds.top < top && scrollBounds.bottom > bottom

        val scrollBounds = Rect()
        scrollView.getHitRect(scrollBounds)
        if (view.getLocalVisibleRect(scrollBounds)) {
            return true
            // Any portion of the imageView, even a single pixel, is within the visible window
        } else {
            return false
            // NONE of the imageView is within the visible window
        }
    }


    override fun onResume() {
        super.onResume()
        Log.d(TAG, "onResume")
        val thread: Thread = object : Thread() {
            override fun run() {
                signalR = SignalRClient(
                    "https://avnetsignalr.iotconnect.io/notificationhub?groupName=" + userDetails.data?.cpId + "-" + uniqueId,
                    this@Data2Fragment
                )

                if (signalR != null) {
                    signalR!!.start()
                }
            }
        }

        thread.start()
    }

    override fun onStop() {
        if (signalR != null) {
            signalR!!.stop()
        }
        Log.d(TAG, "onStop")
        super.onStop()
    }

    private fun initGraph(lineChart: LineChart) {
        lineChart.description.isEnabled = false
//        lineChart.setMaxVisibleValueCount(5)
        lineChart.isDragEnabled = true
        lineChart.setScaleEnabled(false)
        lineChart.setPinchZoom(false)
        lineChart.enableScroll()
        lineChart.setDrawBorders(false)
        lineChart.setDrawGridBackground(false)
        lineChart.setBackgroundColor(ContextCompat.getColor(requireContext(), R.color.white))
//        lineChart.setMaxVisibleValueCount(5)
//        lineChart.xAxis.setLabelCount(5,false)
//        lineChart.setVisibleXRangeMaximum(5F)
//        lineChart.setVisibleXRange(5.0F, 5.0F)
        lineChart.isScrollContainer = true
        lineChart.isHorizontalScrollBarEnabled = true

        lineChart.xAxis.labelRotationAngle = 0F

        lineChart.xAxis.position = XAxis.XAxisPosition.BOTTOM
        lineChart.xAxis.isGranularityEnabled = true
        lineChart.xAxis.granularity = 2.0F
        lineChart.xAxis.setAvoidFirstLastClipping(true)
        lineChart.xAxis.setDrawLimitLinesBehindData(true)
        lineChart.axisRight.isEnabled = true
        lineChart.legend.isEnabled = true
        lineChart.legend.form = Legend.LegendForm.SQUARE
        lineChart.legend.formSize = 9f
        lineChart.legend.textSize = resources.getDimension(R.dimen._3sdp)
        lineChart.legend.xEntrySpace = 20f
        lineChart.legend.textColor = ContextCompat.getColor(requireContext(), R.color.black)
        lineChart.legend.horizontalAlignment = Legend.LegendHorizontalAlignment.CENTER
        lineChart.legend.verticalAlignment = Legend.LegendVerticalAlignment.BOTTOM
        lineChart.legend.setDrawInside(false)


        val xAxis: XAxis = lineChart.xAxis
        xAxis.position = XAxis.XAxisPosition.BOTTOM
        xAxis.setDrawGridLines(true)
        xAxis.granularity = 1f // only intervals of 1 day

        lineChart.axisRight.isEnabled = false

    }

    override fun observeViewModel() {

        dataViewModel._ProgreeBarState.observe(this, Observer {
            when (it) {
                ProgressState.HIDE -> dismissProgressDialog()
                ProgressState.SHOW -> showProgressDialog()
            }
        })
        dataViewModel._attributeList.observe(this, Observer {
            try {
                if (it != null) {
                    val rainbow = context!!.resources.getIntArray(R.array.rainbow)
                    val rainbow2 = context!!.resources.getIntArray(R.array.rainbow2)
                    var j: Int = 0
                    var l: Int = 0

                    for (i in 0 until it.size) {

                        if (((it.get(i).description?.trim().equals("", true)) &&
                                    checkDataTypeGraph(
                                        it.get(i).dataTypeName.toString()
                                    )) || ((it.get(i).description!!.contains(
                                AppConstants.sensorTypeGraph, true
                            )) && checkDataTypeGraph(it.get(i).dataTypeName.toString()))
                        ) {

                            livedataSetsList = ArrayList()

                            var binding: LayoutLineChart2Binding =
                                LayoutLineChart2Binding.inflate(layoutInflater)
                            binding.root.setTag(it.get(i).localName.toString())
                            binding.btnLandscape.setTag(it.get(i).localName.toString())

                            var xAxisLabel: ArrayList<String> = ArrayList()

                            var lineChart: LineChart
                            lineChart = binding.chartBar
                            initGraph(lineChart)


                            if (it.get(i).attributes != null && it.get(i).attributes!!.size > 0) {

                                for (k in 0 until it.get(i).attributes!!.size) {

                                    var values: ArrayList<Entry> = ArrayList()
                                    values = ArrayList()

                                    var set1: LineDataSet =
                                        LineDataSet(values, it.get(i).attributes.get(k).localName)
                                    set1.setDrawIcons(false)
                                    if (!it.get(i).attributes.get(k).attributeColor!!.isEmpty()) {
                                        set1.setCircleColor(
                                            Color.parseColor(
                                                it.get(i).attributes.get(
                                                    k
                                                ).attributeColor
                                            )
                                        )
                                    } else {
                                        set1.setCircleColor(rainbow2[l])
                                    }

//                                    set1.setCircleColor(rainbow2[l])
                                    set1.setDrawCircleHole(true)
                                    set1.setDrawValues(false)
                                    set1.lineWidth = 2F
                                    set1.setCubicIntensity(0.2f)
                                    set1.setDrawHorizontalHighlightIndicator(false)
                                    set1.setMode(LineDataSet.Mode.CUBIC_BEZIER)
                                    if (!it.get(i).attributes.get(k).attributeColor!!.isEmpty()) {
                                        set1.color =
                                            Color.parseColor(it.get(i).attributes.get(k).attributeColor)
                                    } else {
                                        set1.color = rainbow2[l]
                                    }
                                    livedataSetsList.add(set1)
                                    l++
                                    if (l == rainbow2.size) {
                                        l = 0
                                    }
                                }

                            } else {
                                var values: ArrayList<Entry> = ArrayList()
                                values = ArrayList()

                                var set1: LineDataSet = LineDataSet(values, it.get(i).localName)
                                set1.setDrawIcons(false)
                                if (!it.get(i).attributeColor!!.isEmpty()) {
                                    set1.setCircleColor(Color.parseColor(it.get(i).attributeColor))
                                } else {
                                    set1.setCircleColor(rainbow2[j])
                                }
                                set1.setDrawCircleHole(true)
                                set1.setDrawValues(false)
                                set1.lineWidth = 2F
                                set1.setCubicIntensity(0.2f)
                                set1.setDrawHorizontalHighlightIndicator(false)
                                set1.setMode(LineDataSet.Mode.CUBIC_BEZIER)
                                if (!it.get(i).attributeColor!!.isEmpty()) {
                                    set1.color = Color.parseColor(it.get(i).attributeColor)
                                } else {
                                    set1.color = rainbow[j]
                                }
                                livedataSetsList.add(set1)
                            }

                            j++
                            if (j == rainbow.size) {
                                j = 0
                            }

                            val params = LinearLayout.LayoutParams(
                                LinearLayout.LayoutParams.MATCH_PARENT,
                                LinearLayout.LayoutParams.WRAP_CONTENT
                            )
                            params.setMargins(0, 40, 0, 0)
                            binding.root.setLayoutParams(params)

                            binding.btnLandscape.setOnClickListener(View.OnClickListener {
                                landscapeViewTag = binding.btnLandscape.tag.toString()
                                deviceDetailActivity.changeOrientation()

                            })

                            var sensorName = getSubString(it.get(i).description!!)

                            if (sensorName.isEmpty()) {
//                                binding.tvTitle.setText(
//                                    StringUtils.capitalize(
//                                        it.get(i).localName.toString()
//                                    ) /*+ " Sensor"*/
//                                )
                                binding.tvTitle.setText(it.get(i).localName.toString())
                            } else {
//                                binding.tvTitle.setText(
//                                    StringUtils.capitalize(sensorName) /*+ " Sensor"*/
//                                )
                                binding.tvTitle.setText(sensorName)
                            }


                            llMain.addView(binding.root)
                            viewAddedList.add(binding.root)
                            bindingAddedList.add(binding)
                            multipleLiveDataSets.put(
                                it.get(i).localName.toString(), livedataSetsList
                            )

                        } else if ((checkFormatGuage(it.get(i).description!!)) && (it.get(i).description!!.contains(
                                AppConstants.sensorTypeGuage, true
                            )) && checkDataType(it.get(i).dataTypeName.toString())
                        ) {
                            guageList.add(it.get(i).localName.toString())

                            var desc = it.get(i).description

                            var result: String = StringUtils.substringBetween(desc, "[", "]")

                            result = result.replace(" ", "")

                            if (result.contains(AppConstants.sensorTypeGuage, true)) {
                                result = result.replace(AppConstants.sensorTypeGuage, "", true)
                            }

                            result = result.trim()
                            if (result.contains("-")) {
                                result = result.replace("-", ":")
                            }

                            var min: String = "0"
                            var max: String = "100"

                            if (result.contains(":")) {
                                var separated2 = result.split(":").toTypedArray()
                                min = separated2.get(0)
                                max = separated2.get(1)
                            }


                            var binding: LayoutGuageChartBinding =
                                LayoutGuageChartBinding.inflate(layoutInflater)


                            binding.root.setTag(it.get(i).localName.toString())
                            binding.chartSpeedView.maxSpeed = max.toFloat()
                            binding.chartSpeedView.minSpeed = min.toFloat()
                            binding.chartSpeedView.speedTo(min.toFloat(), 0)


                            if (isInteger(min.toDouble())) {
                                binding.tvLow.setText(min.toDouble().toInt().toString())
                            } else {
                                binding.tvLow.setText(roundOffDecimal(min.toDouble()).toString())
                            }
                            if (isInteger(max.toDouble())) {
                                binding.tvHigh.setText(max.toDouble().toInt().toString())
                            } else {
                                binding.tvHigh.setText(roundOffDecimal(max.toDouble()).toString())
                            }

                            val params = LinearLayout.LayoutParams(
                                LinearLayout.LayoutParams.MATCH_PARENT,
                                LinearLayout.LayoutParams.WRAP_CONTENT
                            )
                            params.setMargins(0, 40, 0, 0)
                            binding.root.setLayoutParams(params)


//                            var sensorName = it.get(i).description!!.substring(
//                                test(it.get(i).description!!) + 1,
//                                it.get(i).description!!.length
//                            )

                            var sensorName = getSubString(it.get(i).description!!)

                            if (sensorName.isEmpty()) {
//                                binding.tvTitle.setText(
//                                    StringUtils.capitalize(
//                                        it.get(i).localName.toString()
//                                    ) /*+ " Sensor"*/
//                                )
                                binding.tvTitle.setText(it.get(i).localName.toString())
                            } else {
//                                binding.tvTitle.setText(
//                                    StringUtils.capitalize(sensorName) /*+ " Sensor"*/
//                                )
                                binding.tvTitle.setText(sensorName)
                            }


                            llMain.addView(binding.root)
                            viewAddedList.add(binding.root)
                            bindingAddedList.add(binding)
                            setSpeedView(binding)

                        } else if ((checkFormatSlider(it.get(i).description!!)) && (it.get(i).description!!.contains(
                                AppConstants.sensorTypeSlider, true
                            )) && checkDataType(it.get(i).dataTypeName.toString())
                        ) {
                            sliderSensorList.add(it.get(i).localName.toString())

                            var desc = it.get(i).description

                            var result: String = StringUtils.substringBetween(desc, "[", "]")

                            result = result.replace(" ", "")


                            if (result.contains(AppConstants.sensorTypeSlider, true)) {
                                result = result.replace(AppConstants.sensorTypeSlider, "", true)
                            }

                            result = result.trim()
                            if (result.contains("-")) {
                                result = result.replace("-", ":")
                            }

                            var min: String = "0"
                            var max: String = "100"
                            var stepValue: Double = 0.0
                            var stepCount: Int = 0
                            var isStep: Boolean = false

                            if (result.contains(":")) {
                                var separated2 = result.split(":").toTypedArray()

                                if (separated2.size >= 3) {
                                    min = separated2.get(0)
                                    max = separated2.get(1)

                                    stepValue = separated2.get(2).toDouble()

                                    stepCount =
                                        (((max.toDouble() - min.toDouble()) / stepValue.toDouble()).toInt()) + 1
                                    isStep = true
                                } else {
                                    min = separated2.get(0)
                                    max = separated2.get(1)
                                    isStep = false
                                }

                            }

                            if(isInteger(min.toDouble()) && isInteger(max.toDouble())){
                                isStep=true
                            }else{
                                isStep=false
                            }



                            var binding: LayoutSliderSensorBinding =
                                LayoutSliderSensorBinding.inflate(layoutInflater)
                            binding.root.setTag(it.get(i).localName.toString())
                            binding.slider.max = Math.floor(max.toDouble()).toFloat()
                            binding.slider.min = Math.floor(min.toDouble()).toFloat()

                            stepValue=Math.floor(stepValue)

                            if (!isStep) {
                                stepValue= 0.0
//                                binding.slider.setTickCount(stepCount)
                            }

                            //setting here step value as tag to slider..
                            binding.slider.setTag(stepValue)
                            if (isInteger(min.toDouble())) {
                                binding.tvMin.setText(min.toDouble().toInt().toString())
                            } else {
                                binding.tvMin.setText(roundOffDecimal(Math.floor(min.toDouble())).toInt().toString())
                            }

                            if (isInteger(max.toDouble())) {
                                binding.tvMax.setText(max.toDouble().toInt().toString())
                            } else {
                                binding.tvMax.setText(roundOffDecimal(Math.floor(max.toDouble())).toInt().toString())
                            }

                            val params = LinearLayout.LayoutParams(
                                LinearLayout.LayoutParams.MATCH_PARENT,
                                LinearLayout.LayoutParams.WRAP_CONTENT
                            )
                            params.setMargins(0, 40, 0, 0)
                            binding.root.setLayoutParams(params)

                            var sensorName = getSubString(it.get(i).description!!)

                            if (sensorName.isEmpty()) {
//                                binding.tvTitle.setText(
//                                    StringUtils.capitalize(
//                                        it.get(i).localName.toString()
//                                    ) /*+ " Sensor"*/
//                                )

                                binding.tvTitle.setText(it.get(i).localName.toString())
                            } else {
//                                binding.tvTitle.setText(
//                                    StringUtils.capitalize(sensorName) /*+ " Sensor"*/
//                                )
                                binding.tvTitle.setText(sensorName)
                            }

                            binding.slider.isEnabled = false

                            llMain.addView(binding.root)
                            viewAddedList.add(binding.root)
                            bindingAddedList.add(binding)
                        } else if ((checkFormat2(it.get(i).description!!)) && (it.get(i).description!!.contains(
                                AppConstants.sensorTypeBT, true
                            )) && checkDataTypeForBtLed(it.get(i).dataTypeName.toString())
                        ) {
                            btsensorList.add(it.get(i).localName.toString())

                            var binding: LayoutBtSensorBinding =
                                LayoutBtSensorBinding.inflate(layoutInflater)
                            binding.root.setTag(it.get(i).localName.toString())
                            binding.ivSwitch.setTag(AppConstants.btOff)

                            val params = LinearLayout.LayoutParams(
                                LinearLayout.LayoutParams.MATCH_PARENT,
                                LinearLayout.LayoutParams.WRAP_CONTENT
                            )
                            params.setMargins(0, 40, 0, 0)
                            binding.root.setLayoutParams(params)

                            var sensorName = getSubString(it.get(i).description!!)

                            if (sensorName.isEmpty()) {
//                                binding.tvTitle.setText(
//                                    StringUtils.capitalize(
//                                        it.get(i).localName.toString()
//                                    ) /*+ " Sensor"*/
//                                )
                                binding.tvTitle.setText(it.get(i).localName.toString())
                            } else {
//                                binding.tvTitle.setText(
//                                    StringUtils.capitalize(sensorName)/* + " Sensor"*/
//                                )
                                binding.tvTitle.setText(sensorName)
                            }
                            llMain.addView(binding.root)
                            viewAddedList.add(binding.root)
                            bindingAddedList.add(binding)
                        } else if ((checkFormat2(it.get(i).description!!)) && (it.get(i).description!!.contains(
                                AppConstants.sensorTypeLed, true
                            )) && checkDataTypeForBtLed(it.get(i).dataTypeName.toString())
                        ) {
                            ledSensorList.add(it.get(i).localName.toString())

                            var binding: LayoutLedSensorBinding =
                                LayoutLedSensorBinding.inflate(layoutInflater)
                            binding.root.setTag(it.get(i).localName.toString())
                            val params = LinearLayout.LayoutParams(
                                LinearLayout.LayoutParams.MATCH_PARENT,
                                LinearLayout.LayoutParams.WRAP_CONTENT
                            )
                            params.setMargins(0, 40, 0, 0)
                            binding.root.setLayoutParams(params)

                            var sensorName = getSubString(it.get(i).description!!)

                            if (sensorName.isEmpty()) {
//                                binding.tvTitle.setText(
//                                    StringUtils.capitalize(
//                                        it.get(i).localName.toString()
//                                    ) /*+ " Sensor"*/
//                                )
                                binding.tvTitle.setText(it.get(i).localName.toString())
                            } else {
//                                binding.tvTitle.setText(
//                                    StringUtils.capitalize(sensorName) /*+ " Sensor"*/
//                                )
                                binding.tvTitle.setText(sensorName)
                            }

                            llMain.addView(binding.root)
                            viewAddedList.add(binding.root)
                            bindingAddedList.add(binding)
                        } /*else if (it.get(i).description!!.contains(
                                "notconsidersensor",
                                true
                            )) && ((it.get(i).dataTypeName.equals(
                                AppConstants.sensorDataTypeSTRING
                            ))
                        ) {*/ else if (it.get(i).dataTypeName.equals(
                                AppConstants.sensorDataTypeSTRING, true
                            )
                        ) {
                            notConsiderSensorList.add(it.get(i).localName.toString())

                            var binding: LayoutNoValueSensorBinding =
                                LayoutNoValueSensorBinding.inflate(layoutInflater)
                            binding.root.setTag(it.get(i).localName.toString())
                            val params = LinearLayout.LayoutParams(
                                LinearLayout.LayoutParams.MATCH_PARENT,
                                LinearLayout.LayoutParams.WRAP_CONTENT
                            )
                            params.setMargins(0, 40, 0, 0)
                            binding.root.setLayoutParams(params)

                            var sensorName = getSubString(it.get(i).description!!)

                            if (sensorName.isEmpty()) {
//                                binding.tvTitle.setText(
//                                    StringUtils.capitalize(
//                                        it.get(i).localName.toString()
//                                    ) /*+ " Sensor"*/
//                                )
                                binding.tvTitle.setText(it.get(i).localName.toString())
                            } else {
//                                binding.tvTitle.setText(
//                                    StringUtils.capitalize(sensorName)/* + " Sensor"*/
//                                )
                                binding.tvTitle.setText(sensorName)
                            }
                            llMain.addView(binding.root)
                            viewAddedList.add(binding.root)
                            bindingAddedList.add(binding)
                        }
                    }


                    if (viewAddedList.size > 0) {
                        mBinding.llMain.visibility = View.VISIBLE
                        mBinding.noDataFound.visibility = View.GONE
                    } else {
                        mBinding.llMain.visibility = View.GONE
                        mBinding.noDataFound.visibility = View.VISIBLE
                    }

                }

                baseActivity?.runOnUiThread {
                    Log.d("Tag", "tag")
                }


            } catch (e: Exception) {

                Log.d("Exception", e.printStackTrace().toString())

            }

        })

        dataViewModel._commandList.observe(this, Observer {

            if (it != null && it.size > 0) {
                //bt sensor
                if (btsensorList.size > 0) {

                    for (i in 0 until it.size) {
                        for (j in 0 until viewAddedList.size) {


                            var guid = it.get(i).guid
                            var name = it.get(i).name!!.trim()

//                            name = name!!.replace("set", "", true)
//                            name = name!!.trim()

                            if ((name.equals(
                                    "set" + viewAddedList.get(j).tag.toString().trim(), true
                                )) || (name.equals(
                                    "set " + viewAddedList.get(j).tag.toString().trim(), true
                                ))
                            ) {

                                for (k in 0 until btsensorList.size) {
                                    if ((name.equals(
                                            "set" + btsensorList.get(k), true
                                        )) || (name.equals("set " + btsensorList.get(k), true))
                                    ) {

                                        callClickOfBtSensor(
                                            bindingAddedList.get(j) as LayoutBtSensorBinding,
                                            guid.toString(),
                                            name
                                        )
                                    }
                                }
                            }

                        }
                    }

                }

                //led sensor
                if (ledSensorList.size > 0) {

                    for (i in 0 until it.size) {
                        for (j in 0 until viewAddedList.size) {

                            var guid = it.get(i).guid
                            var name = it.get(i).name!!.trim()
//                            name = name!!.replace("set", "", true)
//                            name = name.trim()


                            Log.d(
                                "tag", "name=$name" + "tag= ${viewAddedList.get(j).tag.toString()}"
                            )


                            if ((name.equals(
                                    "set" + viewAddedList.get(j).tag.toString().trim(), true
                                )) || (name.equals(
                                    "set " + viewAddedList.get(j).tag.toString().trim(), true
                                ))
                            ) {
                                for (k in 0 until ledSensorList.size) {
                                    if ((name.equals(
                                            "set" + ledSensorList.get(k), true
                                        )) || (name.equals("set " + ledSensorList.get(k), true))
                                    ) {
                                        callClickOfLedSensor(
                                            bindingAddedList.get(j) as LayoutLedSensorBinding,
                                            guid.toString(),
                                            name
                                        )
                                    }
                                }


                            }

                        }
                    }

                }

                //slider sensor
                if (sliderSensorList.size > 0) {
                    for (i in 0 until it.size) {
                        for (j in 0 until viewAddedList.size) {

                            var guid = it.get(i).guid
                            var name = it.get(i).name!!.trim()
//                            name = name!!.replace("set", "", true)
//                            name = name.trim()


                            if ((name.equals(
                                    "set" + viewAddedList.get(j).tag.toString().trim(), true
                                ))

                                ||

                                (name.equals(
                                    "set " + viewAddedList.get(j).tag.toString().trim(), true
                                ))
                            ) {

                                for (k in 0 until sliderSensorList.size) {
                                    if ((name.equals(
                                            "set" + sliderSensorList.get(k), true
                                        )) || (name.equals("set " + sliderSensorList.get(k), true))
                                    ) {

                                        callClickOfSliderSensor(
                                            bindingAddedList.get(j) as LayoutSliderSensorBinding,
                                            guid.toString(),
                                            name
                                        )
                                    }
                                }

                            }

                        }
                    }

                }
            }
        })

        dataViewModel._templateResponse.observe(this, Observer {
            if (it != null && it.data?.size!! > 0) {
                for (i in 0 until it.data!!.size) {

                    //btsensor
                    if (btsensorList.size > 0) {
                        for (k in 0 until btsensorList.size) {

                            if (it.data!!.get(i)!!.attributeName!!.equals(
                                    btsensorList.get(k), true
                                )
                            ) {

                                for (l in 0 until viewAddedList.size) {

                                    //bt
                                    if (btsensorList.get(k)
                                            .equals(viewAddedList.get(l).tag.toString(), true)
                                    ) {

                                        if (!it.data!!.get(i)!!.attributeValue?.isEmpty()!!) {


                                            if (isNumeric(it.data!!.get(i)!!.attributeValue!!.toString())) {
                                                if (it.data!!.get(i)!!.attributeValue?.toInt()!! == 0) {
                                                    (bindingAddedList.get(l) as LayoutBtSensorBinding).ivSwitch.setImageResource(
                                                        R.drawable.ic_off
                                                    )
                                                    (bindingAddedList.get(l) as LayoutBtSensorBinding).ivSwitch.setTag(
                                                        AppConstants.btOff
                                                    )
                                                } else {
                                                    (bindingAddedList.get(l) as LayoutBtSensorBinding).ivSwitch.setImageResource(
                                                        R.drawable.ic_on
                                                    )
                                                    (bindingAddedList.get(l) as LayoutBtSensorBinding).ivSwitch.setTag(
                                                        AppConstants.btOn
                                                    )

                                                }
                                            } else {
                                                if (it.data!!.get(i)!!.attributeValue?.equals(
                                                        "false", true
                                                    )!!
                                                ) {

                                                    (bindingAddedList.get(l) as LayoutBtSensorBinding).ivSwitch.setImageResource(
                                                        R.drawable.ic_off
                                                    )
                                                    (bindingAddedList.get(l) as LayoutBtSensorBinding).ivSwitch.setTag(
                                                        AppConstants.btOff
                                                    )
                                                } else {
                                                    (bindingAddedList.get(l) as LayoutBtSensorBinding).ivSwitch.setImageResource(
                                                        R.drawable.ic_on
                                                    )
                                                    (bindingAddedList.get(l) as LayoutBtSensorBinding).ivSwitch.setTag(
                                                        AppConstants.btOn
                                                    )
                                                }
                                            }

                                        }

                                    }

                                }
                            }
                        }
                    }


                    //ledsensor
                    if (ledSensorList.size > 0) {
                        for (k in 0 until ledSensorList.size) {

                            if (it.data!!.get(i)!!.attributeName!!.equals(
                                    ledSensorList.get(k), true
                                )
                            ) {
                                for (l in 0 until viewAddedList.size) {

                                    //bt
                                    if (ledSensorList.get(k).toString()
                                            .equals(viewAddedList.get(l).tag.toString(), true)
                                    ) {

                                        if (!it.data!!.get(i)!!.attributeValue?.isEmpty()!!) {


                                            if (isNumeric(it.data!!.get(i)!!.attributeValue!!.toString())) {

                                                if (it.data!!.get(i)!!.attributeValue?.toInt() == 0) {

                                                    if ((bindingAddedList.get(l) as LayoutLedSensorBinding).tvTitle.tag.toString()
                                                            .equals(AppConstants.disable, true)
                                                    ) {
                                                        (bindingAddedList.get(l) as LayoutLedSensorBinding).ivLed.setImageResource(
                                                            R.drawable.ic_diable_off
                                                        )
                                                    } else {
                                                        (bindingAddedList.get(l) as LayoutLedSensorBinding).ivLed.setImageResource(
                                                            R.drawable.ic_led_off
                                                        )
                                                    }

                                                    (bindingAddedList.get(l) as LayoutLedSensorBinding).ivLed.setTag(
                                                        AppConstants.ledOff
                                                    )

                                                } else {
                                                    if ((bindingAddedList.get(l) as LayoutLedSensorBinding).tvTitle.tag.toString()
                                                            .equals(AppConstants.disable, true)
                                                    ) {

                                                        (bindingAddedList.get(l) as LayoutLedSensorBinding).ivLed.setImageResource(
                                                            R.drawable.ic_disable_on
                                                        )

                                                    } else {
                                                        (bindingAddedList.get(l) as LayoutLedSensorBinding).ivLed.setImageResource(
                                                            R.drawable.ic_led_on
                                                        )
                                                    }

                                                    (bindingAddedList.get(l) as LayoutLedSensorBinding).ivLed.setTag(
                                                        AppConstants.ledOn
                                                    )

                                                }
                                            } else {
                                                if (it.data!!.get(i)!!.attributeValue?.equals(
                                                        "false", true
                                                    )!!
                                                ) {

                                                    if ((bindingAddedList.get(l) as LayoutLedSensorBinding).tvTitle.tag.toString()
                                                            .equals(AppConstants.disable, true)
                                                    ) {

                                                        (bindingAddedList.get(l) as LayoutLedSensorBinding).ivLed.setImageResource(
                                                            R.drawable.ic_diable_off
                                                        )

                                                    } else {
                                                        (bindingAddedList.get(l) as LayoutLedSensorBinding).ivLed.setImageResource(
                                                            R.drawable.ic_led_off
                                                        )
                                                    }

                                                    (bindingAddedList.get(l) as LayoutLedSensorBinding).ivLed.setTag(
                                                        AppConstants.ledOff
                                                    )

                                                } else {
                                                    if ((bindingAddedList.get(l) as LayoutLedSensorBinding).tvTitle.tag.toString()
                                                            .equals(AppConstants.disable, true)
                                                    ) {

                                                        (bindingAddedList.get(l) as LayoutLedSensorBinding).ivLed.setImageResource(
                                                            R.drawable.ic_disable_on
                                                        )

                                                    } else {

                                                        (bindingAddedList.get(l) as LayoutLedSensorBinding).ivLed.setImageResource(
                                                            R.drawable.ic_led_on
                                                        )
                                                    }
                                                    (bindingAddedList.get(l) as LayoutLedSensorBinding).ivLed.setTag(
                                                        AppConstants.ledOn
                                                    )

                                                }
                                            }

//                                            (bindingAddedList.get(l) as LayoutLedSensorBinding).tvTitle.setText(
//                                                StringUtils.capitalize((bindingAddedList.get(l) as LayoutLedSensorBinding).root.tag.toString())
//                                            )

                                        }

                                    }

                                }
                            }
                        }
                    }

                    //slider
                    if (sliderSensorList.size > 0) {
                        for (k in 0 until sliderSensorList.size) {

                            if (it.data!!.get(i)!!.attributeName!!.equals(
                                    sliderSensorList.get(k), true
                                )
                            ) {

                                for (l in 0 until viewAddedList.size) {

                                    //slider
                                    if (sliderSensorList.get(k)
                                            .equals(viewAddedList.get(l).tag.toString(), true)
                                    ) {

                                        if (!it.data!!.get(i)!!.attributeValue?.isEmpty()!!) {

                                            var value = it.data!!.get(i)!!.attributeValue
                                            value = value?.replace(",", "")
                                            Log.d("slider value", value.toString())
                                            (bindingAddedList.get(l) as LayoutSliderSensorBinding).slider.setProgress(
                                                value!!.toFloat()
                                            )
                                            if (isInteger(value.toDouble())) {

                                                (bindingAddedList.get(l) as LayoutSliderSensorBinding).tvProgress.setText(
                                                    value.toDouble().toInt().toString()
                                                )

                                            } else {

                                                (bindingAddedList.get(l) as LayoutSliderSensorBinding).tvProgress.setText(
                                                    roundOffDecimal(value.toDouble()).toString()
                                                )

                                            }


                                        }

                                    }

                                }
                            }
                        }
                    }

                    //guage
                    if (guageList.size > 0) {

                        for (k in 0 until guageList.size) {

                            if (it.data!!.get(i)!!.attributeName!!.equals(guageList.get(k), true)) {

                                for (l in 0 until viewAddedList.size) {

                                    //bt
                                    if (guageList.get(k)
                                            .equals(viewAddedList.get(l).tag.toString(), true)
                                    ) {

                                        if (!it.data!!.get(i)!!.attributeValue?.isEmpty()!!) {

                                            var value = it.data!!.get(i)!!.attributeValue
                                            value = value?.replace(",", "")

                                            (bindingAddedList.get(l) as LayoutGuageChartBinding).chartSpeedView.speedTo(
                                                value?.toFloat()!!
                                            )

                                            if (isInteger(value.toDouble())) {
                                                (bindingAddedList.get(l) as LayoutGuageChartBinding).tvSpeed.setText(
                                                    value.toDouble().toInt().toString()
                                                )
                                            } else {
                                                (bindingAddedList.get(l) as LayoutGuageChartBinding).tvSpeed.setText(
                                                    roundOffDecimal(value.toDouble()).toString()
                                                )
                                            }


                                        }

                                    }

                                }
                            }
                        }
                    }

                    //not consider sensor
                    if (notConsiderSensorList.size > 0) {
                        for (k in 0 until notConsiderSensorList.size) {
                            if (it.data!!.get(i)!!.attributeName!!.equals(
                                    notConsiderSensorList.get(
                                        k
                                    ), true
                                )
                            ) {

                                for (l in 0 until viewAddedList.size) {

                                    //not consider
                                    if (notConsiderSensorList.get(k)
                                            .equals(viewAddedList.get(l).tag.toString(), true)
                                    ) {

                                        if (!it.data!!.get(i)!!.attributeValue?.isEmpty()!!) {

                                            (bindingAddedList.get(l) as LayoutNoValueSensorBinding).tvValue.setText(
                                                it.data!!.get(i)!!.attributeValue?.toString()
                                            )

                                        }

                                    }

                                }
                            }
                        }
                    }

                }
            }
            dataViewModel.isSensorBeginValuesSet = true
        })

    }

    private fun callClickOfBtSensor(binding: LayoutBtSensorBinding, guid: String, name: String) {
        binding.tvTitle.setTag(AppConstants.enable)
        binding.ivSwitch.setTag(AppConstants.btOff)
        binding.ivSwitch.setImageResource(R.drawable.ic_off)
        binding.ivSwitch.setOnClickListener(View.OnClickListener {
            if (binding.ivSwitch.tag.toString().equals(AppConstants.btOn, true)) {
                val request = ExecuteCommandRequest(

                    guid, "0"
                )
                dataViewModel.executeCommandApiCall(guid1, request, binding, R.drawable.ic_off)
                binding.ivSwitch.setTag(AppConstants.btOff)
            } else {
                val request = ExecuteCommandRequest(

                    guid, "1"
                )
                dataViewModel.executeCommandApiCall(guid1, request, binding, R.drawable.ic_on)
                binding.ivSwitch.setTag(AppConstants.btOn)
            }

        })
    }

    private fun callClickOfLedSensor(binding: LayoutLedSensorBinding, guid: String, name: String) {
        binding.tvTitle.setTag(AppConstants.enable)
        binding.ivLed.setImageResource(R.drawable.ic_led_off)
        binding.ivLed.setTag(AppConstants.ledOff)
        binding.ivLed.setOnClickListener(View.OnClickListener {
            if (binding.ivLed.tag.equals(AppConstants.ledOn)) {
                val request = ExecuteCommandRequest(

                    guid, "0"
                )
                dataViewModel.executeCommandApiLedCall(
                    guid1, request, binding, R.drawable.ic_led_off
                )
                binding.ivLed.setTag(AppConstants.ledOff)
            } else {
                val request = ExecuteCommandRequest(

                    guid, "1"
                )
                dataViewModel.executeCommandApiLedCall(
                    guid1, request, binding, R.drawable.ic_led_on
                )
                binding.ivLed.setTag(AppConstants.ledOn)
            }

        })


    }

    private fun callClickOfSliderSensor(
        binding: LayoutSliderSensorBinding, guid: String, name: String
    ) {
        binding.slider.isEnabled = true
        binding.slider.onSeekChangeListener = object : OnSeekChangeListener {
            override fun onSeeking(seekParams: SeekParams?) {

            }

            override fun onStartTrackingTouch(seekBar: IndicatorSeekBar?) {

            }

            override fun onStopTrackingTouch(seekBar: IndicatorSeekBar?) {
                Log.d("progress",seekBar!!.progress.toString())
                //Logic of discrete step values according divide slider with step value..
                try{
                    if(!binding.slider.tag.toString().equals("0.0")){
                        if( binding.slider.progress!=binding.slider.max.toDouble().toInt()){
                            var count: Int = 0

                            while (!checkSeekBarProgress(
                                    count,
                                    binding.slider.progress,
                                    Math.floor(binding.slider.tag.toString().toDouble()).toInt(),
                                    binding.slider.min.toDouble().toInt()
                                )
                            ) {
                                count = count + 1
                            }
                            if (count > 0) {
                                count = count - 1
                            }
                            binding.slider.setProgress(
                                ((binding.slider.tag.toString().toDouble().toInt() * count)+binding.slider.min.toDouble().toInt()).toFloat())
                            binding.tvProgress.setText(binding.slider.progress.toString())
                        }else{
                            binding.slider.setProgress(binding.slider.max)
                            binding.tvProgress.setText(binding.slider.progress.toString())
                        }
                    }
                }catch (e:Exception){
                    Log.d("exception",e.printStackTrace().toString())
                }

                val request = ExecuteCommandRequest(

                    guid, binding.slider.progress.toString()
                )
                dataViewModel.executeCommandApiSliderCall(
                    guid1, request, binding, R.drawable.ic_led_off
                )

                binding.tvProgress.setText(binding.slider.progress.toString())

            }
        }


    }

    private fun checkSeekBarProgress(count: Int, progress: Int, stepvalue: Int,min:Int): Boolean {
        if (progress < ((stepvalue * count)+min)) {
            return true
        } else {
            return false
        }
    }


    class SignalRClient(url: String?, dataFragment: Data2Fragment) {
        var hubConnection: HubConnection
        var dataFragment: Data2Fragment
        private fun handleIncomingMethods() {

            this.hubConnection.on("Reset") { // OK
                Log.e("Reset", "Reset device")
            }

            this.hubConnection.on(
                "ReceiveMessage", { params: Any ->

                    if (dataFragment.dataViewModel.isSensorBeginValuesSet) {
                        Log.e("ReceiveMessage", params.toString())
                        val json = JSONObject(params.toString())
                        val message: JSONObject = json.get("message") as JSONObject
                        val data1: JSONObject = message.get("data") as JSONObject
                        val data2: JSONObject = data1.get("data") as JSONObject
                        if (data2.has("status")) {

                            if (data2.getString("status").equals("on", true)) {

//                            Log.d("StatusVahida","on")

                                dataFragment.deviceDetailActivity.isDeviceOnlineOffline(true)

                                dataFragment.dataViewModel.isConnect = true

                            } else if (data2.getString("status").equals("off", true)) {
//                            Log.d("StatusVahida","off")
                                dataFragment.deviceDetailActivity.isDeviceOnlineOffline(false)
                                dataFragment.dataViewModel.getTemplate(dataFragment.dataViewModel.guid1)
                                dataFragment.dataViewModel.isConnect = false

                            }
                        }


                        if (data2.has("reporting")) {

                            dataFragment.deviceDetailActivity.isDeviceOnlineOffline(true)

                            //live data 2
                            if (dataFragment.multipleLiveDataSets.size > 0) {

                                val json = JSONObject(params.toString())
                                val message: JSONObject = json.get("message") as JSONObject
                                val data1: JSONObject = message.get("data") as JSONObject
                                val data2: JSONObject = data1.get("data") as JSONObject
                                val reporting: JSONObject = data2.get("reporting") as JSONObject


                                dataFragment.chartDataResponse = GsonBuilder().create()
                                    .fromJson(params.toString(), LiveSensorDataResponse::class.java)

                                val currentString =
                                    dataFragment.chartDataResponse.message?.data?.data?.time.toString()
                                val separated = currentString.split(".").toTypedArray()
                                var label = ""
                                if (separated != null) {
                                    label = dataFragment.getDate(separated[0])
                                }

                                dataFragment.xAxisLabel.add(label)




                                for (j in 0 until dataFragment.viewAddedList.size) {

                                    if (dataFragment.multipleLiveDataSets.containsKey(
                                            dataFragment.viewAddedList.get(
                                                j
                                            ).tag.toString()
                                        )
                                    ) {

                                        (dataFragment.bindingAddedList.get(j) as LayoutLineChart2Binding).chartBar.xAxis.valueFormatter =
                                            IndexAxisValueFormatter(dataFragment.xAxisLabel)

                                        (dataFragment.bindingAddedList.get(j) as LayoutLineChart2Binding).chartBar.xAxis.labelCount =
                                            dataFragment.xAxisLabel.size


                                        if (reporting.get(dataFragment.viewAddedList.get(j).tag.toString()) is JSONObject) {

                                            val data: JSONObject =
                                                reporting.get(dataFragment.viewAddedList.get(j).tag.toString()) as JSONObject


                                            for (m in 0 until (dataFragment.multipleLiveDataSets.get(
                                                dataFragment.viewAddedList.get(j).tag.toString()
                                            )!!.size)) {

                                                var lineDataSet =
                                                    dataFragment.multipleLiveDataSets.get(
                                                        dataFragment.viewAddedList.get(j).tag.toString()
                                                    )!!.get(m)

                                                if (data.has(
                                                        dataFragment.multipleLiveDataSets.get(
                                                            dataFragment.viewAddedList.get(
                                                                j
                                                            ).tag.toString()
                                                        )!!.get(m).label
                                                    )
                                                ) {
                                                    val iter: Iterator<String> = data.keys()
                                                    while (iter.hasNext()) {
                                                        val key = iter.next()


                                                        if (key.equals(
                                                                dataFragment.multipleLiveDataSets.get(
                                                                    dataFragment.viewAddedList.get(
                                                                        j
                                                                    ).tag.toString()
                                                                )!!.get(m).label
                                                            )
                                                        ) {

                                                            dataFragment.multipleLiveDataSets.get(
                                                                dataFragment.viewAddedList.get(j).tag.toString()
                                                            )!!.get(m).addEntry(
                                                                Entry(
                                                                    (dataFragment.multipleLiveDataSets.get(
                                                                        dataFragment.viewAddedList.get(
                                                                            j
                                                                        ).tag.toString()
                                                                    )!!
                                                                        .get(m).values.size + 1).toFloat(),
                                                                    data.getString(key)
                                                                        .replace(",", "")
                                                                        .toFloat()
                                                                )
                                                            )

                                                        }

                                                    }

                                                }

                                            }

                                        } else {

                                            if (reporting.has(dataFragment.viewAddedList.get(j).tag.toString())) {

                                                dataFragment.multipleLiveDataSets.get(
                                                    dataFragment.viewAddedList.get(j).tag.toString()
                                                )?.get(0)?.addEntry(
                                                    Entry(
                                                        (dataFragment.multipleLiveDataSets.get(
                                                            dataFragment.viewAddedList.get(
                                                                j
                                                            ).tag.toString()
                                                        )!!.get(0).values.size + 1).toFloat(),
                                                        reporting.getString(
                                                            dataFragment.viewAddedList.get(
                                                                j
                                                            ).tag.toString()
                                                        ).replace(",", "").toFloat()
                                                    )
                                                )

                                            }
                                        }

                                        val data = LineData(
                                            dataFragment.multipleLiveDataSets.get(
                                                dataFragment.viewAddedList.get(
                                                    j
                                                ).tag.toString()
                                            )!! as List<ILineDataSet>?
                                        )
                                        data.setDrawValues(true)
                                        (dataFragment.bindingAddedList.get(j) as LayoutLineChart2Binding).chartBar.data =
                                            data

                                        (dataFragment.bindingAddedList.get(j) as LayoutLineChart2Binding).chartBar.setVisibleXRangeMaximum(
                                            3F
                                        );
                                        (dataFragment.bindingAddedList.get(j) as LayoutLineChart2Binding).chartBar.moveViewToX(
                                            dataFragment.livedataSetsList.get(0).values.size.toFloat()
                                        )
                                        (dataFragment.bindingAddedList.get(j) as LayoutLineChart2Binding).chartBar.notifyDataSetChanged()
                                    }


                                }

                            }

                            //guage
                            if (dataFragment.guageList.size > 0) {

                                val json = JSONObject(params.toString())
                                val message: JSONObject = json.get("message") as JSONObject
                                val data1: JSONObject = message.get("data") as JSONObject
                                val data2: JSONObject = data1.get("data") as JSONObject
                                val reporting: JSONObject = data2.get("reporting") as JSONObject

                                for (i in 0 until dataFragment.guageList.size) {

                                    for (j in 0 until dataFragment.viewAddedList.size) {

                                        if (dataFragment.guageList.get(i).equals(
                                                dataFragment.viewAddedList.get(j).tag.toString(),
                                                true
                                            )
                                        ) {
                                            Log.d(
                                                "label",
                                                (dataFragment.bindingAddedList.get(j) as LayoutGuageChartBinding).root.tag.toString()
                                            )
                                            if (reporting.has((dataFragment.bindingAddedList.get(j) as LayoutGuageChartBinding).root.tag.toString())) {

                                                dataFragment.baseActivity?.runOnUiThread {

                                                    Log.d(
                                                        "Guage min",
                                                        (dataFragment.bindingAddedList.get(j) as LayoutGuageChartBinding).root.tag.toString() + "  " + (dataFragment.bindingAddedList.get(
                                                            j
                                                        ) as LayoutGuageChartBinding).chartSpeedView.minSpeed.toString()
                                                    )
                                                    Log.d(
                                                        "Guage max",
                                                        (dataFragment.bindingAddedList.get(j) as LayoutGuageChartBinding).root.tag.toString() + "  " + (dataFragment.bindingAddedList.get(
                                                            j
                                                        ) as LayoutGuageChartBinding).chartSpeedView.maxSpeed.toString()
                                                    )

                                                    var value = reporting.getString(
                                                        (dataFragment.bindingAddedList.get(j) as LayoutGuageChartBinding).root.tag.toString()
                                                    )
                                                    value = value.replace(",", "")


                                                    (dataFragment.bindingAddedList.get(j) as LayoutGuageChartBinding).chartSpeedView.speedTo(
                                                        value.toFloat()
                                                    )
                                                    if (dataFragment.isInteger(value.toDouble())) {

                                                        (dataFragment.bindingAddedList.get(j) as LayoutGuageChartBinding).tvSpeed.setText(
                                                            value.toDouble().toInt().toString()
                                                        )

                                                    } else {

                                                        (dataFragment.bindingAddedList.get(j) as LayoutGuageChartBinding).tvSpeed.setText(
                                                            dataFragment.roundOffDecimal(
                                                                value.toDouble()
                                                            ).toString()
                                                        )

                                                    }

//                                            (dataFragment.bindingAddedList.get(j) as LayoutGuageChartBinding).tvTitle.setText(
//                                                org.apache.commons.lang3.StringUtils.capitalize(
//                                                    (dataFragment.bindingAddedList.get(
//                                                        j
//                                                    ) as LayoutGuageChartBinding).root.tag.toString() + " Sensor"
//                                                )
//                                            )
                                                }

                                            }
                                        }

                                    }
                                }


                            }

                            //slider data getting updated.
                            if (dataFragment.sliderSensorList.size > 0) {

                                val json = JSONObject(params.toString())
                                val message: JSONObject = json.get("message") as JSONObject
                                val data1: JSONObject = message.get("data") as JSONObject
                                val data2: JSONObject = data1.get("data") as JSONObject
                                val reporting: JSONObject = data2.get("reporting") as JSONObject

                                for (i in 0 until dataFragment.sliderSensorList.size) {
                                    for (j in 0 until dataFragment.viewAddedList.size) {

                                        if (dataFragment.sliderSensorList.get(i).equals(
                                                dataFragment.viewAddedList.get(j).tag.toString(),
                                                true
                                            )
                                        ) {


                                            Log.d(
                                                "label",
                                                (dataFragment.bindingAddedList.get(j) as LayoutSliderSensorBinding).root.tag.toString()
                                            )
                                            if (reporting.has((dataFragment.bindingAddedList.get(j) as LayoutSliderSensorBinding).root.tag.toString())) {

                                                dataFragment.baseActivity?.runOnUiThread {

                                                    var value = reporting.getString(
                                                        (dataFragment.bindingAddedList.get(j) as LayoutSliderSensorBinding).root.tag.toString()
                                                    )
                                                    value = value.replace(",", "")
                                                    Log.d("slider value", value)
                                                    (dataFragment.bindingAddedList.get(j) as LayoutSliderSensorBinding).slider.setProgress(
                                                        value.toFloat()
                                                    )


                                                    if (dataFragment.isInteger(value.toDouble())) {

                                                        (dataFragment.bindingAddedList.get(j) as LayoutSliderSensorBinding).tvProgress.setText(
                                                            value.toDouble().toInt().toString()
                                                        )

                                                    } else {

                                                        (dataFragment.bindingAddedList.get(j) as LayoutSliderSensorBinding).tvProgress.setText(
                                                            dataFragment.roundOffDecimal(value.toDouble())
                                                                .toString()
                                                        )
                                                    }


//                                            (dataFragment.bindingAddedList.get(j) as LayoutSliderSensorBinding).tvTitle.setText(
//                                                org.apache.commons.lang3.StringUtils.capitalize(
//                                                    (dataFragment.bindingAddedList.get(
//                                                        j
//                                                    ) as LayoutSliderSensorBinding).root.tag.toString()
//                                                )
//                                            )
                                                }

                                            }

                                        }
                                    }
                                }


                            }

                            //bt sensor
                            if (dataFragment.btsensorList.size > 0) {


                                val json = JSONObject(params.toString())
                                val message: JSONObject = json.get("message") as JSONObject
                                val data1: JSONObject = message.get("data") as JSONObject
                                val data2: JSONObject = data1.get("data") as JSONObject
                                val reporting: JSONObject = data2.get("reporting") as JSONObject

                                for (i in 0 until dataFragment.btsensorList.size) {
                                    for (j in 0 until dataFragment.viewAddedList.size) {

                                        if (dataFragment.btsensorList.get(i).equals(
                                                dataFragment.viewAddedList.get(j).tag.toString(),
                                                true
                                            )
                                        ) {

                                            Log.d(
                                                "label",
                                                (dataFragment.bindingAddedList.get(j) as LayoutBtSensorBinding).root.tag.toString()
                                            )
                                            if (reporting.has((dataFragment.bindingAddedList.get(j) as LayoutBtSensorBinding).root.tag.toString())) {

                                                dataFragment.baseActivity?.runOnUiThread {

                                                    if (dataFragment.isNumeric(
                                                            reporting.getString(
                                                                (dataFragment.bindingAddedList.get(
                                                                    j
                                                                ) as LayoutBtSensorBinding).root.tag.toString()
                                                            )
                                                        )
                                                    ) {

                                                        if (reporting.getString(
                                                                (dataFragment.bindingAddedList.get(
                                                                    j
                                                                ) as LayoutBtSensorBinding).root.tag.toString()
                                                            ).toInt() == 0
                                                        ) {

                                                            (dataFragment.bindingAddedList.get(j) as LayoutBtSensorBinding).ivSwitch.setImageResource(
                                                                R.drawable.ic_off
                                                            )
                                                            (dataFragment.bindingAddedList.get(j) as LayoutBtSensorBinding).ivSwitch.setTag(
                                                                AppConstants.btOff
                                                            )

                                                        } else {

                                                            (dataFragment.bindingAddedList.get(j) as LayoutBtSensorBinding).ivSwitch.setImageResource(
                                                                R.drawable.ic_on
                                                            )
                                                            (dataFragment.bindingAddedList.get(j) as LayoutBtSensorBinding).ivSwitch.setTag(
                                                                AppConstants.btOn
                                                            )

                                                        }
                                                    } else {
                                                        if (reporting.getString(
                                                                (dataFragment.bindingAddedList.get(
                                                                    j
                                                                ) as LayoutBtSensorBinding).root.tag.toString()
                                                            ).equals("false", true)
                                                        ) {

                                                            (dataFragment.bindingAddedList.get(j) as LayoutBtSensorBinding).ivSwitch.setImageResource(
                                                                R.drawable.ic_off
                                                            )
                                                            (dataFragment.bindingAddedList.get(j) as LayoutBtSensorBinding).ivSwitch.setTag(
                                                                AppConstants.btOff
                                                            )

                                                        } else {

                                                            (dataFragment.bindingAddedList.get(j) as LayoutBtSensorBinding).ivSwitch.setImageResource(
                                                                R.drawable.ic_on
                                                            )
                                                            (dataFragment.bindingAddedList.get(j) as LayoutBtSensorBinding).ivSwitch.setTag(
                                                                AppConstants.btOn
                                                            )

                                                        }
                                                    }


//                                            (dataFragment.bindingAddedList.get(j) as LayoutBtSensorBinding).tvTitle.setText(
//                                                org.apache.commons.lang3.StringUtils.capitalize(
//                                                    (dataFragment.bindingAddedList.get(
//                                                        j
//                                                    ) as LayoutBtSensorBinding).root.tag.toString()
//                                                )
//                                            )

                                                }

                                            }
                                        }

                                    }
                                }


                            }


                            //led data getting updated.
                            if (dataFragment.ledSensorList.size > 0) {


                                val json = JSONObject(params.toString())
                                val message: JSONObject = json.get("message") as JSONObject
                                val data1: JSONObject = message.get("data") as JSONObject
                                val data2: JSONObject = data1.get("data") as JSONObject
                                val reporting: JSONObject = data2.get("reporting") as JSONObject

                                for (i in 0 until dataFragment.ledSensorList.size) {
                                    for (j in 0 until dataFragment.viewAddedList.size) {

                                        if (dataFragment.ledSensorList.get(i).equals(
                                                dataFragment.viewAddedList.get(j).tag.toString(),
                                                true
                                            )
                                        ) {

                                            Log.d(
                                                "label",
                                                (dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).root.tag.toString()
                                            )
                                            if (reporting.has((dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).root.tag.toString())) {
                                                dataFragment.baseActivity?.runOnUiThread {

                                                    if (dataFragment.isNumeric(
                                                            reporting.getString(
                                                                (dataFragment.bindingAddedList.get(
                                                                    j
                                                                ) as LayoutLedSensorBinding).root.tag.toString()
                                                            )
                                                        )
                                                    ) {

                                                        if (reporting.getString(
                                                                (dataFragment.bindingAddedList.get(
                                                                    j
                                                                ) as LayoutLedSensorBinding).root.tag.toString()
                                                            ).toInt() == 0
                                                        ) {

                                                            if ((dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).tvTitle.tag.equals(
                                                                    AppConstants.disable
                                                                )
                                                            ) {

                                                                (dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).ivLed.setImageResource(
                                                                    R.drawable.ic_diable_off
                                                                )
                                                            } else {

                                                                (dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).ivLed.setImageResource(
                                                                    R.drawable.ic_led_off
                                                                )
                                                            }
                                                            (dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).ivLed.setTag(
                                                                AppConstants.ledOff
                                                            )

                                                        } else {

                                                            if ((dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).tvTitle.tag.toString()
                                                                    .equals(
                                                                        AppConstants.disable,
                                                                        true
                                                                    )
                                                            ) {

                                                                (dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).ivLed.setImageResource(
                                                                    R.drawable.ic_disable_on
                                                                )

                                                            } else {

                                                                (dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).ivLed.setImageResource(
                                                                    R.drawable.ic_led_on
                                                                )

                                                            }
                                                            (dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).ivLed.setTag(
                                                                AppConstants.ledOn
                                                            )

                                                        }
                                                    } else {
                                                        if (reporting.getString(
                                                                (dataFragment.bindingAddedList.get(
                                                                    j
                                                                ) as LayoutLedSensorBinding).root.tag.toString()
                                                            ).equals("false", true)
                                                        ) {

                                                            if ((dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).tvTitle.tag.toString()
                                                                    .equals(
                                                                        AppConstants.disable,
                                                                        true
                                                                    )
                                                            ) {

                                                                (dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).ivLed.setImageResource(
                                                                    R.drawable.ic_diable_off
                                                                )

                                                            } else {
                                                                (dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).ivLed.setImageResource(
                                                                    R.drawable.ic_led_off
                                                                )

                                                            }
                                                            (dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).ivLed.setTag(
                                                                AppConstants.ledOff
                                                            )

                                                        } else {
                                                            if ((dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).tvTitle.tag.toString()
                                                                    .equals(
                                                                        AppConstants.disable,
                                                                        true
                                                                    )
                                                            ) {

                                                                (dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).ivLed.setImageResource(
                                                                    R.drawable.ic_disable_on
                                                                )

                                                            } else {

                                                                (dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).ivLed.setImageResource(
                                                                    R.drawable.ic_led_on
                                                                )
                                                            }
                                                            (dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).ivLed.setTag(
                                                                AppConstants.ledOn
                                                            )

                                                        }
                                                    }


//                                            (dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).tvTitle.setText(
//                                                org.apache.commons.lang3.StringUtils.capitalize(
//                                                    (dataFragment.bindingAddedList.get(j) as LayoutLedSensorBinding).root.tag.toString()
//                                                )
//                                            )


                                                }

                                            }
                                        }

                                    }
                                }


                            }

                            if (dataFragment.notConsiderSensorList.size > 0) {


                                val json = JSONObject(params.toString())
                                val message: JSONObject = json.get("message") as JSONObject
                                val data1: JSONObject = message.get("data") as JSONObject
                                val data2: JSONObject = data1.get("data") as JSONObject
                                val reporting: JSONObject = data2.get("reporting") as JSONObject

                                for (i in 0 until dataFragment.notConsiderSensorList.size) {
                                    for (j in 0 until dataFragment.viewAddedList.size) {

                                        if (dataFragment.notConsiderSensorList.get(i).equals(
                                                dataFragment.viewAddedList.get(j).tag.toString(),
                                                true
                                            )
                                        ) {

                                            Log.d(
                                                "label",
                                                (dataFragment.bindingAddedList.get(j) as LayoutNoValueSensorBinding).root.tag.toString()
                                            )
                                            if (reporting.has((dataFragment.bindingAddedList.get(j) as LayoutNoValueSensorBinding).root.tag.toString())) {
                                                dataFragment.baseActivity?.runOnUiThread {
                                                    (dataFragment.bindingAddedList.get(j) as LayoutNoValueSensorBinding).tvValue.setText(
                                                        reporting.getString(
                                                            (dataFragment.bindingAddedList.get(j) as LayoutNoValueSensorBinding).root.tag.toString()
                                                        )
                                                    )
                                                }

                                            }
                                        }

                                    }
                                }


                            }
                        } else {
                            dataFragment.deviceDetailActivity.binding.toolbar.ivDeviceConnect.setColorFilter(
                                R.color.red
                            )
                        }
                    }


                }, Object::class.java
            )

        }


        fun start() {
            try {
                this.hubConnection.start().blockingAwait()
            } catch (e: Exception) {

            }

        }

        fun stop() {
            try {
                this.hubConnection.stop().blockingAwait()
            } catch (e: Exception) {

            }
        }

        init {
            hubConnection = HubConnectionBuilder.create(url).build()
            this.dataFragment = dataFragment
            handleIncomingMethods()
        }
    }

    private fun setSpeedView(layoutGuageChartBinding: LayoutGuageChartBinding) {
        try {
            layoutGuageChartBinding.chartSpeedView.sections.clear()
            var progress = 0F
            progress = 0f
            val section1 = Section(
                0f, 0.8f, ContextCompat.getColor(requireActivity(), R.color.color2AF12C)
            )

            val section3 = Section(
                0.8f, 1f, ContextCompat.getColor(requireActivity(), R.color.colorRedIndicator)
            )

            section3.width = 70F
            section1.width = 70F
            val sectionList = ArrayList<Section>()
            sectionList.add(section1)
            sectionList.add(section3)
            layoutGuageChartBinding.chartSpeedView.addSections(sectionList)

            layoutGuageChartBinding.chartSpeedView.sections[0].color =
                ContextCompat.getColor(requireActivity(), R.color.color2AF12C)

            layoutGuageChartBinding.chartSpeedView.sections[1].color =
                ContextCompat.getColor(requireActivity(), R.color.colorRedIndicator)

            layoutGuageChartBinding.chartSpeedView.speedTo(progress)

        } catch (e: Exception) {
            e.printStackTrace()
        }
    }

    override fun onPause() {
        if (signalR != null) {
            signalR!!.stop()
        }
        super.onPause()
        Log.d(TAG, "onPause")
    }

    override fun onDestroy() {
        if (signalR != null) {
            signalR!!.stop()
        }
        Log.d(TAG, "onDestroy")
        if (isFromDevice) {
            deviceDetailActivity.binding.toolbar.ivDeviceConnect.visibility = View.GONE
        }
        super.onDestroy()
    }

    override fun onConfigurationChanged(newConfig: Configuration) {
        super.onConfigurationChanged(newConfig)
        try {
            val displayMetrics = DisplayMetrics()
            baseActivity?.windowManager?.defaultDisplay?.getMetrics(displayMetrics)

            var width = displayMetrics.widthPixels
            var height = displayMetrics.heightPixels
            Log.d("TAG", "width " + width)
            Log.d("TAG", "height " + height)
            var height2: Int = height - (height / 2.8).toInt()
            Log.d("TAG", "height " + height2)

            val currentOrientation = resources.configuration.orientation
            if (currentOrientation == Configuration.ORIENTATION_LANDSCAPE) {
                Log.v("TAG", "Landscape !!!")
                val params = LinearLayout.LayoutParams(
                    LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.MATCH_PARENT
                )

                params.height = height2
                for (i in 0 until viewAddedList.size) {
                    if (viewAddedList.get(i).tag.equals(landscapeViewTag)) {
                        (bindingAddedList.get(i) as LayoutLineChart2Binding).chartBar.layoutParams =
                            params
                        Log.d("ScrollY", viewAddedList.get(i).y.toInt().toString())
                        scrollValue = viewAddedList.get(i).y
                    } else {
                        viewAddedList.get(i).visibility = View.GONE
                    }
                }

            } else {
                Log.v("TAG", "Portrait !!!")

                val params = LinearLayout.LayoutParams(
                    LinearLayout.LayoutParams.MATCH_PARENT, LinearLayout.LayoutParams.MATCH_PARENT
                )
                params.height = height / 2
                for (i in 0 until viewAddedList.size) {
                    if (viewAddedList.get(i).tag.equals(landscapeViewTag)) {
                        (bindingAddedList.get(i) as LayoutLineChart2Binding).chartBar.layoutParams =
                            params
                        Log.d("ScrollY", viewAddedList.get(i).y.toInt().toString())
                        mBinding.nsvMain.focusOnView(viewAddedList.get(i))
//                        mBinding.nsvMain.scrollY=viewAddedList.get(i).y.toInt()
                    } else {
                        viewAddedList.get(i).visibility = View.VISIBLE
                    }
                }

            }
        } catch (e: Exception) {

        }
    }

    fun NestedScrollView.focusOnView(toView: View) {

        Handler().post(Runnable {
            this.smoothScrollTo(0, toView.top)
        })

    }

    fun checkFormatGuage(value: String): Boolean {
        try {
            if ((value.contains(
                    AppConstants.sensorTypeGuage, true
                ))
            ) {
                if ((value.contains("[")) && (value.contains("]"))) {

                    var result: String = StringUtils.substringBetween(value, "[", "]")

                    result = result.replace(" ", "")

                    if (result.contains(AppConstants.sensorTypeGuage, true)) {
                        result = result.replace(AppConstants.sensorTypeGuage, "", true)
                    }

                    result = result.trim()
                    if (result.contains("-")) {
                        result = result.replace("-", ":")
                    }

                    if (result.contains(":")) {
                        val separated2 = result.split(":").toTypedArray()

                        if (separated2 != null && separated2.size == 2) {
                            Log.d("min max", (separated2.get(0)) + " " + separated2.get(1))

                            if ((!separated2.get(0).isEmpty()) && (!separated2.get(1).isEmpty())) {
                                if ((((separated2.get(0).isDigitsOnly()) && (separated2.get(1)
                                        .isDigitsOnly()))) || (isNumeric((separated2.get(0))) && isNumeric(
                                        (separated2.get(1))
                                    ))
                                ) {

                                    if (separated2.get(1).toDouble() > separated2.get(0)
                                            .toDouble()
                                    ) {
                                        return true
                                    } else {
                                        return false
                                    }

                                } else {
                                    return false
                                }
                            } else {
                                return false
                            }
                        } else {
                            return false
                        }
                    } else {
                        return false
                    }
                } else {
                    return false
                }
            } else {
                return false
            }
        } catch (e: Exception) {
            return false
        }
    }

    fun checkFormatSlider(value: String): Boolean {
        try {
            if ((value.contains(AppConstants.sensorTypeSlider, true))) {

                if ((value.contains("[")) && (value.contains("]"))) {

                    var result: String = StringUtils.substringBetween(value, "[", "]")

                    result = result.replace(" ", "")
                    if (result.contains(AppConstants.sensorTypeSlider, true)) {
                        result = result.replace(AppConstants.sensorTypeSlider, "", true)
                    }

                    result = result.trim()
                    if (result.contains("-")) {
                        result = result.replace("-", ":")
                    }

                    if (result.contains(":")) {
                        val separated2 = result.split(":").toTypedArray()

                        if (separated2 != null && (separated2.size == 2 || separated2.size >= 3)) {
                            Log.d("min max", (separated2.get(0)) + " " + separated2.get(1))

                            if (separated2.size >= 3) {
                                if ((!separated2.get(0).isEmpty()) && (!separated2.get(1)
                                        .isEmpty()) && (!separated2.get(2).isEmpty())
                                ) {

                                    if ((((separated2.get(0).isDigitsOnly()) && (separated2.get(1)
                                            .isDigitsOnly()) && (separated2.get(2)
                                            .isDigitsOnly()))) || (isNumeric((separated2.get(0))) && isNumeric(
                                            (separated2.get(
                                                1
                                            ))
                                        ) && isNumeric((separated2.get(2))))
                                    ) {

                                        if (
                                            (separated2.get(1).toDouble() > separated2.get(0).toDouble()) /*&&
                                            (separated2.get(2).toDouble() > separated2.get(0).toDouble()) */
                                            && (separated2.get(2).toDouble() < separated2.get(1).toDouble())
                                        ) {
                                            return true
                                        } else {
                                            return false
                                        }

                                    } else {
                                        return false
                                    }
                                } else {
                                    return false
                                }
                            } else {
                                if ((!separated2.get(0).isEmpty()) && (!separated2.get(1)
                                        .isEmpty())
                                ) {

                                    if ((((separated2.get(0).isDigitsOnly()) && (separated2.get(1)
                                            .isDigitsOnly()))) || (isNumeric((separated2.get(0))) && isNumeric(
                                            (separated2.get(1))
                                        ))
                                    ) {

                                        if (separated2.get(1).toDouble() > separated2.get(0)
                                                .toDouble()
                                        ) {
                                            return true
                                        } else {
                                            return false
                                        }

                                    } else {
                                        return false
                                    }
                                } else {
                                    return false
                                }
                            }
                        } else {
                            return false
                        }
                    } else {
                        return false
                    }
                } else {
                    return false
                }
            } else {
                return false
            }
        } catch (e: Exception) {
            Log.d("exception",e.printStackTrace().toString())
            return false
        }
    }

    fun isNumeric(str: String) = str.replace(".", "").all { it in '0'..'9' }

    fun checkFormat2(value: String): Boolean {
        try {
            if ((value.contains("[")) && (value.contains("]"))) {
                return true
            } else {
                return false
            }
        } catch (e: Exception) {
            return false
        }

    }

    fun checkDataType(dataType: String): Boolean {
        if ((dataType.equals(AppConstants.sensorDataTypeNumber, true)) || (dataType.equals(
                AppConstants.sensorDataTypeInteger, true
            )) || (dataType.equals(AppConstants.sensorDataTypeDecimal, true)) || (dataType.equals(
                AppConstants.sensorDataTypeLong, true
            ))
        ) {
            return true
        } else {
            return false
        }
    }

    fun checkDataTypeGraph(dataType: String): Boolean {
        if ((dataType.equals(AppConstants.sensorDataTypeNumber, true)) || (dataType.equals(
                AppConstants.sensorDataTypeInteger, true
            )) || (dataType.equals(AppConstants.sensorDataTypeDecimal, true)) || (dataType.equals(
                AppConstants.sensorDataTypeLong, true
            )) || (dataType.equals(AppConstants.sensorDataTypeOBJECT, true))
        ) {
            return true
        } else {
            return false
        }
    }

    fun checkDataTypeForBtLed(dataType: String): Boolean {
        if (/*(dataType.equals(AppConstants.sensorDataTypeBIT, true)) ||
            (dataType.equals(AppConstants.sensorDataTypeInteger, true)) ||*/
            (dataType.equals(AppConstants.sensorDataTypeNumber, true)) || (dataType.equals(
                AppConstants.sensorDataTypeBoolean, true
            ))
        ) {
            return true
        } else {
            return false
        }
    }

    fun isInteger(N: Double): Boolean {
        // Convert float value
        // of N to integer
        val X = N.toInt()
        val temp2 = N - X

        // If N is not equivalent
        // to any integer
        return if (temp2 > 0) {
            false
        } else true
    }

    fun roundOffDecimal(number: Double): Double {
        val df = DecimalFormat("#.##")
        df.roundingMode = RoundingMode.CEILING
        return df.format(number).toDouble()
    }

    fun test(expression: String): Int {

        var index: Int = -1
        for (i in 0 until expression.length) {
            val letter: Char = expression.get(i)
            if (letter == '[') {
                index = i
                break
            }
        }

        if (index != -1) {
            var i: Int
            if (expression[index] != '[') {
                print("-1\n")
                return -1
            }
            val st: Stack<Int> = Stack()
            i = index
            while (i < expression.length) {
                if (expression[i] == '[') {
                    st.push(expression[i].toInt())
                } else if (expression[i] == ']') {
                    st.pop()
                    if (st.empty()) {

                        return i
                    }
                }
                i++
            }
            return -1
        } else {
            return -1
        }


    }

    fun getSubString(expression: String): String {

        return expression.substringAfter("]").trim()

    }

}








