package com.avnet.ebv.view.fragment

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.appcompat.app.AlertDialog
import androidx.drawerlayout.widget.DrawerLayout
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout.OnRefreshListener
import com.avnet.ebv.R
import com.avnet.ebv.ViewModelFactory
import com.avnet.ebv.base.BaseFragment
import com.avnet.ebv.common.SharedPref
import com.avnet.ebv.data.remote.ApiConstant
import com.avnet.ebv.databinding.FragmentDeviceListBinding
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.utils.EndlessScrollRecyclerViewListener
import com.avnet.ebv.utils.NetworkUtils
import com.avnet.ebv.utils.hide
import com.avnet.ebv.utils.show
import com.avnet.ebv.view.activity.DeviceDetailActivity
import com.avnet.ebv.view.activity.MainActivity
import com.avnet.ebv.view.adater.DeviceListAdapter
import com.avnet.ebv.viewmodel.DeviceListViewModel
import javax.inject.Inject


class DeviceListFragment : BaseFragment(), DeviceListAdapter.OnViewClickListener {


    private val TAG: String = "DeviceListFragment"

    @Inject
    lateinit var viewModelFactory: ViewModelFactory

    @Inject
    lateinit var deviceViewModel: DeviceListViewModel

    private lateinit var mBinding: FragmentDeviceListBinding

    lateinit var deviceListAdapter: DeviceListAdapter

//    lateinit var mainActivity: MainActivity

    override fun onAttach(context: Context) {
        super.onAttach(context)

        Log.d(TAG, "onAttach")

//        mainActivity = (context as MainActivity)


    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        mBinding = FragmentDeviceListBinding.inflate(layoutInflater)
        return mBinding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)


        deviceViewModel.getUserIdentity()
        deviceViewModel.getDeviceList()
        deviceViewModel.getTemplate()

        setDeviceListAdapter()
        inits()
        clicks()


        mBinding.swipeRefereshLayout.setOnRefreshListener(OnRefreshListener {

            deviceViewModel._pageNumber.value = 1
            deviceViewModel.callPagination = false
            deviceViewModel.isApiCalled = false
            deviceViewModel.deviceList.clear()
            deviceListAdapter.notifyDataSetChanged()
            deviceViewModel.getDeviceList()
        })

        setPagination()


    }

    override fun initViewModel() {
        deviceViewModel = viewModelFactory.create(DeviceListViewModel::class.java)
        mBinding.lifecycleOwner = this
        deviceViewModel.initializeWithContext(requireContext())
    }

    override fun observeViewModel() {

        deviceViewModel._ProgreeBarState.observe(this, Observer {
            when (it) {
                ProgressState.HIDE -> {
                    if ((!mBinding.swipeRefereshLayout.isRefreshing) && (deviceViewModel.callPagination)) {

                        mBinding.pbProgress.hide()
                    } else {
                        dismissProgressDialog()
                    }
                }
                ProgressState.SHOW -> {
                    if ((!mBinding.swipeRefereshLayout.isRefreshing) && (deviceViewModel.callPagination)) {

                        mBinding.pbProgress.show()
                    } else {
                        if ((!mBinding.swipeRefereshLayout.isRefreshing)) {
                            showProgressDialog()
                        }

                    }

                }
            }
        })

        deviceViewModel.deviceResponse.observe(this, Observer {
            mBinding.swipeRefereshLayout.isRefreshing = false
            deviceViewModel.deviceList.addAll(it.data!!.filter { it?.deviceStatus.toString().equals("Acquired",true) })
            deviceListAdapter.notifyDataSetChanged()
        })

        deviceViewModel.getUserIdentityResponse.observe(this, Observer {

        })

        deviceViewModel._totalDeviceCount.observe(this, Observer {
            if (it == 0) {
                mBinding.noDataFound.show()
                mBinding.rvDevice.hide()
                mBinding.swipeRefereshLayout.isRefreshing = false
            } else {
                mBinding.rvDevice.show()
                mBinding.noDataFound.hide()
            }
        })

        deviceViewModel._templateResponse.observe(this, Observer {
            if (it.data?.size!! > 0) {
                for (i in 0 until it.data!!.size) {
                    if (it.data!!.get(i)?.name.equals(ApiConstant.EBV_MOBILE)) {
                        var guid: String = it.data?.get(i)?.guid.toString()
                        if (!guid.isEmpty()) {
                            SharedPref.instance!!.saveStringSession(
                                getString(R.string.deviceTemplateGuid),
                                guid
                            )
                            Log.d(
                                "MainGuid",
                                SharedPref.instance?.getValueFromPreferences(getString(R.string.deviceTemplateGuid).toString())!!
                            )

                        } else {
                            SharedPref.instance!!.saveStringSession(
                                getString(R.string.deviceTemplateGuid),
                                ApiConstant.deviceTemplateGuid
                            )
                            Log.d(
                                "MainGuid1",
                                SharedPref.instance?.getValueFromPreferences(getString(R.string.deviceTemplateGuid).toString())!!
                            )
                        }
                    }
                }
            }
        })
    }

    private fun setPagination() {
        mBinding.rvDevice.addOnScrollListener(object :
            EndlessScrollRecyclerViewListener() {
            override fun onLoadMore() {
                if (deviceViewModel.callPagination) {
                    if (!deviceViewModel.isApiCalled) {
                        deviceViewModel.isApiCalled = true
                        deviceViewModel.getDeviceList()
                    }
                } else {
                }
            }
        })
    }

    private fun inits() {
        (activity as MainActivity).tvTitle?.text = getString(R.string.devices)
    }

    private fun clicks() {
        (activity as MainActivity).btnAddDevice?.setOnClickListener {
            (activity as MainActivity).navController.navigate(R.id.action_nav_home_to_nav_add_device)
        }
    }

    private fun setDeviceListAdapter() {
        mBinding.rvDevice.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
        deviceListAdapter = DeviceListAdapter(deviceViewModel.deviceList)
        deviceListAdapter.setOnViewClickListener(this)
        mBinding.rvDevice.adapter = deviceListAdapter
    }

    override fun onItemClick(position: Int, view: View) {
        if (view.id == R.id.llMain) {
            deviceListAdapter.notifyDataSetChanged()
            if (NetworkUtils.isNetworkAvailable(requireContext())) {
                val intent = Intent(requireContext(), DeviceDetailActivity::class.java)
                intent.putExtra("Data", deviceViewModel.deviceList[position])

                requireContext().startActivity(intent)
            } else {
                deviceViewModel.showMsg(getString(R.string.str_alert_no_internet))
            }
        } else if (view.id == R.id.rlDelete) {
            val dialogBuilder = AlertDialog.Builder(requireActivity())

            // set message of alert dialog
            dialogBuilder.setMessage(getString(R.string.alert_delete_devidce) + " " + deviceViewModel.deviceList[position]!!.displayName!! + " ?")
                // if the dialog is cancelable
                .setCancelable(false)
                // positive button text and action
                .setPositiveButton(getString(R.string.text_yes)) { dialog, id ->
                    dialog.dismiss()
//                    deviceViewModel.getSingleDeviceDetial(deviceViewModel.deviceList[position]!!.uniqueId!!)
                    deviceViewModel.getDeviceAllocatedList(
                        deviceListAdapter,
                        position,
                        deviceViewModel.deviceList[position]!!.guid!!,
                        SharedPref.instance!!.getValueFromPreferences(getString(R.string.pref_root_guid)).toString()
                    )
                }
                .setNegativeButton(getString(R.string.text_no)) { dialog, id ->
                    dialog.dismiss()
                    deviceListAdapter.viewBinderHelper.closeLayout(position.toString())
//                    deviceListAdapter.notifyItemChanged(position)
                }


            // create dialog box
            val alert = dialogBuilder.create()
            // set title for alert dialog box
            alert.setTitle(R.string.app_name)
            // show alert dialog
            alert.show()
//            alert.getButton(AlertDialog.BUTTON_POSITIVE)
//                .setTextColor(ContextCompat.getColor(requireContext(), R.color.red));

        }


    }

    override fun onStart() {
        super.onStart()
        Log.d(TAG, "onStart")
    }

    override fun onResume() {
        super.onResume()
        Log.d(TAG, "onResume")
        (activity as MainActivity).setSelectedMenuItem(1)
        (activity as MainActivity).tvTitle?.text = getString(R.string.devices)
        (activity as MainActivity).btnAddDevice?.visibility = View.VISIBLE
        (activity as MainActivity).btnAddDevice?.setImageDrawable(resources.getDrawable(R.drawable.ic_add_device))
        (activity as MainActivity).btnMenu?.setImageDrawable(resources.getDrawable(R.drawable.ic_menu))
        (activity as MainActivity).btnMenu?.setOnClickListener {
            (activity as MainActivity).drawerLayout?.openDrawer(Gravity.LEFT)
        }
        (activity as MainActivity).drawerLayout?.setDrawerLockMode(DrawerLayout.LOCK_MODE_UNLOCKED)


    }

    override fun onStop() {

        Log.d(TAG, "onStop")
        super.onStop()
    }

    override fun onPause() {

        super.onPause()
        Log.d(TAG, "onPause")
    }

    override fun onDestroy() {

        Log.d(TAG, "onDestroy")
        super.onDestroy()
    }
}