package com.avnet.ebv.view.fragment

import android.content.BroadcastReceiver
import android.content.Context
import android.content.Intent
import android.content.IntentFilter
import android.content.pm.ActivityInfo
import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.Spinner
import androidx.lifecycle.Observer
import androidx.recyclerview.widget.LinearLayoutManager
import com.avnet.ebv.R
import com.avnet.ebv.ViewModelFactory
import com.avnet.ebv.base.BaseFragment
import com.avnet.ebv.common.SharedPref
import com.avnet.ebv.databinding.FragmentPropertiesBinding
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.model.request.ExecutePropertyRequest
import com.avnet.ebv.model.response.DeviceListResponse
import com.avnet.ebv.model.response.PropertyHistoryDataItem
import com.avnet.ebv.model.response.PropertyLookupDataItem
import com.avnet.ebv.model.response.UserIdentityResponse
import com.avnet.ebv.utils.AppConstants
import com.avnet.ebv.view.activity.DeviceDetailActivity
import com.avnet.ebv.view.adater.PropertyListAdapter
import com.avnet.ebv.viewmodel.PropertiesViewModel
import kotlinx.android.synthetic.main.nav_header_main.view.*
import javax.inject.Inject


class PropertiesFragment() : BaseFragment(), PropertyListAdapter.OnViewClickListener,
    PropertyListAdapter.OnLoadMoreInterface, View.OnClickListener {

    private val TAG: String = "PropertiesFragment"


    private lateinit var mBinding: FragmentPropertiesBinding

    @Inject
    lateinit var propertiesViewModel: PropertiesViewModel

    @Inject
    lateinit var viewModelFactory: ViewModelFactory
    private lateinit var deviceListAdapter: PropertyListAdapter
    private var propertyNameLookUpList: ArrayList<String> = ArrayList()
    private var propertyLocalNameLookUpList: ArrayList<String> = ArrayList()
    private var propertyHistoryList: ArrayList<PropertyHistoryDataItem?> = ArrayList()
    private var propertyLookupList: ArrayList<PropertyLookupDataItem> = ArrayList()

    var bundle = Bundle()
    var deviceInfo: DeviceListResponse.Data? = null

    constructor(bundle: Bundle) : this() {
        this.bundle = bundle
    }

    companion object {
    }

    private val mMessageReceiver: BroadcastReceiver = object : BroadcastReceiver() {
        override fun onReceive(context: Context?, intent: Intent) {
            propertiesViewModel.getPropertyLookList(deviceInfo?.deviceTemplateGuid.toString())
            propertiesViewModel.getPropertyHistoryList((activity as DeviceDetailActivity).deviceInfo?.uniqueId!!)
            setDeviceListAdapter()
        }
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        mBinding = FragmentPropertiesBinding.inflate(layoutInflater)

        deviceInfo = bundle.get("Data") as DeviceListResponse.Data
        return mBinding.root
    }

    override fun initViewModel() {
        mBinding.btnTwinProperty.setOnClickListener(this)
        propertiesViewModel = viewModelFactory.create(PropertiesViewModel::class.java)
        mBinding.lifecycleOwner = this
        propertiesViewModel.initializeWithContext(requireContext())
        mBinding.propertiesData = propertiesViewModel
        propertiesViewModel.deviceInfo.value = deviceInfo
        propertiesViewModel.getPropertyLookList(deviceInfo?.deviceTemplateGuid.toString())
        propertiesViewModel.getPropertyHistoryList((activity as DeviceDetailActivity).deviceInfo?.uniqueId!!)
        setDeviceListAdapter()

    }


    private fun setSpinnerTwinProperty(s: Spinner) {

        val dataAdapter: ArrayAdapter<String> =
            ArrayAdapter<String>(
                requireContext(),
                R.layout.spinner_item_select,
                propertyNameLookUpList
            )
        dataAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        s.adapter = dataAdapter

        s.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onItemSelected(
                parentView: AdapterView<*>?,
                selectedItemView: View?,
                position: Int,
                id: Long
            ) {


            }

            override fun onNothingSelected(parentView: AdapterView<*>?) {
            }
        }
    }


    private fun setDeviceListAdapter() {
        mBinding.rvProperty.layoutManager =
            LinearLayoutManager(requireContext(), LinearLayoutManager.VERTICAL, false)
        deviceListAdapter =
            PropertyListAdapter(
                propertyHistoryList, mBinding.rvProperty, propertyHistoryList.size,
                context!!
            )
        deviceListAdapter.setOnViewClickListener(this)
        deviceListAdapter.setOnLoadMoreListener(this)
        mBinding.rvProperty.adapter = deviceListAdapter
        deviceListAdapter.setLoaded(true)


    }

    override fun observeViewModel() {
        propertiesViewModel._ProgreeBarState.observe(this, Observer {
            when (it) {
                ProgressState.HIDE -> dismissProgressDialog()
                ProgressState.SHOW -> showProgressDialog()
            }
        })

        propertiesViewModel.twinPropertyLookup.observe(this, Observer {
            propertyNameLookUpList.clear()
            for (i in it.data?.data!!) {
                propertyLookupList.add(i!!)
                propertyNameLookUpList.add(i.name.toString())
            }
            setSpinnerTwinProperty(mBinding.spSelectProperty)
        })

        propertiesViewModel.twinPropertyHistory.observe(this, Observer {

            if (it.data?.data != null) {
                propertyHistoryList.clear()
                propertyHistoryList.addAll(it.data.data)
                deviceListAdapter.notifyDataSetChanged()
            }
        })

    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

    }

    //on resume
    override fun onResume() {
        super.onResume()
        Log.d(TAG, "onResume")
        requireActivity().registerReceiver(
            mMessageReceiver,
            IntentFilter(AppConstants.INTENT_REFRESH_PROPERTIES)
        )
    }

    override fun onPause() {
        super.onPause()
        Log.d(TAG, "onPause")
        requireActivity().unregisterReceiver(mMessageReceiver);
    }

    override fun onItemClick(position: Int) {

    }

    override fun onLoadMore() {

    }

    override fun onClick(v: View?) {

        when (v?.id) {

            R.id.btn_twin_property -> {
                if (propertyLookupList.size > 0 && propertiesViewModel.checkIsValid()) {
                    val request = ExecutePropertyRequest(
                        propertyLookupList[mBinding.spSelectProperty.selectedItemPosition].localName,
                        propertiesViewModel.desiredValue.value.toString(),
                        propertyLookupList[mBinding.spSelectProperty.selectedItemPosition].guid
                    )
                    propertiesViewModel.executePropertyApi(
                        deviceInfo!!.uniqueId.toString(),
                        request
                    )
                }else{
                    propertiesViewModel.showMsg(getString(R.string.alert_no_properties_available))
                }
            }
        }
    }


}
