package com.avnet.ebv.view.fragment

import android.os.Bundle
import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.Observer
import com.avnet.ebv.R
import com.avnet.ebv.ViewModelFactory
import com.avnet.ebv.base.BaseFragment
import com.avnet.ebv.common.SharedPref
import com.avnet.ebv.databinding.FragmentStatisticsBinding
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.model.response.UserIdentityResponse
import com.avnet.ebv.utils.AppConstants
import com.avnet.ebv.view.activity.MainActivity
import com.avnet.ebv.viewmodel.StatisticViewModel
import com.google.gson.GsonBuilder
import java.text.DecimalFormat
import javax.inject.Inject
import kotlin.math.roundToInt


class StatisticFragment : BaseFragment() {
    private lateinit var mBinding: FragmentStatisticsBinding

    @Inject
    lateinit var statisticViewModel: StatisticViewModel

    @Inject
    lateinit var viewModelFactory: ViewModelFactory

    private val df: DecimalFormat = DecimalFormat("0")

    companion object {
    }

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        mBinding = FragmentStatisticsBinding.inflate(layoutInflater)
        return mBinding.root
    }

    override fun initViewModel() {

        statisticViewModel = viewModelFactory.create(StatisticViewModel::class.java)
        mBinding.setLifecycleOwner(this)
        mBinding.statisticViewModel = statisticViewModel
        statisticViewModel.initializeWithContext(context!!)

    }

    override fun observeViewModel() {
        statisticViewModel._ProgreeBarState.observe(this, Observer {
            when (it) {
                ProgressState.HIDE -> dismissProgressDialog()
                ProgressState.SHOW -> showProgressDialog()
            }
        })

        statisticViewModel.plan.observe(this, Observer {


            for (i in 0 until (it.data!!.size)) {

                for(j in 0 until (it.data!!.get(i)?.features?.size ?: 0)){

                    if (it.data?.get(i)?.features?.get(j)?.featureName.equals(AppConstants.Users)) {

                        if(it.data?.get(i)?.features?.get(j)?.featureValue?.toDouble()!! > statisticViewModel.userTotal2.value!!){
                            statisticViewModel.userTotal.value =
                                it.data?.get(i)?.features?.get(j)?.featureValue
                            statisticViewModel.userTotal2.value= it.data?.get(i)?.features?.get(j)?.featureValue?.toDouble()
                        }

                    }
                    if (it.data?.get(i)?.features?.get(j)?.featureName.equals(AppConstants.Devices)) {
                        if(it.data?.get(i)?.features?.get(j)?.featureValue?.toDouble()!! > statisticViewModel.deviceTotal2.value!!){
                            statisticViewModel.deviceTotal.value =
                                it.data?.get(i)?.features?.get(j)?.featureValue
                            statisticViewModel.deviceTotal2.value= it.data?.get(i)?.features?.get(j)?.featureValue?.toDouble()
                        }

                    }
                }

                if(it.data?.get(i)?.totalAvailableMessages?.toDouble()!! > statisticViewModel.messageTotal2.value!!){
                    statisticViewModel.messageTotal.value = it.data!!.get(i)?.totalAvailableMessages.toString()
                    statisticViewModel.messageTotal2.value= it.data?.get(i)?.totalAvailableMessages!!.toDouble()
                }

            }



        })

        statisticViewModel.subscribe.observe(this, Observer {

            mBinding.txtPlan.setText(it.planName)

            mBinding.txtUsers.setText(it.noOfUsers.toString() + "/" + statisticViewModel.userTotal.value)
            mBinding.txtDevices.setText(it.noOfDevices.toString() + "/" + statisticViewModel.deviceTotal.value)
            var percentage =
                ((it.totalMessage?.toLong()!! * 100.0) / statisticViewModel.messageTotal.value.toString()
                    .toLong())
            Log.d("percentage", percentage.toString())
            mBinding.txtMessages.setText(
                formatNumber(it.totalMessage?.toLong()!!) + "/" + formatNumber2(
                    statisticViewModel.messageTotal.value.toString().toLong()
                ) + " " + "(" + df.format(percentage) + "%)" + ""
            )
            try{
                val currentString = it.renewalDate
                if(currentString?.contains("+")!!){
                    val separated = currentString.split("+").toTypedArray()

                    if(separated!=null){
                        mBinding.txtPlanRenews.setText(getFullDateWithYear(separated.get(0).toString()))
                    }
                }else if(currentString.contains(" ")){
                    val separated = currentString.split(" ").toTypedArray()

                    if(separated!=null){
                        mBinding.txtPlanRenews.setText(getFullDateWithYear(separated.get(0).toString()))
                    }
                }

            }catch (e:Exception){

            }


        })

    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        statisticViewModel.userDetails = GsonBuilder().create().fromJson(
            SharedPref.instance!!.getValueFromPreferences(
                getString(R.string.pref_user_details)
            ),
            UserIdentityResponse::class.java
        )
        statisticViewModel.getSubscribtionToken()

        clicks()
    }

    override fun onResume() {
        (activity as MainActivity).tvTitle?.text = getString(R.string.statistics)
        (activity as MainActivity).btnAddDevice?.visibility = View.INVISIBLE

        super.onResume()
    }

    private fun clicks() {

    }

    fun formatNumber(count: Long): String {
        try {
            if (count < 1000) return "" + count
            val exp = (Math.log(count.toDouble()) / Math.log(1000.0)).toInt()
            return String.format(
                "%.1f%c", (count / Math.pow(1000.0, exp.toDouble())).roundToInt().toFloat(),
                "kmgtpe"[exp - 1]
            )
        } catch (e: Exception) {
            return ""
        }
    }
    fun formatNumber2(count: Long): String {
        try {
            if (count < 1000) return "" + count
            val exp = (Math.log(count.toDouble()) / Math.log(1000.0)).toInt()
            return String.format(
                "%.1f%c", count / Math.pow(1000.0, exp.toDouble()),
                "kmgtpe"[exp - 1]
            )
        } catch (e: Exception) {
            return ""
        }
    }
}