package com.avnet.ebv.view.fragment.auth.login

import android.os.Bundle
import android.text.method.PasswordTransformationMethod
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.Observer
import com.avnet.ebv.R
import com.avnet.ebv.ViewModelFactory
import com.avnet.ebv.base.BaseFragment
import com.avnet.ebv.databinding.FragmentLoginBinding
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.utils.PasswordUtils
import com.avnet.ebv.view.activity.AuthenticationActivity
import com.avnet.ebv.viewmodel.LoginViewModel
import javax.inject.Inject

class LoginFragment() : BaseFragment(), View.OnClickListener {

    @Inject
    lateinit var viewModelFactory: ViewModelFactory

    @Inject
    lateinit var loginViewModel: LoginViewModel

    private lateinit var binding: FragmentLoginBinding

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentLoginBinding.inflate(layoutInflater)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        binding.edtPassword.setTransformationMethod(PasswordTransformationMethod.getInstance());
        makeLinks(
            binding.tvSignUp,
            Pair(
                "Sign-Up",
                View.OnClickListener {
                    (baseActivity as AuthenticationActivity).navController.navigate(R.id.action_nav_login_to_nav_sign_up)
                })
        )
        binding.ivEye.setOnClickListener(this)
        binding.btnConnect.setOnClickListener(this)


        loginViewModel.initializeWithContext(context!!)

    }

    override fun initViewModel() {
        loginViewModel = viewModelFactory.create(LoginViewModel::class.java)
        binding.setLifecycleOwner(this)
        binding.loginViewModel = loginViewModel
    }

    override fun observeViewModel() {
        loginViewModel._ProgreeBarState.observe(this, Observer {
            when (it) {
                ProgressState.HIDE -> dismissProgressDialog()
                ProgressState.SHOW -> showProgressDialog()
            }
        })
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.ivEye -> {
                if (!binding.edtPassword.text?.isEmpty()!!) {
                    PasswordUtils.showHidePass(binding.ivEye, binding.edtPassword)
                }
            }
            R.id.btnConnect -> {
                if (loginViewModel.checkIsValidEmail()) {

                    loginViewModel.getTokenResponse()

                }
            }
        }
    }
}