package com.avnet.ebv.view.fragment.auth.login

import android.os.Bundle
import android.text.method.PasswordTransformationMethod
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.lifecycle.Observer
import com.avnet.ebv.R
import com.avnet.ebv.ViewModelFactory
import com.avnet.ebv.base.BaseFragment
import com.avnet.ebv.common.SharedPref
import com.avnet.ebv.databinding.FragmentRootUserLoginBinding
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.utils.PasswordUtils
import com.avnet.ebv.view.activity.AuthenticationActivity
import com.avnet.ebv.viewmodel.LoginRootUserViewModel
import javax.inject.Inject

class LoginRootUserFragment() : BaseFragment(), View.OnClickListener {

    @Inject
    lateinit var viewModelFactory: ViewModelFactory

    @Inject
    lateinit var loginRootUserViewModel: LoginRootUserViewModel

    private lateinit var binding: FragmentRootUserLoginBinding

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentRootUserLoginBinding.inflate(layoutInflater)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        SharedPref.instance!!.saveStringSession(
            context!!.getString(R.string.pref_is_root_user_logged_in),
            ""
        )

        binding.edtPassword.setTransformationMethod(PasswordTransformationMethod.getInstance());
        makeLinks(
            binding.tvSignUp,
            Pair(
                "Sign-Up",
                View.OnClickListener {
                    (baseActivity as AuthenticationActivity).navController.navigate(R.id.action_nav_login_to_nav_sign_up)
                })
        )
        binding.ivEye.setOnClickListener(this)
        binding.btnConnect.setOnClickListener(this)


        loginRootUserViewModel.initializeWithContext(context!!)

    }

    override fun initViewModel() {
        loginRootUserViewModel = viewModelFactory.create(LoginRootUserViewModel::class.java)
        binding.setLifecycleOwner(this)
        binding.loginRootUserViewModel = loginRootUserViewModel
    }

    override fun observeViewModel() {

        loginRootUserViewModel._ProgreeBarState.observe(this, Observer {
            when (it) {
                ProgressState.HIDE -> dismissProgressDialog()
                ProgressState.SHOW -> showProgressDialog()
            }
        })

        loginRootUserViewModel.getLoginResponse.observe(this, Observer {



        })


        loginRootUserViewModel.getUserIdentityResponse.observe(this, Observer {

            if ( ((it.data?.data?.parentEntityGuid==null) || (it.data.data?.parentEntityGuid!!.isEmpty())) && (it.data?.data!!.isCompanyAdmin!!)) {
                SharedPref.instance!!.saveStringSession(
                    getString(R.string.pref_root_user),
                    loginRootUserViewModel._emailAddress.value.toString()
                )

                (baseActivity as AuthenticationActivity).navController.navigate(R.id.action_nav_root_user_login_to_nav_login)

            } else {
                loginRootUserViewModel.showMsg(resources.getString(R.string.alert_invalid_root))
            }
        })
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.ivEye -> {
                if (!binding.edtPassword.text?.isEmpty()!!) {
                    PasswordUtils.showHidePass(binding.ivEye, binding.edtPassword)
                }
            }
            R.id.btnConnect -> {
                if (loginRootUserViewModel.checkIsValidEmail()) {

                    loginRootUserViewModel.getTokenResponse()

                }
            }
        }
    }
}