package com.avnet.ebv.view.fragment.auth.signup

import android.os.Bundle
import android.text.method.PasswordTransformationMethod
import android.view.LayoutInflater
import android.view.MotionEvent
import android.view.View
import android.view.ViewGroup
import android.widget.AdapterView
import android.widget.ArrayAdapter
import android.widget.Spinner
import androidx.lifecycle.Observer
import com.avnet.ebv.EbvApplication
import com.avnet.ebv.R
import com.avnet.ebv.ViewModelFactory
import com.avnet.ebv.base.BaseFragment
import com.avnet.ebv.common.SharedPref
import com.avnet.ebv.databinding.FragmentSignupBinding
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.log.AndroidLog
import com.avnet.ebv.utils.KeyboardUtils
import com.avnet.ebv.utils.PasswordUtils
import com.avnet.ebv.utils.hide
import com.avnet.ebv.viewmodel.SignUpViewModel
import com.hbb20.CountryCodePicker.OnCountryChangeListener
import javax.inject.Inject


class SignUpFragment() : BaseFragment(), View.OnClickListener {

    @Inject
    lateinit var viewModelFactory: ViewModelFactory

    @Inject
    lateinit var signUpViewModel: SignUpViewModel

    private lateinit var binding: FragmentSignupBinding

    lateinit var countryAdapter: ArrayAdapter<String>
    var selectCountry: ArrayList<String> = ArrayList()
    lateinit var stateAdapter: ArrayAdapter<String>
    var selectState: ArrayList<String> = ArrayList()


    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        binding = FragmentSignupBinding.inflate(layoutInflater)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)


        signUpViewModel._rootusername.value= SharedPref.instance!!.getValueFromPreferences(EbvApplication.context.getString(R.string.pref_root_user_email))
        signUpViewModel._rootpassword.value= SharedPref.instance!!.getValueFromPreferences(EbvApplication.context.getString(R.string.pref_root_user_password))
        initToolbar()
        setSpinnerCountry(binding.spSelectCountry)
        setSpinnerState(binding.spSelectState)
        signUpViewModel.getCountryList()
        setViewListener()

    }

    private fun setViewListener() {
        binding.ccp.setOnCountryChangeListener(OnCountryChangeListener {
            signUpViewModel._code.value = binding.ccp.selectedCountryCode
            var selectedPosition = 0
            if (signUpViewModel._countryResponse.value != null) {
                for (i in 0 until (signUpViewModel._countryResponse.value?.data?.size ?: 0)) {
                    if (signUpViewModel._countryResponse.value?.data?.get(i)?.countryCode.equals(
                            binding.ccp.selectedCountryNameCode
                        )
                    ) {
                        selectedPosition = i
                    }
                }
                binding.spSelectCountry.setSelection(selectedPosition)
            }
        })
        binding.btnSubmit.setOnClickListener(this)
        binding.ivEye.setOnClickListener(this)
        binding.ivEye1.setOnClickListener(this)
        binding.ivEye2.setOnClickListener(this)
        binding.btnCancel.setOnClickListener(this)

        binding.edtRootPassword.setTransformationMethod(PasswordTransformationMethod.getInstance());
        binding.edtPassword.setTransformationMethod(PasswordTransformationMethod.getInstance());
        binding.edtConfirmPassword.setTransformationMethod(PasswordTransformationMethod.getInstance());
    }

    private fun initToolbar() {
        binding.viewToolbar.ivMenu.hide()
        binding.viewToolbar.ivAdd.hide()
        binding.viewToolbar.tvTitle.setText("Sign Up")
    }

    override fun initViewModel() {
        signUpViewModel = viewModelFactory.create(SignUpViewModel::class.java)
        signUpViewModel.initializeWithContext(requireContext())
        binding.setLifecycleOwner(this)
        binding.signUpViewModel = signUpViewModel

    }

    override fun observeViewModel() {
        signUpViewModel._ProgreeBarState.observe(this, Observer {
            when (it) {
                ProgressState.HIDE -> dismissProgressDialog()
                ProgressState.SHOW -> showProgressDialog()
            }
        })

        signUpViewModel.getCountryListResponse.observe(this, Observer {

            var stringCountryCodeDefault = ""
            selectCountry.clear()
            for (i in 0 until (it.data?.size ?: 0)) {
                selectCountry.add(it.data?.get(i)?.name.toString())
                stringCountryCodeDefault =
                    stringCountryCodeDefault + it.data?.get(i)?.countryCode.toString()
                if (i != (it.data?.size?.minus(1))) {
                    stringCountryCodeDefault = stringCountryCodeDefault + ","
                }
            }

            AndroidLog.d("ccp default", stringCountryCodeDefault)
            binding.ccp.setCustomMasterCountries(stringCountryCodeDefault)
            countryAdapter.notifyDataSetChanged()
        })

        signUpViewModel.getStateListResponse.observe(this, Observer {
            selectState.clear()
            for (i in 0 until (it.data?.size ?: 0)) {
                selectState.add(it.data?.get(i)?.name.toString())
            }
            stateAdapter.notifyDataSetChanged()
        })

        signUpViewModel._country.observe(this, Observer {
            if (it != null) {
                binding.ccp.setCountryForNameCode(it.countryCode)
                signUpViewModel.getStateList(it.guid.toString())
            }
        })
    }

    fun setSpinnerCountry(s: Spinner) {
        selectCountry.add("India")
        selectCountry = ArrayList()
        countryAdapter =
            this.context.let {
                ArrayAdapter<String>(
                    it!!,
                    R.layout.spinner_item_select,
                    selectCountry
                )
            }

        countryAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        s.adapter = countryAdapter

        s.setOnTouchListener(object : View.OnTouchListener {
            override fun onTouch(v: View?, event: MotionEvent?): Boolean {
                KeyboardUtils.forceCloseKeyboard(binding.rootView)
                return false
            }
        })


        s.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onItemSelected(
                parentView: AdapterView<*>?,
                selectedItemView: View?,
                position: Int,
                id: Long
            ) {
                binding.spSelectCountry.requestFocus()
                KeyboardUtils.forceCloseKeyboard(binding.rootView)
                if (signUpViewModel._countryResponse.value != null) {
                    signUpViewModel._country.value =
                        signUpViewModel._countryResponse.value?.data?.get(position)

                }
            }

            override fun onNothingSelected(parentView: AdapterView<*>?) {
            }
        }
    }

    fun setSpinnerState(s: Spinner) {

        selectState = ArrayList()

        stateAdapter =
            this.context.let {
                ArrayAdapter<String>(
                    it!!,
                    R.layout.spinner_item_select,
                    selectState
                )
            }

        stateAdapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item)
        s.adapter = stateAdapter

        s.onItemSelectedListener = object : AdapterView.OnItemSelectedListener {
            override fun onItemSelected(
                parentView: AdapterView<*>?,
                selectedItemView: View?,
                position: Int,
                id: Long
            ) {
                binding.spSelectState.requestFocus()
                if (signUpViewModel._stateResponse.value != null) {
                    signUpViewModel._state.value =
                        signUpViewModel._stateResponse.value?.data?.get(position)

                }
            }

            override fun onNothingSelected(parentView: AdapterView<*>?) {
            }
        }
    }

    override fun onClick(v: View?) {
        when (v?.id) {
            R.id.ivEye -> {
                if (!binding.edtRootPassword.text?.isEmpty()!!) {
                    PasswordUtils.showHidePass(binding.ivEye, binding.edtRootPassword)
                }
            }  R.id.ivEye1 -> {
                if (!binding.edtPassword.text?.isEmpty()!!) {
                    PasswordUtils.showHidePass(binding.ivEye1, binding.edtPassword)
                }
            }  R.id.ivEye2 -> {
                if (!binding.edtConfirmPassword.text?.isEmpty()!!) {
                    PasswordUtils.showHidePass(binding.ivEye2, binding.edtConfirmPassword)
                }
            }
            R.id.btnSubmit -> {
                if (signUpViewModel.checkIsValid()) {
                    signUpViewModel.getEntityLookUpList()
                }
            }R.id.btnCancel -> {
               baseActivity?.onBackPressed()
            }
        }
    }

}