package com.avnet.ebv.viewmodel

import android.content.Context
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import android.widget.Toast
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.avnet.ebv.R
import com.avnet.ebv.base.BaseActivity
import com.avnet.ebv.base.BaseViewModel
import com.avnet.ebv.data.DeviceRepositorySource
import com.avnet.ebv.data.Resource
import com.avnet.ebv.data.error.ErrorManager
import com.avnet.ebv.data.error.mapper.ErrorMapper
import com.avnet.ebv.data.remote.ApiConstant
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.model.request.ExecuteCommandRequest
import com.avnet.ebv.model.response.*
import com.avnet.ebv.utils.AppConstants
import com.avnet.ebv.utils.NetworkUtils
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.launch
import javax.inject.Inject
import kotlin.collections.ArrayList
import kotlin.collections.HashMap
import kotlin.collections.set

class CommandViewModel @Inject
constructor(private val deviceRepository: DeviceRepositorySource) : BaseViewModel() {

    lateinit var context: Context
    var commandLookup = MutableLiveData<Resource<CommandLookupResponse>>()
    var commandHistory = MutableLiveData<Resource<CommandHistoryResponse>>()
    var refreshData = MutableLiveData<Resource<CommandHistoryResponse>>()
    var executeCommand = MutableLiveData<Resource<ExecuteCommandResponse>>()
    var desiredValue: MutableLiveData<String> = MutableLiveData()
    var command: MutableLiveData<String> = MutableLiveData()
    var _ProgreeBarState: MutableLiveData<ProgressState> = MutableLiveData()
    var _ProgreeBarState1: MutableLiveData<ProgressState> = MutableLiveData()

    var deviceInfo: MutableLiveData<DeviceListResponse.Data?> = MutableLiveData()

    var propertyHistoryList = ArrayList<CommandHistoryDataItem?>()

    var _pageNumber = MutableLiveData<Int>()
    var _pageSize = MutableLiveData<String>()
    var _totalDeviceCount = MutableLiveData<Int>()
    var isApiCalled: Boolean = false
    var callPagination: Boolean = false


    init {
        desiredValue.value = ""
        command.value = ""
        _pageNumber.value = 1
        _pageSize.value = "20"
        _totalDeviceCount.value = 0
        propertyHistoryList = ArrayList()
        _totalDeviceCount.value = -1
    }

    override val errorManager: ErrorManager
        get() = ErrorManager(ErrorMapper())

    fun initializeWithContext(context: Context) {
        this.context = context
    }

    fun getCommandLookList(templateGuid: String) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.HIDE
            viewModelScope.launch {
                deviceRepository.commandLookupList(templateGuid).collect() {
                    if (it.data?.status == AppConstants.STATUS_200) {
                        _ProgreeBarState.value = ProgressState.HIDE
                        commandLookup.value = it

                    }else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    }else {
                        _ProgreeBarState.value = ProgressState.HIDE
//                        showError(it.errorBody)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun getCommandHistoryList(deviceGuid: String) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState1.value = ProgressState.SHOW
            viewModelScope.launch {
                val params = HashMap<String, String?>()
                params[ApiConstant.pageNumber] = _pageNumber.value.toString()
                params[ApiConstant.pageSize] = _pageSize.value
                params[ApiConstant.deviceGuid] = deviceGuid

                deviceRepository.commandHistoryList(params).collect() {
                    _ProgreeBarState1.value = ProgressState.HIDE
                    if (it.data?.status == AppConstants.STATUS_200) {
                        commandHistory.value = it
                        _totalDeviceCount.value = it.data.count!!
                        if (it.data.count != propertyHistoryList.size) {
                            _pageNumber.value = _pageNumber.value!!.plus(1)
                            callPagination = true
                            isApiCalled = false
                        } else {
                            callPagination = false
                        }

                    } else if (it.errorBody?.status == AppConstants.STATUS_204) {
                        if (propertyHistoryList.size == 0) {
                            _totalDeviceCount.value = 0
                        }
                    }else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun checkIsValid(): Boolean {
        return if (desiredValue.value.equals("", ignoreCase = true)) {
            showMsg(context.getString(R.string.alert_enter_parameter_value))
            false
        } else {
            true
        }
    }

    fun executeCommandApiCalL(commandGuId: String, commandRequest: ExecuteCommandRequest) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {

                deviceRepository.executeCommand(commandGuId, commandRequest).collect() {
                    _ProgreeBarState.value = ProgressState.HIDE
                    executeCommand.value = it
                    if (executeCommand.value!!.data?.status == AppConstants.STATUS_200) {
                        desiredValue.value = ""
                        _pageNumber.value = 1
                        getCommandHistoryList(deviceInfo.value?.guid.toString())
                        showMsg(it.data?.message.toString())

                    }else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }

    }

    fun showMsg(msg: String) {
        val inflater: LayoutInflater = (context as BaseActivity).layoutInflater
        val layout: View =
            inflater.inflate(R.layout.custom_toast, (context as BaseActivity).findViewById(R.id.custom_toast_container))

        val text: TextView = layout.findViewById(R.id.tvToast)
        text.setText(msg)

        val toast = Toast(context)
        toast.setGravity(Gravity.BOTTOM, 0, 40)
        toast.duration = Toast.LENGTH_LONG
        toast.setView(layout)
        toast.show()
    }

    fun refereshTokenExpired() {
//        SharedPref.instance!!.deleteAllPreferences()
//        val intent = Intent(context, AuthenticationActivity()::class.java)
//        (context as BaseActivity).startActivity(intent)
//        (context as BaseActivity).finish()
    }

    fun showError(errorBody: ErrorRespose) {
        if (errorBody.error != null && errorBody.error!!.size > 0) {
            showMsg(errorBody.error!!.get(0)?.message.toString())
        } else {
            showMsg(errorBody.message.toString())
        }
    }

}
