package com.avnet.ebv.viewmodel

import android.content.Context
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import android.widget.Toast
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.avnet.ebv.R
import com.avnet.ebv.base.BaseActivity
import com.avnet.ebv.base.BaseViewModel
import com.avnet.ebv.data.DeviceRepositorySource
import com.avnet.ebv.data.Resource
import com.avnet.ebv.data.error.ErrorManager
import com.avnet.ebv.data.error.mapper.ErrorMapper
import com.avnet.ebv.data.remote.ApiConstant
import com.avnet.ebv.databinding.LayoutBtSensorBinding
import com.avnet.ebv.databinding.LayoutLedSensorBinding
import com.avnet.ebv.databinding.LayoutSliderSensorBinding
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.model.request.ExecuteCommandRequest
import com.avnet.ebv.model.response.*
import com.avnet.ebv.utils.AppConstants
import com.avnet.ebv.utils.NetworkUtils
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.launch
import javax.inject.Inject


class DataViewModel @Inject
constructor(private val deviceRepository: DeviceRepositorySource) : BaseViewModel() {

    lateinit var context: Context

    var _ProgreeBarState: MutableLiveData<ProgressState> = MutableLiveData()

    override val errorManager: ErrorManager
        get() = ErrorManager(ErrorMapper())


    var _attributeTemplateResponse = MutableLiveData<AttributeTemplateResponse>()
    var _deviceTemplateResponse = MutableLiveData<DeviceTemplateResponse>()
    var _templateResponse = MutableLiveData<TemplateResponse>()

    var count = MutableLiveData<Int>()
    var isConnect: Boolean
    var isValueChanged: Boolean
    var view: View? =null
    var _attributeList1 = ArrayList<AttributeTemplateResponse.Data>()
    var _attributeList = MutableLiveData<ArrayList<AttributeTemplateResponse.Data>>()
    var _commandList = MutableLiveData<ArrayList<DeviceTemplateResponse.Data.Command>>()
    var _pageNumber: Int = 1
    lateinit var deviceGuidId: String
    lateinit var guid1: String
    var executeCommand = MutableLiveData<Resource<ExecuteCommandResponse>>()
    var isSensorBeginValuesSet: Boolean

    init {
        isConnect = false
        isValueChanged = false
        isSensorBeginValuesSet = false
    }

    fun initializeWithContext(context: Context) {
        this.context = context
    }

    fun getTemplateAttributeList(deviceTemplateGuid: String) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {
                val params = HashMap<String, String?>()
                params[ApiConstant.pageNumber] = _pageNumber.toString()
                params[ApiConstant.pageSize] = "20"

                deviceRepository.requestTemplateAttributeList(params, deviceTemplateGuid)
                    .collect() {

                        if (it.data?.status == AppConstants.STATUS_200) {
                            count.value = it.data.count!!
                            _attributeList1.addAll((it.data.data as ArrayList<AttributeTemplateResponse.Data>?)!!)

                            if (_attributeList1.size < count.value!!) {
                                _pageNumber = _pageNumber + 1
                                getTemplateAttributeList(deviceGuidId)
                            } else {
                                _attributeList.value = _attributeList1
                                getDeviceTemplateList(deviceGuidId)
                            }

                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            isSensorBeginValuesSet=true
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            isSensorBeginValuesSet=true
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            isSensorBeginValuesSet=true
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun getDeviceTemplateList(deviceTemplateGuid: String) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {
                val params = HashMap<String, String?>()
//                params[ApiConstant.pageNumber] = "1"
//                params[ApiConstant.pageSize] = "20"

                deviceRepository.requestDeviceTemplateList(params, deviceTemplateGuid)
                    .collect() {

                        if (it.data?.status == AppConstants.STATUS_200) {
                            _deviceTemplateResponse.value = it.data!!
                            _commandList.value =
                                it.data.data?.get(0)?.commands as ArrayList<DeviceTemplateResponse.Data.Command>?
//                            if (!isConnect) {
                                getTemplate(guid1)
//                            }
                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            isSensorBeginValuesSet=true
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            isSensorBeginValuesSet=true
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            isSensorBeginValuesSet=true
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun getTemplate(deviceTemplateGuid: String) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {

                deviceRepository.requestTemplateDevice(deviceTemplateGuid)
                    .collect() {

                        if (it.data?.status == AppConstants.STATUS_200) {
                            _ProgreeBarState.value = ProgressState.HIDE
                            _templateResponse.value = it.data!!

                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            isSensorBeginValuesSet=true
                            _templateResponse.value = it.data!!
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            isSensorBeginValuesSet=true
                            _templateResponse.value = it.data!!
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            isSensorBeginValuesSet=true
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun executeCommandApiCall(
        commandGuId: String,
        commandRequest: ExecuteCommandRequest,
        binding: LayoutBtSensorBinding,
        resId: Int
    ) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {

                deviceRepository.executeCommand(commandGuId, commandRequest).collect() {
                    _ProgreeBarState.value = ProgressState.HIDE
                    executeCommand.value = it
                    if (executeCommand.value!!.data?.status == AppConstants.STATUS_200) {
                        showMsg(it.data?.message.toString())
                        binding.ivSwitch.setImageResource(resId)
                        isValueChanged=true
                        view=binding.crdView
                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }

    }

    fun executeCommandApiLedCall(
        commandGuId: String,
        commandRequest: ExecuteCommandRequest,
        binding: LayoutLedSensorBinding,
        resId: Int
    ) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {

                deviceRepository.executeCommand(commandGuId, commandRequest).collect() {
                    _ProgreeBarState.value = ProgressState.HIDE
                    executeCommand.value = it
                    if (executeCommand.value!!.data?.status == AppConstants.STATUS_200) {
                        showMsg(it.data?.message.toString())
                        binding.ivLed.setImageResource(resId)
                        isValueChanged=true
                        view=binding.crdView
                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        showError(it.errorBody!!)
                    }

                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }

    }

    fun executeCommandApiSliderCall(
        commandGuId: String,
        commandRequest: ExecuteCommandRequest,
        binding: LayoutSliderSensorBinding,
        resId: Int
    ) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {

                deviceRepository.executeCommand(commandGuId, commandRequest).collect() {
                    _ProgreeBarState.value = ProgressState.HIDE
                    executeCommand.value = it
                    if (executeCommand.value!!.data?.status == AppConstants.STATUS_200) {
                        showMsg(it.data?.message.toString())
                        isValueChanged=true
                        view=binding.crdView
                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }

    }

    fun showMsg(msg: String) {
        val inflater: LayoutInflater = (context as BaseActivity).layoutInflater
        val layout: View =
            inflater.inflate(R.layout.custom_toast, (context as BaseActivity).findViewById(R.id.custom_toast_container))

        val text: TextView = layout.findViewById(R.id.tvToast)
        text.setText(msg)

        val toast = Toast(context)
        toast.setGravity(Gravity.BOTTOM, 0, 40)
        toast.duration = Toast.LENGTH_LONG
        toast.setView(layout)
        toast.show()
    }

    fun refereshTokenExpired() {
        _ProgreeBarState.value = ProgressState.HIDE
//        SharedPref.instance!!.deleteAllPreferences()
//        val intent = Intent(context, AuthenticationActivity()::class.java)
//        (context as BaseActivity).startActivity(intent)
//        (context as BaseActivity).finish()
    }

    fun showError(errorBody: ErrorRespose) {
        _ProgreeBarState.value = ProgressState.HIDE
        if (errorBody.error != null && errorBody.error!!.size > 0) {
            showMsg(errorBody.error!!.get(0)?.message.toString())
        } else {
            showMsg(errorBody.message.toString())
        }
    }

}

