package com.avnet.ebv.viewmodel

import android.content.Context
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import android.widget.Toast
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.avnet.ebv.R
import com.avnet.ebv.base.BaseActivity
import com.avnet.ebv.base.BaseViewModel
import com.avnet.ebv.data.DeviceRepositorySource
import com.avnet.ebv.data.error.ErrorManager
import com.avnet.ebv.data.error.mapper.ErrorMapper
import com.avnet.ebv.data.remote.ApiConstant
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.model.response.AttributeTemplateResponse
import com.avnet.ebv.model.response.DeviceTemplateResponse
import com.avnet.ebv.model.response.ErrorRespose
import com.avnet.ebv.model.response.HistoryDataResponse
import com.avnet.ebv.utils.AppConstants
import com.avnet.ebv.utils.NetworkUtils
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.launch
import javax.inject.Inject


class HistoryDataViewModel @Inject
constructor(private val deviceRepository: DeviceRepositorySource) : BaseViewModel() {

    lateinit var context: Context

    var _ProgreeBarState: MutableLiveData<ProgressState> = MutableLiveData()

    override val errorManager: ErrorManager
        get() = ErrorManager(ErrorMapper())

    var _deviceTemplateResponse = MutableLiveData<DeviceTemplateResponse>()

    //    var _attributeList = MutableLiveData<ArrayList<DeviceTemplateResponse.Data.Attribute>>()
    var _attributeFeedList = MutableLiveData<ArrayList<HistoryDataResponse.Data.Feed>>()
    var uniqueId = MutableLiveData<String>()
    var fromDate = MutableLiveData<String>()
    var toDate = MutableLiveData<String>()
    var _pageNumber: Int = 1

    var count = MutableLiveData<Int>()
    var _attributeList1 = ArrayList<AttributeTemplateResponse.Data>()
    var _attributeList = MutableLiveData<ArrayList<AttributeTemplateResponse.Data>>()
    lateinit var deviceTemplateGuid: String

    fun initializeWithContext(context: Context) {
        this.context = context
    }

    fun getHistoryData() {


        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {
                val params = HashMap<String, String?>()

                deviceRepository.requestHistoryDataList(
                    uniqueId.value.toString(),
                    fromDate.value.toString(), toDate.value.toString()
                )
                    .collect() {
                        _ProgreeBarState.value = ProgressState.HIDE
                        if (it.data?.status == AppConstants.STATUS_200) {

                            if(it.data.data?.feed!=null){
                                _attributeFeedList.value =
                                    (it.data.data?.feed as ArrayList<HistoryDataResponse.Data.Feed>?)!!
                            }


                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

//    fun getDevihceTemplateList(deviceTemplateGuid: String) {
//        if (NetworkUtils.isNetworkAvailable(context)) {
//            viewModelScope.launch {
//                val params = HashMap<String, String?>()
////                params[ApiConstant.pageNumber] = "1"
////                params[ApiConstant.pageSize] = "20"
//
//                deviceRepository.requestDeviceTemplateList(params, deviceTemplateGuid)
//                    .collect() {
//                        _ProgreeBarState.value = ProgressState.HIDE
//                        if (it.data?.status == AppConstants.STATUS_200) {
//                            _deviceTemplateResponse.value = it.data!!
//                            _attributeList.value =
//                                (it.data.data?.get(0)?.attributes as ArrayList<DeviceTemplateResponse.Data.Attribute>?)!!
//
//
//                        } else if (it.errorBody?.status == AppConstants.STATUS_204) {
//
//                        } else {
//                            showMsg(it.errorBody?.message.toString())
//                        }
//                    }
//            }
//        } else {
//            showMsg(context.getString(R.string.str_alert_no_internet))
//        }
//    }

    fun getTemplateAttributeList(deviceTemplateGuid: String) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {
                val params = HashMap<String, String?>()
                params[ApiConstant.pageNumber] = _pageNumber.toString()
                params[ApiConstant.pageSize] = "20"
                _ProgreeBarState.value = ProgressState.SHOW
                deviceRepository.requestTemplateAttributeList(params, deviceTemplateGuid)
                    .collect() {
                        _ProgreeBarState.value = ProgressState.HIDE
                        if (it.data?.status == AppConstants.STATUS_200) {
                            count.value = it.data.count!!
                            _attributeList1.addAll((it.data.data as ArrayList<AttributeTemplateResponse.Data>?)!!)

                            if (_attributeList1.size < count.value!!) {
                                _pageNumber = _pageNumber + 1
                                getTemplateAttributeList(deviceTemplateGuid)
                            } else {
                                _attributeList.value = _attributeList1

                                getHistoryData()
                            }

                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun showMsg(msg: String) {
        val inflater: LayoutInflater = (context as BaseActivity).layoutInflater
        val layout: View =
            inflater.inflate(R.layout.custom_toast, (context as BaseActivity).findViewById(R.id.custom_toast_container))

        val text: TextView = layout.findViewById(R.id.tvToast)
        text.setText(msg)

        val toast = Toast(context)
        toast.setGravity(Gravity.BOTTOM, 0, 40)
        toast.duration = Toast.LENGTH_LONG
        toast.setView(layout)
        toast.show()
    }

    fun refereshTokenExpired() {
//        SharedPref.instance!!.deleteAllPreferences()
//        val intent = Intent(context, AuthenticationActivity()::class.java)
//        (context as BaseActivity).startActivity(intent)
//        (context as BaseActivity).finish()
    }

    fun showError(errorBody: ErrorRespose) {
        if (errorBody.error != null && errorBody.error!!.size > 0) {
            showMsg(errorBody.error!!.get(0)?.message.toString())
        } else {
            showMsg(errorBody.message.toString())
        }
    }
}

