package com.avnet.ebv.viewmodel

import android.content.Context
import android.content.Intent
import android.util.Log
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import android.widget.Toast
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.avnet.ebv.R
import com.avnet.ebv.base.BaseActivity
import com.avnet.ebv.base.BaseViewModel
import com.avnet.ebv.common.SharedPref
import com.avnet.ebv.data.DataRepositorySource
import com.avnet.ebv.data.DeviceRepositorySource
import com.avnet.ebv.data.Resource
import com.avnet.ebv.data.error.ErrorManager
import com.avnet.ebv.data.error.mapper.ErrorMapper
import com.avnet.ebv.data.remote.ApiConstant
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.model.request.LoginRequest
import com.avnet.ebv.model.response.ErrorRespose
import com.avnet.ebv.model.response.LoginResponse
import com.avnet.ebv.model.response.TokenResponse
import com.avnet.ebv.model.response.UserIdentityResponse
import com.avnet.ebv.utils.AppConstants
import com.avnet.ebv.utils.NetworkUtils
import com.avnet.ebv.utils.ValidationUtils
import com.avnet.ebv.view.activity.AuthenticationActivity
import com.avnet.ebv.view.activity.MainActivity
import com.google.gson.Gson
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.launch
import javax.inject.Inject


class LoginViewModel @Inject
constructor(
    private val dataRepositoryRepository: DataRepositorySource,
    private val deviceRepository: DeviceRepositorySource
) : BaseViewModel() {

    lateinit var context: Context

    var _ProgreeBarState: MutableLiveData<ProgressState> = MutableLiveData()

    override val errorManager: ErrorManager
        get() = ErrorManager(ErrorMapper())


    val getTokenResponse: LiveData<Resource<TokenResponse>> get() = _tokenResponse
    private var _tokenResponse = MutableLiveData<Resource<TokenResponse>>()

    val getLoginResponse: LiveData<Resource<LoginResponse>> get() = _loginResponse
    private var _loginResponse = MutableLiveData<Resource<LoginResponse>>()

    val getUserIdentityResponse: LiveData<Resource<UserIdentityResponse>> get() = _userIdentity
    private var _userIdentity = MutableLiveData<Resource<UserIdentityResponse>>()

    var _emailAddress: MutableLiveData<String> = MutableLiveData()
    var _password: MutableLiveData<String> = MutableLiveData()

    init {
//        _emailAddress.value = "ebv.test2@yopmail.com"
//        _password.value = "Softweb#123"
//        _emailAddress.value = "testuser222@mailcatch.com"
//        _password.value = "Test#123"
//        _emailAddress.value = "thibault.richard@ebv.com"
//        _password.value = "@Abcdefg8"
        _emailAddress.value = ""
        _password.value = ""
    }

    fun initializeWithContext(context: Context) {
        this.context = context
    }

    fun getTokenResponse() {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {
                dataRepositoryRepository.requestGetToken().collect() {


                    if (it.data?.status == AppConstants.STATUS_200) {
                        _tokenResponse.value = it


                        SharedPref.instance!!.saveStringSession(
                            context.getString(R.string.pref_auth_token),
                            _tokenResponse.value!!.data?.data
                        )
                        Log.d(
                            "auth token",
                            SharedPref.instance!!.getValueFromPreferences(context.getString(R.string.pref_auth_token))
                                .toString()
                        )

                        getLoginResponse(
                            LoginRequest(
                                _emailAddress.value?.trim(),
                                _password.value
                            )
                        )
                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        _ProgreeBarState.value = ProgressState.HIDE
                        showError(it.errorBody)
//                        refereshTokenExpired()
                    } else {
                        _ProgreeBarState.value = ProgressState.HIDE
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }

    }

    fun getLoginResponse(loginRequest: LoginRequest) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {

                dataRepositoryRepository.requestLogin(loginRequest).collect() {
                    if (it.data?.status == AppConstants.STATUS_200) {
                        _loginResponse.value = it
                        _ProgreeBarState.value = ProgressState.SHOW

                        SharedPref.instance!!.saveStringSession(
                            context.getString(R.string.pref_bearer_token),
                            _loginResponse.value!!.data?.accessToken
                        )
                        SharedPref.instance!!.saveStringSession(
                            context.getString(R.string.pref_refresh_token),
                            _loginResponse.value!!.data?.refreshToken
                        )

//                        navigateToMain()
                        SharedPref.instance!!.saveStringSession(
                            context.getString(R.string.pref_is_user_logged_in),
                            "1"
                        )

                        getUserIdentity()

                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        _ProgreeBarState.value = ProgressState.HIDE
                        if (it.errorBody.error != null && it.errorBody.error!!.size > 0) {
                            if (it.errorBody.error!!.get(0)?.message.toString()
                                    .equals(context.resources.getString(R.string.alert_solution_not_found),true)
                            ) {
                                showMsg(context.resources.getString(R.string.alert_plan_expired))
                            } else {
                                showMsg(it.errorBody.error!!.get(0)?.message.toString())
                            }

                        } else {
                            if (it.errorBody.message.toString()
                                    .equals(context.resources.getString(R.string.alert_solution_not_found),true)
                            ) {
                                showMsg(context.resources.getString(R.string.alert_plan_expired))
                            } else {
                                showMsg(it.errorBody.message.toString())
                            }

                        }
//                        showError(it.errorBody)
//                        refereshTokenExpired()
                    } else {
                        _ProgreeBarState.value = ProgressState.HIDE
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    private fun navigateToMain() {
        val destIntent = MainActivity.newIntent(context)
        context.startActivity(destIntent)
        (context as AuthenticationActivity).finish()
    }

    fun getUserIdentity() {
        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {
                deviceRepository.requestGetUserIdentity().collect() {
                    _userIdentity.value = it
                    if (_userIdentity.value!!.data?.status == AppConstants.STATUS_200) {
                        _ProgreeBarState.value = ProgressState.HIDE

                        SharedPref.instance!!.saveStringSession(
                            context.getString(R.string.pref_user_details),
                            Gson().toJson(it.data)
                        )

                        SharedPref.instance!!.saveStringSession(
                            context.getString(R.string.pref_entity_guid),
                            it.data?.data?.entityGuid
                        )

                        getDeviceList()

                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        _ProgreeBarState.value = ProgressState.HIDE
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        _ProgreeBarState.value = ProgressState.HIDE
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }

    }


    fun getDeviceList() {
        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {
                val params = HashMap<String, String?>()
                params[ApiConstant.pageNumber] = "1"
                params[ApiConstant.pageSize] = "10"

                dataRepositoryRepository.requestDeviceList(params).collect() {
                    _ProgreeBarState.value = ProgressState.HIDE
                    if (it.data?.status == AppConstants.STATUS_200) {
                        var intent = Intent(context, MainActivity()::class.java)
                        intent.putExtra(AppConstants.startDest, 1)
                        context.startActivity(intent)
                        (context as AuthenticationActivity).finish()

                    } else if (it.errorBody?.status == AppConstants.STATUS_204) {
                        var intent = Intent(context, MainActivity()::class.java)
                        intent.putExtra(AppConstants.startDest, 0)
                        context.startActivity(intent)
                        (context as AuthenticationActivity).finish()
                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun showMsg(msg: String) {
        val inflater: LayoutInflater = (context as BaseActivity).layoutInflater
        val layout: View =
            inflater.inflate(
                R.layout.custom_toast,
                (context as BaseActivity).findViewById(R.id.custom_toast_container)
            )

        val text: TextView = layout.findViewById(R.id.tvToast)
        text.setText(msg)

        val toast = Toast(context)
        toast.setGravity(Gravity.BOTTOM, 0, 40)
        toast.duration = Toast.LENGTH_LONG
        toast.setView(layout)
        toast.show()
    }


    fun refereshTokenExpired() {
//        SharedPref.instance!!.deleteAllPreferences()
//        val intent = Intent(context, AuthenticationActivity()::class.java)
//        (context as BaseActivity).startActivity(intent)
//        (context as BaseActivity).finish()
    }

    fun showError(errorBody: ErrorRespose) {
        if (errorBody.error != null && errorBody.error!!.size > 0) {
            showMsg(errorBody.error!!.get(0)?.message.toString())
        } else {
            showMsg(errorBody.message.toString())
        }
    }

    /**
     * Check validation for require filed
     */
    fun checkIsValidEmail(): Boolean {
        return if (_emailAddress.value.equals("", ignoreCase = true)) {
            showMsg(context.getString(R.string.alert_enter_email_id))
            false
        } else if (!ValidationUtils.isValidEmail(_emailAddress.value)) {
            showMsg(context.getString(R.string.alert_invalid_email))
            false
        } else if (_password.value.equals("", ignoreCase = true)) {
            showMsg(context.getString(R.string.alert_enter_password))
            false
        } else {
            true
        }
    }

}

