package com.avnet.ebv.viewmodel

import android.content.Context
import android.content.Intent
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import android.widget.Toast
import androidx.lifecycle.*
import com.avnet.ebv.R
import com.avnet.ebv.base.BaseActivity
import com.avnet.ebv.base.BaseViewModel
import com.avnet.ebv.common.SharedPref
import com.avnet.ebv.data.DeviceRepositorySource
import com.avnet.ebv.data.Resource
import com.avnet.ebv.data.error.ErrorManager
import com.avnet.ebv.data.error.mapper.ErrorMapper
import com.avnet.ebv.data.remote.ApiConstant
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.model.response.DeviceListResponse
import com.avnet.ebv.model.response.ErrorRespose
import com.avnet.ebv.model.response.UserIdentityResponse
import com.avnet.ebv.utils.AppConstants
import com.avnet.ebv.utils.NetworkUtils
import com.avnet.ebv.view.activity.AuthenticationActivity
import com.avnet.ebv.view.activity.MainActivity
import com.google.gson.Gson
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.launch
import javax.inject.Inject


class MainViewModel @Inject
constructor(
    private val deviceRepository: DeviceRepositorySource
) : BaseViewModel() {

    override val errorManager: ErrorManager
        get() = ErrorManager(ErrorMapper())

    lateinit var context: Context

    var _ProgreeBarState: MutableLiveData<ProgressState> = MutableLiveData()



    var deviceList = ArrayList<DeviceListResponse.Data?>()
    var startDest = MutableLiveData<Int>()


    var _pageNumber = MutableLiveData<Int>()
    var _pageSize = MutableLiveData<String>()
    var _totalDeviceCount = MutableLiveData<Int>()
    var isApiCalled: Boolean = false
    var callPagination: Boolean = false
    init {
        _pageNumber.value = 1
        _pageSize.value = "12"
        _totalDeviceCount.value=0
        deviceList=ArrayList()
        _totalDeviceCount.value=-1
        startDest.value=-1
    }

    fun initializeWithContext(context: Context) {
        this.context = context
    }

    fun getDeviceList() {
        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {
                val params = HashMap<String, String?>()
                params[ApiConstant.pageNumber] = "1"
                params[ApiConstant.pageSize] = "10"

                deviceRepository.requestDeviceList(params).collect() {
                    _ProgreeBarState.value = ProgressState.HIDE
                    if (it.data?.status == AppConstants.STATUS_200) {
                        startDest.value=1
                    } else if (it.errorBody?.status == AppConstants.STATUS_204) {
                        startDest.value=0
                    }else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }



    fun showMsg(msg: String) {
        val inflater: LayoutInflater = (context as BaseActivity).layoutInflater
        val layout: View =
            inflater.inflate(R.layout.custom_toast, (context as BaseActivity).findViewById(R.id.custom_toast_container))

        val text: TextView = layout.findViewById(R.id.tvToast)
        text.setText(msg)

        val toast = Toast(context)
        toast.setGravity(Gravity.BOTTOM, 0, 40)
        toast.duration = Toast.LENGTH_LONG
        toast.setView(layout)
        toast.show()
    }

    fun refereshTokenExpired() {
//        SharedPref.instance!!.deleteAllPreferences()
//        val intent = Intent(context, AuthenticationActivity()::class.java)
//        (context as BaseActivity).startActivity(intent)
//        (context as BaseActivity).finish()
    }

    fun showError(errorBody: ErrorRespose) {
        if (errorBody.error != null && errorBody.error!!.size > 0) {
            showMsg(errorBody.error!!.get(0)?.message.toString())
        } else {
            showMsg(errorBody.message.toString())
        }
    }
}
