package com.avnet.ebv.viewmodel

import android.content.Context
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import android.widget.Toast
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.avnet.ebv.R
import com.avnet.ebv.base.BaseActivity
import com.avnet.ebv.base.BaseViewModel
import com.avnet.ebv.common.SharedPref
import com.avnet.ebv.data.DataRepositorySource
import com.avnet.ebv.data.Resource
import com.avnet.ebv.data.error.ErrorManager
import com.avnet.ebv.data.error.mapper.ErrorMapper
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.log.AndroidLog
import com.avnet.ebv.model.request.CreateRoleRequest
import com.avnet.ebv.model.request.LoginRequest
import com.avnet.ebv.model.request.NewEntityRequest
import com.avnet.ebv.model.response.*
import com.avnet.ebv.utils.AppConstants
import com.avnet.ebv.utils.NetworkUtils
import com.avnet.ebv.utils.ValidationUtils
import com.avnet.ebv.view.activity.AuthenticationActivity
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.launch
import javax.inject.Inject


class SignUpViewModel @Inject
constructor(private val dataRepositoryRepository: DataRepositorySource) : BaseViewModel() {

    lateinit var context: Context

    var _ProgreeBarState: MutableLiveData<ProgressState> = MutableLiveData()

    override val errorManager: ErrorManager
        get() = ErrorManager(ErrorMapper())

    val getTokenResponse: LiveData<Resource<TokenResponse>> get() = _tokenResponse
    private var _tokenResponse = MutableLiveData<Resource<TokenResponse>>()

    val getLoginResponse: LiveData<Resource<LoginResponse>> get() = _loginResponse
    private var _loginResponse = MutableLiveData<Resource<LoginResponse>>()

    val getCountryListResponse: LiveData<CountryListResponse> get() = _countryResponse
    var _countryResponse = MutableLiveData<CountryListResponse>()

    val getStateListResponse: LiveData<StateResponseList> get() = _stateResponse
    var _stateResponse = MutableLiveData<StateResponseList>()

    val getEntityLookUpResponse: LiveData<EntityLookUpResponse> get() = _entityLookUpResponse
    private var _entityLookUpResponse = MutableLiveData<EntityLookUpResponse>()

    val getRoleLookUpResponse: LiveData<RoleLookUpResponse> get() = _roleLookUpResponse
    private var _roleLookUpResponse = MutableLiveData<RoleLookUpResponse>()

    val getAddNewEntityResponse: LiveData<NewEntityResponse> get() = _addNewEntityResponse
    private var _addNewEntityResponse = MutableLiveData<NewEntityResponse>()


    val createRoleResponse: LiveData<CreateRoleResponse> get() = _createRoleResponse
    private var _createRoleResponse = MutableLiveData<CreateRoleResponse>()

    val getRolePermissionsResponse: LiveData<GetRolePermissionsResponse> get() = _getRolePermissionsResponse
    private var _getRolePermissionsResponse = MutableLiveData<GetRolePermissionsResponse>()

    var _rootusername: MutableLiveData<String> = MutableLiveData()
    var _rootpassword: MutableLiveData<String> = MutableLiveData()
    var _firstName: MutableLiveData<String> = MutableLiveData()
    var _lastName: MutableLiveData<String> = MutableLiveData()
    var _emailAddress: MutableLiveData<String> = MutableLiveData()
    var _newPassword: MutableLiveData<String> = MutableLiveData()
    var _confirmPassword: MutableLiveData<String> = MutableLiveData()
    var _address: MutableLiveData<String> = MutableLiveData()
    var _code: MutableLiveData<String> = MutableLiveData()
    var _contact: MutableLiveData<String> = MutableLiveData()
    var _country: MutableLiveData<CountryListResponse.Data> = MutableLiveData()
    var _state: MutableLiveData<StateResponseList.Data> = MutableLiveData()
    var _city: MutableLiveData<String> = MutableLiveData()
    var _postalcode: MutableLiveData<String> = MutableLiveData()
    var _parentguid: MutableLiveData<String> = MutableLiveData()
    var _roleguid: MutableLiveData<String> = MutableLiveData()
    var _timezoneguid: MutableLiveData<String> = MutableLiveData()

    init {
        _rootusername.value = ""
        _rootpassword.value = ""
        _firstName.value = ""
        _lastName.value = ""
        _emailAddress.value = ""
        _address.value = ""
        _code.value = ""
        _contact.value = ""
        _parentguid.value = ""
        _roleguid.value = ""
        _timezoneguid.value = "E39F8004-31BC-42CE-B97A-C0668BD1F965"
        _newPassword.value = ""
        _confirmPassword.value = ""
        _city.value = ""
        _postalcode.value = ""
    }

    fun initializeWithContext(context: Context) {
        this.context = context
    }

    fun getCountryList() {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {
                dataRepositoryRepository.requestCountryList().collect() {
                    _ProgreeBarState.value = ProgressState.HIDE

                    if (it.data?.status == AppConstants.STATUS_200) {
                        _countryResponse.value = it.data!!
                        _country.value = _countryResponse.value?.data?.get(0)
                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun getStateList(countryGuid: String) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {
                dataRepositoryRepository.requestStateList(countryGuid).collect() {
                    _ProgreeBarState.value = ProgressState.HIDE
                    if (it.data?.status == AppConstants.STATUS_200) {
                        _stateResponse.value = it.data!!
                        _state.value = _stateResponse.value?.data?.get(0)
                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun getTokenResponse() {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {
                dataRepositoryRepository.requestGetToken().collect() {
                    if (it.data?.status == AppConstants.STATUS_200) {
                        _tokenResponse.value = it
                        _ProgreeBarState.value = ProgressState.SHOW
                        SharedPref.instance!!.saveStringSession(
                            context.getString(R.string.pref_auth_token),
                            _tokenResponse.value!!.data?.data
                        )
                        getLoginResponse(
                            LoginRequest(
                                _rootusername.value,
                                _rootpassword.value
                            )
                        )
                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        _ProgreeBarState.value = ProgressState.HIDE
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        _ProgreeBarState.value = ProgressState.HIDE
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }

    }

    fun getLoginResponse(loginRequest: LoginRequest) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {
                dataRepositoryRepository.requestLogin(loginRequest).collect() {

                    if (it.data?.status == AppConstants.STATUS_200) {
                        _ProgreeBarState.value = ProgressState.SHOW
                        _loginResponse.value = it
                        SharedPref.instance!!.saveStringSession(
                            context.getString(R.string.pref_bearer_token),
                            _loginResponse.value!!.data?.accessToken
                        )

                        getEntityLookUpList()

                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        _ProgreeBarState.value = ProgressState.HIDE
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        _ProgreeBarState.value = ProgressState.HIDE
                        showError(it.errorBody!!)
                    }
                    AndroidLog.d("token", _loginResponse.value!!.data?.accessToken.toString())
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun getEntityLookUpList() {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {

                dataRepositoryRepository.requestEntityLookUp().collect() {
                    if (it.data?.status == AppConstants.STATUS_200) {
                        _ProgreeBarState.value = ProgressState.SHOW
                        _entityLookUpResponse.value = it.data!!
                        for (i in 0 until (it.data.data?.size ?: 0)) {
                            if (it.data.data?.get(i)?.parentEntityGuid == null) {
                                _parentguid.value = it.data.data?.get(i)?.guid!!
                            }
                        }
                        getRoleLookUpList()

                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        _ProgreeBarState.value = ProgressState.HIDE
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        _ProgreeBarState.value = ProgressState.HIDE
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun getRoleLookUpList() {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {

                dataRepositoryRepository.requestRoleLookUp().collect() {
                    if (it.data?.status == AppConstants.STATUS_200) {
                        _ProgreeBarState.value = ProgressState.SHOW
                        _roleLookUpResponse.value = it.data!!

//                        for (i in 0 until (it.data.data?.size ?: 0)) {
//                            if (it.data.data?.get(i)?.description.equals("Mobile User", true)) {
//
//                                _roleguid.value = it.data.data?.get(i)?.guid!!
//                                break
//                            }
//                        }

                        for (i in 0 until (it.data.data?.size ?: 0)) {

                            if ((it.data.data?.get(i)?.description.equals("Mobile User", true))) {
                                _roleguid.value = it.data.data?.get(i)?.guid!!
                                break
                            }

                        }

                        if (!_roleguid.value!!.isEmpty()) {
                        var newEntityRequest: NewEntityRequest
                        var userInfo: NewEntityRequest.UserInfo
                        var contactNo = ""
                        if (!_contact.value?.isEmpty()!!) {
                            contactNo = _code.value + "-" + _contact.value
                        }
                        userInfo = NewEntityRequest.UserInfo(
                            contactNo,
                            _firstName.value,
                            true,
                            _lastName.value,
                            _roleguid.value,
                            false,
                            _timezoneguid.value,
                            _emailAddress.value?.trim(),
                            false,
                            _newPassword.value
                        )
                        //todo changed name from first name +last name to emailid....
                        newEntityRequest = NewEntityRequest(
                            _address.value,
                            _city.value,
                            _country.value?.guid,
                            _emailAddress.value?.trim(),
                            _parentguid.value,
                            _state.value?.guid,
                            _timezoneguid.value,
                            userInfo,
                            _postalcode.value
                        )

                        getAddNewEntityResponse(newEntityRequest)
                        } else {
                            var solutions: ArrayList<String>
                            solutions = ArrayList()
                            solutions.add(AppConstants.solution_guid)
                            var createRoleRequest =
                                CreateRoleRequest("Mobile User", "Mobile User", solutions)
                            createRoleApi(createRoleRequest)
                        }


                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        _ProgreeBarState.value = ProgressState.HIDE
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        _ProgreeBarState.value = ProgressState.HIDE
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun getAddNewEntityResponse(addNewEntityRequest: NewEntityRequest) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {

                dataRepositoryRepository.requestAddNewEntity(addNewEntityRequest).collect() {
                    _ProgreeBarState.value = ProgressState.HIDE
                    if (it.data?.status == AppConstants.STATUS_200) {
                        _addNewEntityResponse.value = it.data!!
                        showMsg(it.data.message.toString())
                        (context as AuthenticationActivity).navController.navigateUp()

                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun createRoleApi(createRoleRequest: CreateRoleRequest) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {


                dataRepositoryRepository.requestCreateRole(createRoleRequest).collect() {
                    _ProgreeBarState.value = ProgressState.HIDE
                    if (it.data?.status == AppConstants.STATUS_200) {
                        _createRoleResponse.value = it.data!!

                        _roleguid.value = it.data.data.get(0).newId
//                        showMsg(it.data.message.toString())

                        var newEntityRequest: NewEntityRequest
                        var userInfo: NewEntityRequest.UserInfo
                        var contactNo = ""
                        if (!_contact.value?.isEmpty()!!) {
                            contactNo = _code.value + "-" + _contact.value
                        }
                        userInfo = NewEntityRequest.UserInfo(
                            contactNo,
                            _firstName.value,
                            true,
                            _lastName.value,
                            _roleguid.value,
                            false,
                            _timezoneguid.value,
                            _emailAddress.value?.trim(),
                            false,
                            _newPassword.value
                        )
                        //todo changed name from first name +last name to emailid....
                        newEntityRequest = NewEntityRequest(
                            _address.value,
                            _city.value,
                            _country.value?.guid,
                            _emailAddress.value?.trim(),
                            _parentguid.value,
                            _state.value?.guid,
                            _timezoneguid.value,
                            userInfo,
                            _postalcode.value
                        )

                        getAddNewEntityResponse(newEntityRequest)


                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun getRolePermissionsApi(roleGuid: String) {
        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {
                dataRepositoryRepository.requestGetRolePermissions(roleGuid).collect() {
                    _ProgreeBarState.value = ProgressState.HIDE
                    if (it.data?.status == AppConstants.STATUS_200) {
                        _getRolePermissionsResponse.value = it.data!!
                        showMsg(it.data.message.toString())

                    } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                        showError(it.errorBody)
                        refereshTokenExpired()
                    } else {
                        showError(it.errorBody!!)
                    }
                }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    /**
     * Check validation for require filed
     */
    fun checkIsValid(): Boolean {
        return if (_rootusername.value.equals("", ignoreCase = true)) {
            showMsg(context.getString(R.string.alert_enter_root_email_id))
            false
        } else if (!ValidationUtils.isValidEmail(_rootusername.value)) {
            showMsg(context.getString(R.string.alert_root_invalid_email))
            false
        } else if (_rootpassword.value.equals("", ignoreCase = true)) {
            showMsg(context.getString(R.string.alert_enter_root_password))
            false
        } else if (!ValidationUtils.isValidPassword(_rootpassword.value.toString())) {
            showMsg(context.getString(R.string.alert_enter_valid_password))
            false
        } else if (_firstName.value.equals("", ignoreCase = true)) {
            showMsg(context.getString(R.string.alert_enter_first_name))
            false
        } else if (_lastName.value.equals("", ignoreCase = true)) {
            showMsg(context.getString(R.string.alert_enter_last_name))
            false
        } else if (_emailAddress.value.equals("", ignoreCase = true)) {
            showMsg(context.getString(R.string.alert_enter_email_id))
            false
        } else if (!ValidationUtils.isValidEmail(_emailAddress.value)) {
            showMsg(context.getString(R.string.alert_invalid_email))
            false
        } else if (_newPassword.value.equals("", ignoreCase = true)) {
            showMsg(context.getString(R.string.alert_enter_password))
            false
        } else if (!ValidationUtils.isValidPassword(_newPassword.value.toString())) {
            showMsg(context.getString(R.string.alert_enter_valid_password))
            false
        } else if (_confirmPassword.value.equals("", ignoreCase = true)) {
            showMsg(context.getString(R.string.alert_enter_confirm_password))
            false
        } else if (!_newPassword.value.equals(_confirmPassword.value)) {
            showMsg(context.getString(R.string.alert_password_matches))
            false
        }/* else if (_address.value.equals("", ignoreCase = true)) {
            showMsg(context.getString(R.string.alert_enter_address))
            false
        } else if (_contact.value.equals("", ignoreCase = true)) {
            showMsg(context.getString(R.string.alert_enter_contact))
            false
        } else if (_city.value.equals("", ignoreCase = true)) {
            showMsg(context.getString(R.string.alert_enter_city))
            false
        } else if (_postalcode.value.equals("", ignoreCase = true)) {
            showMsg(context.getString(R.string.alert_enter_postal_code))
            false
        }*/ else {
            true
        }
    }


    fun showMsg(msg: String) {
        val inflater: LayoutInflater = (context as BaseActivity).layoutInflater
        val layout: View =
            inflater.inflate(R.layout.custom_toast, (context as BaseActivity).findViewById(R.id.custom_toast_container))

        val text: TextView = layout.findViewById(R.id.tvToast)
        text.setText(msg)

        val toast = Toast(context)
        toast.setGravity(Gravity.BOTTOM, 0, 40)
        toast.duration = Toast.LENGTH_LONG
        toast.setView(layout)
        toast.show()
    }

    fun refereshTokenExpired() {
//        SharedPref.instance!!.deleteAllPreferences()
//        val intent = Intent(context, AuthenticationActivity()::class.java)
//        (context as BaseActivity).startActivity(intent)
//        (context as BaseActivity).finish()
    }

    fun showError(errorBody: ErrorRespose) {
        if (errorBody.error != null && errorBody.error!!.size > 0) {
            showMsg(errorBody.error!!.get(0)?.message.toString())
        } else {
            showMsg(errorBody.message.toString())
        }
    }

    override fun onCleared() {
        super.onCleared()
    }

}

