package com.avnet.ebv.viewmodel

import android.content.Context
import android.util.Log
import android.view.Gravity
import android.view.LayoutInflater
import android.view.View
import android.widget.TextView
import android.widget.Toast
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.avnet.ebv.EbvApplication
import com.avnet.ebv.R
import com.avnet.ebv.base.BaseActivity
import com.avnet.ebv.base.BaseViewModel
import com.avnet.ebv.common.SharedPref
import com.avnet.ebv.data.DeviceRepositorySource
import com.avnet.ebv.data.error.ErrorManager
import com.avnet.ebv.data.error.mapper.ErrorMapper
import com.avnet.ebv.data.remote.ApiConstant
import com.avnet.ebv.enums.ProgressState
import com.avnet.ebv.model.response.ErrorRespose
import com.avnet.ebv.model.response.UserIdentityResponse
import com.avnet.ebv.model.response.subscribe.PlanResponse
import com.avnet.ebv.model.response.subscribe.SubscriberResponse
import com.avnet.ebv.utils.AppConstants
import com.avnet.ebv.utils.NetworkUtils
import kotlinx.coroutines.flow.collect
import kotlinx.coroutines.launch
import javax.inject.Inject


class StatisticViewModel @Inject
constructor(private val deviceRepository: DeviceRepositorySource) : BaseViewModel() {

    lateinit var context: Context

    var _ProgreeBarState: MutableLiveData<ProgressState> = MutableLiveData()

    override val errorManager: ErrorManager
        get() = ErrorManager(ErrorMapper())

    fun initializeWithContext(context: Context) {
        this.context = context
    }

    lateinit var userDetails: UserIdentityResponse

    var customerId: MutableLiveData<String> = MutableLiveData()
    var plan = MutableLiveData<PlanResponse>()
    var subscribe = MutableLiveData<SubscriberResponse>()
    var messageTotal = MutableLiveData<String>()
    var deviceTotal = MutableLiveData<String>()
    var userTotal = MutableLiveData<String>()
    var userTotal2 = MutableLiveData<Double>()
    var deviceTotal2 = MutableLiveData<Double>()
    var messageTotal2 = MutableLiveData<Double>()

    init {
        userTotal2.value = 0.0
        deviceTotal2.value = 0.0
        messageTotal2.value = 0.0
    }


    fun getSubscribtionToken() {

        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {
                val params = HashMap<String, String?>()
                params[ApiConstant.clientId] = ApiConstant.clientIdValue
                params[ApiConstant.clientSecret] = ApiConstant.clientSecretValue
                params[ApiConstant.userName] = SharedPref.instance!!.getValueFromPreferences(
                    context.getString(R.string.pref_root_user_email)
                )

                deviceRepository.requestSubscribeToken(params)
                    .collect() {

                        if (it.data?.status == AppConstants.STATUS_200) {
                            SharedPref.instance!!.saveStringSession(
                                context.getString(R.string.pref_auth_token_subscribe),
                                it.data.accessToken
                            )

                            Log.d(
                                "token", SharedPref.instance!!.getValueFromPreferences(
                                    EbvApplication.context.getString(
                                        R.string.pref_auth_token_subscribe
                                    )
                                ).toString()
                            )
                            getConsumerId(
                                SharedPref.instance!!.getValueFromPreferences(
                                    EbvApplication.context.getString(
                                        R.string.pref_company_guid
                                    )
                                ).toString()
                            )

                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            _ProgreeBarState.value = ProgressState.HIDE
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            _ProgreeBarState.value = ProgressState.HIDE
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun getConsumerId(iotConnectCompanyId: String) {

        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {
                val params = HashMap<String, String?>()

                deviceRepository.requestConsumerId(iotConnectCompanyId)
                    .collect() {
                        customerId.value = it.data?.string().toString()
                        Log.d("customerId", customerId.value.toString())
                        getPlans()

                    }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun getSubscriber() {

        if (NetworkUtils.isNetworkAvailable(context)) {
            _ProgreeBarState.value = ProgressState.SHOW
            viewModelScope.launch {
                val params = HashMap<String, String?>()

                deviceRepository.requestSubscriber(
                    ApiConstant.environmentId,
                    customerId.value.toString()
                )
                    .collect() {

                        if (it.data?.status == AppConstants.STATUS_200) {
                            _ProgreeBarState.value = ProgressState.HIDE
                            subscribe.value = it.data!!

                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            _ProgreeBarState.value = ProgressState.HIDE
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            _ProgreeBarState.value = ProgressState.HIDE
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun getPlans() {
        if (NetworkUtils.isNetworkAvailable(context)) {
            viewModelScope.launch {
                val params = HashMap<String, String?>()
                params[ApiConstant.consumerid] = customerId.value.toString()
                params[ApiConstant.pageNo] = "1"
                params[ApiConstant.pageSize] = "10"
                deviceRepository.requestPlans(
                    ApiConstant.environmentId,
                    params
                )
                    .collect() {
                        if (it.data?.status == AppConstants.STATUS_200) {
                            plan.value = it.data!!
                            getSubscriber()

                        } else if (it.errorBody?.status == AppConstants.STATUS_401) {
                            _ProgreeBarState.value = ProgressState.HIDE
                            showError(it.errorBody)
                            refereshTokenExpired()
                        } else {
                            _ProgreeBarState.value = ProgressState.HIDE
                            showError(it.errorBody!!)
                        }
                    }
            }
        } else {
            showMsg(context.getString(R.string.str_alert_no_internet))
        }
    }

    fun showMsg(msg: String) {
        val inflater: LayoutInflater = (context as BaseActivity).layoutInflater
        val layout: View =
            inflater.inflate(R.layout.custom_toast, (context as BaseActivity).findViewById(R.id.custom_toast_container))

        val text: TextView = layout.findViewById(R.id.tvToast)
        text.setText(msg)

        val toast = Toast(context)
        toast.setGravity(Gravity.BOTTOM, 0, 40)
        toast.duration = Toast.LENGTH_LONG
        toast.setView(layout)
        toast.show()
    }

    fun refereshTokenExpired() {
//        SharedPref.instance!!.deleteAllPreferences()
//        val intent = Intent(context, AuthenticationActivity()::class.java)
//        (context as BaseActivity).startActivity(intent)
//        (context as BaseActivity).finish()
    }

    fun showError(errorBody: ErrorRespose) {
        if (errorBody.error != null && errorBody.error!!.size > 0) {
            showMsg(errorBody.error!!.get(0)?.message.toString())
        } else {
            showMsg(errorBody.message.toString())
        }
    }
}
